// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "cell/geom_data.h"
#include "utils/error_macros.h"
#include "utils/utility.h"
#include "utils/string_utils.h"
#include "utils/min_max.h"
#include <string>
#include <fstream>
#include <iostream>

//-----------------------------------------------------------------------------
void Geom_data::read_geom_data ( std::ifstream & IN, double r )
{
    if ( !IN.good() )
        cell_error ( "Unusable input stream." );

    std::string inputString;
    size_t n_points, n_faces, n_cells, n_face_points, n_cell_faces;
    int index;
    double x, y, z;

    IN >> inputString;

    //check header
    if ( inputString != "Cells" )
        cell_error ( ( "Unexpected file format. " + inputString ).c_str() );

    //read number of entites
    IN >> n_points >> n_faces >> n_cells;

    std::cout << n_points << " points; " << n_faces << " faces; " << n_cells << " cells\n";

    //read points
    points.reserve ( n_points );
    std::vector< Point > new_points;

    std::vector< Point > *pts_p;

    if ( r > 0.0 )
    {
        new_points.reserve ( n_points );
        pts_p = &new_points;
    }
    else
        pts_p = &points;

    double max_cor[3], min_cor[3], diff[3];
    for ( int i = 0; i < n_points; ++i )
    {
        IN >> x >> y >> z;
        pts_p->push_back ( Point ( x, y, z ) );
        if ( r > 0.0 )
        {
            if ( i == 0 )
            {
                max_cor[0] = min_cor[0] = x;
                max_cor[1] = min_cor[1] = y;
                max_cor[2] = min_cor[2] = z;
            }
            else
            {
                max_cor[0] = minmax::max ( max_cor[0], x );
                min_cor[0] = minmax::min ( min_cor[0], x );
                max_cor[1] = minmax::max ( max_cor[1], y );
                min_cor[1] = minmax::min ( min_cor[1], y );
                max_cor[2] = minmax::max ( max_cor[2], z );
                min_cor[2] = minmax::min ( min_cor[2], z );
            }
        }
    }

    //randomize points
    if ( r > 0.0 )
    {
        r = minmax::min ( r, 0.5 );
        srand ( ( unsigned ) time ( NULL ) );

        std::vector< Point >::iterator p_iter, p_end;

        for ( p_iter = pts_p->begin(), p_end = pts_p->end(); p_iter != p_end; ++p_iter )
        {
            double X = rand() * r * ( max_cor[0] - min_cor[0] ) / RAND_MAX;
            double Y = rand() * r * ( max_cor[1] - min_cor[1] ) / RAND_MAX;
            double Z = rand() * r * ( max_cor[2] - min_cor[2] ) / RAND_MAX;

            Vector v ( X, Y, Z );
            points.push_back ( *p_iter + v );
        }
    }
    //read faces points indices

    face_points_ind.reserve ( n_faces );
    std::vector< size_t > points_indices;

    for ( int i = 0; i < n_faces; ++i )
    {
        IN >> n_face_points;

        points_indices.clear();
        points_indices.reserve ( n_face_points );

        for ( int j = 0; j < n_face_points; ++j )
        {
            IN >> index;
            points_indices.push_back ( index );
        }
        face_points_ind.push_back ( points_indices );
    }
    //read cells
    face_to_cells_map.resize ( n_faces, Index_pair ( -1, -1 ) );
    cell_faces_ind.reserve ( n_cells );
    std::vector< Index > faces_indices;

    for ( int i = 0; i < n_cells; ++i )
    {
        bool reversed = false;

        IN >> n_cell_faces;

        faces_indices.clear();
        faces_indices.reserve ( n_cell_faces );
        for ( int j = 0; j < n_cell_faces; ++j )
        {
            IN >> inputString;
            from_string< int > ( index, inputString );

            if ( inputString[0] == '-' )
            {
                reversed = true;
                index = -index;
                face_to_cells_map[ index ].second = i;
            }
            else
            {
                reversed = false;
                face_to_cells_map[ index ].first = i;
            }
            faces_indices.push_back ( Index ( index, reversed ) );
        }
        cell_faces_ind.push_back ( faces_indices );
    }

    //group faces to walls if needed

    if ( IN >> inputString && inputString != "Walls" )
        cell_error ( ( "Unexpected file format. " + inputString ).c_str() );

    int n;
    while ( IN >> n )
    {
        walls.push_back ( std::set< size_t >() );
        for ( int i = 0; i < n; ++ i )
        {
            size_t ind;
            IN >> ind;
            walls.back().insert ( ind );
        }
    }
}
//-----------------------------------------------------------------------------
void Geom_data::read_mesh_file ( std::ifstream & in, Index_on_vec_function &funct )
{
    if ( !in.good() )
        cell_error ( "Unusable input stream." );

    size_t id, n_points, n_faces, n_cells, n_face_points, n_cell_faces, n1, n2;
    double x, y, z;
    std::string text;

    in >> n_points >> text;
    if ( text != "nodes" )
        cell_error ( ( "Unexpected file format. " + text ).c_str() );

    points.resize ( n_points );

    for ( int i = 0; i < n_points; ++i )
    {
        in >> id >> text >> x >> y >> z;
        if ( text != ":" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );
        if ( id >= points.size() )
            points.resize ( id+1 );
        points[id] = Point ( x, y, z );
    }

    in >> n_faces >> text;
    if ( text != "faces" )
        cell_error ( ( "Unexpected file format. " + text ).c_str() );

    face_points_ind.resize ( n_faces );
    std::vector< size_t > points_indices;

    for ( int i = 0; i < n_faces; ++i )
    {
        in >> id >> text >> n_face_points;
        if ( text != ":" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );
        in >> text;
        if ( text != "," )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );

        if ( id >= face_points_ind.size() )
            face_points_ind.resize ( id+1 );

        points_indices.clear();
        points_indices.reserve ( n_face_points );

        for ( int j = 0; j < n_face_points; ++j )
        {
            size_t index;
            in >> index;
            points_indices.push_back ( index );
        }
        face_points_ind[id] = points_indices;
    }

    face_to_cells_map.resize ( n_faces, Index_pair ( -1, -1 ) );

    in >> n_cells >> text;
    if ( in.eof() )
        return;
    if ( text != "elements" )
        cell_error ( ( "Unexpected file format. " + text ).c_str() );

    cell_faces_ind.resize ( n_cells );
    std::vector< Index > faces_indices;
    for ( int i = 0; i < n_cells; ++i )
    {
        in >> id >> text >> n_cell_faces;
        if ( text != ":" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );
        in >> text;
        if ( text != "," )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );

        if ( id >= cell_faces_ind.size() )
            cell_faces_ind.resize ( id+1 );

        faces_indices.clear();
        faces_indices.reserve ( n_cell_faces );
        for ( int j = 0; j < n_cell_faces; ++j )
        {
            int index;
            bool reversed;

            in >> text;
            from_string< int > ( index, text );

            if ( text[0] == '-' )
            {
                reversed = true;
                index = -index;
                face_to_cells_map[ index ].second = i;
            }
            else
            {
                reversed = false;
                face_to_cells_map[ index ].first = i;
            }
            faces_indices.push_back ( Index ( index, reversed ) );
        }
        cell_faces_ind[id] = faces_indices;

        in >> text >> n1;
        if ( text != "(" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );
        in >> text >> n2;
        if ( text != "," )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );
        in >> text;
        if ( text != ")" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );

        funct.resize ( n_cells );
        std::vector< size_t > v ( 2 );
        v[0] = n1;
        v[1] = n2;

        funct.set ( id, v );
    }

}
//-----------------------------------------------------------------------------
void Geom_data::statistics ( std::ostream &out, bool detailed ) const
{
    out << "Geom data: " << this << " statistics.\n";
    out << "number of points: " << numb_points() << "\n";
    out << "number of faces: " << numb_faces() << "\n";


    if ( detailed )
    {
        for ( size_t i = 0; i < numb_faces(); ++i )
        {
            out << i << " : " << numb_face_points ( i ) << ",";
            Face_point_ind_const_iterator f_iter, f_end;
            for ( f_iter = face_point_ind_begin ( i ), f_end = face_point_ind_end ( i ); f_iter != f_end; ++f_iter )
            {
                out << " " << *f_iter;
            }
            out << "\n";
        }
    }
    out << "number of cells: " << numb_cells() << "\n";
    if ( detailed )
    {
        for ( size_t i = 0; i < numb_cells(); ++i )
        {
            out << i << " : " << numb_cell_faces ( i ) << ",";

            Cell_face_ind_const_iterator c_iter, c_end;
            for ( c_iter = cell_face_ind_begin ( i ), c_end = cell_face_ind_end ( i ); c_iter != c_end; ++c_iter )
            {
                out << " ";
                if ( c_iter->reversed() )
                    out << "-";
                out << c_iter->index();
            }
            out << "\n";
        }
    }
}
//-----------------------------------------------------------------------------
