// Copyright (C) 2010 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#ifndef _AFFINE_MAP_SHELL_H_
#define _AFFINE_MAP_SHELL_H_

#include "utils/types.h"
#include "cell/cell_handle.h"
#include "material/generic_material.h"
#include "matrix/DenseMatrix.h"
#include "matrix/Matrix3.h"
#include "matrix/rot_mat.h"
#include "fem/quadrature_rule.h"
#include "fem/quadrature_rule_surface.h"
#include "fem/Finite_element.h"
#include "utils/string_utils.h"

/// This class represents the affine map from the reference element to the current element for the shells

#undef DEBUG_PRINT
// #define DEBUG_PRINT

template<class ME>
class Affine_map_shell
{
public:
  typedef Mesh::Face_iterator Element_iterator;
  typedef Mesh::Face_const_iterator Element_const_iterator;
  typedef ME Mesh_element;

  Element_iterator elements_begin( Mesh& m ) {
    return m.faces_begin();
  }
  Element_const_iterator elements_begin( Mesh& m ) const {
    return m.faces_begin();
  }
  Element_iterator elements_end( Mesh& m ) {
    return m.faces_end();
  }
  Element_const_iterator elements_end( Mesh& m ) const {
    return m.faces_end();
  }

  enum Update_flags
  {
    UPDATE_NONE     = 0,
    UPDATE_J0       = 1,
    UPDATE_J        = 2,
    UPDATE_DEFGRAD  = 5,
    UPDATE_ALL      = 7
  };
  /// Constructor
  Affine_map_shell(Finite_element const* fe, Quadrature_rule const* qr, Quadrature_rule_surface const* sqr, cell::Tissue_handle const& t = cell::INVALID_TISSUE);
  /// Destructor
  ~Affine_map_shell();
  /// Get quadrature rule for the map
  Quadrature_rule const& q_rule() const {
    return *m_qrule;
  }
  /// Get surface quadrature rule for the map
  Quadrature_rule_surface const& surf_q_rule() const {
    return *m_qrule_surf;
  }
  /// Get positions of dofs at given Mesh_node in global matrix/vector
  inline void vertexmap(size_t *vertex_nodes, Mesh_node const *vertex, const Mesh& mesh) const;
  /// Compute map from local to global degrees of freedom
  void nodemap(size_t *nodes, const ME& cell, const Mesh& mesh) const;
  /// Compute map from local to global coordinates
  void pointmap(Point *points, const size_t *components) const;
  /// Update map for current element
  void update(ME const& cell);
  /// Return cell of current element
  inline const ME& cell() const {
    return *m_cell;
  }
  /// Return associated finite element
  inline const Finite_element& element() const {
    return *m_fe;
  }
  /// Return associated tissue
  inline const cell::Tissue_handle& tissue() const {
    return m_tissue;
  }
  /// Return material properties of the compartment of current cell
  Generic_material const* material() const {
    return m_material;
  }
  /// determinant of jacobian of initial deformatrion at integration points
  double det0(int i) const {
    return m_det0[i];
  }
  /// determinant of jacobian of current deformation at integration points
  double det(int i) const {
    return m_det[i];
  }
  /// determinant of deformation gradient at integration points
  double def_grad_det(int i) const {
    return m_dg_det[i];
  }
  /// jacobian of initial deformatrion at integration points
  DenseMatrix const& J0(int i) const {
    return m_Jacobian0[i];
  }
  /// jacobian of current deformation at integration points
  DenseMatrix const& J(int i) const {
    return m_Jacobian[i];
  }
  /// inverse jacobian of initial deformatrion at integration points
  DenseMatrix const& invJ0(int i) const {
    return m_Jacobian0[i];
  }
  /// inverse jacobian of current deformation at integration points
  DenseMatrix const& invJ(int i) const {
    return m_Jacobian[i];
  }
  Matrix3 const& def_grad(int i) const {
    return m_def_gradient[i];
  }
  /// get the right Cauchy-Green tensor, C = Ft*F at integration point
  void right_cauchy_green( Matrix3s &C, int i ) const;
  /// get the left Cauchy-Green tensor, b = F*Ft at integration point
  void left_cauchy_green( Matrix3s &b, int i ) const;
  /// get the cauchy stress at integration point i
  Matrix3s const& stress(int i) const {
    return m_stress[i];
  }
  /// Z coordinate of i-th integration point
  double int_point_Z(int i) const {
    return m_z[i];
  }
  /// initial thickness of shell at node i
  double h0(int i) const {
    return m_h0[i];
  }
  /// initial point of current cell at node i
  Point const& P0(int i) const {
    return m_p0[i];
  }
  /// current point of current cell at node i
  Point const& P(int i) const {
    return m_p[i];
  }
  /// initial director of current cell at node i
  Vector const& D0(int i) const {
    return m_D0[i];
  }
  /// current director of current cell at node i
  Vector const& D(int i) const {
    return m_D[i];
  }
  /// shape function global derivatives matrix (B-matrix) at integration points and element nodes
  DenseMatrix const& sfd_matrix(int i, int a) const {
    return sfd_matrix_v[i][a];
  }
  /// Get update flag
  Update_flags flag() const {
    return m_flag;
  }
  ///S et update flag
  void set_flag( Update_flags f ) {
    m_flag = f;
  }
  /// Get shape function value at integration point l and node a
  double shape_fun(int l, int a) const {
    return shape_f[l][a];
  }
  /// Get shape function local derivative in 1st direction at integration point l and node a
  double shape_fun_der_x(int l, int a) const {
    return shape_f_x[l][a];
  }
  /// Get shape function local derivative in 2nd direction at integration point l and node a
  double shape_fun_der_y(int l, int a) const {
    return shape_f_y[l][a];
  }
  /// Get shape function vector at integration point l
  std::vector<double> const& shape_fun(int l) const {
    return shape_f[l];
  }
  /// Get shape function local derivative vector in 1st direction at integration point l
  std::vector<double> const& shape_fun_der_x(int l) const {
    return shape_f_x[l];
  }
  /// Get shape function local derivative vector in 2nd direction at integration point l
  std::vector<double> const& shape_fun_der_y(int l) const {
    return shape_f_y[l];
  }
  ///update stress at integration points for element. Needs update of the map for element first
  void update_stress();
  
  void update_local_coord(int n_el_nodes, int _int_pts);
  ///get coordinate rotation matrix to lamina coordinate system at integration point
  rot_mat const& coord_rot(int i) const {
    return m_coord_rot[i];
  }
  private:
    /// Current cell
    ME const* m_cell;
    /// Associated finite element
    Finite_element const *m_fe;
    /// Material properties of current cell
    Generic_material const *m_material;
    /// Associated quadrature rule
    Quadrature_rule const *m_qrule;
    /// Associated quadrature rule
    Quadrature_rule_surface const *m_qrule_surf;

    /// shape functions values at body integration points [n_int_pts][n_elem_nodes]
    std::vector< std::vector<double> > shape_f;
    /// shape functions local deriviatives values at body integration points, x=xi, y=eta, z=zeta
    std::vector< std::vector<double> > shape_f_x;
    std::vector< std::vector<double> > shape_f_y;
    ///z position of each integration point in body coordinates
    std::vector<double> m_z;
    ///integration weights
    double *weights;
    ///Tissue pointer for definition of compartment materials
    cell::Tissue_handle m_tissue;
    ///shape function global derivatives matrix (B-matrix) at integration points and element nodes
    std::vector< std::vector<DenseMatrix> >sfd_matrix_v;
    ///flag for different update modes
    Update_flags m_flag;
    ///Jacobian of current deformation from reference shape
    std::vector< DenseMatrix > m_Jacobian;
    ///inverse Jacobian of current deformation from reference shape
    std::vector< DenseMatrix > m_inv_Jacobian;
    std::vector<double> m_det;
    ///Jacobian of initial deformation from reference shape
    std::vector< DenseMatrix > m_Jacobian0;
    ///Jacobian of initial deformation from reference shape
    std::vector< DenseMatrix > m_inv_Jacobian0;
    std::vector<double> m_det0;
    std::vector< Matrix3 > m_def_gradient;
    std::vector<double> m_dg_det;
    /// Vertices of current cell
    std::vector<Point> m_p;
    /// Directors of current cell
    std::vector<Vector> m_D;
    /// Initial thicknes of current cell
    std::vector<double> m_h0;
    /// Initial vertices of current cell
    std::vector<Point> m_p0;
    /// Initial directors of current cell
    std::vector<Vector> m_D0;
    /// Cauchy stress at each integration point in global coordinates
    std::vector<Matrix3s> m_stress;
    /// Rotation matrix from local to global lamina coordinates at each integration point
    std::vector<rot_mat> m_coord_rot ;

    void fill_sdf_matrix_v();
    void zero_sdf_matrix_v();
    //global coordinates element points and shape functions need to be updated for this function
    void fill_coord_rot(int i, rot_mat& r, int nen);
    //calculate jacobian, its inverse  and determiant
    double calculate_jacobian(DenseMatrix &J, DenseMatrix &Ji, int j, int n_elem_nodes, std::vector<Point> const& m_p, std::vector<Vector> const& m_D);

    };
    //-----------------------------------------------------------------------------
    template<class ME>
    Affine_map_shell<ME>::Affine_map_shell(Finite_element const* fe, Quadrature_rule const* qr, Quadrature_rule_surface const* sqr, cell::Tissue_handle const& t ) : m_fe(fe), m_qrule(qr), m_qrule_surf(sqr)/*, m_cof_v(std::vector< std::vector< double > >(3))*/, m_tissue(t)
    {
      size_t n_node_dof = fe->tensordim();
      size_t n_int_pts = qr->spacedim();
      size_t n_elem_nodes = fe->spacedim()/n_node_dof;

      sfd_matrix_v.resize( n_int_pts, std::vector<DenseMatrix>(n_elem_nodes, DenseMatrix(6, n_node_dof) ) );
      zero_sdf_matrix_v();

      Point *integration_points = new Point[n_int_pts];
      weights = new double[n_int_pts];
      qr->points(integration_points);
      qr->weights(weights);
      m_z.resize( n_int_pts );

      shape_f.resize(n_int_pts, std::vector<double>(n_elem_nodes) );
      shape_f_x.resize(n_int_pts, std::vector<double>(n_elem_nodes) );
      shape_f_y.resize(n_int_pts, std::vector<double>(n_elem_nodes) );
      //make space for data updated for each mesh element
      m_Jacobian.resize( n_int_pts, DenseMatrix(3,3) );
      m_inv_Jacobian.resize( n_int_pts, DenseMatrix(3,3) );
      m_det.resize( n_int_pts );

      m_Jacobian0.resize( n_int_pts, DenseMatrix(3,3) );
      m_inv_Jacobian0.resize( n_int_pts, DenseMatrix(3,3) );
      m_det0.resize( n_int_pts );

      m_def_gradient.resize( n_int_pts );
      m_dg_det.resize( n_int_pts );
      m_stress.resize( n_int_pts );
      m_coord_rot.resize( n_int_pts );

      m_p.resize(n_elem_nodes);
      m_D.resize(n_elem_nodes);

      m_h0.resize(n_elem_nodes);
      m_p0.resize(n_elem_nodes);
      m_D0.resize(n_elem_nodes);

      for (int j = 0; j < n_int_pts; j++ )
      {
        double int_p[2] = {integration_points[j].x(), integration_points[j].y()};
        m_z[j] = integration_points[j].z();

        for (int i = 0; i < n_elem_nodes; i++)
        {

          shape_f[j][i] = m_fe->function(i, int_p);
          shape_f_x[j][i] = m_fe->derivative(i, int_p, Finite_element::X);
          shape_f_y[j][i] = m_fe->derivative(i, int_p, Finite_element::Y);
        }
      }
      delete [] integration_points;
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    Affine_map_shell<ME>::~Affine_map_shell()
    {
      delete [] weights;
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::fill_sdf_matrix_v()
    {
      size_t n_node_dof = m_fe->tensordim();
      size_t n_elem_nodes = m_fe->spacedim()/n_node_dof;
      size_t n_int_pts = m_qrule->spacedim();

      double Nx, Ny, Nz, Mx, My, Mz;
      for (int j = 0; j < n_int_pts; ++j)
      {
        std::vector<double> const& Nr = shape_f_x[j];
        std::vector<double> const& Ns = shape_f_y[j];
        std::vector<double> const& N = shape_f[j];
        DenseMatrix const& J = m_inv_Jacobian[j];
        double gt = m_z[j];
        for ( int a = 0; a < n_elem_nodes; ++a )
        {
          double vNr = Nr[a];
          double vNs = Ns[a];
          double vN = N[a];
          double za = 0.5*gt*m_h0[a];

          double temp1 = J(0,0)*vNr+J(1,0)*vNs;
          double temp2 = 0.5*m_h0[a]*vN;
          Nx = temp1;
          Mx = za*temp1 + J(2,0)*temp2;

          temp1 = J(0,1)*vNr+J(1,1)*vNs;
          Ny = temp1;
          My = za*temp1 + J(2,1)*temp2;

          temp1 = J(0,2)*vNr+J(1,2)*vNs;
          Nz = temp1;
          Mz = za*temp1 + J(2,2)*temp2;

          //0th row
          sfd_matrix_v[j][a].set(0, 0, Nx);
          sfd_matrix_v[j][a].set(0, 3, Mx);
          //1st row
          sfd_matrix_v[j][a].set(1, 1, Ny);
          sfd_matrix_v[j][a].set(1, 4, My);
          //2nd row
          sfd_matrix_v[j][a].set(2, 2, Nz);
          sfd_matrix_v[j][a].set(2, 5, Mz);
          //3rd row
          sfd_matrix_v[j][a].set(3, 0, Ny);
          sfd_matrix_v[j][a].set(3, 1, Nx);
          sfd_matrix_v[j][a].set(3, 3, My);
          sfd_matrix_v[j][a].set(3, 4, Mx);
          //4th row
          sfd_matrix_v[j][a].set(4, 1, Nz);
          sfd_matrix_v[j][a].set(4, 2, Ny);
          sfd_matrix_v[j][a].set(4, 4, Mz);
          sfd_matrix_v[j][a].set(4, 5, My);
          //5th row
          sfd_matrix_v[j][a].set(5, 0, Nz);
          sfd_matrix_v[j][a].set(5, 2, Nx);
          sfd_matrix_v[j][a].set(5, 3, Mz);
          sfd_matrix_v[j][a].set(5, 5, Mx);
        }
      }
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::zero_sdf_matrix_v()
    {
      size_t n_elem_nodes = m_fe->spacedim()/m_fe->tensordim();
      int size = sfd_matrix_v.size();

      for ( int i = 0; i < size; ++i )
      {
        for ( int a = 0; a < n_elem_nodes; ++a )
          sfd_matrix_v[i][a].zero();
      }
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    double Affine_map_shell<ME>::calculate_jacobian(DenseMatrix &J, DenseMatrix &Ji, int j, int n_elem_nodes, std::vector<Point> const& m_p, std::vector<Vector> const& m_D)
    {
      J(0, 0) = J(0, 1) = J(0, 2) = 0.0;
      J(1, 0) = J(1, 1) = J(1, 2) = 0.0;
      J(2, 0) = J(2, 1) = J(2, 2) = 0.0;

      std::vector<double> const& Hrj = shape_f_x[j];
      std::vector<double> const& Hsj = shape_f_y[j];
      std::vector<double> const& Hj = shape_f[j];

      #ifdef DEBUG_PRINT
      std::ofstream d_out( (globals.outDir + "det_matrix.txt").c_str() );
      std::ofstream s_out( (globals.outDir + "setup.txt").c_str() );
      #endif

      double det;
      for ( int i = 0; i < n_elem_nodes; ++i )
      {
        const double& Hri = Hrj[i];
        const double& Hsi = Hsj[i];
        const double& Hi = Hj[i];

        const double& x = m_p[i].x();
        const double& y = m_p[i].y();
        const double& z = m_p[i].z();

        const double& dx = m_D[i].x();
        const double& dy = m_D[i].y();
        const double& dz = m_D[i].z();

        double za = 0.5*m_z[j]*m_h0[i];
        double temp = 0.5*m_h0[i]*Hi;

        J(0, 0) += Hri*x + Hri*za*dx;
        J(0, 1) += Hsi*x + Hsi*za*dx;
        J(0, 2) += temp*dx;
        J(1, 0) += Hri*y + Hri*za*dy;
        J(1, 1) += Hsi*y + Hsi*za*dy;
        J(1, 2) += temp*dy;
        J(2, 0) += Hri*z + Hri*za*dz;
        J(2, 1) += Hsi*z + Hsi*za*dz;
        J(2, 2) += temp*dz;
      }
      // determinant
      det = J(0, 0)*(J(1, 1)*J(2, 2) - J(1, 2)*J(2, 1)) +
      J(0, 1)*(J(1, 2)*J(2, 0) - J(2, 2)*J(1, 0)) +
      J(0, 2)*(J(1, 0)*J(2, 1) - J(1, 1)*J(2, 0));

      if (det != 0)
      {
        double deti = 1.0 / det;

        Ji(0, 0) =  deti*(J(1, 1)*J(2, 2) - J(1, 2)*J(2, 1));
        Ji(1, 0) =  deti*(J(1, 2)*J(2, 0) - J(1, 0)*J(2, 2));
        Ji(2, 0) =  deti*(J(1, 0)*J(2, 1) - J(1, 1)*J(2, 0));

        Ji(0, 1) =  deti*(J(0, 2)*J(2, 1) - J(0, 1)*J(2, 2));
        Ji(1, 1) =  deti*(J(0, 0)*J(2, 2) - J(0, 2)*J(2, 0));
        Ji(2, 1) =  deti*(J(0, 1)*J(2, 0) - J(0, 0)*J(2, 1));

        Ji(0, 2) =  deti*(J(0, 1)*J(1, 2) - J(1, 1)*J(0, 2));
        Ji(1, 2) =  deti*(J(0, 2)*J(1, 0) - J(0, 0)*J(1, 2));
        Ji(2, 2) =  deti*(J(0, 0)*J(1, 1) - J(0, 1)*J(1, 0));
      }
      else
      {
        cell_error("Zero Jacobian");
      }
      return fabs(det);
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>:: update(ME const& cell)
    {
      m_cell = &cell;
      m_material = cell.material();
      typename ME::Edge_use_const_circulator n_circ = cell.edge_uses_begin(), n_start = cell.edge_uses_begin();
      int i = 0;
      do {
        Node_handle nh = n_circ->node();
        m_p[i] = nh->point();
        m_D[i] = nh->director();
        m_p0[i] = nh->point0();
        m_D0[i] = nh->director0();
        m_h0[i] = nh->h0();

        ++i;
      } while (++n_circ != n_start);

      size_t n_node_dof = m_fe->tensordim();
      size_t n_elem_nodes = m_fe->spacedim()/n_node_dof;
      size_t n_int_pts = m_qrule->spacedim();
      size_t int_shape_dim = m_qrule->spacedim();

      update_local_coord(n_elem_nodes, n_int_pts);
      if (i != n_elem_nodes)
        cell_error(("Face: " + to_string(cell.id()) + " : Wrong number of face nodes." + to_string(i) + " vs " + to_string(n_elem_nodes)).c_str());

      for (int j = 0; j < n_int_pts; ++j)
      {
        if (m_flag & UPDATE_J)
        {
          m_det[j] = calculate_jacobian( m_Jacobian[j], m_inv_Jacobian[j], j, n_elem_nodes, m_p, m_D);
          fill_sdf_matrix_v();
        }

        if ((m_flag & UPDATE_J0) || (m_flag & UPDATE_DEFGRAD))
        {
          m_det0[j] = calculate_jacobian( m_Jacobian0[j], m_inv_Jacobian0[j], j, n_elem_nodes, m_p0, m_D0);
        }
        if (m_flag & UPDATE_DEFGRAD)
        {
          double NX, NY, NZ, MX, MY, MZ;

          std::vector<double> const& Hrj = shape_f_x[j];
          std::vector<double> const& Hsj = shape_f_y[j];
          std::vector<double> const& Hj = shape_f[j];

          Matrix3 &F = m_def_gradient[j];
          DenseMatrix const& J0i = m_inv_Jacobian0[j];

          F(0, 0) = F(0, 1) = F(0, 2) = 0;
          F(1, 0) = F(1, 1) = F(1, 2) = 0;
          F(2, 0) = F(2, 1) = F(2, 2) = 0;

          for ( int i = 0; i < n_elem_nodes; ++i )
          {
            const double& Hri = Hrj[i];
            const double& Hsi = Hsj[i];
            const double& Hi = Hj[i];

            const double& x = m_p[i].x();
            const double& y = m_p[i].y();
            const double& z = m_p[i].z();

            const double& dx = m_D[i].x();
            const double& dy = m_D[i].y();
            const double& dz = m_D[i].z();

            double za = 0.5*m_z[j]*m_h0[i];
            double temp = 0.5*m_h0[i]*Hi;

            NX = J0i(0, 0)*Hri+J0i(1, 0)*Hsi;
            NY = J0i(0, 1)*Hri+J0i(1, 1)*Hsi;
            NZ = J0i(0, 2)*Hri+J0i(1, 2)*Hsi;

            MX = za*J0i(0, 0)*Hri + za*J0i(1, 0)*Hsi + J0i(2, 0)*temp;
            MY = za*J0i(0, 1)*Hri + za*J0i(1, 1)*Hsi + J0i(2, 1)*temp;
            MZ = za*J0i(0, 2)*Hri + za*J0i(1, 2)*Hsi + J0i(2, 2)*temp;

            //deformation gradient F
            F(0, 0) += NX*x + MX*dx;
            F(0, 1) += NY*x + MY*dx;
            F(0, 2) += NZ*x + MZ*dx;
            F(1, 0) += NX*y + MX*dy;
            F(1, 1) += NY*y + MY*dy;
            F(1, 2) += NZ*y + MZ*dy;
            F(2, 0) += NX*z + MX*dz;
            F(2, 1) += NY*z + MY*dz;
            F(2, 2) += NZ*z + MZ*dz;
          }

          m_dg_det[j] = F(0, 0)*(F(1, 1)*F(2, 2) - F(1, 2)*F(2, 1)) +
          F(0, 1)*(F(1, 2)*F(2, 0) - F(2, 2)*F(1, 0)) +
          F(0, 2)*(F(1, 0)*F(2, 1) - F(1, 1)*F(2, 0));
        }
      }
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::nodemap(size_t *nodes, const ME& cell, const Mesh& mesh) const
    {
      m_fe->nodemap(nodes, cell, mesh);
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::vertexmap(size_t *vertex_nodes, Mesh_node const *vertex, const Mesh& mesh) const
    {
      m_fe->vertexmap(vertex_nodes, vertex, mesh);
    }
    //-----------------------------------------------------------------------------
    /// get the right Cauchy-Green tensor, C = Ft*F
    template<class ME>
    void Affine_map_shell<ME>::right_cauchy_green( Matrix3s &C, int i ) const
    {
      Matrix3 const& F = def_grad(i);
      C.xx() = F(0,0)*F(0,0)+F(1,0)*F(1,0)+F(2,0)*F(2,0); // = C(0,0)
      C.yy() = F(0,1)*F(0,1)+F(1,1)*F(1,1)+F(2,1)*F(2,1); // = C(1,1)
      C.zz() = F(0,2)*F(0,2)+F(1,2)*F(1,2)+F(2,2)*F(2,2); // = C(2,2)
      C.xy() = F(0,0)*F(0,1)+F(1,0)*F(1,1)+F(2,0)*F(2,1); // = C(0,1)
      C.yz() = F(0,1)*F(0,2)+F(1,1)*F(1,2)+F(2,1)*F(2,2); // = C(1,2)
      C.xz() = F(0,0)*F(0,2)+F(1,0)*F(1,2)+F(2,0)*F(2,2); // = C(0,2)
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::left_cauchy_green( Matrix3s &b, int i ) const
    {
      Matrix3 const& F = def_grad(i);
      b.xx() = F(0,0)*F(0,0)+F(0,1)*F(0,1)+F(0,2)*F(0,2); // = b(0,0)
      b.yy() = F(1,0)*F(1,0)+F(1,1)*F(1,1)+F(1,2)*F(1,2); // = b(1,1)
      b.zz() = F(2,0)*F(2,0)+F(2,1)*F(2,1)+F(2,2)*F(2,2); // = b(2,2)
      b.xy() = F(0,0)*F(1,0)+F(0,1)*F(1,1)+F(0,2)*F(1,2); // = b(0,1)
      b.yz() = F(1,0)*F(2,0)+F(1,1)*F(2,1)+F(1,2)*F(2,2); // = b(1,2)
      b.xz() = F(0,0)*F(2,0)+F(0,1)*F(2,1)+F(0,2)*F(2,2); // = b(0,2)
    }
    //-----------------------------------------------------------------------------
    //map should be updated prior to this
    template<class ME>
    void Affine_map_shell<ME>::update_stress()
    {
      assert(material() != NULL);
      size_t nint = q_rule().spacedim();
      for ( int l = 0; l < nint; ++l )
      {
        material()->stress( m_stress[l], *this, l );
      }
    }
    //-----------------------------------------------------------------------------
    //global coordinates element points and shape functions need to be updated for this function
    template<class ME>
    void Affine_map_shell<ME>::fill_coord_rot(int j, rot_mat& r, int nen)
    {
      std::vector<double> const& Hrj = shape_f_x[j];
      std::vector<double> const& Hsj = shape_f_y[j];
      Vector3 t1, t2;
      for ( int i = 0; i < nen; ++i )
      {
        const double& Hri = Hrj[i];
        const double& Hsi = Hsj[i];
        const Vector3 x(m_p[i].x(), m_p[i].y(), m_p[i].z());

        t1 += x * Hri;
        t2 += x * Hsi;
      }
      Vector3 p = t1^t2;
      p.unit();
      Vector3 ta = t1+t2;
      ta.unit();
      Vector3 tb = p^ta;
      tb.unit();

      Vector3 tl1 = (ta - tb) * HALF_SQRT2;
      Vector3 tl2 = (ta + tb) * HALF_SQRT2;
      m_coord_rot[j].assign( tl1, tl2, p );
    }
    //-----------------------------------------------------------------------------
    template<class ME>
    void Affine_map_shell<ME>::update_local_coord(int n_el_nodes, int n_int_pts)
    {
      //get local lamina system
      for (int j = 0; j < n_int_pts; ++j)
      {
        fill_coord_rot(j ,m_coord_rot[j], n_el_nodes);
      }
    }
    //-----------------------------------------------------------------------------
    #endif
    