// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _BC_EVALUATOR_H_
#define _BC_EVALUATOR_H_

#include "mesh/mesh.h"
#include "fem/Boundary_condition.h"
#include "fem/direct_boundary_condition.h"
#include "fem/Boundary_value.h"
#include "utils/function_objects.h"
#include <iterator>
#include <set>
#include <list>
//-----------------------------------------------------------------------------
template<class ME>
struct BC_evaluator_element
{
    BC_evaluator_element( Node_handle n) : m_node_p(n) {}
    BC_evaluator_element() {}
    Node_handle m_node_p;
};
///BC_evaluator evaluates which nodes satisfy boundary conditions and stores them for processing
//-----------------------------------------------------------------------------

template<class Condition_base, class ME>
class BC_evaluator
{
public:

//  typedef typename Condition_base::Category       Category;
    typedef BC_evaluator_element<ME>                    BCE_Element;
    typedef std::list< BCE_Element >                    BCe_Element_list;
    typedef typename BCe_Element_list::iterator         BCe_Element_iterator;
    typedef typename BCe_Element_list::const_iterator   BCe_Element_const_iterator;

    typedef CGAL::Iterator_project< BCe_Element_iterator, Project_BC_node< BCE_Element > > Node_iterator;

    BC_evaluator() :m_bc(NULL) {};
    BC_evaluator( Condition_base *bc, Mesh& mesh );
    ///sets boundary conditions for a set of nodes given by a range of iterators to Node_handles. If once is 'true' the nodes to apply conditions (BCe_Element_list) will be selected only once at the start, otherwise all the nodes in the range of iterators will be added
    template<class Iter>
    void set_bc( Condition_base *bc, Iter beg, Iter end, bool once = true);
    ///sets boundary conditions for a set of nodes given by a range of Handle_iterators to Node_handles. If once is 'true' the nodes to apply conditions (BCe_Element_list) will be selected only once at the start, otherwise all the nodes in the range of iterators will be added
    template<class Iter>
    void set_bc( Condition_base *bc, Handle_iterator<Iter> n_iter, Handle_iterator<Iter> n_end, bool once = true );
    ///sets boundary conditions for a specific node
    void set_bc( Condition_base *bc, Node_handle nh );

    void eval(Boundary_value& value, Mesh_node const& n, size_t i) const;

    bool contains_point( ME* el, Point& p ) const;

    int size_of_nodes() const {
        return m_boundary.size();
    }
    Condition_base const* boundary_condition() const {
        return m_bc;
    }

    BCe_Element_iterator bc_elements_begin() {
        return m_boundary.begin();
    }
    BCe_Element_iterator bc_elements_end() {
        return m_boundary.end();
    }

    BCe_Element_const_iterator bc_elements_begin() const {
        return m_boundary.begin();
    }
    BCe_Element_const_iterator bc_elements_end() const {
        return m_boundary.end();
    }

    Node_iterator nodes_begin() {
        return m_boundary.begin();
    }
    Node_iterator nodes_end() {
        return m_boundary.end();
    }

private:
    BCe_Element_list m_boundary;
    Condition_base const* m_bc;
};
//Iter is a Handle_iterator to Node_handle
//-----------------------------------------------------------------------------
template<class Condition_base, class ME> template< class Iter >
void BC_evaluator<Condition_base, ME>::set_bc ( Condition_base *bc, Handle_iterator<Iter> n_iter, Handle_iterator<Iter> n_end, bool once )
{
    if (once)
    {
        while ( n_iter != n_end )
        {
            set_bc(bc, n_iter );
            ++n_iter;
        }
    }
    else
    {
        m_bc = bc;
        while ( n_iter != n_end )
        {
            m_boundary.push_back(BC_evaluator_element<Mesh_face>(n_iter.handle()));
            ++n_iter;
        }
    }
}
//Iter is a normal iterator to Node_handle
//-----------------------------------------------------------------------------
template<class Condition_base, class ME> template< class Iter>
void BC_evaluator<Condition_base, ME>::set_bc( Condition_base *bc, Iter n_iter, Iter n_end, bool once )
{
    if (once)
    {
        while ( n_iter != n_end )
        {
            set_bc(bc, *n_iter );
            ++n_iter;
        }
    }
    else
    {
        m_bc = bc;
        while ( n_iter != n_end )
        {
            m_boundary.push_back(BC_evaluator_element<Mesh_face>(*n_iter));
            ++n_iter;
        }

    }
}
//-----------------------------------------------------------------------------
template<class Condition_base, class ME>
BC_evaluator<Condition_base, ME>::BC_evaluator( Condition_base *bc, Mesh& mesh )
{
    set_bc( bc, mesh.nodes_begin(), mesh.nodes_end() );
}
//-----------------------------------------------------------------------------
#endif
