// Copyright (C) 2005-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include "mesh/mesh.h"
#include "fem/FEM.h"
#include "fem/Finite_element.h"
#include "fem/Discrete_function.h"
#include "utils/error_macros.h"

//-----------------------------------------------------------------------------
Discrete_function::Discrete_function()
        :  _x(NULL), _mesh(0), _element(0),
        _vectordim(1), component(0), mixed_offset(0), component_offset(0),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Vector mesh and element need to be specified later or are automatically
    // chosen during assembly.
}
//-----------------------------------------------------------------------------
Discrete_function::Discrete_function(DenseVector& x)
        : _x(&x), _mesh(0), _element(0),
        _vectordim(1), component(0), mixed_offset(0), component_offset(0),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Mesh and element need to be specified later or are automatically
    // chosen during assembly.
}
//-----------------------------------------------------------------------------
Discrete_function::Discrete_function(DenseVector& x, Mesh& mesh)
        : _x(&x), _mesh(&mesh), _element(0),
        _vectordim(1), component(0), mixed_offset(0), component_offset(0),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Element needs to be specified later or are automatically
    // chosen during assembly.
}
//-----------------------------------------------------------------------------
Discrete_function::Discrete_function(DenseVector& x, Mesh& mesh, Finite_element const& element)
        : _x(&x), _mesh(&mesh), _element(&element),
        _vectordim(1), component(0), mixed_offset(0), component_offset(0),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Update vector dimension from element
    updateVectorDimension();
}
//-----------------------------------------------------------------------------
Discrete_function::Discrete_function(Mesh& mesh, Finite_element const& element)
        : _x(0), _mesh(&mesh), _element(&element),
        _vectordim(1), component(0), mixed_offset(0), component_offset(0),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Update vector dimension from element
    updateVectorDimension();

    // Allocate local storage
    size_t size = FEM::size(mesh, element);
    _x = new DenseVector(size);
    vector_local = true;
}
//-----------------------------------------------------------------------------
Discrete_function::Discrete_function(const Discrete_function& f)
        : _x(0), _mesh(f._mesh), _element(f._element),
        _vectordim(f._vectordim), component(f.component),
        mixed_offset(f.mixed_offset), component_offset(f.component_offset),
        vector_local(false), mesh_local(false), element_local(false)
{
    // Create a new vector and copy the values
    cell_assert(f._x);
    _x = new DenseVector();
    *_x =* f._x;
    vector_local = true;
}
//-----------------------------------------------------------------------------
Discrete_function::~Discrete_function()
{
    if ( vector_local )
        delete _x;
    if ( mesh_local )
        delete _mesh;
}
//-----------------------------------------------------------------------------
double Discrete_function::operator()(const Point& p, size_t i)
{
    cell_error("Discrete functions cannot be evaluated at arbitrary points.");
    return 0.0;
}
////-----------------------------------------------------------------------------
double Discrete_function::operator() (const Mesh_node* vertex, size_t i)
{
    cell_assert(_x && _mesh && _element);
    local.init(*_element);

    // Get vertex nodes for all components
    _element->vertexmap(local.vertex_nodes, vertex, *_mesh);
    // Pick value
    return (*_x)(mixed_offset + local.vertex_nodes[component + i]);
}
//-----------------------------------------------------------------------------
void Discrete_function::sub(size_t i)
{
    // Check that we have an element
    if ( !_element )
        cell_error("Unable to pick sub function or component of function since no element has been attached.");

    // Check that we have a mesh
    if ( !_mesh )
        cell_error("Unable to pick sub function or component of function since no mesh has been attached.");
}
//-----------------------------------------------------------------------------
void Discrete_function::copy(const Discrete_function& f)
{
    cell_assert(f._x);

    // Initialize local data if not already done
    if ( !vector_local )
    {
        _x = new DenseVector();
        vector_local = true;
    }
    // Copy values to vector
    *_x = *f._x;

    // Copy pointers to mesh and element
    _mesh = f._mesh;
    _element = f._element;
}
//-----------------------------------------------------------------------------
size_t Discrete_function::vectordim() const
{
    return _vectordim;
}
//-----------------------------------------------------------------------------
DenseVector& Discrete_function::vector()
{
    cell_assert(_x);
    return *_x;
}
//-----------------------------------------------------------------------------
Mesh& Discrete_function::mesh()
{
    cell_assert(_mesh);
    return *_mesh;
}
//-----------------------------------------------------------------------------
Finite_element const& Discrete_function::element()
{
    cell_assert(_element);
    return *_element;
}
//-----------------------------------------------------------------------------
void Discrete_function::attach(DenseVector& x, bool local)
{
    // Delete old vector if local
    if ( vector_local )
        delete _x;
    // Attach new vector
    _x = &x;
    vector_local = local;
}
//-----------------------------------------------------------------------------
void Discrete_function::attach(Mesh& mesh, bool local)
{
    // Delete old mesh if local
    if ( mesh_local )
        delete _mesh;
    // Attach new mesh
    _mesh = &mesh;
    mesh_local = local;
}
//-----------------------------------------------------------------------------
void Discrete_function::attach(Finite_element const& element, bool local)
{
    // Attach new mesh
    _element = &element;
    element_local = local;

    // Recompute vector dimension
    updateVectorDimension();
}
//-----------------------------------------------------------------------------
void Discrete_function::init(Mesh& mesh, Finite_element& element)
{
    std::cout << "Reinitializing discrete function" << std::endl;

    // Reset data
    _mesh = &mesh;
    _element = &element;
    component = 0;
    mixed_offset = 0;
    component_offset = 0;

    // Update vector dimension from element
    updateVectorDimension();

    // Reinitialize local storage
    size_t size = FEM::size(mesh, element);
    if ( !vector_local )
    {
        _x = new DenseVector(size);
        vector_local = true;
    }
    else
    {
        _x->init(size);
        *_x = 0.0;
    }
}
//-----------------------------------------------------------------------------
void Discrete_function::updateVectorDimension()
{
    cell_assert(_element);
    _vectordim = _element->tensordim();

}
//-----------------------------------------------------------------------------
void Discrete_function::interpolate(double * coefficients, Affine_map_shell<Mesh_face> const& map, Finite_element& element)
{
    _element = &element;
    // Initialize local data (if not already initialized correctly)
    local.init(*_element);
    map.nodemap(local.dofs, map.cell(), *_mesh);
    // Compute positions in global vector by adding offsets
    for (size_t i = 0; i < _element->spacedim(); i++)
        local.dofs[i] = local.dofs[i] + mixed_offset + component_offset;
    // Get values
    _x->get(coefficients, local.dofs, _element->spacedim());
}
//-----------------------------------------------------------------------------
void Discrete_function::info(bool detail) const
{
    std::cout << "function vector :" << *_x << "\n";
    std::cout << "vector dim :" << vectordim() << "\n";
    if (detail)
    {
        std::cout << "function mesh :";
        _mesh->statistics();
        std::cout << "function element :" << _element->spec() << "\n";
    }
}
//-----------------------------------------------------------------------------
