// Copyright (C) 2005-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __DISCRETE_FUNCTION_H
#define __DISCRETE_FUNCTION_H

#include "fem/Generic_function.h"
#include "fem/Finite_element.h"

class Mesh;

/// This class implements the functionality for a discrete function
/// of some given finite element space, defined by a vector of degrees
/// of freedom, a mesh, and a finite element.

class Discrete_function : public Generic_function
{
public:
    /// Create empty discrete function with no data
    Discrete_function();

    /// Create discrete function from given data
    Discrete_function(DenseVector& x);

    /// Create discrete function from given data
    Discrete_function(DenseVector& x, Mesh& mesh);

    /// Create discrete function from given data
    Discrete_function(DenseVector& x, Mesh& mesh, Finite_element const& element);

    /// Create discrete function from given data (automatically create vector)
    Discrete_function(Mesh& mesh, Finite_element const& element);

    /// Copy constructor
    Discrete_function(const Discrete_function& f);

    /// Destructor
    ~Discrete_function();

    /// Evaluate function at given point
    double operator() (const Point& point, size_t i);

    /// Evaluate function at given vertex
    double operator() (const Mesh_node* vertex, size_t i);

    /// Restrict to sub function or component (if possible)
    void sub(size_t i);

    /// Copy data from given function
    void copy(const Discrete_function& f);

    /// Compute interpolation of function onto local finite element space
    void interpolate(double* coefficients, Affine_map_shell<Mesh_face> const& map, Finite_element& element);

    /// Return vector dimension of function
    size_t vectordim() const;

    /// Return vector associated with function (if any)
    DenseVector& vector();

    /// Return mesh associated with function (if any)
    Mesh& mesh();

    /// Return element associated with function (if any)
    Finite_element const& element();

    /// Attach vector to function
    void attach(DenseVector& x, bool local);

    /// Attach mesh to function
    void attach(Mesh& mesh, bool local);

    /// Attach finite element to function
    void attach(Finite_element const& element, bool local);

    /// Reinitialize to given data (automatically create vector)
    void init(Mesh& mesh, Finite_element& element);

    /// print debuging information
    void info( bool detail = false) const;
protected:

    /// Update vector dimension from current element
    void updateVectorDimension();

    // Pointer to degrees of freedom
    DenseVector* _x;
    Mesh* _mesh;
    Finite_element const* _element;
    size_t _vectordim;
    size_t component;
    size_t mixed_offset;

    size_t component_offset;
    bool vector_local;
    bool mesh_local;
    bool element_local;
    ;
};
//-----------------------------------------------------------------------------
#endif
