// Copyright (C) 2007 Pawel Krupinski
// Licensed under the GNU GPL Version 2.

#ifndef _FEM_H_
#define _FEM_H_

#include <boost/timer.hpp>

#include "matrix/uBlasMatrix.h"
#include "matrix/SparseMatrix.h"
#include "matrix/uBlasVector.h"
#include "matrix/DenseVector.h"

#include "fem/Affine_map_shell.h"
#include "fem/Finite_element.h"
#include "fem/Boundary_value.h"
#include "fem/Boundary_condition.h"
#include "fem/Bilinear_form.h"
#include "fem/Linear_form.h"
#include "fem/Load_case.h"
#include "fem/fem_utils.h"
#include "fem/BC_evaluator.h"

#include "utils/geometry_utils.h"

#include "io/VTK_ostream.h"

class FEM
{
public:
    /// Assemble bilinear form
    template<class AFF>
    static void assemble ( Bilinear_form& a, SparseMatrix& A, Mesh& mesh, AFF &map, VTK_ofstream &o, bool init = true );
    /// Assemble spring stiffness
    static void assemble ( Bilinear_form& a, SparseMatrix& A, Mesh& mesh, cell::Tissue_handle const& t, VTK_ofstream &o, bool init = true );
    /// Assemble bilinear form
    template<class AFF>
    static void assemble ( Bilinear_form& a, SparseMatrix& A, Mesh& mesh, AFF &map, bool init = true );
    /// Assemble bilinear form with load (for load dependendent stiffness)
    template<class AFF>
    static void assemble_surface_loads ( Bilinear_form& a, SparseMatrix& A, Load_case const& lc, Mesh& mesh, AFF &map, bool init = true );
    /// Assemble linear form
    template<class AFF>
    static void assemble ( Linear_form& L, DenseVector& b, Mesh& mesh, AFF &map, bool init = true );
    /// Assemble linear form with surface load
    template<class AFF>
    static void assemble_surface_loads ( Linear_form& L, DenseVector& b, Load_case const& lc, Mesh& mesh, AFF &map, bool init = true );
    /// Assemble linear form with point load
    template<class AFF>
    static void assemble_point_loads ( DenseVector& b, Load_case const& lc, Mesh& mesh, AFF &map, bool init = true );
    ///assemble line loads
    template<class AFF>
    static void assemble_line_loads ( DenseVector& b, Load_case const& lc, Mesh& mesh, AFF &map, bool init = true );
    /// Assemble spring stiffness
    template<class AFF>
    static void assemble_springs ( SparseMatrix& A, Mesh& mesh, AFF &map );
    template<class AFF>
    static void assemble_springs ( SparseMatrix& A, Mesh& mesh, AFF &map, bool init)
    {
      if( init )
      {
        // Get finite element
        Finite_element const& element = map.element();
        // Initialize global matrix
        const size_t M = size ( mesh, element );
        const size_t nz = nzsize ( mesh, element );
        A.init ( M, M, nz );
        A.zero();
      }
      assemble_springs ( A, mesh, map );
    }
    template<class AFF>
    static void assemble_spring_internal_forces (DenseVector& b, Mesh& mesh, AFF &map);
    template<class AFF>
    static void assemble_spring_internal_forces (DenseVector& b, Mesh& mesh, AFF &map, bool init)
    {
      if( init )
      {
        Finite_element const& element = map.element();
        const size_t M = size ( mesh, element );
        b.init(M);;
        b.zero();
      }
      assemble_spring_internal_forces ( b, mesh, map );
    }

    /// Apply boundary conditions to matrix and vector
    template<class Condition_base, class AFF>
    static void applyBC ( SparseMatrix& A, DenseVector& b, Mesh& mesh, AFF &map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );
    /// Apply boundary conditions to vector.
    template<class Condition_base, class AFF>
    static void applyBC ( DenseVector& b, Mesh& mesh, AFF &map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );
    /// Apply boundary conditions to residual vector.
    template<class Condition_base, class AFF>
    static void applyBC_residual ( DenseVector& b, Mesh& mesh, AFF &map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );
    ///Set bending degrees of freedom to identity assumig they occupy lower half of matrix
    static void Indent_bending_DOF ( SparseMatrix& A );
    /// Count the degrees of freedom
    static size_t size ( const Mesh& mesh, const Finite_element& element );
    /// Estimate the maximum number of nonzeros in each row
    static size_t nzsize ( const Mesh& mesh, const Finite_element& element );
    /// Lump matrix
    static void lump ( const SparseMatrix& M, DenseVector& m );
    /// Display assembly data (useful for debugging)
    static void disp ( const Mesh& mesh, const Finite_element& element );

private:
    /// Apply boundary conditions on mesh
    template< class Condition_base, class AFF>
    static void applyBC_3D ( SparseMatrix& A, DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );

    /// Apply boundary conditions to the vector
    template< class Condition_base, class AFF>
    static void applyBC_3D ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );

    /// Apply boundary conditions to the residual vector (residual vector comes with minus sign)
    template< class Condition_base, class AFF>
    static void applyBC_3D_residual ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e );

    /// Check that dimension of the mesh matches the form
    static void checkdims ( Bilinear_form& a, const Mesh& mesh );

    /// Check that dimension of the mesh matches the form
    static void checkdims ( Linear_form& L, const Mesh& mesh );

    /// Check number of nonzeros in each row
    //static void checknz(const SparseMatrix& A, size_t nz);

};
//-----------------------------------------------------------------------------
template<>
inline  void FEM::assemble ( Bilinear_form& a, SparseMatrix& A, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{

    // Get finite elements
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();

    // Initialize local data
    const size_t m = test_element.spacedim();
    const size_t n = trial_element.spacedim();
    double* block = new double[m*n];
    size_t* test_nodes = new size_t[m];
    size_t* trial_nodes = new size_t[n];

    // Initialize global matrix
    const size_t M = size ( mesh, test_element );
    const size_t N = size ( mesh, trial_element );
    const size_t nz = nzsize ( mesh, trial_element );
    if ( init )
    {
        A.init ( M, N, nz );
        A.zero();
    }
    // Start a progress session
#ifdef CELL_INFO
    cell_info ( "Assembling matrix of size %d x %d.", M, N );
#endif
    // Iterate over all cells in the mesh
    Affine_map_shell<Mesh_face>::Element_iterator el_iter;
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    //use temporary matrix for faster assembly of sparse matrix
    A.unapply();
    for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
    {
        // Update affine map
        map.update ( *el_iter );
        map.update_stress();
        el_iter->fem_data().update ( map );
        // Update form
        a.update ( map );
        // Compute maps from local to global degrees of freedom
        map.nodemap ( test_nodes, *el_iter, mesh );
        map.nodemap ( trial_nodes, *el_iter, mesh );
        // Compute element matrix
        a.eval ( block, map );
        // Add element matrix to global matrix
        A.add ( block, m, test_nodes, n, trial_nodes );
    }
    A.apply();
    // Delete data
    delete [] block;
    delete [] test_nodes;
    delete [] trial_nodes;
}
//-----------------------------------------------------------------------------
template<>
inline  void FEM::assemble ( Bilinear_form& a, SparseMatrix& A, Mesh& mesh, Affine_map_shell<Mesh_face> &map, VTK_ofstream &o, bool init )
{
    // Get finite elements
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();

    // Initialize local data
    const size_t m = test_element.spacedim();
    const size_t n = trial_element.spacedim();
    double* block = new double[m*n];
    size_t* test_nodes = new size_t[m];
    size_t* trial_nodes = new size_t[n];

    // Initialize global matrix
    const size_t M = size ( mesh, test_element );
    const size_t N = size ( mesh, trial_element );
    const size_t nz = nzsize ( mesh, trial_element );
    if ( init )
    {
        A.init ( M, N, nz );
        A.zero();
    }
    // Start a progress session
#ifdef CELL_INFO
    cell_info ( "Assembling matrix of size %d x %d.", M, N );
#endif
    // Iterate over all cells in the mesh
    Affine_map_shell<Mesh_face>::Element_iterator el_iter;
    //for (CellIterator cell(mesh); !cell.end(); ++cell)
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    std::vector<Matrix3s> loc_data_tens ( 12, Matrix3s() );
    //use temporary matrix for faster assembly of sparse matrix
    A.unapply();
    for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
    {
        // Update affine map
        map.update ( *el_iter );
        map.update_stress();
        el_iter->fem_data().update ( map );
        // Update form
        a.update ( map );
        // Compute maps from local to global degrees of freedom
        map.nodemap ( test_nodes, *el_iter, mesh );
        map.nodemap ( trial_nodes, *el_iter, mesh );
        // Compute element matrix
        a.eval ( block, map );
        // Add element matrix to global matrix
        A.add ( block, m, test_nodes, n, trial_nodes );
        std::vector<Matrix3s> glob_data_tens ( 12, Matrix3s() );;
        for ( int i= 0; i < 12; ++i )
        {
            glob_data_tens[i] = map.stress ( i );
        }
        rotate_element_data ( loc_data_tens, glob_data_tens, map, true );
    }
    A.apply();
    // Delete data
    delete [] block;
    delete [] test_nodes;
    delete [] trial_nodes;
}
//-----------------------------------------------------------------------------
template<>
inline void FEM::assemble_surface_loads ( Bilinear_form& a, SparseMatrix& A, Load_case const& lc, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{
    // Get finite elements
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();

    // Initialize local data
    const size_t m = test_element.spacedim();
    const size_t n = trial_element.spacedim();
    double* block = new double[m*n];
    size_t* test_nodes = new size_t[m];
    size_t* trial_nodes = new size_t[n];

    // Initialize global matrix
    const size_t M = size ( mesh, test_element );
    const size_t N = size ( mesh, trial_element );
    const size_t nz = nzsize ( mesh, trial_element );
    if ( init )
    {
        A.init ( M, N, nz );
        A.zero();
    }
    // Start a progress session
#ifdef CELL_INFO
    cell_info ( "Assembling matrix of size %d x %d.", M, N );
#endif
    //use temporary matrix for faster assembly of sparse matrix
    A.unapply();

    // Iterate over all cells in the mesh
    Affine_map_shell<Mesh_face>::Element_iterator el_iter;
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
    {
        // Update affine map
        map.update ( *el_iter );
        map.update_stress();
        el_iter->fem_data().update ( map );
        // Update form
        a.update ( map );
        // Compute maps from local to global degrees of freedom
        map.nodemap ( test_nodes, *el_iter, mesh );
        map.nodemap ( trial_nodes, *el_iter, mesh );
        // Compute element matrix
        a.eval ( block, map );
        // Add element matrix to global matrix
        A.add ( block, m, test_nodes, n, trial_nodes );
    }
    Load_case::Surface_load_const_iterator ld_iter, ld_end;

    for ( ld_iter = lc.surface_loads_begin(), ld_end = lc.surface_loads_end(); ld_iter != ld_end; ++ld_iter )
    {
        Surface_load const&sl = **ld_iter;
        double pressure = sl.load() * sl.load_factor(lc.time());
        double l = sl.load_position();
        //Check if load subsurfaces were defined
        if ( sl.is_localized() )
        {
            Surface_load::Load_const_iterator l_iter, l_end;
            for ( l_iter = sl.loads_begin(), l_end = sl.loads_end(); l_iter != l_end; ++l_iter )
            {
                Mesh_face const&f = *l_iter->face;
                pressure *=l_iter->load_factor;
                l = l_iter->load_position;

                a.update ( pressure, &l );
                // Update affine map
                map.update ( f );
                map.update_stress();
                f.fem_data().update ( map );
                // Update form
                a.update ( map );
                // Compute maps from local to global degrees of freedom
                map.nodemap ( test_nodes, f, mesh );
                map.nodemap ( trial_nodes, f, mesh );
                // Compute element matrix
                a.eval ( block, map );
                // Add element matrix to global matrix
                A.add ( block, m, test_nodes, n, trial_nodes );
            }
        }
        else
        {
            a.update ( pressure, &l );
            // Iterate over all cells in the mesh
            Affine_map_shell<Mesh_face>::Element_iterator el_iter;
            for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
            {
                Mesh_face &f = *el_iter;
                // Update affine map
                map.update ( f );
                map.update_stress();
                f.fem_data().update ( map );
                // Update form
                a.update ( map );
                // Compute maps from local to global degrees of freedom
                map.nodemap ( test_nodes, f, mesh );
                map.nodemap ( trial_nodes, f, mesh );
                // Compute element matrix
                a.eval ( block, map );
                // Add element matrix to global matrix
                A.add ( block, m, test_nodes, n, trial_nodes );
            }
        }
    }
    A.apply();
    // Delete data
    delete [] block;
    delete [] test_nodes;
    delete [] trial_nodes;
}
//-----------------------------------------------------------------------------
template<>
inline void FEM::assemble ( Linear_form& L, DenseVector& b, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{
    // Get finite element
    Finite_element& test_element = L.test();
    // Initialize local data
    const size_t m = test_element.spacedim();
    double* block = new double[m];
    size_t* test_nodes = new size_t[m];

    // Initialize global vector
    const size_t M = size ( mesh, test_element );
    if ( init )
    {
        b.init ( M );
        b = 0.0;
    }
    // Start a progress session
#ifdef CELL_INFO
    cell_info ( "Assembling vector of size %d.", M );
#endif
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    // Iterate over all cells in the mesh
    Affine_map_shell<Mesh_face>::Element_iterator el_iter;
    for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
    {
        Mesh_face &f = *el_iter;
        map.update ( *el_iter );
        map.update_stress();
        L.update ( map );
        // Compute map from local to global degrees of freedom
        map.nodemap ( test_nodes, *el_iter, mesh );
        // Compute element matrix
        L.eval ( block, map );
        // Add element matrix to global matrix
        b.add ( block, test_nodes, m );

    }
    // Delete data
    delete [] block;
    delete [] test_nodes;
}
//-----------------------------------------------------------------------------
template<>
inline void FEM::assemble_surface_loads ( Linear_form& L, DenseVector& b, Load_case const& lc, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{

    // Get finite element
    Finite_element& test_element = L.test();
    // Initialize local data
    const size_t m = test_element.spacedim();
    double* block = new double[m];
    size_t* test_nodes = new size_t[m];
    // Initialize global vector
    const size_t M = size ( mesh, test_element );
    if ( init )
    {
        b.init ( M );
        b = 0.0;
    }
    // Start a progress session
#ifdef CELL_INFO
    cell_info ( "Assembling vector of size %d.", M );
#endif
    //Make all updates for given element
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    // Iterate over load elements
    Load_case::Surface_load_const_iterator ld_iter, ld_end;
    for ( ld_iter = lc.surface_loads_begin(), ld_end = lc.surface_loads_end(); ld_iter != ld_end; ++ld_iter )
    {
        Surface_load const&sl = **ld_iter;
        double pressure = sl.load() *sl.load_factor(lc.time());
        double l = sl.load_position();
        //Check if load subsurfaces were defined
        if ( sl.is_localized() )
        {
            Surface_load::Load_const_iterator l_iter, l_end;
            for ( l_iter = sl.loads_begin(), l_end = sl.loads_end(); l_iter != l_end; ++l_iter )
            {
                Mesh_face const&f = *l_iter->face;
                pressure *=l_iter->load_factor;
                l = l_iter->load_position;
                L.update ( pressure, &l );
                // Update affine map
                map.update ( f );
                map.update_stress();
                // Update form
                L.update ( map );
                // Compute map from local to global degrees of freedom
                map.nodemap ( test_nodes, f, mesh );
                // Compute element matrix
                L.eval ( block, map );
                // Add element matrix to global matrix
                b.add ( block, test_nodes, m );
            }
        }
        else
        {
            L.update ( pressure, &l );
            // Iterate over all cells in the mesh
            Affine_map_shell<Mesh_face>::Element_iterator el_iter;
            for ( el_iter = map.elements_begin ( mesh ); el_iter != map.elements_end ( mesh ); ++el_iter )
            {
                Mesh_face &f = *el_iter;
                // Update affine map
                map.update ( f );
                map.update_stress();
                // Update form
                L.update ( map );
                // Compute map from local to global degrees of freedom
                map.nodemap ( test_nodes, *el_iter, mesh );
                // Compute element matrix
                L.eval ( block, map );
                // Add element matrix to global matrix
                b.add ( block, test_nodes, m );
            }
        }
    }
    // Delete data
    delete [] block;
    delete [] test_nodes;
}
//-----------------------------------------------------------------------------
template<>
inline void FEM::assemble_line_loads ( DenseVector& b, Load_case const& lc, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{
    // Get finite element
    Finite_element const& test_element = map.element();
    // Initialize global vector
    const size_t M = size ( mesh, test_element );
    size_t n = test_element.tensordim();
    //indices of node dofs in the matrix/vector
    size_t* nodes0 = new size_t[n];
    size_t* nodes1 = new size_t[n];
    if ( init )
    {
        b.init ( M );
        b = 0.0;
    }
    // Start a progress session
#ifdef CELL_INFO    
    cell_info ( "Assembling vector of size %d.", M );
#endif    
    //Make all updates for given element
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    // Iterate over load elements
    Load_case::Line_load_const_iterator ld_iter, ld_end;
    for ( ld_iter = lc.line_loads_begin(), ld_end = lc.line_loads_end(); ld_iter != ld_end; ++ld_iter )
    {
        Line_load const& cl = **ld_iter;
        double load = cl.load() * cl.load_factor(lc.time()) * 0.5; //0.5 is averaging for two nodes
        Line_load::Load_const_iterator l_iter, l_end;
        for ( l_iter = cl.loads_begin(), l_end = cl.loads_end(); l_iter != l_end; ++l_iter )
        {
            Edge_use_handle const& eu = l_iter->edge;
            Node_handle const& n0 = eu->node();
            Node_handle const& n1 = eu->other_node();
            Vector v ( n0->point(), n1->point() );
            double len = sqrt ( v.squared_length() );
            map.vertexmap ( nodes0, n0.ptr(), mesh );
            map.vertexmap ( nodes1, n1.ptr(), mesh );


            if ( l_iter->load_factor == Vector ( 0,0,0 ) ) //take normal to the egde in the face plane if load vector unspecified
            {
              Vector force0 = CGAL::cross_product(v, normalized(n0->director()));
              Vector force1 = CGAL::cross_product(v, normalized(n1->director()));
              for ( int i = 0; i < 3; ++i )
                {
                    b.add ( nodes0[i], force0[i]* load );
                    b.add ( nodes1[i], force1[i]* load );
                }
            }
            else{
            Vector force =  l_iter->load_factor * load * len; //multiply by edge length to get force from line force density
                for ( int i = 0; i < 3; ++i )
                {
                    b.add ( nodes0[i], force[i] );
                    b.add ( nodes1[i], force[i] );
                }
            }
        }

    }

    // Delete data
    delete [] nodes0;
    delete [] nodes1;

}
//-----------------------------------------------------------------------------
template<>
inline void FEM::assemble_point_loads ( DenseVector& b, Load_case const& lc, Mesh& mesh, Affine_map_shell<Mesh_face> &map, bool init )
{

    // Get finite element
    Finite_element const& test_element = map.element();

    // Initialize global vector
    const size_t M = size ( mesh, test_element );
    size_t n = test_element.tensordim();
    //indices of node dofs in the matrix/vector
    size_t* nodes = new size_t[n];
    if ( init )
    {
        b.init ( M );
        b = 0.0;
    }
    // Start a progress session
#ifdef CELL_INFO    
    cell_info ( "Assembling vector of size %d.", M );
#endif    
    //Make all updates for given element
    map.set_flag ( Affine_map_shell<Mesh_face>::UPDATE_ALL );
    // Iterate over load elements
    Load_case::Concentrated_load_const_iterator ld_iter, ld_end;
    for ( ld_iter = lc.concentrated_loads_begin(), ld_end = lc.concentrated_loads_end(); ld_iter != ld_end; ++ld_iter )
    {
        Concentrated_load const& cl = **ld_iter;
        double load = cl.load() * cl.load_factor(lc.time());

        Concentrated_load::Load_const_iterator l_iter, l_end;
        for ( l_iter = cl.loads_begin(), l_end = cl.loads_end(); l_iter != l_end; ++l_iter )
        {
            Mesh_node const* n = &*l_iter->node;
            map.vertexmap ( nodes, n, mesh );
            Vector force = l_iter->load_factor * load;

            for ( int i = 0; i < 3; ++i )
                b.add ( nodes[i], force[i] );
        }
    }
    // Delete data
    delete [] nodes;
}
//-----------------------------------------------------------------------------
template <class Condition_base, class AFF>
void FEM::applyBC_3D ( SparseMatrix& A, DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{

    size_t m = 0;
    size_t* rows = new size_t[b.size() ];

    Boundary_value bv;
    //number of node degrees of freedom
    size_t n = map.element().tensordim();
    //indices of node dofs in the matrix/vector
    size_t* nodes = new size_t[n];

    typename BC_evaluator<Condition_base, typename AFF::Mesh_element>::Node_iterator n_iter, n_end;

    for ( n_iter = BC_e.nodes_begin(), n_end = BC_e.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Mesh_node * vert = &*n_iter;
        map.vertexmap ( nodes, vert, mesh );
        for ( int i = 0; i < n; ++i )
        {
            bv.reset();
            BC_e.eval ( bv, *vert, i );

            if ( bv.is_set() )
            {
                int node = nodes[i];
                rows[m++] = node;
                b ( node ) = bv.value;
            }
        }
    }
    A.ident ( m, rows );
    // Delete data
    delete [] nodes;
    delete [] rows;
}
//-----------------------------------------------------------------------------
template <class Condition_base, class AFF>
void FEM::applyBC_3D ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{
//   size_t m = 0;
    Boundary_value bv;
    //number of node degrees of freedom
    size_t n = map.element().tensordim();
    //indices of node dofs in the matrix/vector
    size_t* nodes = new size_t[n];

    typename BC_evaluator<Condition_base, typename AFF::Mesh_element>::Node_iterator n_iter, n_end;

    for ( n_iter = BC_e.nodes_begin(), n_end = BC_e.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Mesh_node *vert = &*n_iter;
        map.vertexmap ( nodes, vert, mesh );
        for ( int i = 0; i < n; ++i )
        {
            bv.reset();
            BC_e.eval ( bv, *vert, i );

            if ( bv.is_set() )
            {
                int node = nodes[i];
                b ( node ) = bv.value;
            }
        }
    }
    // Delete data
    delete [] nodes;
}
//-----------------------------------------------------------------------------
template <class Condition_base, class AFF>
void FEM::applyBC_3D_residual ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{
    Boundary_value bv;
    //number of node degrees of freedom
    size_t n = map.element().tensordim();
    //indices of node dofs in the matrix/vector
    size_t* nodes = new size_t[n];

    typename BC_evaluator<Condition_base, typename AFF::Mesh_element>::Node_iterator n_iter, n_end;
    for ( n_iter = BC_e.nodes_begin(), n_end = BC_e.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Mesh_node * vert = &*n_iter;

        map.vertexmap ( nodes, vert, mesh );
        for ( int i = 0; i < n; ++i )
        {
            bv.reset();
            BC_e.eval ( bv, *vert, i );

            if ( bv.is_set() )
            {
                int node = nodes[i];
//                 b ( node ) = - bv.value; //residual vector comes with minus sign
                b ( node ) = 0.0;
            }
        }

    }
    // Delete data
    delete [] nodes;
}
//-----------------------------------------------------------------------------
template<class Condition_base, class AFF>
void FEM::applyBC ( SparseMatrix& A, DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{
#ifdef CELL_INFO  
    cell_info ( "Applying boundary conditions." );
#endif    
    applyBC_3D ( A, b, mesh, map, BC_e );
}
//-----------------------------------------------------------------------------
template<class Condition_base, class AFF>
void FEM::applyBC ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{
#ifdef CELL_INFO  
    cell_info ( "Applying boundary conditions." );
#endif    
    applyBC_3D ( b, mesh, map, BC_e );
}
//-----------------------------------------------------------------------------
template<class Condition_base, class AFF>
void FEM::applyBC_residual ( DenseVector& b, Mesh& mesh, AFF& map, BC_evaluator<Condition_base, typename AFF::Mesh_element> & BC_e )
{
#ifdef CELL_INFO  
    cell_info ( "Applying residual boundary conditions." );
#endif    
    applyBC_3D_residual ( b, mesh, map, BC_e );
}
//-----------------------------------------------------------------------------
inline void FEM::Indent_bending_DOF ( SparseMatrix& A )
{
    //set bending part of stiffnes matrix to identity
    size_t beg = A.size ( 0 ) /2;
    size_t* rows = new size_t[beg];
    for ( int i = 0; i < beg; ++i )
    {
        rows[i] = beg+i;
    }
    A.ident_insert ( beg, rows );
    // Delete data
    delete [] rows;
}
//-----------------------------------------------------------------------------
#undef CELL_INFO
#endif
