// Copyright (C) 2003-2006 Johan Hoffman, Johan Jansson and Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Garth N. Wells 2005
// Modified by Pawel Krupinski 2007
//
// The class Function serves as the envelope class and holds a pointer
// to a letter class that is a subclass of Generic_function. All the
// functionality is handled by the specific implementation (subclass).

#include "fem/Discrete_function.h"
#include "fem/ConstantFunction.h"
#include "fem/Function.h"
#include "utils/error_macros.h"

//-----------------------------------------------------------------------------
Function::Function ( double value ) : f ( NULL ), _type ( constant ) 
{
    f = new ConstantFunction ( value );
}
//-----------------------------------------------------------------------------
Function::Function() : f ( NULL ), _type ( discrete ) 
{
    f = new Discrete_function();
}
//-----------------------------------------------------------------------------
Function::Function ( DenseVector& x )
    :
    f ( NULL ), _type ( discrete ) 
{
    f = new Discrete_function ( x );
}
//-----------------------------------------------------------------------------
Function::Function ( DenseVector& x, Mesh& mesh )
    :
    f ( NULL ), _type ( discrete ) 
{
    f = new Discrete_function ( x, mesh );
}
//-----------------------------------------------------------------------------
Function::Function ( DenseVector& x, Mesh& mesh, Finite_element const& element )
    : f ( NULL ), _type ( discrete ) 
{
    f = new Discrete_function ( x, mesh, element );
}
//-----------------------------------------------------------------------------
Function::Function ( Mesh& mesh, Finite_element& element )
    :
    f ( NULL ), _type ( discrete ) 
{
    f = new Discrete_function ( mesh, element );
}
//-----------------------------------------------------------------------------
Function::Function ( const Function& f )
    : f ( 0 ), _type ( f._type ) 
{
    switch ( f._type )
    {
    case discrete:
        this->f = new Discrete_function ( * ( ( Discrete_function * ) f.f ) );
        break;
    default:
        cell_error ( "Unknown function type." );
    }
}
//-----------------------------------------------------------------------------
Function::~Function()
{
    delete f;
}
//-----------------------------------------------------------------------------
double Function::eval ( const Point& p, size_t i )
{
    cell_info ( "User-defined functions must overload double Function::eval(const Point& p, size_t i)." );
    cell_error ( "Missing eval() for user-defined function." );

    return 0.0;
}
//-----------------------------------------------------------------------------
Function Function::operator[] ( const size_t i )
{
    // Create copy
    Function f ( *this );

    // Restrict copy to sub function or component
    f.f->sub ( i );

    return f;
}
//-----------------------------------------------------------------------------
const Function& Function::operator= ( const Function& f )
{
    switch ( f._type )
    {
    case discrete:
        // Don't delete data if not necessary (don't want to recreate vector)
        if ( _type == discrete )
        {
            ( ( Discrete_function * ) this->f )->copy ( * ( ( Discrete_function * ) f.f ) );
        }
        else
        {
            delete this->f;
            this->f = new Discrete_function ( * ( ( Discrete_function * ) f.f ) );
        }
        break;
    default:
        cell_error ( "Unknown function type." );
    }
    return *this;
}
//-----------------------------------------------------------------------------
void Function::init ( Mesh& mesh, Finite_element& element )
{
    if ( _type != discrete )
    {
        delete f;
        f = new Discrete_function ( mesh, element );
    }
    else
    {
        ( ( Discrete_function* ) f )->init ( mesh, element );
    }

    _type = discrete;
}

//-----------------------------------------------------------------------------
