// Copyright (C) 2003-2006 Johan Hoffman, Johan Jansson and Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Garth N. Wells 2005.
// Modified by Pawel Krupinski 2007

#ifndef __FUNCTION_H
#define __FUNCTION_H

#include "fem/Discrete_function.h"
#include "fem/Affine_map_shell.h"
#include "utils/types.h"

class Mesh;
class Finite_element;

/// This class represents a function that can be evaluated on a
/// mesh. The actual representation of the function can vary, but
/// the typical representation is in terms of a mesh, a vector of
/// degrees of freedom and a finite element that determines the
/// distribution of degrees of freedom on the mesh.

class Function
{
public:

    /// Create function with given constant value
    Function ( double value );

    /// Create empty discrete function (no vector mesh or finite element attached)
    Function();

    /// Create discrete function (mesh and finite element chosen automatically)
    Function ( DenseVector& x );

    /// Create discrete function (finite element chosen automatically)
    Function ( DenseVector& x, Mesh& mesh );

    /// Create discrete function
    Function ( DenseVector& x, Mesh& mesh, Finite_element const& element );

    /// Create discrete function (vector created automatically)
    Function ( Mesh& mesh, Finite_element& element );

    /// Copy constructor
    Function ( const Function& f );

    /// Destructor
    virtual ~Function();

    /// Evaluate function at given point (must be implemented for user-defined function)
    virtual double eval ( const Point& p, size_t i = 0 );

    /// Evaluate function at given point
    inline double operator() ( const Point& p, size_t i = 0 )
    {
        return ( *f ) ( p, i );
    }

    /// Evaluate function at given vertex
    inline double operator() ( const Mesh_node* vertex, size_t i = 0 ) const
    {
        return ( *f ) ( vertex, i );
    }

    /// Pick sub function (of mixed function) or component of vector-valued function
    Function operator[] ( const size_t i );

    /// Assignment operator
    const Function& operator= ( const Function& f );

    /// Compute interpolation of function onto local finite element space
    inline void interpolate ( double * coefficients, Affine_map_shell<Mesh_face> const& map, Finite_element& element );

    /// Return vector dimension of function
    inline size_t vectordim() const
    {
        return f->vectordim();
    }

    /// Return vector associated with function (if any)
    inline DenseVector& vector()
    {
        return f->vector();
    }

    /// Return mesh associated with function (if any)
    inline Mesh& mesh() const
    {
        return f->mesh();
    }

    /// Return element associated with function (if any)
    inline Finite_element const& element()
    {
        return f->element();
    }

    /// Attach vector to function (data local if set, use with caution)
    inline void attach ( DenseVector& x, bool local = false )
    {
        f->attach ( x, local );
    }

    /// Attach mesh to function (data local if set, use with caution)
    inline void attach ( Mesh& mesh, bool local = false )
    {
        f->attach ( mesh, local );
    }

    /// Attach finite element to function (data local if set, use with caution)
    inline void attach ( Finite_element const& element, bool local = false )
    {
        f->attach ( element, local );
    }

    /// Reinitialize to discrete function (vector created automatically)
    void init ( Mesh& mesh, Finite_element& element );

    /// Return current type of function
    enum Type { constant/*, user, functionpointer*/, discrete };
    inline Type type() const
    {
        return _type;
    }
    /// print debug information
    inline void info ( bool detail = false ) const
    {
        f->info ( detail );
    }
private:

    // Pointer to current implementation (letter base class)
    Generic_function* f;

    // Current function type
    Type _type;

};
//-----------------------------------------------------------------------------

void Function::interpolate ( double * coefficients, Affine_map_shell<Mesh_face> const& map, Finite_element& element )
{
    // Delegate function call
    f->interpolate ( coefficients, map, element );
}
//-----------------------------------------------------------------------------
#endif
