// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _LOAD_TIME_LINE_H_
#define _LOAD_TIME_LINE_H_

//-----------------------------------------------------------------------------
class Load_time_line
{
public:
    // a function which returns multiplicative factor for loads depending on time
    // time and return values should vary between 0 and 1
    virtual double get_load_factor(double t) = 0;
};
//-----------------------------------------------------------------------------
namespace load_functions
{
//-----------------------------------------------------------------------------
//load which linearly increases in time
class Identity: public Load_time_line
{
public:
    double get_load_factor(double t)
    {
        return t;
    }
};
//-----------------------------------------------------------------------------
//load which increases linearly up to treshold and then stays constant. Initial delay is t0.
class Tresholded: public Load_time_line
{
public:
    Tresholded(double t, double t0 = 0.0): m_treshold(t), m_t0(t0), m_a(1.0/(t-t0)), m_b(t0/(t0-t)) {}
    double get_load_factor(double t)
    {
        if (t > m_t0)
        {
            if (t <= m_treshold)
                return t*m_a+m_b;
            else
                return 1;
        }
        return 0;
    }
private:
  const double m_treshold;
  const double m_t0;
  const double m_a;
  const double m_b;
};
//-----------------------------------------------------------------------------
//load which increases linearly up to treshold, then stays constant and then goes to 0. Initial delay is t0. Time at which it starts to decrease is is t1, reaches 0 at t2
class Trapez: public Load_time_line
{
public:
    Trapez(double t, double t0 = 0.0, double t1 = 1.0, double t2 = 1.0): m_treshold(t), m_t0(t0), m_t1(t1), m_t2(t2), m_a(1.0/(t-t0)), m_b(t0/(t0-t)), m_a1(1.0/(t1-t2)), m_b1(t2/(t2-t1)) {}
    double get_load_factor(double t)
    {
        if (t >= m_t0 && t <= m_t2)
        {
            if (t <= m_treshold)
                return t*m_a+m_b;
            else if(t >= m_t1)
                return t*m_a1+m_b1;
            else
              return 1.0;
        }
        return 0;
    }
private:
  const double m_treshold;
  const double m_a;
  const double m_b;
  const double m_a1;
  const double m_b1;
  const double m_t0;
  const double m_t1;
  const double m_t2;
};
//-----------------------------------------------------------------------------
//load which increases linearly up to treshold and then is 0 (sharp jump). Initial delay is t0.
class Tresholded_0: public Load_time_line
{
public:
    Tresholded_0(double t, double t0 = 0.0): m_treshold(t), m_t0(t0), m_a(1.0/(t-t0)), m_b(t0/(t-t0)) {}
    double get_load_factor(double t)
    {
      if(t > m_t0 && t <= m_treshold)
        return t*m_a+m_b;
      else
        return 0;
    }
private:
  const double m_treshold;
  const double m_t0;
  const double m_a;
  const double m_b;
};
//-----------------------------------------------------------------------------
//load which increases linearly up to treshold and then decreases linearly to 0 (triangular load shape). Initial delay is t0. Time at which it reaches 0 is t1
class Tresholded_tr: public Load_time_line
{
public:
    Tresholded_tr(double t, double t0 = 0.0, double t1 = 1.0): m_treshold(t), m_t0(t0), m_t1(t1), m_a(1.0/(t-t0)), m_b(t0/(t0-t)), m_a1(1.0/(t-t1)), m_b1(t1/(t1-t)) {}
    double get_load_factor(double t)
    {
        if (t >= m_t0 && t <= m_t1)
        {
            if (t <= m_treshold)
                return t*m_a+m_b;
            else
                return t*m_a1+m_b1;
        }
        return 0;
    }
private:
  const double m_treshold;
  const double m_a;
  const double m_b;
  const double m_a1;
  const double m_b1;
  const double m_t0;
  const double m_t1;
};
//-----------------------------------------------------------------------------
//load which turns on after agiven time and increases linearly to full value
class Delayed: public Load_time_line
{
public:
    Delayed(double t): m_treshold(t), m_inv_tr(1.0/(1.0-t)) {}
    double get_load_factor(double t)
    {
      double d = t-m_treshold;
      if(d>0)
        return d*m_inv_tr;
      else
        return 0.0;
    }
private:
  double m_treshold;
  double m_inv_tr;
};
//-----------------------------------------------------------------------------
}
#endif
