// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "fem/Affine_map_shell.h"
#include "matrix/Vector3.h"
#include "matrix/Matrix3.h"
#include "matrix/DenseMatrix.h"
#include "matrix/DenseVector.h"
#include "fem/fem_utils.h"
//-----------------------------------------------------------------------------
void reduce_tangent_matrix ( DenseMatrix &d )
{
    //pick the normal to the lamina component
    double d22_inv = 1.0/d ( 2,2 );
    for ( int i = 0; i < 2; ++i )
    {
        double di2 = d ( i,2 ) *d22_inv;
        for ( int j = 0; j < 2; ++j )
        {
            d ( i,j ) -= di2*d ( 2,j );
        }
        for ( int j = 3; j < 6; ++j )
        {
            d ( i,j ) -= di2*d ( 2,j );
        }
        d ( i,2 ) = 0;
        d ( 2,i ) = 0;
    }
    for ( int i = 3; i < 6; ++i )
    {
        double di2 = d ( i,2 ) *d22_inv;
        for ( int j = 0; j < 2; ++j )
        {
            d ( i,j ) -= di2*d ( 2,j );
        }
        for ( int j = 3; j < 6; ++j )
        {
            d ( i,j ) -= di2*d ( 2,j );
        }
        d ( i,2 ) = 0;
        d ( 2,i ) = 0;
    }
    d ( 2,2 ) = 0;
}
//-----------------------------------------------------------------------------
double norm ( DenseVector &v, int n_nodes )
{
    //dofs are laid out sequentialy in the vector. First x for all nodes than y,z,Dx,Dy,Dz
    DenseVector::Data::const_iterator x_iter = v.vec().begin();
    DenseVector::Data::const_iterator y_iter = x_iter+n_nodes;
    DenseVector::Data::const_iterator z_iter = y_iter+n_nodes;
    double norm = 0.0;
    for ( int i = 0; i < n_nodes; ++i, ++x_iter, ++y_iter, ++z_iter )
    {
        norm += ( *x_iter ) * ( *x_iter ) + ( *y_iter ) * ( *y_iter ) + ( *z_iter ) * ( *z_iter );
    }
    return sqrt ( norm );
}
//-----------------------------------------------------------------------------
void rotate_B_matrix ( DenseMatrix &res,  DenseMatrix const& b, rot_mat const& rot, bool init )
{
    if ( init )
        res.zero();
    double b0 = b ( 0,0 ), b1 = b ( 1,1 ), b2 = b ( 2,2 );
    double h0 = b ( 0,3 ), h1 = b ( 1,4 ), h2 = b ( 2,5 );
    Vector3 Nglob ( b0,b1,b2 );
    Vector3 Nlam = rot.inv_rotate ( Nglob );
    Vector3 Mglob ( h0,h1,h2 );
    Vector3 Mlam = rot.inv_rotate ( Mglob );
    b0 = Nlam.x;
    b1 = Nlam.y;
    b2 = Nlam.z;
    h0 = Mlam.x;
    h1 = Mlam.y;
    h2 = Mlam.z;

    for ( int i = 0; i < 3; ++i )
    {
        res ( 0,i ) = b0*rot ( 0,i );
        res ( 1,i ) = b1*rot ( 1,i );
        res ( 2,i ) = b2*rot ( 2,i );

        res ( 3,i ) = b1*rot ( 0,i ) + b0*rot ( 1,i );
        res ( 4,i ) = b2*rot ( 1,i ) + b1*rot ( 2,i );
        res ( 5,i ) = b2*rot ( 0,i ) + b0*rot ( 2,i );

        int ind = 3+i;
        res ( 0,ind ) = h0*rot ( 0,i );
        res ( 1,ind ) = h1*rot ( 1,i );
        res ( 2,ind ) = h2*rot ( 2,i );

        res ( 3,ind ) = h1*rot ( 0,i ) + h0*rot ( 1,i );
        res ( 4,ind ) = h2*rot ( 1,i ) + h1*rot ( 2,i );
        res ( 5,ind ) = h2*rot ( 0,i ) + h0*rot ( 2,i );
    }
}
//-----------------------------------------------------------------------------
void inv_rotate_B_matrix ( DenseMatrix &res,  DenseMatrix const& b, rot_mat const& rot, bool init )
{
    if ( init )
        res.zero();
    double b0 = b ( 0,0 ), b1 = b ( 1,1 ), b2 = b ( 2,2 );
    double h0 = b ( 0,3 ), h1 = b ( 1,4 ), h2 = b ( 2,5 );
    Vector3 Nglob ( b0,b1,b2 );
    Vector3 Nlam = rot.inv_rotate ( Nglob );
    Vector3 Mglob ( h0,h1,h2 );
    Vector3 Mlam = rot.inv_rotate ( Mglob );
    b0 = Nlam.x;
    b1 = Nlam.y;
    b2 = Nlam.z;
    h0 = Mlam.x;
    h1 = Mlam.y;
    h2 = Mlam.z;

    for ( int i = 0; i < 3; ++i )
    {
        res ( 0,i ) = b0*rot ( i,0 );
        res ( 1,i ) = b1*rot ( i,1 );
        res ( 2,i ) = b2*rot ( i,2 );

        res ( 3,i ) = b1*rot ( i,0 ) + b0*rot ( i,1 );
        res ( 4,i ) = b2*rot ( i,1 ) + b1*rot ( i,2 );
        res ( 5,i ) = b2*rot ( i,0 ) + b0*rot ( i,2 );

        int ind = 3+i;
        res ( 0,ind ) = h0*rot ( i,0 );
        res ( 1,ind ) = h1*rot ( i,1 );
        res ( 2,ind ) = h2*rot ( i,2 );

        res ( 3,ind ) = h1*rot ( i,0 ) + h0*rot ( i,1 );
        res ( 4,ind ) = h2*rot ( i,1 ) + h1*rot ( i,2 );
        res ( 5,ind ) = h2*rot ( i,0 ) + h0*rot ( i,2 );
    }

}
//-----------------------------------------------------------------------------
void to_voigt_vector ( Matrix3s const&m, DenseVector &v )
{
    v ( 0 ) = m.xx();
    v ( 1 ) = m.yy();
    v ( 2 ) = m.zz();
    v ( 3 ) = m.xy();
    v ( 4 ) = m.yz();
    v ( 5 ) = m.xz();
}
//-----------------------------------------------------------------------------
void from_voigt_vector ( DenseVector const&v, Matrix3s &m )
{
    m.xx() = v ( 0 );
    m.yy() = v ( 1 );
    m.zz() = v ( 2 );
    m.xy() = v ( 3 );
    m.yz() = v ( 4 );
    m.xz() = v ( 5 );
}
//-----------------------------------------------------------------------------
void adjust_shell_lumped_mass_mat ( DenseVector &lum_m, Affine_map_shell<Mesh_face> const& map, double a )
{
    int n = lum_m.size();
    int n_nodes = n/6;
    int const i_rot_dof = 3; //starting index of rotational defrees of freedom
    int offset = n_nodes*3;
    for ( int i = 0; i < offset; ++i )
    {

        lum_m ( i+offset ) = lum_m ( i+offset ) *a;

    }


}
//-----------------------------------------------------------------------------
