// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _LINE_LOAD_H_
#define _LINE_LOAD_H_

#include "mesh/mesh_handle.h"
#include "fem/timed_load.h"
#include <vector>
#include <iterator>
//-----------------------------------------------------------------------------
struct Edge_use_load
{
    Edge_use_load(Edge_use_handle e, Vector l, double t) : edge(e), load_factor(l), load_position(t) {}
    Edge_use_handle edge;
    Vector load_factor; //line force density. Has to be multiplied by the edge length to get the force
    double load_position;
};
//-----------------------------------------------------------------------------
class Line_load : public Timed_load
{
public:
    typedef std::vector<Edge_use_load>::iterator Load_iterator;
    typedef std::vector<Edge_use_load>::const_iterator Load_const_iterator;

    ///construct with specified pressure (default = 0.0) and load position t in the director direction reference space (t should be in range -1:1, default = 0.0)
    Line_load( double p=0.0, double t=0.0): m_load(p), m_load_pos(t) {}

    ///add Edge_use_load on specified edge, with specified load_factor and load position
    void add(Edge_use_handle f , Vector l = Vector(0,0,0), double t = 0.0)
    {
        m_edges.push_back(Edge_use_load(f,l,t));
    }
    ///add Edge_use_load on edges within iterators, with specified load_factor and load position
    template<class I>
    void add(I from, I beyond , Vector l = Vector(0,0,0), double t = 0.0)
    {
        base_add(typename std::iterator_traits<I>::iterator_category(), from, beyond , l, t);
    }

    /// get load value of this Line_load
    double load() const {
        return m_load;
    }
    /// get time position of this Line_load
    double load_position() const {
        return m_load_pos;
    }
    /// check if Line_load applies to all edges in the mesh (false) or only to subset of edges (true)
    bool is_localized() const {
        return !m_edges.empty();
    }

    /// iterators over Edge_use_loads
    Load_iterator loads_begin() {
        return m_edges.begin();
    }
    Load_iterator loads_end() {
        return m_edges.end();
    }
    Load_const_iterator loads_begin() const {
        return m_edges.begin();
    }
    Load_const_iterator loads_end() const {
        return m_edges.end();
    }

private:

    template<class I>
    void base_add(std::bidirectional_iterator_tag, I from, I beyond , Vector l, double t)
    {
        while (from != beyond)
        {
            m_edges.push_back(Edge_use_load(*from,l,t));
            ++from;
        }
    }

    template<class I>
    void base_add(std::random_access_iterator_tag, I from, I beyond , Vector l, double t)
    {
        m_edges.reserve(beyond-from);
        while (from != beyond)
        {
            m_edges.push_back(Edge_use_load(*from,l,t));
            ++from;
        }
    }

    double m_load;
    double m_load_pos;
    std::vector<Edge_use_load> m_edges;
};
//-----------------------------------------------------------------------------
#endif
