// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#define _QUAD_AXIAL_LOAD_
#ifdef _QUAD_AXIAL_LOAD_

#include "io/VTK_mesh_io.h"
#include "io/VTK_ostream.h"

#include "mesh/mesh.h"
#include "mesh/mesh_manipulator.h"
#include "mesh/boundary_builder.h"
#include "mesh_generator/mesh_generator.h"

#include "utils/param_init.h"
#include "utils/globals.h"

#include "fem/FEM.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/elements/Quad4P_rule.h"

#include "material/isotropic_material.h"
#include "material/anisotropic_material.h"

#include "solver/Elasticity_shell.h"
#include "solver/boundary_conditions.h"
#include "solver/direct_boundary_conditions.h"
#include "solver/Newton_solver.h"
#include "solver/newmark_solver.h"

#include "predicates/node_predicates.h"
#include "matrix/principal_directions.h"
//-----------------------------------------------------------------------------
int main ( int ac, char* av[] )
{
    Mesh_generator* mg = Mesh_generator::get_instance();

    parse_options ( ac, av );

    std::string f_name = "square";
    std::cout << "Creating mesh\n";
    Mesh_handle m = mg->create_regular_polygon_mesh ( 1.0, 4, PI/4.0 );

    Mesh_handle new_m = m;

    Mesh_manipulator manip ( new_m );
    manip.init_nodes ( 0.01 );
    new_m->statistics();

    std::cout << "Setting material\n";
    Isotropic_material material ( 400.0, 0.2 );
    material.info();

    manip.set_material ( &material, new_m->faces_begin(), new_m->faces_end() );

    Quadrature_rule *q_rule = new Quad12P_rule();
    Quadrature_rule_surface *q_rule_surf = new Quad4P_rule();

    Elasticity_shell::Bilinear_form a;
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();

    Affine_map_shell<Mesh_face> aff_map ( &test_element, q_rule, q_rule_surf );

    std::cout << "Setting boundary conditions\n";
    //point predicate lists for bc
    point_predicates::Predicate_list<> p_list01, p_list02, p_list03, p_list04;
    node_predicates::Predicate_list p_list1, p_list2, p_list3, p_list4;

    point_predicates::X_position< std::greater<double> > x_nod_pos01 ( 0.5 );
    point_predicates::X_position< std::less<double> > x_nod_pos02 ( -0.5 );
    point_predicates::Y_position< std::greater<double> > y_nod_pos01 ( 0.5 );
    point_predicates::Y_position< std::less<double> > y_nod_pos02 ( -0.5 );

    //top right node
    p_list01.add ( &x_nod_pos01 );
    p_list01.add ( &y_nod_pos01 );
    //top left node
    p_list02.add ( &x_nod_pos02 );
    p_list02.add ( &y_nod_pos01 );
    //bottom right node
    p_list03.add ( &x_nod_pos01 );
    p_list03.add ( &y_nod_pos02 );
    //bottom left node
    p_list04.add ( &x_nod_pos02 );
    p_list04.add ( &y_nod_pos02 );

    //get the nodes to apply forces to
    node_predicates::X_position< std::greater<double> > x_nod_pos1 ( 0.5 );
    node_predicates::X_position< std::less<double> > x_nod_pos2 ( -0.5 );
    node_predicates::Y_position< std::greater<double> > y_nod_pos1 ( 0.5 );
    node_predicates::Y_position< std::less<double> > y_nod_pos2 ( -0.5 );
    //top right node
    p_list1.add ( &x_nod_pos1 );
    p_list1.add ( &y_nod_pos1 );
    //top left node
    p_list2.add ( &x_nod_pos2 );
    p_list2.add ( &y_nod_pos1 );
    //bottom right node
    p_list3.add ( &x_nod_pos1 );
    p_list3.add ( &y_nod_pos2 );
    //bottom left node
    p_list4.add ( &x_nod_pos2 );
    p_list4.add ( &y_nod_pos2 );
    std::vector<Node_handle> sel_nodes1, sel_nodes2, sel_nodes3, sel_nodes4;
    select ( new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter ( sel_nodes1 ), p_list1 ); //top right node
    select ( new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter ( sel_nodes2 ), p_list2 ); //top left node
    select ( new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter ( sel_nodes3 ), p_list3 ); //bottom right node
    select ( new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter ( sel_nodes4 ), p_list4 ); //bottom left node

    std::vector<Node_handle>::iterator it, e_it;

    double force = 4;

    Concentrated_load * load1 = new Concentrated_load ( force );
    for ( it = sel_nodes1.begin(), e_it = sel_nodes1.end(); it != e_it; ++it )
    {
        load1->add ( *it, Vector ( 1,0,0 ) );
    }
    Concentrated_load * load3 = new Concentrated_load ( force );
    for ( it = sel_nodes3.begin(), e_it = sel_nodes3.end(); it != e_it; ++it )
        load3->add ( *it, Vector ( 1,0,0 ) );

    Fix_nodes *fix_yz = new Fix_nodes ( false, true, true, false );
    Fix_nodes *fix_xz = new Fix_nodes ( true, false, true, false );
    Fix_nodes *fix_all = new Fix_nodes ( true, true, true, false );
    Fix_nodes *fix_z = new Fix_nodes ( false, false, true, false );

    BC_evaluator<Direct_boundary_condition, Mesh_face> dbc_e2;
    dbc_e2.set_bc ( fix_xz, sel_nodes2.begin(), sel_nodes2.end() ); //top left node
    BC_evaluator<Direct_boundary_condition, Mesh_face> dbc_e3;
    dbc_e3.set_bc ( fix_yz, sel_nodes3.begin(), sel_nodes3.end() ); //bottom right node
    BC_evaluator<Direct_boundary_condition, Mesh_face> dbc_e4;
    dbc_e4.set_bc ( fix_all, sel_nodes4.begin(), sel_nodes4.end() ); //bottom left node
    BC_evaluator<Direct_boundary_condition, Mesh_face> dbc_eZ ( fix_z, *new_m ); //all nodes

    BC_evaluator_list<Direct_boundary_condition, Mesh_face> DBC_list;
	
    DBC_list.add ( &dbc_e2 );
    DBC_list.add ( &dbc_e3 );
    DBC_list.add ( &dbc_e4 );
    DBC_list.add ( &dbc_eZ );

    Load_case lc ( new_m );
    lc.add_load ( load1 );
    lc.add_load ( load3 );

    const int N_STEPS = 100;

    PVD_file result ( globals.outDir + "figure2abP2_viz.pvd" );
    std::ofstream spreadsheet ( ( globals.outDir + "fig2aP2shell.data" ).c_str() );

    result.set ( "write_face_flags", true );
    result.set ( "write_elastic_mod", true );
    result.set ( "write_shell_thickness", true );
    result.set ( "write_strain", true );
    Newton_solver n_solver ( aff_map, new_m );

//     n_solver.set_output(result, Newton_solver::result);
    n_solver.set_output ( spreadsheet, Newton_solver::spreadsheet );

    const int INIT_STEPS = 100;

    std::cout << "Solving for mu = 0.2 (please wait)...\n";
    n_solver.solve ( INIT_STEPS, DBC_list, lc, N_STEPS );
    spreadsheet.close();

    n_solver.reset();
    manip.reset_to_init_mesh_state();
    lc.update_time ( 0 );
    PVD_file result2 ( globals.outDir + "figure2abP4_viz.pvd" );
    std::ofstream spreadsheet2 ( ( globals.outDir + "fig2aP4shell.data" ).c_str() );
    material.update ( 400.0, 0.4 );
//     n_solver.set_output(result2, Newton_solver::result);
    n_solver.set_output ( spreadsheet2, Newton_solver::spreadsheet );
    std::cout << "Solving for mu = 0.4 (please wait)...\n";
    n_solver.solve ( INIT_STEPS, DBC_list, lc, N_STEPS );
    spreadsheet2.close();

    return EXIT_SUCCESS;
}
//-----------------------------------------------------------------------------
#endif
