// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#define _QUAD_ANISOTROPY_DIRECTION_
#ifdef _QUAD_ANISOTROPY_DIRECTION_

#include "io/VTK_mesh_io.h"
#include "io/VTK_ostream.h"

#include "mesh/mesh.h"
#include "mesh/mesh_manipulator.h"
#include "mesh/boundary_builder.h"

#include "utils/param_init.h"
#include "utils/globals.h"

#include "fem/FEM.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/elements/Quad4P_rule.h"

#include "material/isotropic_material.h"
#include "material/anisotropic_material.h"

#include "solver/Elasticity_shell.h"
#include "solver/boundary_conditions.h"
#include "solver/Newton_solver.h"
#include "solver/newmark_solver.h"

#include "predicates/node_predicates.h"
#include "matrix/principal_directions.h"

//-----------------------------------------------------------------------------
int main( int ac, char* av[] )
{
//     VTK_ofstream vtk_out;
    
    parse_options( ac, av );
    //BEGIN input
    std::cout << "Creating mesh\n";
    std::ifstream IN( (globals.initDir + "divided_quad2.cif").c_str() );
    Mesh_handle m = Boundary_builder::build_mesh( IN );
    //END input

    Mesh_handle new_m = m;
    Mesh_manipulator manip(new_m);
    manip.init_nodes(0.01);
    
    std::cout << "Setting material\n";
    Anisotropic_material material( 400, 400, 0.1 );
    material.info();
    
    manip.set_material( &material, new_m->faces_begin(), new_m->faces_end());
    manip.set_fiber_direction( Vector3(1,0,0), new_m->faces_begin(), new_m->faces_end());


    Quadrature_rule *q_rule = new Quad12P_rule();
    Quadrature_rule_surface *q_rule_surf = new Quad4P_rule();
    Elasticity_shell::Bilinear_form a;
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();
    Affine_map_shell<Mesh_face> aff_map(&test_element, q_rule, q_rule_surf);

    std::cout << "Setting boundary conditions\n";
    point_predicates::Predicate_list<> p_list01, p_list02, p_list03, p_list04;
    node_predicates::Predicate_list p_list1, p_list2, p_list3, p_list4;

    point_predicates::X_position< std::greater<double> > x_nod_pos01(0.7);
    point_predicates::X_position< std::less<double> > x_nod_pos02(-0.7);
    point_predicates::Y_position< std::greater<double> > y_nod_pos01(0.7);
    point_predicates::Y_position< std::less<double> > y_nod_pos02(-0.7);

    //top right node
    p_list01.add(&x_nod_pos01);
    p_list01.add(&y_nod_pos01);
    //top left node
    p_list02.add(&x_nod_pos02);
    p_list02.add(&y_nod_pos01);
    //bottom right node
    p_list03.add(&x_nod_pos01);
    p_list03.add(&y_nod_pos02);
    //bottom left node
    p_list04.add(&x_nod_pos02);
    p_list04.add(&y_nod_pos02);

    //get the nodes to apply forces to
    node_predicates::X_position< std::greater<double> > x_nod_pos1(0.7);
    node_predicates::X_position< std::less<double> > x_nod_pos2(-0.7);
    node_predicates::Y_position< std::greater<double> > y_nod_pos1(0.7);
    node_predicates::Y_position< std::less<double> > y_nod_pos2(-0.7);
    //top right node
    p_list1.add(&x_nod_pos1);
    p_list1.add(&y_nod_pos1);
    //top left node
    p_list2.add(&x_nod_pos2);
    p_list2.add(&y_nod_pos1);
    //bottom right node
    p_list3.add(&x_nod_pos1);
    p_list3.add(&y_nod_pos2);
    //bottom left node
    p_list4.add(&x_nod_pos2);
    p_list4.add(&y_nod_pos2);
    std::vector<Node_handle> sel_nodes1, sel_nodes2, sel_nodes3, sel_nodes4;
    select(new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter(sel_nodes1), p_list1);//top right node
    select(new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter(sel_nodes2), p_list2);//top left node
    select(new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter(sel_nodes3), p_list3);//bottom right node
    select(new_m->nodes_begin(), new_m->nodes_end(), std::back_inserter(sel_nodes4), p_list4);//bottom left node
    std::vector<Node_handle>::iterator it, e_it;

    double force1 = 100;
    double force2 = 400;

    Concentrated_load * load1_x = new Concentrated_load(force2);
    for (it = sel_nodes1.begin(), e_it = sel_nodes1.end(); it != e_it; ++it)//top right node
        load1_x->add(*it, Vector(1,0,0));
    Concentrated_load * load1_y = new Concentrated_load(force1);
    for (it = sel_nodes1.begin(), e_it = sel_nodes1.end(); it != e_it; ++it)//top right node
        load1_y->add(*it, Vector(0,1,0));

    Concentrated_load * load2_x = new Concentrated_load(-force2);
    for (it = sel_nodes2.begin(), e_it = sel_nodes2.end(); it != e_it; ++it)//top left node
        load2_x->add(*it, Vector(1,0,0));
    Concentrated_load * load2_y = new Concentrated_load(force1);
    for (it = sel_nodes2.begin(), e_it = sel_nodes2.end(); it != e_it; ++it)//top left node
        load2_y->add(*it, Vector(0,1,0));

    Concentrated_load * load3_x = new Concentrated_load(force2);
    for (it = sel_nodes3.begin(), e_it = sel_nodes3.end(); it != e_it; ++it)//bottom right node
        load3_x->add(*it, Vector(1,0,0));
    Concentrated_load * load3_y = new Concentrated_load(-force1);
    for (it = sel_nodes3.begin(), e_it = sel_nodes3.end(); it != e_it; ++it)//bottom right node
        load3_y->add(*it, Vector(0,1,0));

    Concentrated_load * load4_x = new Concentrated_load(-force2);
    for (it = sel_nodes4.begin(), e_it = sel_nodes4.end(); it != e_it; ++it)//bottom left node
        load4_x->add(*it, Vector(1,0,0));
    Concentrated_load * load4_y = new Concentrated_load(-force1);
    for (it = sel_nodes4.begin(), e_it = sel_nodes4.end(); it != e_it; ++it)//bottom left node
        load4_y->add(*it, Vector(0,1,0));

    Point_predicate *true_pred  = new point_predicates::True();
    Fix *fix_yz = new Fix(false, true, true, false, *true_pred);
    Fix *fix_xz = new Fix(true, false, true, false, *true_pred);
    Fix *fix_all = new Fix(true, true, true, false, *true_pred);
    Fix_Z *fix_z = new Fix_Z();

    BC_evaluator<Boundary_condition, Mesh_face> bc_e2;
    bc_e2.set_bc( fix_xz, sel_nodes2.begin(), sel_nodes2.end() );//top left node
    BC_evaluator<Boundary_condition, Mesh_face> bc_e3;
    bc_e3.set_bc( fix_yz, sel_nodes3.begin(), sel_nodes3.end() );//bottom right node
    BC_evaluator<Boundary_condition, Mesh_face> bc_e4;
    bc_e4.set_bc( fix_all, sel_nodes4.begin(), sel_nodes4.end() );//bottom left node
    BC_evaluator<Boundary_condition, Mesh_face> bc_eZ( fix_z, *new_m );//all nodes


    BC_evaluator_list<Boundary_condition, Mesh_face> BC_list;
    BC_list.add(&bc_eZ);

    Load_case lc(new_m);
    lc.add_load(load1_x);
    lc.add_load(load1_y);
    lc.add_load(load2_x);
    lc.add_load(load2_y);
    lc.add_load(load3_x);
    lc.add_load(load3_y);
    lc.add_load(load4_x);
    lc.add_load(load4_y);

    const int N_STEPS = 50;

    manip.set_mesh(new_m);
    manip.init_nodes(0.01);
    material.update(400000, 800000, 0.2);

    State_interpolator stress_int(new_m, aff_map);
    std::ofstream spreadsheet((globals.outDir + "fig2Dshell.data").c_str());
    std::cout << "Solving (please wait)...\n";
    for (double thet = 0; thet < PI; thet += PI/100)
    {
        double ang = 180.0*(thet/PI);
//         PVD_file pvd_f1(globals.outDir + "figure2d_viz.pvd");
//         pvd_f1.set("write_strain", true);
//         pvd_f1.set("write_fiber_direction", true);

        Newmark_solver nm_solver(aff_map, new_m);
        nm_solver.add("Newmark relative tolerance", 1e-6);
        nm_solver.add("Newmark absolute tolerance", 1e-16);
        nm_solver.add ( "Newmark gamma", 0.0 );

        lc.update_time(0.0);
        manip.reset_to_init_mesh_state();
        Vector3 r_vec(cos(thet),sin(thet),0);
        std::cout << "fiber vector = (" << r_vec.x << ", " << r_vec.y << ", " << r_vec.z << ", " << ")\n";

        manip.set_fiber_direction( r_vec, new_m->faces_begin(), new_m->faces_end());
        nm_solver.setup_case(&lc, &BC_list);
        Function rho(100000.0);
        nm_solver.construct_forms(rho);
        double lf = 0;

        for (int n = 0; n < N_STEPS; ++n)
        {
            lf += 1.0/N_STEPS;
            nm_solver.assemble_matrices();
            nm_solver.assemble_load(&lc, lf);
//             int n_iter = nm_solver.solve_explicit_overdamped(1000000, 0.001, pvd_f1, 1);
            int n_iter = nm_solver.solve_explicit_overdamped(1000000, 0.001);
            manip.displace_mesh( nm_solver.get_u() );
        }
        stress_int.interpolate_stress();

        //BEGIN file output
        principal_directions pd;
        Node_handle n = new_m->node(0);
        spreadsheet << thet << " " << ang << " ";
        pd.calculate_principal_directions( stress_int.stress(n), true );
        for ( int i = 0 ; i < 3; ++i )
        {
            spreadsheet << pd.val(i) << " ";
        }

        double s1 = pd.val(0)-pd.val(1);
        double s2 = pd.val(0)-pd.val(2);
        double s3 = pd.val(1)-pd.val(2);
        double misses = 0.5*(s1*s1 + s2*s2 + s3*s3);
        spreadsheet << std::sqrt(misses);

        spreadsheet << "\n";
        //END file output
    }
    return EXIT_SUCCESS;


}
//-----------------------------------------------------------------------------

#endif
