// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
//pressurized plate simulation
#define _MAIN_PRESSURIZED_PLATE_
#ifdef _MAIN_PRESSURIZED_PLATE_

#include "io/VTK_mesh_io.h"
#include "io/VTK_ostream.h"

#include "mesh/mesh.h"
#include "mesh/mesh_manipulator.h"
#include "mesh/boundary_builder.h"
#include "mesh_generator/mesh_generator.h"
#include "mesh/mesh_builder.h"

#include "utils/param_init.h"
#include "utils/globals.h"

#include "fem/FEM.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/elements/Quad4P_rule.h"

#include "material/isotropic_material.h"
#include "material/anisotropic_material.h"

#include "solver/Elasticity_shell.h"
#include "solver/boundary_conditions.h"
#include "solver/direct_boundary_conditions.h"
#include "solver/Newton_solver.h"
#include "solver/newmark_solver.h"

#include "predicates/node_predicates.h"
#include "predicates/edge_use_predicates.h"
#include "matrix/principal_directions.h"
//-----------------------------------------------------------------------------
int main( int ac, char* av[] )
{
    Mesh_generator* mg = Mesh_generator::get_instance();
    VTK_ofstream vtk_out;
    
    parse_options( ac, av );
    
    std::cout << "Creating mesh\n";
    Mesh_handle new_m= mg->create_quad_mesh(10, 20, 20);
    new_m->set_type(Mesh::SURFACE);
    Mesh_manipulator manip(new_m);
    manip.translate_mesh(Vector(-5,-5,0));
    manip.init_nodes(1);
    new_m->statistics();

    std::cout << "Setting material\n";
    Isotropic_material material( 4000, .2 );
    material.info();
    manip.set_material( &material, new_m->faces_begin(), new_m->faces_end());

    Quadrature_rule *q_rule = new Quad12P_rule();
    Quadrature_rule_surface *q_rule_surf = new Quad4P_rule();

    Elasticity_shell::Bilinear_form a;
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();

    Affine_map_shell<Mesh_face> aff_map(&test_element, q_rule, q_rule_surf);
    
//BEGIN boundary conditions and load
    std::cout << "Setting boundary conditions\n";
    std::list<Edge_use_handle> sel_edges, sel_edges2;
    edge_use_predicates::Predicate_list eupl;

    eupl.add ( new edge_use_predicates::Boundary() );
    select ( new_m->edge_uses_begin(), new_m->edge_uses_end(), std::back_inserter ( sel_edges ),eupl );

    std::list<Node_handle> sel_nodes, sel_nodes2;
    select_edge_use_nodes(sel_edges.begin(), sel_edges.end(), std::back_inserter ( sel_nodes )); 

    Fix *fix_all = new Fix(true, true, true, false, point_predicates::True());

    BC_evaluator<Boundary_condition, Mesh_face> bc_e1;
    bc_e1.set_bc( fix_all, sel_nodes.begin(), sel_nodes.end());//bc for cell nodes

    BC_evaluator_list<Boundary_condition, Mesh_face> BC_list;
    BC_list.add(&bc_e1);

    Load_case lc(new_m);
    Surface_load * s_load = new Surface_load(4);
    lc.add_load(s_load);
    
//END boundary conditions and load
    const int N_STEPS = 40;

//BEGIN implicit solver 

    PVD_file result ( globals.outDir + "figure2ef_viz.pvd" );
    Newton_solver n_solver(aff_map, new_m);

    n_solver.set_output(result, Newton_solver::result);
    std::cout << "Solving (please wait)...\n";
    n_solver.solve(N_STEPS, BC_list, lc, N_STEPS);
    
    State_interpolator & st_int = n_solver.state_interpolator();
    std::ofstream stress_file((globals.outDir + "missesStressShell.dat").c_str());
    std::ofstream strain_file((globals.outDir + "missesStrainShell.dat").c_str());
    st_int.interpolate_tensors();
    std::vector<principal_directions> princ(new_m->size_of_nodes());
    Mesh::Node_iterator n_iter, n_end;
    for ( n_iter = new_m->nodes_begin(), n_end = new_m->nodes_end(); n_iter != n_end; ++n_iter)
    {
       principal_directions  stress_pd;
       stress_pd.calculate_principal_directions( st_int.stress(n_iter), true );
       double s1 = stress_pd.val(0) - stress_pd.val(1);
       double s2 = stress_pd.val(1) - stress_pd.val(2);
       double s3 = stress_pd.val(2) - stress_pd.val(0);
       stress_file << n_iter->id() << ", " << sqrt(0.5*(s1*s1+s2*s2+s3*s3)) << "\n";
       principal_directions  strain_pd;
       strain_pd.calculate_principal_directions( st_int.strain(n_iter), true );

       double st1 = strain_pd.val(0) - strain_pd.val(1);
       double st2 = strain_pd.val(1);
       double st3 = strain_pd.val(0);
       strain_file << n_iter->id() << ", " << 1.0/(1.0+0.2)*sqrt(0.5*(st1*st1+st2*st2+st3*st3)) << "\n";
    }
//END implicit solver 
    return EXIT_SUCCESS;
}
//-----------------------------------------------------------------------------
#endif
