// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#define _MAIN_PRESSURIZED_MERISTEM_
#ifdef _MAIN_PRESSURIZED_MERISTEM_

#include "io/VTK_mesh_io.h"
#include "io/VTK_ostream.h"

#include "mesh/mesh.h"
#include "mesh/mesh_builder.h"
#include "mesh_generator/mesh_generator.h"
#include "mesh/mesh_manipulator.h"

#include "utils/param_init.h"
#include "utils/globals.h"

#include "fem/FEM.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/elements/Quad4P_rule.h"

#include "material/isotropic_material.h"

#include "solver/Elasticity_shell.h"
#include "solver/boundary_conditions.h"
#include "solver/Newton_solver.h"
#include "solver/newmark_solver.h"

#include "predicates/node_predicates.h"
#include "predicates/edge_use_predicates.h"

//-----------------------------------------------------------------------------
int main ( int ac, char* av[] )
{
    Mesh_generator* mg = Mesh_generator::get_instance();
    parse_options ( ac, av );

    std::cout << "Creating mesh\n";
    std::ifstream init_in ( ( globals.initDir + "/finit/S1C.init" ).c_str() );
    Mesh_handle m = import_from_spring_format ( init_in );
    m->set_type ( Mesh::SURFACE );

    Mesh_builder *builder =  Mesh_builder::get_instance();

    Mesh_handle new_m = builder->subdivide_mesh ( m );
    Mesh_manipulator manip ( new_m );
    manip.set_mesh ( new_m );
    manip.init_nodes ( 1 );

    std::cout << "Setting material\n";
    Isotropic_material material ( 40, 0.2 );
    manip.set_material ( &material, new_m->faces_begin(), new_m->faces_end() );
    material.info();
    Quadrature_rule *q_rule = new Quad12P_rule();
    Quadrature_rule_surface *q_rule_surf = new Quad4P_rule();

    Elasticity_shell::Bilinear_form a;
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();
    Affine_map_shell<Mesh_face> aff_map ( &test_element, q_rule, q_rule_surf );

    std::cout << "Setting boundary conditions\n";
    std::list<Edge_use_handle> sel_edges, sel_edges2;
    edge_use_predicates::Predicate_list eupl;
    eupl.add ( new edge_use_predicates::Boundary() );
    select ( new_m->edge_uses_begin(), new_m->edge_uses_end(), std::back_inserter ( sel_edges ),eupl );

    std::list<Node_handle> sel_nodes, sel_nodes2;
    select_edge_use_nodes ( sel_edges.begin(), sel_edges.end(), std::back_inserter ( sel_nodes ) );

    Fix *fix_all = new Fix ( true, true, true, false, point_predicates::True() );

    BC_evaluator<Boundary_condition, Mesh_face> bc_e1;
    bc_e1.set_bc ( fix_all, sel_nodes.begin(), sel_nodes.end() ); //bc for cell nodes

    BC_evaluator_list<Boundary_condition, Mesh_face> BC_list;
    BC_list.add ( &bc_e1 );

    Load_case lc ( new_m );
    Surface_load * s_load = new Surface_load ( 0.01 );
    lc.add_load ( s_load );
    const int N_STEPS = 40;

    std::cout << "Solving (please wait)...\n";
    PVD_file result ( globals.outDir + "figureS1c.pvd" );
    result.set ( "write_strain", true );

    Newton_solver n_solver ( aff_map, new_m );
    n_solver.set_output ( result, Newton_solver::result );
    n_solver.solve ( N_STEPS, BC_list, lc, N_STEPS );

    return EXIT_SUCCESS;
}
//-----------------------------------------------------------------------------
#endif
