// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#define _PRESSURIZED_TEMPLATES_
#ifdef _PRESSURIZED_TEMPLATES_

#include "mesh/mesh.h"
#include "mesh/mesh_builder.h"
#include "mesh_generator/mesh_generator.h"
#include "mesh/mesh_manipulator.h"

#include "utils/param_init.h"
#include "utils/globals.h"

#include "fem/FEM.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/elements/Quad4P_rule.h"

#include "material/isotropic_material.h"

#include "solver/Elasticity_shell.h"
#include "solver/boundary_conditions.h"
#include "solver/Newton_solver.h"
#include "solver/newmark_solver.h"

#include "predicates/node_predicates.h"
#include "predicates/edge_use_predicates.h"
//-----------------------------------------------------------------------------
int main ( int ac, char* av[] )
{
    parse_options ( ac, av );
    std::cout << "Creating mesh\n";
    std::ifstream init_in ( ( globals.initDir + "/finit/S1D.init" ).c_str() );
    Mesh_handle m = import_from_spring_format ( init_in );

    Mesh_builder *builder =  Mesh_builder::get_instance();
    Mesh_handle new_m = builder->subdivide_mesh ( m );
    new_m->set_type ( Mesh::SURFACE );
    Mesh_manipulator manip ( new_m );
    manip.init_nodes ( 1 );
    new_m->statistics();

    std::cout << "Setting material\n";
    Isotropic_material material ( 90.0, 0.2 );
    manip.set_material ( &material, new_m->faces_begin(), new_m->faces_end() );
    material.info();

    Quadrature_rule *q_rule = new Quad12P_rule();
    Quadrature_rule_surface *q_rule_surf = new Quad4P_rule();
    Elasticity_shell::Bilinear_form a;
    Finite_element& test_element = a.test();
    Finite_element& trial_element = a.trial();
    Affine_map_shell<Mesh_face> aff_map ( &test_element, q_rule, q_rule_surf );

    std::cout << "Setting boundary conditions\n";
    BC_evaluator_list<Boundary_condition, Mesh_face> BC_list;
    Load_case lc ( new_m );
    Surface_load * s_load = new Surface_load ( 1.0e-1 );
    lc.add_load ( s_load );

    const int N_STEPS = 40;

    std::cout << "Solving (please wait)...\n";
    State_interpolator stress_int ( new_m, aff_map );
    PVD_file pvd_f1 ( globals.outDir + "figureS1d.pvd" );

    Newmark_solver nm_solver ( aff_map, new_m );
    nm_solver.set ( "Newmark relative tolerance", 1e-6 );
    nm_solver.set ( "Newmark absolute tolerance", 1e-9 );
    nm_solver.set ( "Newmark gamma", 0.0 );
    pvd_f1.set ( "write_strain", true );

    nm_solver.setup_case ( &lc, &BC_list );
    Function rho ( 50.0 );
    nm_solver.construct_forms ( rho );
    double lf = 0;

    for ( int n = 0; n < N_STEPS; ++n )
    {
        lf += 1.0/N_STEPS;
        nm_solver.assemble_matrices();
        nm_solver.assemble_load ( &lc, lf );
        int n_iter = nm_solver.solve_explicit_overdamped ( 100000, 0.001, pvd_f1, 3 );
        manip.displace_mesh ( nm_solver.get_u() );
    }

    return EXIT_SUCCESS;
}
//-----------------------------------------------------------------------------

#endif
