// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "io/VTK_mesh_io.h"
#include "utils/error_macros.h"
#include "fem/Function.h"
//-----------------------------------------------------------------------------
void print_surface_mesh( VTK_ostream& out, Mesh const& m, Function const* f )
{
    int n_nodes = m.size_of_nodes();
    int n_elements = m.size_of_faces();

    out <<"<Piece NumberOfPoints=\"" << (int)n_nodes << "\" NumberOfCells=\"" << (int)n_elements << "\">\n";
    out << "<Points>\n";
    out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    Mesh::Node_const_iterator n_iter, n_end;
    if (f == NULL)
        for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() << " " << p.y() << " " << p.z() << "\n";
        }
    else
    {
        Function & u = *const_cast<Function *>(f);
        for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() + u(n_iter.ptr(), 0)<< " " << p.y() + u(n_iter.ptr(), 1) << " " << p.z() + u(n_iter.ptr(), 2) << "\n";
        }
    }
    out << "</DataArray>\n";
    out << "</Points>\n";
    out << "<PointData Scalars=\"Id\" Normals=\"Normals\">\n";
    out << "<DataArray type=\"Int32\" Name=\"Id\" format=\"ascii\">\n";
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        out << int(n_iter->id()) << " ";
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray type=\"Float32\" Name=\"Normals\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    if (f == NULL)
        for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        {
            Vector const& v = n_iter->director();
            out << v.x() << " " << v.y() << " " << v.z() << "\n";
        }
    else
    {
        Function & u = *const_cast<Function *>(f);
        for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        {
            Vector const& v = n_iter->director();
            out << v.x() + u(n_iter.ptr(), 3) << " " << v.y() + u(n_iter.ptr(), 4) << " " << v.z() + u(n_iter.ptr(), 5) << "\n";
        }
    }
//      out << "\n";
    out << "</DataArray>\n";
    out << "</PointData>\n";
    out << "<Cells>  \n";
    out << "<DataArray  type=\"Int32\"  Name=\"connectivity\"  format=\"ascii\">\n";
    Mesh::Face_const_iterator f_iter, f_end;
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        Mesh_face const&f = *f_iter;
        Mesh_face::Edge_use_const_circulator fe_circ = f.edge_uses_begin(), fe_start = f.edge_uses_begin();
        do {
            out << (int)fe_circ->node()->id() << " ";
        } while (++fe_circ != fe_start);
    }
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"offsets\"  format=\"ascii\">\n";

    int offset = 0;
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        offset += int(f_iter->size_of_nodes());
        out << offset << " ";
    }
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"ascii\">\n";
    for ( int i = 0; i < n_elements; ++i )
        out << "7 ";
    out << "\n";
    out << "</DataArray>\n";

    out << "</Cells>\n";

    out << "<CellData>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"flag\"  format=\"ascii\">\n";
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        out << f_iter->flag() << " ";
    }
    out << "</DataArray>\n";
    out << "</CellData>\n";

    out << "</Piece>\n";
}
//-----------------------------------------------------------------------------
void print_function( VTK_ostream& out, Function const& f )
{
    Mesh &m = f.mesh();
    assert( m.type() == Mesh::SURFACE );
    int n_nodes = m.size_of_nodes();
    int n_elements = m.size_of_faces();

    out <<"<Piece NumberOfPoints=\"" << (int)n_nodes << "\" NumberOfCells=\"" << (int)n_elements << "\">\n";
    out << "<Points>\n";
    out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    Mesh::Node_const_iterator n_iter, n_end;

    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Point const& p = n_iter->point();
        out << p.x() << " " << p.y() << " " << p.z() << "\n";
    }
    out << "</DataArray>\n";
    out << "</Points>\n";
    out << "<PointData Scalars=\"Id\" Vectors=\"node_load\" Normals=\"Normals\">\n";
    out << "<DataArray type=\"Int32\" Name=\"Id\" format=\"ascii\">\n";
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        out << int(n_iter->id()) << " ";
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray type=\"Float32\" Name=\"Normals\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Vector const& v = n_iter->director();
        out << v.x() << " " << v.y() << " " << v.z() << "\n";
    }
//      out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray type=\"Float32\" Name=\"node_load\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        const Mesh_node *n = &*n_iter;
        out << f(n, 0) << " " << f(n, 1) << " " << f(n, 2) << "\n";
    }
    out << "</DataArray>\n";
    out << "<DataArray type=\"Float32\" Name=\"node_moments\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        const Mesh_node *n = &*n_iter;
        out << f(n, 3) << " " << f(n, 4) << " " << f(n, 5) << "\n";
    }
    out << "</DataArray>\n";
    out << "</PointData>\n";
    out << "<Cells>  \n";
    out << "<DataArray  type=\"Int32\"  Name=\"connectivity\"  format=\"ascii\">\n";
    Mesh::Face_const_iterator f_iter, f_end;
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        Mesh_face const&f = *f_iter;
        Mesh_face::Edge_use_const_circulator fe_circ = f.edge_uses_begin(), fe_start = f.edge_uses_begin();
        do {
            out << (int)fe_circ->node()->id() << " ";
        } while (++fe_circ != fe_start);
    }
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"offsets\"  format=\"ascii\">\n";

    int offset = 0;
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        offset += int(f_iter->size_of_nodes());
        out << offset << " ";
    }
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"ascii\">\n";
    for ( int i = 0; i < n_elements; ++i )
        out << "7 ";
    out << "\n";
    out << "</DataArray>\n";

    out << "</Cells>\n";
    out << "</Piece>\n";
}
//-----------------------------------------------------------------------------
VTK_ostream& operator<<( VTK_ostream& out, Mesh const& m )
{
    print_surface_mesh( out, m, NULL );
}
//-----------------------------------------------------------------------------
void print_displaced_mesh( VTK_ostream& out, Mesh const& m, Function const& u )
{
    print_surface_mesh( out, m, &u );
}
//-----------------------------------------------------------------------------
