// // Copyright (C) 2008 Pawel Krupinski.
// // Licensed under the GNU GPL Version 2.
//
// #include "cell/tissue.h"
// #include "cell/cell_utils.h"
// #include "organism/baseSolver.h"
// #include "driver/simulation_data.h"
// #include "driver/driver.h"
#include "mesh/mesh_handle.h"
#include "mesh/mesh.h"
#include "io/pvd_file.h"
#include "fem/Function.h"
#include <fstream>
// #include "utils/index_map_element.h"
#include "postprocessing/stress_interpolator.h"
#include "matrix/principal_directions.h"
#include "material/isotropic_material.h"
// #include "fem/Function.h"
// #include "utils/error_macros.h"
// #include "utils/string_utils.h"
// #include "postprocessing/temporal_data.h"
// #include "postprocessing/stress_evaluator.h"
// #include "fem/stress_recovery.h"
// #include <list>
// #include <map>
#include <set>
#include <boost/functional/hash.hpp>
#include <boost/unordered_map.hpp>
const double WALL_RELATIVE_THICKNESS = 0.1;
const double ANISOTROPYTRESHOLD = 1e-15;
// //----------------------------------------------------------------------------
PVD_file::PVD_file(const std::string filename):filename(filename), m_counter(0)
{
    add("output_mode", "2D"); //defines two or three dimmensional output mode "2D" or "3D"
    add("write_face_flags", false);
    add("write_point_flags", false);
    add("append_spring_tension", false); //add tension data to cell edges
    add("write_fiber_direction", false);
    add("write_compartment_id", false);
    add("write_cell_id", false);
    add("write_shell_thickness", false);
    
    add("write_stress", true);//write nodal stress
    add("write_stress_tensor", false);//write tensor form in addition to principal directions
    add("write_stress_anisotropy", true);
    add("write_strain", false);//write nodal strain
    add("write_strain_tensor", false);//write tensor form in addition to principal directions
    add("write_strain_anisotropy", true);
    
    add("write_strain_stress_cosine", false);//write nodal strain stress cosine
    add("write_elastic_mod", false);
//      add("use_final_mesh", true);
//      add("mark_elements", false);
//      add("use_signed_stress", true);
//      add("update_geometry", false);
}
// //----------------------------------------------------------------------------
PVD_file::~PVD_file() {}
// //----------------------------------------------------------------------------
void PVD_file::operator<<(Mesh_handle const& mesh)
{
    // Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    // Write pvd file
    pvdFileWrite(m_counter);
    // Write headers
    VTKHeaderOpen(mesh);
    // Write mesh
    write_mesh_2D(mesh);
    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
//      message(1, "Saved mesh %s (%s) to file %s in VTK format.",
//                      mesh.name().c_str(), mesh.label().c_str(), filename.c_str());
}
//----------------------------------------------------------------------------
void PVD_file::write(Mesh_handle const& mesh, double t, State_interpolator const* si)
{
    //Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    //Write pvd file
    pvdFileWrite(m_counter, t);
    //Write headers
    VTKHeaderOpen(mesh);
    //Write mesh
    write_mesh_2D(mesh, NULL, &si->map());
    write_point_data_header();
    append_point_data(mesh);
    std::vector<Vector_data>::iterator it, end;
    for (it = m_output_data.begin(), end = m_output_data.end(); it != end; ++it)
    {
        append_point_data(*(it->function), it->name, it->name+"_moment");
    }
    if (si)
    {
      if(get("write_stress"))
        append_nodal_stress(*si);
      if(get("write_strain"))
        append_nodal_strain(*si);
      if(get("write_strain_stress_cosine"))
        append_nodal_strain_stress_cosine(*si);
    }
    write_point_data_footer();
    //Close headers
    VTKHeaderClose();
    //Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write(Mesh_handle const& mesh, Function const& u, double t, State_interpolator const* si)
{
    // Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    // Write pvd file
    pvdFileWrite(m_counter, t);
    // Write headers
    VTKHeaderOpen(mesh);
    // Write mesh
    write_mesh_2D(mesh, &u, &si->map());
    write_point_data_header();
    append_point_data(mesh);
    std::vector<Vector_data>::iterator it, end;
    for (it = m_output_data.begin(), end = m_output_data.end(); it != end; ++it)
    {
        append_point_data(*(it->function), it->name, it->name+"_moment");
    }
    if (si)
    {
      if(get("write_stress"))
        append_nodal_stress(*si);
      if(get("write_strain"))
        append_nodal_strain(*si);
      if(get("write_strain_stress_cosine"))
        append_nodal_strain_stress_cosine(*si);
    }
    write_point_data_footer();
    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write(Mesh_handle const& mesh, cell::Tissue_handle const& ts, const Function& u, double t, State_interpolator const* si)
{
    std::string mode = get("output_mode");
    // Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    // Write pvd file
    pvdFileWrite(m_counter, t);

    if (!mode.compare("2D"))
    {
        // Write headers
        VTKHeaderOpen(mesh);
        // Write mesh
        write_mesh_2D(mesh, &u);
        write_point_data_header();
        append_point_data(mesh);
        std::vector<Vector_data>::iterator it, end;
        for (it = m_output_data.begin(), end = m_output_data.end(); it != end; ++it)
        {
            append_point_data(*(it->function), it->name, it->name+"_moment");
        }
        if (si)
            append_nodal_stress(*si);
        write_point_data_footer();
//         if ( has_cell_data() )
//         {
//             write_cell_data_header();
//             write_cell_data(mesh, ts);
//             write_cell_data_footer();
//         }
    }
//     else if (!mode.compare("3D"))
//     {
//         std::streampos pt_pos, cl_pos;
//         // Write headers
//         VTKHeaderOpen(pt_pos, cl_pos);
//         int n_pts, n_cl;
//         // Write mesh
//         write_mesh_3D(mesh, ts, n_pts, n_cl, &u);
//         VTKHeaderUpdate(n_pts, n_cl, pt_pos, cl_pos);
//     }
    else
        cell_error("unrecognized output mode. Allowed values are \'2D\' and \'3D\'.");

    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write(Mesh_handle const& mesh, cell::Tissue_handle const& ts, double t, State_interpolator const* si)
{
    std::string mode = get("output_mode");
    // Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    // Write pvd file
    pvdFileWrite(m_counter, t);

    if (!mode.compare("2D"))
    {
//       std::cout << "mode 2D\n";
        // Write headers
        VTKHeaderOpen(mesh);
        // Write mesh
//         std::cout << "write mesh\n";
        if (si)
            write_mesh_2D(mesh, NULL, &si->map() );
        else
            write_mesh_2D(mesh, NULL, NULL);
//         std::cout << "write point data\n";
        write_point_data_header();
        append_point_data(mesh);
        std::vector<Vector_data>::iterator it, end;
        for (it = m_output_data.begin(), end = m_output_data.end(); it != end; ++it)
        {
            append_point_data(*(it->function), it->name, it->name+"_moment");
        }
        if (si)
            append_nodal_stress(*si);
        write_point_data_footer();
//         if ( has_cell_data() )
//         {
// //           std::cout << "cell data\n";
//             write_cell_data_header();
//             if (si)
//               write_cell_data(mesh, ts, &si->map());
//             else
//               write_cell_data(mesh, ts);
//             write_cell_data_footer();
//         }
    }
//     else if (!mode.compare("3D"))
//     {
//         std::streampos pt_pos, cl_pos;
//         // Write headers
//         VTKHeaderOpen(pt_pos, cl_pos);
//         int n_pts, n_cl;
//         // Write mesh
//         write_mesh_3D(mesh, ts, n_pts, n_cl, NULL);
//         VTKHeaderUpdate(n_pts, n_cl, pt_pos, cl_pos);
//     }
    else
        cell_error("unrecognized output mode. Allowed values are \'2D\' and \'3D\'.");
    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write_tensors(State_interpolator const& si, double t)
{
    Mesh_handle const& mesh = si.mesh();
    // Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    // Write pvd file
    pvdFileWrite(m_counter, t);
    // Write headers
    VTKHeaderOpen(mesh);
    // Write mesh
    write_mesh_2D(mesh);
    write_point_data_header();
    append_point_data(mesh);
    if(get("write_stress"))
      append_nodal_stress(si);
    if(get("write_strain"))
      append_nodal_strain(si);
    write_point_data_footer();
    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write_loads(Function const& f, double t)
{
    Mesh_handle mesh(&f.mesh());
    //Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    //Write pvd file
    pvdFileWrite(m_counter, t);
    //Write headers
    VTKHeaderOpen(mesh);
    //Write mesh
    write_mesh_2D(mesh);
    write_point_data_header();
    append_point_data(mesh);
    append_point_data(f, "node_load", "node_moment");
    write_point_data_footer();
    //Close headers
    VTKHeaderClose();
    //Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write_loads(Function const& f, Function const& u, double t)
{
    Mesh_handle mesh(&f.mesh());
    //Update vtu file name and clear file
    vtuNameUpdate(m_counter);
    //Write pvd file
    pvdFileWrite(m_counter, t);
    //Write headers
    VTKHeaderOpen(mesh);
    //Write mesh
    write_mesh_2D(mesh, &u);
    write_point_data_header();
    append_point_data(mesh);
    append_point_data(f, "node_load", "node_moment");
    write_point_data_footer();
    //Close headers
    VTKHeaderClose();
    //Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write_loads(Face_handle fh, double* loads, double t)
{
    size_t ind = fh->id();
    //Update vtu file name and clear file
    vtuNameUpdate(ind, m_counter);
    //Write pvd file
    pvdFileWrite(ind, m_counter, t);
    //Write headers
    VTKHeaderOpen(fh->size_of_nodes(), 1);
    //Write mesh
//   write_face_2D(fh);
    write_point_data_header();
    append_point_data(fh, loads, "node_load", "node_moment");
    write_point_data_footer();
    //Close headers
    VTKHeaderClose();
    //Increase the number of time updates
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::write_mesh_2D(Mesh_handle const& mesh, Function const* f, PVD_file::Affine_map* map) const
{
//     assert( mesh->type() == Mesh::SURFACE );
    int n_nodes = mesh->size_of_nodes();
    int n_elements = mesh->size_of_faces();

    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    out << "<Points>\n";
    out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    Mesh::Node_const_iterator n_iter, n_end;

    if (f == NULL)
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() << " " << p.y() << " " << p.z() << "\n";
        }
    else
    {
        Function &u = *const_cast<Function *>(f);
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() + u(&*n_iter, 0) << " "
            << p.y() + u(&*n_iter, 1) << " "
            << p.z() + u(&*n_iter, 2) << "\n";
        }
    }
    out << "</DataArray>\n";
    out << "</Points>\n";
    out << "<Cells>  \n";
    out << "<DataArray  type=\"Int32\"  Name=\"connectivity\"  format=\"ascii\">\n";
    Mesh::Face_const_iterator f_iter, f_end;
    for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
    {
        Mesh_face const&f = *f_iter;
        Mesh_face::Edge_use_const_circulator fe_circ = f.edge_uses_begin(), fe_start = f.edge_uses_begin();
        do {
            out << (int)fe_circ->node()->id() << " ";
        } while (++fe_circ != fe_start);
    }
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"offsets\"  format=\"ascii\">\n";

    int offset = 0;
    for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
    {
        offset += int(f_iter->size_of_nodes());
        out << offset << " ";
    }
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"ascii\">\n";
    for ( int i = 0; i < n_elements; ++i )
        out << "7 ";
    out << "\n";
    out << "</DataArray>\n";

    out << "</Cells>\n";

    bool write_flags = get("write_face_flags");
    bool write_fiber = get("write_fiber_direction");
    bool write_elast = get("write_elastic_mod");
    bool write_thick = get("write_shell_thickness");
    bool write_cell_dat = write_flags || write_fiber || write_elast;
    if (write_cell_dat)
        out << "<CellData>\n";
    if (write_flags)
    {
        out << "<DataArray  type=\"Int32\"  Name=\"face_flag\"  format=\"ascii\">\n";
        for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
        {
            out << f_iter->flag() << " ";
        }
        out << "</DataArray>\n";
    }
    if (write_fiber)
    {

        out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" Name=\"fiber_direction\" format=\"ascii\">\n";
        if (map != NULL)
            for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
            {

                Vector3 fiber = f_iter->fem_data().fiber;
                map->update(*(f_iter.ptr()));
                // deformation gradient
                Matrix3 const& F = map->def_grad(0);
                // calculate the current material axis lam*a = F*a0;
                Vector3 a = F*fiber;
                a.unit();
                out << a.x << " " << a.y << " " << a.z << "\n";
            }
        else
        {
            for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
            {
                Vector3 fiber = f_iter->fem_data().fiber;
                out << fiber.x << " " << fiber.y << " " << fiber.z << "\n";
            }
        }
        out << "</DataArray>\n";
    }
    if (write_elast)
    {
        out << "<DataArray type=\"Float64\" Name=\"elastic modulus\" format=\"ascii\">\n";
        for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
        {
            double E = 0.0;
            Isotropic_material* mater = dynamic_cast<Isotropic_material*>(f_iter->material());
            if (mater)
                E = mater->E();
            out << E << "\n";
        }
        out << "</DataArray>\n";
    }
    if (write_thick)
    {
        out << "<DataArray type=\"Float64\" Name=\"shell thickness\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Vector const& v = n_iter->director();
            out << sqrt(v.squared_length()) << " ";
        }
        out << "\n";
        out << "</DataArray>\n";
    }
    if (write_cell_dat)
        out << "</CellData>\n";
    // Close file
    out.flush();
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::write_mesh_2D(Mesh_handle const& mesh, cell::Tissue_handle const& t, Function const* f, PVD_file::Affine_map* map) const
{
    int n_nodes = mesh->size_of_nodes();
    int n_elements = mesh->size_of_faces();

    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    out << "<Points>\n";
    out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    Mesh::Node_const_iterator n_iter, n_end;

    if (f == NULL)
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() << " " << p.y() << " " << p.z() << "\n";
        }
    else
    {
        Function &u = *const_cast<Function *>(f);
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() + u(&*n_iter, 0) << " "
            << p.y() + u(&*n_iter, 1) << " "
            << p.z() + u(&*n_iter, 2) << "\n";
        }
    }
    out << "</DataArray>\n";
    out << "</Points>\n";
    out << "<Cells>  \n";
    out << "<DataArray  type=\"Int32\"  Name=\"connectivity\"  format=\"ascii\">\n";
    Mesh::Face_const_iterator f_iter, f_end;
    for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
    {
        Mesh_face const&f = *f_iter;
        Mesh_face::Edge_use_const_circulator fe_circ = f.edge_uses_begin(), fe_start = f.edge_uses_begin();
        do {
            out << (int)fe_circ->node()->id() << " ";
        } while (++fe_circ != fe_start);
    }
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"offsets\"  format=\"ascii\">\n";

    int offset = 0;
    for ( f_iter = mesh->faces_begin(), f_end = mesh->faces_end(); f_iter != f_end; ++f_iter )
    {
        offset += int(f_iter->size_of_nodes());
        out << offset << " ";
    }
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"ascii\">\n";
    for ( int i = 0; i < n_elements; ++i )
        out << "7 ";
    out << "\n";
    out << "</DataArray>\n";

    out << "</Cells>\n";

    // Close file
    out.flush();
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::write_edges(Mesh_handle const& mesh, double t, bool marked, int flag)
{
    typedef Mesh::Edge_const_iterator Iter;
    typedef std::set< Node_handle >::const_iterator     NPI;
    typedef std::map<size_t,size_t> Node_index_map;
    typedef Node_index_map::const_iterator NIIter;
    //prepare data structures for output
    Mesh::Edge_container edges;
    if (marked)
        edges.reserve(mesh->size_of_edges());

    std::set< Node_handle > nodes;
    Node_index_map map;

    Iter e_it, e_end;
    for ( e_it = mesh->edges_begin(), e_end = mesh->edges_end(); e_it != e_end; ++ e_it )
    {
        if ( marked )
        {
            if (e_it->flag() == flag )
                edges.push_back(e_it.handle());
            else
                continue;
        }
        Node_handle const& n0 = e_it->node0();
        nodes.insert ( n0 );
        Node_handle const& n1 = e_it->node1();
        nodes.insert ( n1 );
    }
    size_t count = 0;
    for (NPI it = nodes.begin(); it != nodes.end(); ++it, ++count)
    {
        map.insert ( std::make_pair( (*it )->id(), count ) );
    }
    size_t n_edges = 0;
    if ( marked )
        n_edges = edges.size();
    else
        n_edges = mesh->size_of_edges();
    size_t npts = nodes.size();

    vtuNameUpdate ( m_counter );
    // Write pvd file
    pvdFileWrite ( m_counter, t );
    // Write headers
    VTKHeaderOpen ( npts,  n_edges );

    //write the output
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    out << "<Points>\n";
    out << "<DataArray type=\"Float64\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for (NPI it = nodes.begin(); it != nodes.end(); ++it)
    {
        Point const& p = (*it)->point();
        out << p.x() << " "
        << p.y() << " "
        << p.z() << "\n";

    }
    out << "</DataArray>\n";
    out << "</Points>\n";

    out << "<Cells>  \n";
    out << "<DataArray  type=\"Int32\"  Name=\"connectivity\"  format=\"ascii\">\n";

    if ( marked )
    {
        e_it = Iter(edges.begin());
        e_end = Iter(edges.end());
    }
    else
    {
        e_it = mesh->edges_begin();
        e_end = mesh->edges_end();
    }

    for (;e_it != e_end; ++e_it)
    {
        size_t i1 = map.find(e_it->node0()->id())->second;
        size_t i2 = map.find(e_it->node1()->id())->second;
        out << i1 << " " << i2 << " ";
    }
    out << "\n";
    out << "</DataArray>\n";
    out << "<DataArray  type=\"Int32\"  Name=\"offsets\"  format=\"ascii\">\n";

    int offset = 0;

    for (int i = 0; i < n_edges; ++i)
    {
        offset += 2;
        out << offset << " ";
    }
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray  type=\"UInt8\"  Name=\"types\"  format=\"ascii\">\n";
    for ( int i = 0; i < n_edges; ++i )
        out << "3 ";
    out << "\n";
    out << "</DataArray>\n";

    out << "</Cells>\n";

    // Close file
    out.flush();
    out.close();

    // Close headers
    VTKHeaderClose();
    // Increase the number of times we have saved the mesh
    m_counter++;
}
//----------------------------------------------------------------------------
void PVD_file::append_nodal_stress(State_interpolator const& si) const
{
    bool w_stress_tens = get("write_stress_tensor");
    bool w_aniso = get("write_stress_anisotropy");
    // Open file
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    Mesh_handle const& mesh = si.mesh();

    Mesh::Node_const_iterator n_iter, n_end;
    int count;

    std::string name = "stress_";
    std::vector<principal_directions> princ(mesh->size_of_nodes());
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        princ[n_iter->id()].calculate_principal_directions( si.stress(n_iter), true );
    if (w_stress_tens)
    {
        //write stress tensor collect stress principal directions for all nodes
        out << "<DataArray type=\"Float64\" Name=\"stress tensor\" NumberOfComponents=\"9\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            Matrix3s const& s = si.stress(n_iter);
            out << s.xx() << " " << s.xy() << " " << s.xz()<< " ";
            out << s.xy() << " " << s.yy() << " " << s.yz()<< " ";
            out << s.xz() << " " << s.yz() << " " << s.zz() << "\n";
        }
        out << "</DataArray>\n";
    }
    //write each principal direction in turn
    for ( int i = 0 ; i < 3; ++i )
    {
        std::string name_n = name + to_string(i);
        out << "<DataArray type=\"Float64\" Name=\"" << name_n << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            principal_directions const& pi = princ[n_iter->id()];
            double eig_val = pi.val(i);
            Matrix3 const& eig_vec = pi.vec();
            out << eig_vec(0, i)*eig_val << " " <<  eig_vec(1, i)*eig_val << " " <<  eig_vec(2, i)*eig_val << "\n";
        }
        out << "</DataArray>\n";
    }
    //anisotropy
    if (w_aniso)
    {
        out << "<DataArray type=\"Float64\" Name=\"stress anisotropy\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            principal_directions const& pi = princ[n_iter->id()];
            double eig_val0 = pi.val(0);
            double eig_val1 = pi.val(1);
            assert(eig_val0>=eig_val1);
            double aniso = 0.0;
            if (eig_val0 > ANISOTROPYTRESHOLD)
                aniso = (eig_val0-eig_val1)/eig_val0;
            out << aniso << "\n";
        }
        out << "</DataArray>\n";
    }
// Close file
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::append_nodal_strain(State_interpolator const& si) const
{
    bool w_tens = get("write_strain_tensor");
    bool w_aniso = get("write_strain_anisotropy");

    // Open file
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    Mesh_handle const& mesh = si.mesh();

    Mesh::Node_const_iterator n_iter, n_end;
    int count;

    std::string name = "strain_";
    std::vector<principal_directions> princ(mesh->size_of_nodes());
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        princ[n_iter->id()].calculate_principal_directions( si.strain(n_iter), true );
    if (w_tens)
    {
        //write stress tensor collect stress principal directions for all nodes
        out << "<DataArray type=\"Float64\" Name=\"strain tensor\" NumberOfComponents=\"9\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            Matrix3s const& s = si.strain(n_iter);
            out << s.xx() << " " << s.xy() << " " << s.xz()<< " ";
            out << s.xy() << " " << s.yy() << " " << s.yz()<< " ";
            out << s.xz() << " " << s.yz() << " " << s.zz() << "\n";
        }
        out << "</DataArray>\n";
    }
    //write each principal direction in turn
    for ( int i = 0 ; i < 3; ++i )
    {
        std::string name_n = name + to_string(i);
        out << "<DataArray type=\"Float64\" Name=\"" << name_n << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            principal_directions const& pi = princ[n_iter->id()];
            double eig_val = pi.val(i);
            Matrix3 const& eig_vec = pi.vec();
            out << eig_vec(0, i)*eig_val << " " <<  eig_vec(1, i)*eig_val << " " <<  eig_vec(2, i)*eig_val << "\n";
        }
        out << "</DataArray>\n";
    }
    //anisotropy
    if (w_aniso)
    {
        out << "<DataArray type=\"Float64\" Name=\"strain anisotropy\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
        {
            principal_directions const& pi = princ[n_iter->id()];
            double eig_val0 = pi.val(0);
            double eig_val1 = pi.val(1);
            assert(eig_val0>=eig_val1);
            double aniso = 0.0;
            if (eig_val0 > ANISOTROPYTRESHOLD)
                aniso = (eig_val0-eig_val1)/eig_val0;
            out << aniso << "\n";
        }
        out << "</DataArray>\n";
    }
// Close file
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::append_nodal_strain_stress_cosine(State_interpolator const& si) const
{
    // Open file
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);
    Mesh_handle const& mesh = si.mesh();
    Mesh::Node_const_iterator n_iter, n_end;
    int count;

    std::string name = "strain_stress_cosine";
    std::vector<principal_directions> strain_princ(mesh->size_of_nodes()), stress_princ(mesh->size_of_nodes());
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
    {
        strain_princ[n_iter->id()].calculate_principal_directions( si.strain(n_iter), true );
        stress_princ[n_iter->id()].calculate_principal_directions( si.stress(n_iter), true );
    }
    //write cosine between max principal strain and stress directions (alpha=180-alpha)
    out << "<DataArray type=\"Float64\" Name=\"" << name << "\" format=\"ascii\">\n";
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(), count = 0; n_iter != n_end; ++n_iter, ++count )
    {
        principal_directions const& strain_pi = strain_princ[n_iter->id()];
        principal_directions const& stress_pi = stress_princ[n_iter->id()];
        double c = strain_pi.vec(0) * stress_pi.vec(0);
        out << fabs(c) << " ";
    }
    out << "\n";
    out << "</DataArray>\n";
// Close file
    out.close();
}
//END comment
//----------------------------------------------------------------------------
void PVD_file::pvdFileWrite(size_t num, double time)
{
    std::fstream pvdFile;

    if ( num == 0)
    {
        // Open pvd file
        pvdFile.open(filename.c_str(), std::ios::out|std::ios::trunc);
        // Write header
        pvdFile << "<?xml version=\"1.0\"?> " << std::endl;
        pvdFile << "<VTKFile type=\"Collection\" version=\"0.1\" > " << std::endl;
        pvdFile << "<Collection> " << std::endl;
    }
    else
    {
        // Open pvd file
        pvdFile.open(filename.c_str(),  std::ios::out|std::ios::in);
        pvdFile.seekp(mark);

    }
    // Remove directory path from name for pvd file
    std::string fname;
    fname.assign(vtu_filename, filename.find_last_of("/") + 1, vtu_filename.size());

    if (time < 0.0)
        time = num;
    // Data file name
    pvdFile << std::setiosflags(std::ios::scientific);
    pvdFile << std::setprecision(10);
    pvdFile << "<DataSet timestep=\"" << time << "\" part=\"0\"" << " file=\"" <<  fname <<  "\"/>" << std::endl;
    mark = pvdFile.tellp();

    // Close headers
    pvdFile << "</Collection> " << std::endl;
    pvdFile << "</VTKFile> " << std::endl;

    // Close file
    pvdFile.close();
}
//----------------------------------------------------------------------------
void PVD_file::pvdFileWrite(size_t part_num, size_t num,double time)
{
    std::fstream pvdFile;

    if ( num == 0 && part_num == 0)
    {
        // Open pvd file
        pvdFile.open(filename.c_str(), std::ios::out|std::ios::trunc);
        // Write header
        pvdFile << "<?xml version=\"1.0\"?> " << std::endl;
        pvdFile << "<VTKFile type=\"Collection\" version=\"0.1\" > " << std::endl;
        pvdFile << "<Collection> " << std::endl;
    }
    else
    {
        // Open pvd file
        pvdFile.open(filename.c_str(),  std::ios::out|std::ios::in);
        pvdFile.seekp(mark);

    }
    // Remove directory path from name for pvd file
    std::string fname;
    fname.assign(vtu_filename, filename.find_last_of("/") + 1, vtu_filename.size());

    if (time < 0.0)
        time = num;
    // Data file name
    pvdFile << "<DataSet timestep=\"" << time << "\" part=\"" << part_num << "\"" << " file=\"" <<  fname <<  "\"/>" << std::endl;
    mark = pvdFile.tellp();

    // Close headers
    pvdFile << "</Collection> " << std::endl;
    pvdFile << "</VTKFile> " << std::endl;

    // Close file
    pvdFile.close();
}
// //----------------------------------------------------------------------------
void PVD_file::VTKHeaderOpen(Mesh_handle const& mesh) const
{
    // Open file
    FILE *fp = fopen(vtu_filename.c_str(), "a");

    // Write headers
    fprintf(fp, "<VTKFile type=\"UnstructuredGrid\"  version=\"0.1\"   >\n");
    fprintf(fp, "<UnstructuredGrid>  \n");
    fprintf(fp, "<Piece  NumberOfPoints=\" %8u\"  NumberOfCells=\" %8u\">  \n",
            mesh->size_of_nodes(), mesh->size_of_faces());

    // Close file
    fclose(fp);
    fflush(fp);
}
//----------------------------------------------------------------------------
void PVD_file::VTKHeaderOpen(int npts, int ncells) const
{
    // Open file
    FILE *fp = fopen(vtu_filename.c_str(), "a");
    // Write headers
    fprintf(fp, "<VTKFile type=\"UnstructuredGrid\"  version=\"0.1\"   >\n");
    fprintf(fp, "<UnstructuredGrid>  \n");
    fprintf(fp, "<Piece  NumberOfPoints=\" %8u\"  NumberOfCells=\" %8u\">  \n",
            npts,ncells);
    // Close file
    fclose(fp);
    fflush(fp);
}

//----------------------------------------------------------------------------
void PVD_file::VTKHeaderOpen(std::streampos &pt_pos , std::streampos &cl_pos) const
{
    std::ofstream os(vtu_filename.c_str()/*, std::ios_base::out*/);

    os << "<VTKFile type=\"UnstructuredGrid\" version=\"0.1\">\n<UnstructuredGrid>\n";
    os << "<Piece  NumberOfPoints=\"";
    os.width(12);
    pt_pos = os.tellp();
    os << " ";
    os << "\" NumberOfCells=\"";
    os.width(12);
    cl_pos = os.tellp();
    os << " ";
    os << "\">\n";
    os.close();
}
//----------------------------------------------------------------------------
void PVD_file::VTKHeaderUpdate(int npts, int ncells, std::streampos pt_pos , std::streampos cl_pos)
{
    std::ofstream os(vtu_filename.c_str(), std::ios::ate | std::ios::out | std::ios::in);
    os.seekp(pt_pos);
    os << npts;
    os.seekp(cl_pos);
    os << ncells;
    os.close();
}
//----------------------------------------------------------------------------
void PVD_file::VTKHeaderClose() const
{
    // Open file
    FILE *fp = fopen(vtu_filename.c_str(), "a");

    // Close headers
    fprintf(fp, "</Piece>\n</UnstructuredGrid>\n</VTKFile>");

    // Close file
    fclose(fp);
    fflush(fp);
}
//----------------------------------------------------------------------------
void PVD_file::vtuNameUpdate(size_t counter)
{
    std::string filestart, extension;
    std::ostringstream fileid, newfilename;

    fileid.fill('0');
    fileid.width(6);

    filestart.assign(filename, 0, filename.find("."));
    extension.assign(filename, filename.find("."), filename.size());

    fileid << counter;
    newfilename << filestart << fileid.str() << ".vtu";

    vtu_filename = newfilename.str();

    // Make sure file is empty
    FILE* fp = fopen(vtu_filename.c_str(), "w");
    fclose(fp);
}

//----------------------------------------------------------------------------
void PVD_file::vtuNameUpdate(size_t num_part, size_t num)
{
    std::string filestart, extension;
    std::ostringstream fileid, newfilename;

    fileid.fill('0');
    fileid.width(13);

    filestart.assign(filename, 0, filename.find("."));
    extension.assign(filename, filename.find("."), filename.size());

    fileid << num << "_" << num_part;
    newfilename << filestart << fileid.str() << ".vtu";

    vtu_filename = newfilename.str();

    // Make sure file is empty
    FILE* fp = fopen(vtu_filename.c_str(), "w");
    fclose(fp);
}
//-----------------------------------------------------------------------------
void PVD_file::append_point_data(Function const& f, std::string d, std::string r) const
{
    Mesh_handle mesh(&f.mesh());
    Mesh::Node_const_iterator n_iter, n_end;
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    out << "<DataArray type=\"Float64\" Name=\"" << d << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";

    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
    {
        const Mesh_node *n = &*n_iter;
        out << f(n, 0) << " " << f(n, 1) << " " << f(n, 2) << "\n";
    }
    out << "</DataArray>\n";

    out << "<DataArray type=\"Float64\" Name=\"" << r << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
    {
        const Mesh_node *n = &*n_iter;
        out << f(n, 3) << " " << f(n, 4) << " " << f(n, 5) << "\n";
    }
    out << "</DataArray>\n";

    out.flush();
    out.close();
}
//-----------------------------------------------------------------------------
//for quad shell block is 6*4=24 doubles. Maping is  idof*nnodes+inode (dnode ids change first then degrees of freedon)
void PVD_file::append_point_data(Face_handle fh, double * block, std::string d, std::string r) const
{
//     Mesh_face::Node_circulator n_start = fh->nodes_begin(), n_circ = n_start;
    Mesh_face::Edge_use_circulator eu_start = fh->edge_uses_begin(), eu_circ = eu_start;
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);

    out << "<DataArray type=\"Int32\" Name=\"Id\" format=\"ascii\">\n";
    do {
        out << eu_circ->id() << " ";
    } while (++eu_circ != eu_start);
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray type=\"Float64\" Name=\"Normals\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    eu_start = fh->edge_uses_begin(), eu_circ = eu_start;
    do
    {
        Vector const& v = eu_circ->node()->director();
        out << v.x() << " " << v.y() << " " << v.z() << "\n";
    } while (++eu_circ != eu_start);
    //      out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray type=\"Float64\" Name=\"" << d << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";

    eu_start = fh->edge_uses_begin(), eu_circ = eu_start;
    do
    {
        const Node_handle n = eu_circ->node();
    } while (++eu_circ != eu_start);
    out << "</DataArray>\n";

    out << "<DataArray type=\"Float64\" Name=\"" << r << "\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    eu_start = fh->edge_uses_begin(), eu_circ = eu_start;
    do
    {
        const Node_handle n = eu_circ->node();
    } while (++eu_circ != eu_start);
    out << "</DataArray>\n";

    out.flush();
    out.close();
}
//-----------------------------------------------------------------------------
void PVD_file::append_point_data(Mesh_handle const& mesh) const
{
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);
    Mesh::Node_const_iterator n_iter, n_end;

    out << "<DataArray type=\"Int32\" Name=\"Id\" format=\"ascii\">\n";
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
        out << int(n_iter->id()) << " ";
    out << "\n";
    out << "</DataArray>\n";

    out << "<DataArray type=\"Float64\" Name=\"Normals\" NumberOfComponents=\"3\" format=\"ascii\">\n";
    for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
    {
        Vector const& v = n_iter->director();
        out << v.x() << " " << v.y() << " " << v.z() << "\n";
    }
//      out << "\n";
    out << "</DataArray>\n";
    if (get("write_point_flags"))
    {
        out << "<DataArray type=\"Int32\" Name=\"point_flags\" format=\"ascii\">\n";
        for ( n_iter = mesh->nodes_begin(), n_end = mesh->nodes_end(); n_iter != n_end; ++n_iter )
            out << n_iter->flag() << " ";
        out << "\n";
        out << "</DataArray>\n";
    }
    out.flush();
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::write_point_data_header() const
{
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);
    out << "<PointData Scalars=\"Id\" Normals=\"Normals\">\n";
    out.close();
}
//----------------------------------------------------------------------------
void PVD_file::write_point_data_footer() const
{
    std::ofstream out(vtu_filename.c_str(), std::ios_base::app);
    out << "</PointData>\n";
    out.close();
}
//-----------------------------------------------------------------------------
void PVD_file::attach_vector_function(Function const& u, std::string n, bool unique_name)
{
    if (unique_name)
    {
        std::vector<Vector_data>::iterator it, end = m_output_data.end();
        for (it = m_output_data.begin(); it != end; ++it)
        {
            if (!(it->name.compare(n)))
                break;
        }
        if (it == end)
            m_output_data.push_back(Vector_data(u,n));
    }
    else
        m_output_data.push_back(Vector_data(u,n));
}
//-----------------------------------------------------------------------------
bool PVD_file::has_cell_data() const
{
    return ( get("write_fiber_direction") || get("write_compartment_id") || get("write_cell_id") );
}
//-----------------------------------------------------------------------------
