// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _PVD_FILE_H_
#define _PVD_FILE_H_
#include <fstream>
#include "cell/cell_handle.h"
#include "mesh/mesh_handle.h"
#include "parameter/Parametrized.h"
class Function;
template<class ME> class Affine_map_shell;
class BaseSolver;
class Driver;
class State_interpolator;
//-----------------------------------------------------------------------------
struct Vector_data
{ 
  Vector_data(Function const& u, std::string n) :function(&u), name(n) {}
  Function const *function; 
  std::string name; 
};
//-----------------------------------------------------------------------------
class PVD_file: public Parametrized
{
public:
    typedef std::list<Point> PointList;
    typedef PointList::iterator Point_iterator;
    typedef std::list< std::vector<size_t> > CellList;
    typedef CellList::iterator Cell_iterator;
    typedef Affine_map_shell<Mesh_face> Affine_map ;

    PVD_file();

    PVD_file(const std::string filename);
    ~PVD_file();

    void operator<< (Mesh_handle const& m);

    void write(Mesh_handle const& mesh, double t = -1.0, State_interpolator const* si=NULL);
    void write(Mesh_handle const& mesh, const Function& u, double t = -1.0, State_interpolator const* si=NULL);
    void write(const Mesh_handle& mesh, const cell::Tissue_handle& ts, const Function& u, double t = -1.0, State_interpolator const* si=NULL);
    void write(const Mesh_handle& mesh, const cell::Tissue_handle& ts, double t = -1.0, State_interpolator const* si=NULL);
    ///Write stress output
    void write_tensors(State_interpolator const& si, double t = -1.0);
    ///Write stress output where displacement is given by the function
    void write_tensors(State_interpolator const& si, const Function& u, double t = -1.0);
    ///Write tension in springs if present
    void write_string_tension(Mesh_handle const& mesh, cell::Tissue_handle const& th, double t = -1.0);
    ///Write loads for the whole mesh from the function
    void write_loads(Function const& f, double t = -1.0);
    ///Write loads for the mesh displaced by the function
    void write_loads(Function const& f, Function const& u, double t = -1.0);
    ///Write loads for a single face from the block of values as it comes from 'eval' function
    void write_loads(Face_handle fh, double* block, double t = -1.0);
    void write_molecular_step_2D(Driver const& d);
    /// Write edges of the mesh. All of them or only marked with specified flag
    void write_edges(Mesh_handle const& mesh, double t = -1.0, bool marked = false, int flag  = 0);
    
    void write_point_data_header() const;
    void write_point_data_footer() const;
    //write mesh with single layer for each shared plant wall
    void write_mesh_2D(Mesh_handle const& mesh, Function const* f = NULL, PVD_file::Affine_map* map = NULL) const;
    //write mesh with separate layers for each shared plant wall
    void write_mesh_2D(const Mesh_handle& mesh, const cell::Tissue_handle& t, const Function* f = NULL, PVD_file::Affine_map* map = NULL) const;
    void pvdFileWrite(size_t num, double time = -1.0);
    void pvdFileWrite(size_t part_num, size_t num, double time = -1.0);
    std::string get_vtu_filename() const 
    {
        return vtu_filename;
    }
    size_t counter() const 
    {
        return m_counter;
    }
    void vtuNameUpdate(size_t counter);
    void vtuNameUpdate(size_t part_num, size_t num);
    //attaches vector function to output with thhe mesh (force, load itd)
    void attach_vector_function(Function const& u, std::string n, bool unique_nqame = true);
    void reset_vector_functions() {m_output_data.clear();}
private:
    void VTKHeaderOpen(Mesh_handle const& mesh) const;
    void VTKHeaderOpen(int npts, int ncells) const;
    void VTKHeaderOpen(std::streampos &pt_pos , std::streampos &cl_pos) const;
    void VTKHeaderUpdate(int npts, int ncells, std::streampos pt_pos , std::streampos cl_pos);
    void VTKHeaderClose() const;
    
    /// Write two vectors for the whole mesh from a six dimensional function
    void append_point_data(Function const& f, std::string d, std::string r) const;
    /// Write two vectors for the face from a block data
    void append_point_data(Face_handle fh, double * block, std::string d, std::string r) const;
    /// Write just node ids and normals for the nesh
    void append_point_data(Mesh_handle const& mesh) const;
    void append_nodal_stress(State_interpolator const& si) const;
    void append_nodal_strain(State_interpolator const& si) const;
    void append_nodal_strain_stress_cosine(State_interpolator const& si) const;
    void append_string_tension(Mesh_handle const& mesh, cell::Tissue_handle const& th);
    ///check if cell data output is requested
    bool has_cell_data() const;
    void write_cell_outline_2D(Mesh_handle const& mesh, cell::Tissue_handle const& t) const;
    void data_write( const BaseSolver& solver, cell::Tissue_handle const& t, const Mesh_handle& mesh, PVD_file::PointList& vtu_points, PVD_file::CellList& vtu_cells );

    //struct collecting vector data for output
private:
    std::ios::pos_type mark;
    std::string filename;
    std::string vtu_filename;
    size_t m_counter;
    size_t m_part_counter;
    std::vector<Vector_data> m_output_data;
};

//-----------------------------------------------------------------------------
#endif
