// Copyright (C) 2012 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "material/fiber_manager.h"
#include "fem/principial_stress.h"
#include "postprocessing/stress_interpolator.h"

#include "mesh/mesh_utils.h"
#include "matrix/principal_directions.h"
//-----------------------------------------------------------------------------
void Align_with_stress::align_material_fibers(Mesh_handle &m)
{
    Mesh::Face_iterator it, it_end;
    principal_directions princ;
    double eig_val[3];
    std::vector<Vector3> eig_vecs(3);

    for (it = m->faces_begin(), it_end = m->faces_end(); it != it_end; ++it)
    {
        Matrix3s const& estress = m_state_int.element_stress(it.handle());
        princ.calculate_principal_directions(estress, true);
        Matrix3 const& eig_vec = princ.vec();

        for (size_t i=0; i<3; ++i)
        {
            eig_val[i] = princ.val(i);
            eig_vecs[i] = eig_vec.col(i);
        }

        //find a director vector at the center by averaging node directors
        Vector3 dir = average_director(it.handle());
        //find eigenvector which is most parallel to dirctor;
        double max_proj = 0;
        int index  = 0;
        for (size_t i=0; i<3; ++i)
        {
            double proj = fabs(eig_vecs[i] * dir);
            if (proj > max_proj)
            {
                max_proj = proj;
                index = i;
            }
        }
        //choose next largest eigenvalue exept of index one
        if (index == 0)
            index = 1;
        else
            index = 0;
        eig_vecs[index].unit();
        it->fem_data().fiber = eig_vecs[index];
    }
}
//-----------------------------------------------------------------------------
void Align_with_stress_projection_treshold::align_material_fibers(Mesh_handle &m)
{
    Mesh::Face_iterator it, it_end;
    principal_directions princ;
    double eig_val[3];
    std::vector<Vector3> eig_vecs(3);

    for (it = m->faces_begin(), it_end = m->faces_end(); it != it_end; ++it)
    {
        Vector3 fib = it->fem_data().fiber;
        Matrix3s const& estress = m_state_int.element_stress(it.handle());
        princ.calculate_principal_directions(estress, true);
        Matrix3 const& eig_vec = princ.vec();

        for (size_t i=0; i<3; ++i)
        {
            eig_val[i] = princ.val(i);
            eig_vecs[i] = eig_vec.col(i);
        }

        //find a director vector at the center by averaging node directors
        Vector3 dir = average_director(it.handle());
        //find eigenvector which is most parallel to director;
        double max_proj = 0;
        int index  = 0;
        for (size_t i=0; i<3; ++i)
        {
            double proj = fabs(eig_vecs[i] * dir);
            if (proj > max_proj)
            {
                max_proj = proj;
                index = i;
            }
        }
        //choose next largest eigenvalue exept of index one
        int index1 = 0;
        if (index == 0)
        {
            index = 1;
            index1 = 2;
        }
        else
        {
            if (index == 1)
                index1 = 2;
            else if (index == 2)
                index1 = 1;
            index = 0;
        }
        eig_vecs[index].unit();

        //calculate normal stress in the direction of fibers
        double stress_proj = fib * (estress * fib);
        double t = (eig_val[index]-stress_proj)/stress_proj;
        if (t>m_treshold)
            it->fem_data().fiber = eig_vecs[index];
    }
}
//-----------------------------------------------------------------------------
void Align_with_strain::align_material_fibers(Mesh_handle &m)
{
    Mesh::Face_iterator it, it_end;
    principal_directions princ;
    double eig_val[3];
    std::vector<Vector3> eig_vecs(3);

    for (it = m->faces_begin(), it_end = m->faces_end(); it != it_end; ++it)
    {
        Matrix3s const& etens = m_state_int.element_strain(it.handle());
        princ.calculate_principal_directions(etens, true);
        Matrix3 const& eig_vec = princ.vec();
        for (size_t i=0; i<3; ++i)
        {
            eig_val[i] = princ.val(i);
            eig_vecs[i] = eig_vec.col(i);
        }

        //find a director vector at the center by averaging node directors
        Vector3 dir = average_director(it.handle());
        //find eigenvector which is most parallel to dirctor;
        double max_proj = 0;
        int index  = 0;
        for (size_t i=0; i<3; ++i)
        {
            double proj = fabs(eig_vecs[i] * dir);
            if (proj > max_proj)
            {
                max_proj = proj;
                index = i;
            }
        }
        //choose next largest eigenvalue exept of index one
        if (index == 0)
            index = 1;
        else
            index = 0;
        //get vector perpendicular to max eigen vector and dir simultaneously
        Vector3 rvec = eig_vecs[index]^ dir;
        rvec.unit();
        it->fem_data().fiber = rvec;
    }
}
//-----------------------------------------------------------------------------
