#ifndef _MAT3D_H_
#define _MAT3D_H_

#include "matrix/Vector3.h"
#include <ostream>
#include <cassert>
//-----------------------------------------------------------------------------
class Matrix3; 
class Matrix3s; 
class Matrix3d; 
//-----------------------------------------------------------------------------
class Matrix3d
{
public:
  Matrix3d(){}

  Matrix3d(double a);
  Matrix3d(double a0, double a1, double a2);

  Matrix3d& operator = (const Matrix3d& m);
  Matrix3d& operator = (double a);

  double operator () (int i, int j) const;
  double& diag(int i);
  const double& diag(int i) const;

  Matrix3d operator + (const Matrix3d& m) const;
  Matrix3d operator - (const Matrix3d& m) const;
  Matrix3d operator * (const Matrix3d& m) const;
  Matrix3d operator * (double a) const;
  Matrix3d operator / (double a) const;

  Matrix3s operator + (const Matrix3s& m) const;
  Matrix3s operator - (const Matrix3s& m) const;
  Matrix3s operator * (const Matrix3s& m) const;

  Matrix3 operator + (const Matrix3& m) const;
  Matrix3 operator - (const Matrix3& m) const;
  Matrix3 operator * (const Matrix3& m) const;

  Matrix3d& operator += (const Matrix3d& m);
  Matrix3d& operator -= (const Matrix3d& m);
  Matrix3d& operator *= (const Matrix3d& m);
  Matrix3d& operator *= (double a);
  Matrix3d& operator /= (double a);

  Vector3 operator * (const Vector3& r) const;

  double tr() const;
  double det() const;
  void clean();
  
protected:
  double	d[3];

  friend class Matrix3;
  friend class Matrix3s;
  friend class Matrix3a;
};
//-----------------------------------------------------------------------------
inline Matrix3d operator * (double a, const Matrix3d& d) { return d*a; }
//-----------------------------------------------------------------------------
class Matrix3s
{
protected:
  enum {
    XX = 0,
    XY = 1,
    YY = 2,
    XZ = 3,
    YZ = 4,
    ZZ = 5 };
public:
  Matrix3s(){}
  Matrix3s(double xx, double yy, double zz, double xy, double yz, double xz);
  Matrix3s(const Matrix3d& d);

  Matrix3s& operator = (const Matrix3s& m);

  double& operator () (int i, int j);
  const double& operator () (int i, int j) const;

  double& xx() { return m[XX]; }
  double& yy() { return m[YY]; }
  double& zz() { return m[ZZ]; }
  double& xy() { return m[XY]; }
  double& yz() { return m[YZ]; }
  double& xz() { return m[XZ]; }

  const double& xx() const { return m[XX]; }
  const double& yy() const { return m[YY]; }
  const double& zz() const { return m[ZZ]; }
  const double& xy() const { return m[XY]; }
  const double& yz() const { return m[YZ]; }
  const double& xz() const { return m[XZ]; }

  Matrix3s operator + (const Matrix3d& d) const;
  Matrix3s operator - (const Matrix3d& d) const;
  Matrix3s operator * (const Matrix3d& d) const;

  Matrix3s operator + (const Matrix3s& t) const;
  Matrix3s operator - (const Matrix3s& t) const;
  Matrix3s operator * (const Matrix3s& t) const;
  Matrix3s operator * (double g) const;
  Matrix3s operator / (double g) const;

  Matrix3s& operator += (const Matrix3s& t);
  Matrix3s& operator -= (const Matrix3s& t);
  Matrix3s& operator *= (const Matrix3s& t);
  Matrix3s& operator *= (double g);
  Matrix3s& operator /= (double g);

  Matrix3s& operator += (const Matrix3d& d);
  Matrix3s& operator -= (const Matrix3d& d);

  Vector3 operator * (const Vector3& r) const;

  double tr() const;
  double det() const;
  Matrix3s inverse() const;
  void zero();
  Matrix3s dev() const;
  void eigen(double d[3], Vector3 r[3] = 0);
  void clean();
  
protected:
  double m[6];//order xx, xy, yy, xz, yz, zz

  friend class Matrix3d;
  friend class Matrix3;
};
//-----------------------------------------------------------------------------
inline Matrix3s operator * (double a, const Matrix3s& m) { return m*a; }
//-----------------------------------------------------------------------------
class Matrix3
{
public:
  Matrix3() {}
  Matrix3(double a00, double a01, double a02,
        double a10, double a11, double a12,
        double a20, double a21, double a22);

  Matrix3(double m[3][3]);

  Matrix3(const Matrix3d& m);
  Matrix3(const Matrix3s& m);

  Matrix3& operator = (const Matrix3d& m);
  Matrix3& operator = (const Matrix3s& m);
  Matrix3& operator = (const double m[3][3]);

  double& operator () (int i, int j);
  const double& operator () (int i, int j) const;
  double* operator [] (int i);

  Matrix3 operator + (const Matrix3& m) const;
  Matrix3 operator - (const Matrix3& m) const;
  Matrix3 operator * (const Matrix3& m) const;
  Matrix3 operator * (double a) const;
  Matrix3 operator / (double a) const;

  Matrix3 operator + (const Matrix3d& m) const;
  Matrix3 operator - (const Matrix3d& m) const;
  Matrix3 operator * (const Matrix3d& m) const;

  Matrix3 operator + (const Matrix3s& m) const;
  Matrix3 operator - (const Matrix3s& m) const;
  Matrix3 operator * (const Matrix3s& m) const;

  Matrix3& operator += (const Matrix3& m);
  Matrix3& operator -= (const Matrix3& m);
  Matrix3& operator *= (const Matrix3& m);
  Matrix3& operator *= (double a);
  Matrix3& operator /= (double a);

  Matrix3& operator += (const Matrix3d& m);
  Matrix3& operator -= (const Matrix3d& m);
  Matrix3& operator *= (const Matrix3d& m);

  Matrix3& operator += (const Matrix3s& m);
  Matrix3& operator -= (const Matrix3s& m);
  Matrix3& operator *= (const Matrix3s& m);

  Vector3 operator * (const Vector3& r) const;
  double det() const;
  double trace() const;
  void zero();
  void unit();
  Vector3 col(int j) const;
  Matrix3s sym() const;
  Matrix3 inverse() const;
  Matrix3 transpose() const;
  Matrix3 transinv() const;
  void skew(const Vector3& v);
  void clean();
  
protected:
  double d[3][3];

  friend class Matrix3d;
  friend class Matrix3s;
};
//-----------------------------------------------------------------------------
inline void Matrix3d::clean() 
{
  d[0]=d[1]=d[2]=0.0;
}
//-----------------------------------------------------------------------------
inline void Matrix3s::clean() 
{
  m[0]=m[1]=m[2]=m[3]=m[4]=m[5]=0.0;
} 
//-----------------------------------------------------------------------------
inline void Matrix3::clean() 
{
  d[0][0]=d[1][0]=d[2][0]=0.0;
  d[0][1]=d[1][1]=d[2][1]=0.0;
  d[0][2]=d[1][2]=d[2][2]=0.0;
}
//-----------------------------------------------------------------------------
inline Matrix3 operator & (const Vector3& a, const Vector3& b)
{
  return Matrix3(a.x*b.x, a.x*b.y, a.x*b.z,
               a.y*b.x, a.y*b.y, a.y*b.z,
               a.z*b.x, a.z*b.y, a.z*b.z);
}
//-----------------------------------------------------------------------------
inline Matrix3s dyad(const Vector3& a)
{
  return Matrix3s(a.x*a.x, a.y*a.y, a.z*a.z, a.x*a.y, a.y*a.z, a.x*a.z);
}
//-----------------------------------------------------------------------------
// c_ij = a_i*b_j + a_j*b_i
inline Matrix3s dyads(const Vector3& a, const Vector3& b)
{
  return Matrix3s(2.0*a.x*b.x, 2.0*a.y*b.y, 2.0*a.z*b.z, a.x*b.y + a.y*b.x, a.y*b.z + a.z*b.y, a.x*b.z + a.y*b.z);
}
//-----------------------------------------------------------------------------
inline Matrix3d::Matrix3d(double a) { d[0] = d[1] = d[2] = a; }
//-----------------------------------------------------------------------------
inline Matrix3d::Matrix3d(double a0, double a1, double a2) { d[0] = a0; d[1] = a1; d[2] = a2; }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator = (const Matrix3d& m) { d[0] = m.d[0]; d[1] = m.d[1]; d[2] = m.d[2]; return (*this); }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator = (double a) { d[0] = d[1] = d[2] = a; return (*this); }
//-----------------------------------------------------------------------------
inline double Matrix3d::operator () (int i, int j) const { return (i==j? d[i] : 0); }
//-----------------------------------------------------------------------------
inline double& Matrix3d::diag(int i) { return d[i]; }
//-----------------------------------------------------------------------------
inline const double& Matrix3d::diag(int i) const { return d[i]; }
//-----------------------------------------------------------------------------
inline Matrix3d Matrix3d::operator + (const Matrix3d& m) const { return Matrix3d(d[0]+m.d[0],d[1]+m.d[1],d[2]+m.d[2]);}
//-----------------------------------------------------------------------------
inline Matrix3d Matrix3d::operator - (const Matrix3d& m) const { return Matrix3d(d[0]-m.d[0],d[1]-m.d[1],d[2]-m.d[2]);}
//-----------------------------------------------------------------------------
inline Matrix3d Matrix3d::operator * (const Matrix3d& m) const { return Matrix3d(d[0]*m.d[0],d[1]*m.d[1],d[2]*m.d[2]);}
//-----------------------------------------------------------------------------
inline Matrix3d Matrix3d::operator * (double a) const { return Matrix3d(d[0]*a, d[1]*a, d[2]*a); }
//-----------------------------------------------------------------------------
inline Matrix3d Matrix3d::operator / (double a) const { a = 1./a; return Matrix3d(d[0]*a, d[1]*a, d[2]*a); }
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3d::operator + (const Matrix3s& m) const
{
  return Matrix3s(
    d[0]+m.m[Matrix3s::XX],
    d[1]+m.m[Matrix3s::YY],
    d[2]+m.m[Matrix3s::ZZ],
    m.m[Matrix3s::XY],
    m.m[Matrix3s::YZ],
    m.m[Matrix3s::XZ]
  );
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3d::operator - (const Matrix3s& m) const
{
  return Matrix3s(
    d[0]-m.m[Matrix3s::XX],
    d[1]-m.m[Matrix3s::YY],
    d[2]-m.m[Matrix3s::ZZ],
    -m.m[Matrix3s::XY],
    -m.m[Matrix3s::YZ],
    -m.m[Matrix3s::XZ]
  );
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3d::operator * (const Matrix3s& m) const
{
  return Matrix3s(
    d[0]*m.m[Matrix3s::XX],
    d[1]*m.m[Matrix3s::YY],
    d[2]*m.m[Matrix3s::ZZ],
    d[0]*m.m[Matrix3s::XY],
    d[1]*m.m[Matrix3s::YZ],
    d[0]*m.m[Matrix3s::XZ]
  );
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3d::operator + (const Matrix3& m) const
{
  return Matrix3(d[0]+m.d[0][0], m.d[0][1], m.d[0][2],
               m.d[1][0], d[1]+m.d[1][1], m.d[1][2],
               m.d[2][0], m.d[2][1], d[2]+m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3d::operator - (const Matrix3& m) const
{
  return Matrix3(d[0]-m.d[0][0], -m.d[0][1], -m.d[0][2],
               -m.d[1][0], d[1]-m.d[1][1], -m.d[1][2],
               -m.d[2][0], -m.d[2][1], d[2]-m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3d::operator * (const Matrix3& m) const
{
  return Matrix3(d[0]*m.d[0][0], d[0]*m.d[0][1], d[0]*m.d[0][2],
               d[1]*m.d[1][0], d[1]*m.d[1][1], d[1]*m.d[1][2],
               d[2]*m.d[2][0], d[2]*m.d[2][1], d[2]*m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator += (const Matrix3d& m) { d[0] += m.d[0]; d[1] += m.d[1]; d[2] += m.d[2]; return (*this); }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator -= (const Matrix3d& m) { d[0] -= m.d[0]; d[1] -= m.d[1]; d[2] -= m.d[2]; return (*this); }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator *= (const Matrix3d& m) { d[0] *= m.d[0]; d[1] *= m.d[1]; d[2] *= m.d[2]; return (*this); }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator *= (double a) { d[0] *= a; d[1] *= a; d[2] *= a; return (*this); }
//-----------------------------------------------------------------------------
inline Matrix3d& Matrix3d::operator /= (double a) { a = 1./a; d[0] *= a; d[1] *= a; d[2] *= a; return (*this); }
//-----------------------------------------------------------------------------
inline Vector3 Matrix3d::operator * (const Vector3& r) const { return Vector3(r.x*d[0], r.y*d[1], r.z*d[2]); }
//-----------------------------------------------------------------------------
inline double Matrix3d::tr() const { return d[0]+d[1]+d[2]; }
//-----------------------------------------------------------------------------
inline double Matrix3d::det() const { return d[0]*d[1]*d[2]; }
//-----------------------------------------------------------------------------
inline Matrix3s::Matrix3s(double xx, double yy, double zz, double xy, double yz, double xz)
{
  m[XX] = xx;
  m[XY] = xy;
  m[YY] = yy;
  m[XZ] = xz;
  m[YZ] = yz;
  m[ZZ] = zz;
}
//-----------------------------------------------------------------------------
inline Matrix3s::Matrix3s(const Matrix3d& d)
{
  m[XX] = d.d[0];
  m[YY] = d.d[1];
  m[ZZ] = d.d[2];
  m[XY] = m[YZ] = m[XZ] = 0.;
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator = (const Matrix3s& d)
{
  m[XX] = d.m[XX];
  m[YY] = d.m[YY];
  m[ZZ] = d.m[ZZ];
  m[XY] = d.m[XY];
  m[YZ] = d.m[YZ];
  m[XZ] = d.m[XZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline double& Matrix3s::operator ()(int i, int j)
{
  const int n[] = {0, 1, 3};
  return (i<=j? m[n[j]+i] : m[n[i]+j]);
}
//-----------------------------------------------------------------------------
inline const double& Matrix3s::operator ()(int i, int j) const
{
  const int n[] = {0, 1, 3};
  return (i<=j? m[n[j]+i] : m[n[i]+j]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator + (const Matrix3d& d) const
{
  return Matrix3s(m[XX]+d.d[0], m[YY]+d.d[1], m[ZZ]+d.d[2], m[XY], m[YZ], m[XZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator - (const Matrix3d& d) const
{
  return Matrix3s(m[XX]-d.d[0], m[YY]-d.d[1], m[ZZ]-d.d[2], m[XY], m[YZ], m[XZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator * (const Matrix3d& d) const
{
  return Matrix3s(m[XX]*d.d[0], m[YY]*d.d[1], m[ZZ]*d.d[2], m[XY]*d.d[1], m[YZ]*d.d[2], m[XZ]*d.d[2]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator +(const Matrix3s& t) const
{
  return Matrix3s(m[XX]+t.m[XX], m[YY]+t.m[YY], m[ZZ]+t.m[ZZ], m[XY]+t.m[XY], m[YZ]+t.m[YZ],m[XZ]+t.m[XZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator -(const Matrix3s& t) const
{
  return Matrix3s(m[XX]-t.m[XX], m[YY]-t.m[YY], m[ZZ]-t.m[ZZ], m[XY]-t.m[XY], m[YZ]-t.m[YZ],m[XZ]-t.m[XZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator *(const Matrix3s& t) const
{
  return Matrix3s(
    m[XX]*t.m[XX]+m[XY]*t.m[XY]+m[XZ]*t.m[XZ],
    m[XY]*t.m[XY]+m[YY]*t.m[YY]+m[YZ]*t.m[YZ],
    m[XZ]*t.m[XZ]+m[YZ]*t.m[YZ]+m[ZZ]*t.m[ZZ],
    m[XX]*t.m[XY]+m[XY]*t.m[YY]+m[XZ]*t.m[YZ],
    m[XY]*t.m[XZ]+m[YY]*t.m[YZ]+m[YZ]*t.m[ZZ],
    m[XX]*t.m[XZ]+m[XY]*t.m[YZ]+m[XZ]*t.m[ZZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator * (double g) const
{
  return Matrix3s(m[XX]*g, m[YY]*g, m[ZZ]*g, m[XY]*g, m[YZ]*g, m[XZ]*g);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::operator / (double g) const
{
  g = 1.0/g;
  return Matrix3s(m[XX]*g, m[YY]*g, m[ZZ]*g, m[XY]*g, m[YZ]*g, m[XZ]*g);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator += (const Matrix3s& t)
{
  m[XX] += t.m[XX]; m[YY] += t.m[YY]; m[ZZ] += t.m[ZZ];
  m[XY] += t.m[XY]; m[YZ] += t.m[YZ]; m[XZ] += t.m[XZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator -= (const Matrix3s& t)
{
  m[XX] -= t.m[XX]; m[YY] -= t.m[YY]; m[ZZ] -= t.m[ZZ];
  m[XY] -= t.m[XY]; m[YZ] -= t.m[YZ]; m[XZ] -= t.m[XZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator *= (const Matrix3s& t)
{
  double xx = m[XX]*t.m[XX]+m[XY]*t.m[XY]+m[XZ]*t.m[XZ];
  double yy = m[XY]*t.m[XY]+m[YY]*t.m[YY]+m[YZ]*t.m[YZ];
  double zz = m[XZ]*t.m[XZ]+m[YZ]*t.m[YZ]+m[ZZ]*t.m[ZZ];
  double xy = m[XX]*t.m[XY]+m[XY]*t.m[YY]+m[XZ]*t.m[YZ];
  double yz = m[XY]*t.m[XZ]+m[YY]*t.m[YZ]+m[YZ]*t.m[ZZ];
  double xz = m[XX]*t.m[XZ]+m[XY]*t.m[YZ]+m[XZ]*t.m[ZZ];

  m[XX] = xx; m[YY] = yy; m[ZZ] = zz;
  m[XY] = xy; m[YZ] = yz; m[XZ] = xz;

  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator *= (double g)
{
  m[XX] *= g; m[YY] *= g; m[ZZ] *= g;
  m[XY] *= g; m[YZ] *= g; m[XZ] *= g;
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator /= (double g)
{
  g = 1./g;
  m[XX] *= g; m[YY] *= g; m[ZZ] *= g;
  m[XY] *= g; m[YZ] *= g; m[XZ] *= g;
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator += (const Matrix3d& d)
{
  m[XX] += d.d[0];
  m[YY] += d.d[1];
  m[ZZ] += d.d[2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3s& Matrix3s::operator -= (const Matrix3d& d)
{
  m[XX] -= d.d[0];
  m[YY] -= d.d[1];
  m[ZZ] -= d.d[2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Vector3 Matrix3s::operator* (const Vector3& r) const
{
  return Vector3(
    r.x*m[XX]+r.y*m[XY]+r.z*m[XZ],
    r.x*m[XY]+r.y*m[YY]+r.z*m[YZ],
    r.x*m[XZ]+r.y*m[YZ]+r.z*m[ZZ]
  );
}
//-----------------------------------------------------------------------------
inline double Matrix3s::tr() const
{
  return m[XX]+m[YY]+m[ZZ];
}
//-----------------------------------------------------------------------------
inline double Matrix3s::det() const
{
  return (m[XX]*(m[YY]*m[ZZ] - m[YZ]*m[YZ])
  + m[XY]*(m[YZ]*m[XZ] - m[ZZ]*m[XY])
  + m[XZ]*(m[XY]*m[YZ] - m[YY]*m[XZ]));
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::inverse() const
{
  double D = det();
  assert(D != 0);
  D = 1/D;
  return Matrix3s(D*(m[YY]*m[ZZ] - m[YZ]*m[YZ]),//xx
                D*(m[XX]*m[ZZ] - m[XZ]*m[XZ]),//yy
                D*(m[XX]*m[YY] - m[XY]*m[XY]),//zz
                D*(m[XZ]*m[YZ] - m[XY]*m[ZZ]),//xy
                D*(m[XZ]*m[XY] - m[XX]*m[YZ]),//yz
                D*(m[XY]*m[YZ] - m[YY]*m[XZ]));//xz             
}
//-----------------------------------------------------------------------------
inline void Matrix3s::zero()
{
  m[0] = m[1] = m[2] = m[3] = m[4] = m[5] = 0;
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3s::dev() const
{
  double t = (m[XX]+m[YY]+m[ZZ])/3.0;
  return Matrix3s(m[XX]-t, m[YY]-t, m[ZZ]-t, m[XY], m[YZ], m[XZ]);
}
//-----------------------------------------------------------------------------

inline Matrix3::Matrix3(double a00, double a01, double a02,
                    double a10, double a11, double a12,
                    double a20, double a21, double a22)
{
  d[0][0] = a00; d[0][1] = a01; d[0][2] = a02;
  d[1][0] = a10; d[1][1] = a11; d[1][2] = a12;
  d[2][0] = a20; d[2][1] = a21; d[2][2] = a22;
}
//-----------------------------------------------------------------------------
inline Matrix3::Matrix3(double m[3][3])
{
  d[0][0] = m[0][0]; d[0][1] = m[0][1]; d[0][2] = m[0][2];
  d[1][0] = m[1][0]; d[1][1] = m[1][1]; d[1][2] = m[1][2];
  d[2][0] = m[2][0]; d[2][1] = m[2][1]; d[2][2] = m[2][2];
}
//-----------------------------------------------------------------------------
inline Matrix3::Matrix3(const Matrix3d& m)
{
  d[0][0] = m.d[0]; d[1][1] = m.d[1]; d[2][2] = m.d[2];
  d[0][1] = d[1][0] = 0;
  d[1][2] = d[2][1] = 0;
  d[0][2] = d[2][0] = 0;
}
//-----------------------------------------------------------------------------
inline Matrix3::Matrix3(const Matrix3s& m)
{
  d[0][0] = m.m[Matrix3s::XX];
  d[1][1] = m.m[Matrix3s::YY];
  d[2][2] = m.m[Matrix3s::ZZ];
  d[0][1] = d[1][0] = m.m[Matrix3s::XY];
  d[1][2] = d[2][1] = m.m[Matrix3s::YZ];
  d[0][2] = d[2][0] = m.m[Matrix3s::XZ];
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator = (const Matrix3d& m)
{
  d[0][0] = m.d[0];
  d[1][1] = m.d[1];
  d[2][2] = m.d[2];
  d[0][1] = d[1][0] = 0;
  d[1][2] = d[2][1] = 0;
  d[0][2] = d[2][0] = 0;
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator = (const Matrix3s& m)
{
  d[0][0] = m.m[Matrix3s::XX];
  d[1][1] = m.m[Matrix3s::YY];
  d[2][2] = m.m[Matrix3s::ZZ];
  d[0][1] = d[1][0] = m.m[Matrix3s::XY];
  d[1][2] = d[2][1] = m.m[Matrix3s::YZ];
  d[0][2] = d[2][0] = m.m[Matrix3s::XZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator = (const double m[3][3])
{
  d[0][0] = m[0][0]; d[0][1] = m[0][1]; d[0][2] = m[0][2];
  d[1][0] = m[1][0]; d[1][1] = m[1][1]; d[1][2] = m[1][2];
  d[2][0] = m[2][0]; d[2][1] = m[2][1]; d[2][2] = m[2][2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline double& Matrix3::operator () (int i, int j) { return d[i][j]; }
inline const double& Matrix3::operator () (int i, int j) const { return d[i][j]; }
inline double* Matrix3::operator [] (int i) { return d[i]; }
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator + (const Matrix3& m) const
{
  return Matrix3( d[0][0]+m.d[0][0], d[0][1]+m.d[0][1], d[0][2]+m.d[0][2],
                d[1][0]+m.d[1][0], d[1][1]+m.d[1][1], d[1][2]+m.d[1][2],
                d[2][0]+m.d[2][0], d[2][1]+m.d[2][1], d[2][2]+m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator - (const Matrix3& m) const
{
  return Matrix3( d[0][0]-m.d[0][0], d[0][1]-m.d[0][1], d[0][2]-m.d[0][2],
                d[1][0]-m.d[1][0], d[1][1]-m.d[1][1], d[1][2]-m.d[1][2],
                d[2][0]-m.d[2][0], d[2][1]-m.d[2][1], d[2][2]-m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator * (const Matrix3& m) const
{
  return Matrix3(d[0][0]*m.d[0][0]+d[0][1]*m.d[1][0]+d[0][2]*m.d[2][0],
               d[0][0]*m.d[0][1]+d[0][1]*m.d[1][1]+d[0][2]*m.d[2][1],
               d[0][0]*m.d[0][2]+d[0][1]*m.d[1][2]+d[0][2]*m.d[2][2],
               d[1][0]*m.d[0][0]+d[1][1]*m.d[1][0]+d[1][2]*m.d[2][0],
               d[1][0]*m.d[0][1]+d[1][1]*m.d[1][1]+d[1][2]*m.d[2][1],
               d[1][0]*m.d[0][2]+d[1][1]*m.d[1][2]+d[1][2]*m.d[2][2],
               d[2][0]*m.d[0][0]+d[2][1]*m.d[1][0]+d[2][2]*m.d[2][0],
               d[2][0]*m.d[0][1]+d[2][1]*m.d[1][1]+d[2][2]*m.d[2][1],
               d[2][0]*m.d[0][2]+d[2][1]*m.d[1][2]+d[2][2]*m.d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator * (double a) const
{
  return Matrix3(d[0][0]*a, d[0][1]*a, d[0][2]*a,
               d[1][0]*a, d[1][1]*a, d[1][2]*a,
               d[2][0]*a, d[2][1]*a, d[2][2]*a);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator / (double a) const
{
  a = 1./a;
  return Matrix3(d[0][0]*a, d[0][1]*a, d[0][2]*a,
               d[1][0]*a, d[1][1]*a, d[1][2]*a,
               d[2][0]*a, d[2][1]*a, d[2][2]*a);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator + (const Matrix3d& m) const
{
  return Matrix3( d[0][0]+m.d[0], d[0][1], d[0][2],
                d[1][0], d[1][1]+m.d[1], d[1][2],
                d[2][0], d[2][1], d[2][2]+m.d[2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator - (const Matrix3d& m) const
{
  return Matrix3( d[0][0]-m.d[0], d[0][1], d[0][2],
                d[1][0], d[1][1]-m.d[1], d[1][2],
                d[2][0], d[2][1], d[2][2]-m.d[2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator * (const Matrix3d& m) const
{
  return Matrix3( d[0][0]*m.d[0], d[0][1]*m.d[1], d[0][2]*m.d[2],
                d[1][0]*m.d[0], d[1][1]*m.d[1], d[1][2]*m.d[2],
                d[2][0]*m.d[0], d[2][1]*m.d[1], d[2][2]*m.d[2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator + (const Matrix3s& m) const
{
  return Matrix3(d[0][0]+m.m[m.XX], d[0][1]+m.m[m.XY], d[0][2]+m.m[m.XZ],
               d[1][0]+m.m[m.XY], d[1][1]+m.m[m.YY], d[1][2]+m.m[m.YZ],
               d[2][0]+m.m[m.XZ], d[2][1]+m.m[m.YZ], d[2][2]+m.m[m.ZZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator - (const Matrix3s& m) const
{
  return Matrix3(d[0][0]-m.m[m.XX], d[0][1]-m.m[m.XY], d[0][2]-m.m[m.XZ],
               d[1][0]-m.m[m.XY], d[1][1]-m.m[m.YY], d[1][2]-m.m[m.YZ],
               d[2][0]-m.m[m.XZ], d[2][1]-m.m[m.YZ], d[2][2]-m.m[m.ZZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::operator * (const Matrix3s& m) const
{
  return Matrix3(
    d[0][0]*m.m[m.XX] + d[0][1]*m.m[m.XY] + d[0][2]*m.m[m.XZ],
    d[0][0]*m.m[m.XY] + d[0][1]*m.m[m.YY] + d[0][2]*m.m[m.YZ],
    d[0][0]*m.m[m.XZ] + d[0][1]*m.m[m.YZ] + d[0][2]*m.m[m.ZZ],
    d[1][0]*m.m[m.XX] + d[1][1]*m.m[m.XY] + d[1][2]*m.m[m.XZ],
    d[1][0]*m.m[m.XY] + d[1][1]*m.m[m.YY] + d[1][2]*m.m[m.YZ],
    d[1][0]*m.m[m.XZ] + d[1][1]*m.m[m.YZ] + d[1][2]*m.m[m.ZZ],
    d[2][0]*m.m[m.XX] + d[2][1]*m.m[m.XY] + d[2][2]*m.m[m.XZ],
    d[2][0]*m.m[m.XY] + d[2][1]*m.m[m.YY] + d[2][2]*m.m[m.YZ],
    d[2][0]*m.m[m.XZ] + d[2][1]*m.m[m.YZ] + d[2][2]*m.m[m.ZZ]);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator += (const Matrix3& m)
{
  d[0][0] += m.d[0][0]; d[0][1] += m.d[0][1]; d[0][2] += m.d[0][2];
  d[1][0] += m.d[1][0]; d[1][1] += m.d[1][1]; d[1][2] += m.d[1][2];
  d[2][0] += m.d[2][0]; d[2][1] += m.d[2][1]; d[2][2] += m.d[2][2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator -= (const Matrix3& m)
{
  d[0][0] -= m.d[0][0]; d[0][1] -= m.d[0][1]; d[0][2] -= m.d[0][2];
  d[1][0] -= m.d[1][0]; d[1][1] -= m.d[1][1]; d[1][2] -= m.d[1][2];
  d[2][0] -= m.d[2][0]; d[2][1] -= m.d[2][1]; d[2][2] -= m.d[2][2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator *= (const Matrix3& m)
{
  double d00 = d[0][0]*m.d[0][0]+d[0][1]*m.d[1][0]+d[0][2]*m.d[2][0];
  double d01 = d[0][0]*m.d[0][1]+d[0][1]*m.d[1][1]+d[0][2]*m.d[2][1];
  double d02 = d[0][0]*m.d[0][2]+d[0][1]*m.d[1][2]+d[0][2]*m.d[2][2];
  double d10 = d[1][0]*m.d[0][0]+d[1][1]*m.d[1][0]+d[1][2]*m.d[2][0];
  double d11 = d[1][0]*m.d[0][1]+d[1][1]*m.d[1][1]+d[1][2]*m.d[2][1];
  double d12 = d[1][0]*m.d[0][2]+d[1][1]*m.d[1][2]+d[1][2]*m.d[2][2];
  double d20 = d[2][0]*m.d[0][0]+d[2][1]*m.d[1][0]+d[2][2]*m.d[2][0];
  double d21 = d[2][0]*m.d[0][1]+d[2][1]*m.d[1][1]+d[2][2]*m.d[2][1];
  double d22 = d[2][0]*m.d[0][2]+d[2][1]*m.d[1][2]+d[2][2]*m.d[2][2];

  d[0][0] = d00; d[0][1] = d01; d[0][2] = d02;
  d[1][0] = d10; d[1][1] = d11; d[1][2] = d12;
  d[2][0] = d20; d[2][1] = d21; d[2][2] = d22;

  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator *= (double a)
{
  d[0][0]*=a; d[0][1]*=a; d[0][2]*=a;
  d[1][0]*=a; d[1][1]*=a; d[1][2]*=a;
  d[2][0]*=a; d[2][1]*=a; d[2][2]*=a;
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator /= (double a)
{
  a = 1./a;
  d[0][0]*=a; d[0][1]*=a; d[0][2]*=a;
  d[1][0]*=a; d[1][1]*=a; d[1][2]*=a;
  d[2][0]*=a; d[2][1]*=a; d[2][2]*=a;
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator += (const Matrix3d& m)
{
  d[0][0] += m.d[0];
  d[1][1] += m.d[1];
  d[2][2] += m.d[2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator -= (const Matrix3d& m)
{
  d[0][0] -= m.d[0];
  d[1][1] -= m.d[1];
  d[2][2] -= m.d[2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator *= (const Matrix3d& m)
{
  d[0][0] *= m.d[0]; d[0][1] *= m.d[1]; d[0][2] *= m.d[2];
  d[1][0] *= m.d[0]; d[1][1] *= m.d[1]; d[1][2] *= m.d[2];
  d[2][0] *= m.d[0]; d[2][1] *= m.d[1]; d[2][2] *= m.d[2];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator += (const Matrix3s& m)
{
  d[0][0] += m.m[m.XX]; d[0][1] += m.m[m.XY]; d[0][2] += m.m[m.XZ];
  d[1][0] += m.m[m.XY]; d[1][1] += m.m[m.YY]; d[1][2] += m.m[m.YZ];
  d[2][0] += m.m[m.XZ]; d[2][1] += m.m[m.YZ]; d[2][2] += m.m[m.ZZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator -= (const Matrix3s& m)
{
  d[0][0] -= m.m[m.XX]; d[0][1] -= m.m[m.XY]; d[0][2] -= m.m[m.XZ];
  d[1][0] -= m.m[m.XY]; d[1][1] -= m.m[m.YY]; d[1][2] -= m.m[m.YZ];
  d[2][0] -= m.m[m.XZ]; d[2][1] -= m.m[m.YZ]; d[2][2] -= m.m[m.ZZ];
  return (*this);
}
//-----------------------------------------------------------------------------
inline Matrix3& Matrix3::operator *= (const Matrix3s& m)
{
  double d00 = d[0][0]*m.m[m.XX]+d[0][1]*m.m[m.XY]+d[0][2]*m.m[m.XZ];
  double d01 = d[0][0]*m.m[m.XY]+d[0][1]*m.m[m.YY]+d[0][2]*m.m[m.YZ];
  double d02 = d[0][0]*m.m[m.XZ]+d[0][1]*m.m[m.YZ]+d[0][2]*m.m[m.ZZ];
  double d10 = d[1][0]*m.m[m.XX]+d[1][1]*m.m[m.XY]+d[1][2]*m.m[m.XZ];
  double d11 = d[1][0]*m.m[m.XY]+d[1][1]*m.m[m.YY]+d[1][2]*m.m[m.YZ];
  double d12 = d[1][0]*m.m[m.XZ]+d[1][1]*m.m[m.YZ]+d[1][2]*m.m[m.ZZ];
  double d20 = d[2][0]*m.m[m.XX]+d[2][1]*m.m[m.XY]+d[2][2]*m.m[m.XZ];
  double d21 = d[2][0]*m.m[m.XY]+d[2][1]*m.m[m.YY]+d[2][2]*m.m[m.YZ];
  double d22 = d[2][0]*m.m[m.XZ]+d[2][1]*m.m[m.YZ]+d[2][2]*m.m[m.ZZ];

  d[0][0] = d00; d[0][1] = d01; d[0][2] = d02;
  d[1][0] = d10; d[1][1] = d11; d[1][2] = d12;
  d[2][0] = d20; d[2][1] = d21; d[2][2] = d22;

  return (*this);
}

//-----------------------------------------------------------------------------
inline Vector3 Matrix3::operator * (const Vector3& r) const
{
  return Vector3(d[0][0]*r.x+d[0][1]*r.y+d[0][2]*r.z,
               d[1][0]*r.x+d[1][1]*r.y+d[1][2]*r.z,
               d[2][0]*r.x+d[2][1]*r.y+d[2][2]*r.z);
}
//-----------------------------------------------------------------------------
inline double Matrix3::det() const
{
  return (d[0][0]*(d[1][1]*d[2][2] - d[1][2]*d[2][1])
  + d[0][1]*(d[1][2]*d[2][0] - d[2][2]*d[1][0])
  + d[0][2]*(d[1][0]*d[2][1] - d[1][1]*d[2][0]));
}
//-----------------------------------------------------------------------------
inline double Matrix3::trace() const { return d[0][0]+d[1][1]+d[2][2]; }
//-----------------------------------------------------------------------------
inline void Matrix3::unit()
{
  d[0][0] = d[1][1] = d[2][2] = 1;
  d[0][1] = d[1][0] = 0;
  d[0][2] = d[2][0] = 0;
  d[1][2] = d[2][1] = 0;
}
//-----------------------------------------------------------------------------
inline void Matrix3::zero()
{
  d[0][0] = d[0][1] = d[0][2] = 0;
  d[1][0] = d[1][1] = d[1][2] = 0;
  d[2][0] = d[2][1] = d[2][2] = 0;
}
//-----------------------------------------------------------------------------
inline Vector3 Matrix3::col(int j) const
{
  return Vector3(d[0][j], d[1][j], d[2][j]);
}
//-----------------------------------------------------------------------------
inline Matrix3s Matrix3::sym() const
{
  return Matrix3s(
    d[0][0],
    d[1][1],
    d[2][2],
    0.5*(d[0][1]+d[1][0]),
                0.5*(d[1][2]+d[2][1]),
                0.5*(d[0][2]+d[2][0]));
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::inverse() const
{
  double D = det();
  assert(D != 0);
  D = 1/D;

  return Matrix3(D*(d[1][1]*d[2][2] - d[1][2]*d[2][1]),
               D*(d[0][2]*d[2][1] - d[0][1]*d[2][2]),
               D*(d[0][1]*d[1][2] - d[1][1]*d[0][2]),
               D*(d[1][2]*d[2][0] - d[1][0]*d[2][2]),
               D*(d[0][0]*d[2][2] - d[0][2]*d[2][0]),
               D*(d[0][2]*d[1][0] - d[0][0]*d[1][2]),
               D*(d[1][0]*d[2][1] - d[1][1]*d[2][0]),
               D*(d[0][1]*d[2][0] - d[0][0]*d[2][1]),
               D*(d[0][0]*d[1][1] - d[0][1]*d[1][0]));
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::transpose() const
{
  return Matrix3(d[0][0], d[1][0], d[2][0],
               d[0][1], d[1][1], d[2][1],
               d[0][2], d[1][2], d[2][2]);
}
//-----------------------------------------------------------------------------
inline Matrix3 Matrix3::transinv() const
{
  double D = det();
  assert(D != 0);
  D = 1/D;

  return Matrix3(D*(d[1][1]*d[2][2] - d[1][2]*d[2][1]), // xx
               D*(d[1][2]*d[2][0] - d[1][0]*d[2][2]), // yx
               D*(d[1][0]*d[2][1] - d[1][1]*d[2][0]), // zx
               D*(d[0][2]*d[2][1] - d[0][1]*d[2][2]), // xy
               D*(d[0][0]*d[2][2] - d[0][2]*d[2][0]), // yy
               D*(d[0][1]*d[2][0] - d[0][0]*d[2][1]), // zy
               D*(d[0][1]*d[1][2] - d[1][1]*d[0][2]), // xz
               D*(d[0][2]*d[1][0] - d[0][0]*d[1][2]), // yz
               D*(d[0][0]*d[1][1] - d[0][1]*d[1][0])); // zz
}
//-----------------------------------------------------------------------------
inline void Matrix3::skew(const Vector3& v)
{
  d[0][0] =    0; d[0][1] = -v.z; d[0][2] =  v.y;
  d[1][0] =  v.z; d[1][1] =    0; d[1][2] = -v.x;
  d[2][0] = -v.y; d[2][1] =  v.x; d[2][2] =    0;
}
//-----------------------------------------------------------------------------
inline std::ostream& operator<<(std::ostream& s, Matrix3s const& m)
{
  s << m.xx() << " " << m.xy() << " " << m.xz() << "\n";
  s << m.xy() << " " << m.yy() << " " << m.yz() << "\n";
  s << m.xz() << " " << m.yz() << " " << m.zz();
  return s;
}
//-----------------------------------------------------------------------------
inline std::ostream& operator<<(std::ostream& s, Matrix3 const& m)
{
  s << m(0,0) << " " << m(0,1) << " " << m(0,2) << "\n";
  s << m(1,0) << " " << m(1,1) << " " << m(1,2) << "\n";
  s << m(2,0) << " " << m(2,1) << " " << m(2,2);
  return s;
}
//-----------------------------------------------------------------------------
#endif

