// Copyright (C) 2006-2008 Garth N. Wells
// Modified by Anders Logg 2006.
// Modified by Dag Lindbo 2008.
//
#include "boost/tuple/tuple.hpp"
#include "UmfpackLUSolver.h"
#include "SparseMatrix.h"
#include "DenseVector.h"
#include "utils/error_macros.h"
#include "matrix/SparseMatrix.h"
#include "matrix/uBlasMatrix.h"

#include <fstream>
#include "utils/globals.h"

#define HAS_UMFPACK
extern "C"
{
#ifdef HAS_UMFPACK
#include <umfpack.h>
#endif
}

/// NoDeleter is a customised deleter intended for use with smart pointers.
class NoDeleter
{
public:
  void operator() (const void *p) {}
};

#ifdef HAS_UMFPACK

  class UmfpackIntSymbolicDeleter
  {
  public:
    void operator() (void* symbolic)
    {
      if (symbolic)
      {
        umfpack_di_free_symbolic(&symbolic);
        symbolic = 0;
      }
    }
  };

  class UmfpackLongIntSymbolicDeleter
  {
  public:
    void operator() (void* symbolic)
    {
      if (symbolic)
      {
        umfpack_dl_free_symbolic(&symbolic);
        symbolic = 0;
      }
    }
  };

  class UmfpackIntNumericDeleter
  {
  public:
    void operator() (void* numeric)
    {
      if (numeric)
      {
        umfpack_di_free_numeric(&numeric);
        numeric = 0;
      }
    }
  };

  class UmfpackLongIntNumericDeleter
  {
  public:
    void operator() (void* numeric)
    {
      if (numeric)
      {
        umfpack_dl_free_numeric(&numeric);
        numeric = 0;
      }
    }
  };
#endif
//-----------------------------------------------------------------------------
UmfpackLUSolver::UmfpackLUSolver()
{
}
//-----------------------------------------------------------------------------
UmfpackLUSolver::UmfpackLUSolver(boost::shared_ptr<const SparseMatrix> A)
                               : A(A)
{
}
//-----------------------------------------------------------------------------
UmfpackLUSolver::~UmfpackLUSolver()
{
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::set_operator(const boost::shared_ptr<const SparseMatrix> A)
{
  symbolic.reset();
  numeric.reset();
  this->A = A;
  assert(this->A);
}
//-----------------------------------------------------------------------------
const SparseMatrix& UmfpackLUSolver::get_operator() const
{
  if (!A)
    cell_error("Operator for linear solver has not been set.");
  return *A;
}
//-----------------------------------------------------------------------------
size_t UmfpackLUSolver::solve(DenseVector& x, const DenseVector& b, bool erase)
{
  assert(A);

  // Check dimensions
  size_t M = A->size(0);
  size_t N = A->size(1);
  if ( N != b.size() )
    cell_error("Non-matching dimensions for linear system.");
  // Set tolerances and other parameters
    const bool report  = get("LU report");
    const bool reuse_fact   = get("LU reuse_factorization");
    const bool same_pattern = get("LU same_nonzero_pattern");
    if ( erase )
      x.init(b.size());
    // Read parameters if not done
    if ( !parameters_read )
        readParameters();
    // Write a message
    if ( report )
      cell_info("Solving linear system of size %d x %d (Umfpack direct LU solver).", M, N);
  // Perform symbolic factorization if required
  if (!symbolic)
    symbolic_factorize();
  else if (!reuse_fact && !same_pattern)
    symbolic_factorize();

  // Perform numerical factorization if required
  if (!numeric)
    numeric_factorize();
  else if (!reuse_fact)
    numeric_factorize();
  // Solve
  return solve_factorized(x, b);
}
//-----------------------------------------------------------------------------
size_t UmfpackLUSolver::solve(const SparseMatrix& A, DenseVector& x, const DenseVector& b, bool erase)
{
  boost::shared_ptr<const SparseMatrix> _A(&A, NoDeleter());
  set_operator(_A);
  return solve(x, b, erase);
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::symbolic_factorize()
{
  if (!A)
    cell_error("A matrix must be assocoated with UmfpackLUSolver to peform a symbolic factorisation.");

  // Clear any old factorizations
  symbolic.reset();
  numeric.reset();

  // Get matrix data
  boost::tuple<const std::size_t*, const std::size_t*, const double*, int> data = A->data();
  const std::size_t* Ap  = boost::get<0>(data);
  const std::size_t* Ai  = boost::get<1>(data);
  const double*      Ax  = boost::get<2>(data);
  const size_t         nnz = boost::get<3>(data);

  // Check dimensions and get number of non-zeroes
  const size_t M   = A->size(0);
  const size_t N   = A->size(1);
  assert(nnz >= M);

  // Perform symbolic factorisation
  symbolic = umfpack_factorize_symbolic(M, N, Ap, Ai, Ax);
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::numeric_factorize()
{
  if (!A)
    cell_error("A matrix must be assocoated with UmfpackLUSolver to peform a factorisation.");

  boost::tuple<const std::size_t*, const std::size_t*, const double*, int> data = A->data();
  const std::size_t* Ap  = boost::get<0>(data);
  const std::size_t* Ai  = boost::get<1>(data);
  const double*      Ax  = boost::get<2>(data);
  const size_t         nnz = boost::get<3>(data);

  const size_t M   = A->size(0);
  const size_t N   = A->size(1);
  assert(nnz >= M);

  assert(symbolic);
  numeric.reset();

  // Perform LU factorisation
  numeric = umfpack_factorize_numeric(Ap, Ai, Ax, symbolic.get());
}
//-----------------------------------------------------------------------------
size_t UmfpackLUSolver::solve_factorized(DenseVector& x, const DenseVector& b) const
{
  if (!A)
    cell_error("No matrix associated with UmfpackLUSolver. Cannot perform factorized_solve.");

  assert(A->size(0) == A->size(0));
  assert(A->size(0) == b.size());

  // Resize x if required
  if (A->size(1) != x.size())
    x.resize(A->size(1));

  if (!symbolic)
    cell_error("No symbolic factorisation. Please call UmfpackLUSolver::factorize_symbolic().");

  if (!numeric)
    cell_error("No LU factorisation. Please call UmfpackLUSolver::factorize_numeric().");

  // Get matrix data
  boost::tuple<const std::size_t*, const std::size_t*, const double*, int> data = A->data();
  const std::size_t* Ap  = boost::get<0>(data);
  const std::size_t* Ai  = boost::get<1>(data);
  const double*      Ax  = boost::get<2>(data);

  // Solve for tranpose since we use compressed rows and UMFPACK expected compressed columns
  umfpack_solve(Ap, Ai, Ax, x.data(), b.data(), numeric.get(),A);

  return 1;
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::readParameters()
{
  parameters_read = true;
}
//-----------------------------------------------------------------------------
#ifdef HAS_UMFPACK
//-----------------------------------------------------------------------------
boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_symbolic(size_t M, size_t N, const std::size_t* Ap, const std::size_t* Ai, const double* Ax)
{
  assert(Ap);
  assert(Ai);
  assert(Ax);

  void* symbolic = 0;
  boost::scoped_ptr<double> dnull;

  // Symbolic factorisation step (reordering, etc)
  if (sizeof(std::size_t) == sizeof(int))
  {
    const int* _Ap = reinterpret_cast<const int*>(Ap);
    const int* _Ai = reinterpret_cast<const int*>(Ai);
    long int status = umfpack_di_symbolic(M, N, _Ap, _Ai, Ax, &symbolic, dnull.get(), dnull.get());
    umfpack_check_status(status, "symbolic");
    return boost::shared_ptr<void>(symbolic, UmfpackIntSymbolicDeleter());
  }
  else if (sizeof(std::size_t) == sizeof(UF_long))
  {
    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
    long int status = umfpack_dl_symbolic(M, N, _Ap, _Ai, Ax, &symbolic, dnull.get(), dnull.get());
    umfpack_check_status(status, "symbolic");
    return boost::shared_ptr<void>(symbolic, UmfpackLongIntSymbolicDeleter());
  }
  else
    cell_error("Could not determine correct types for casting integers to pass to UMFPACK.");

  return boost::shared_ptr<void>();
}
//-----------------------------------------------------------------------------
boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
                                                 const std::size_t* Ai,
                                                 const double* Ax,
                                                 void* symbolic)
{
  assert(Ap);
  assert(Ai);
  assert(Ax);
  assert(symbolic);

  void* numeric = 0;
  boost::scoped_ptr<double> dnull;

  // Factorization step
  long int status = 0;
  if (sizeof(std::size_t) == sizeof(int))
  {
    const int* _Ap = reinterpret_cast<const int*>(Ap);
    const int* _Ai = reinterpret_cast<const int*>(Ai);
    status = umfpack_di_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(), dnull.get());
    umfpack_check_status(status, "numeric");
    return boost::shared_ptr<void>(numeric, UmfpackIntNumericDeleter());
  }
  else if (sizeof(std::size_t) == sizeof(UF_long))
  {
    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
    status = umfpack_dl_numeric(_Ap, _Ai, Ax, symbolic, &numeric, dnull.get(), dnull.get());
    umfpack_check_status(status, "numeric");
    return boost::shared_ptr<void>(numeric, UmfpackLongIntNumericDeleter());
  }
  else
    cell_error("Could not determine correct types for casting integers to pass to UMFPACK.");

  return boost::shared_ptr<void>();
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
                                    const std::size_t* Ai,
                                    const double* Ax, double* x,
                                    const double* b, void* numeric,
                                    boost::shared_ptr<const SparseMatrix> a)
{
  assert(Ap);
  assert(Ai);
  assert(Ax);
  assert(x);
  assert(b);
  assert(numeric);

  boost::scoped_ptr<double> dnull;

  // Solve system. We assume CSR storage, but UMFPACK expects CSC, so solve
  // for the transpose
  long int status = 0;
  if (sizeof(std::size_t) == sizeof(int))
  {
    const int* _Ap = reinterpret_cast<const int*>(Ap);
    const int* _Ai = reinterpret_cast<const int*>(Ai);
    status = umfpack_di_solve(UMFPACK_At, _Ap, _Ai, Ax, x, b, numeric, dnull.get(), dnull.get());
  }
  else if (sizeof(std::size_t) == sizeof(UF_long))
  {
    const UF_long* _Ap = reinterpret_cast<const UF_long*>(Ap);
    const UF_long* _Ai = reinterpret_cast<const UF_long*>(Ai);
    
    status = umfpack_dl_solve(UMFPACK_At, _Ap, _Ai, Ax, x, b, numeric, dnull.get(), dnull.get());
  }
  else
    cell_error("Could not determine correct types for casting integers to pass to UMFPACK.");

  umfpack_check_status(status, "solve");
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::umfpack_check_status(long int status,
                                            std::string function)
{
  if(status == UMFPACK_OK)
    return;

  // Printing which UMFPACK function is returning an warning/error
  std::cout << "UMFPACK problem related to call to " << function << std::endl;

  if(status == UMFPACK_WARNING_singular_matrix)
    cell_warning("UMFPACK reports that the matrix being solved is singular.");
  else if(status == UMFPACK_ERROR_out_of_memory)
    cell_error("UMFPACK has run out of memory solving a system.");
  else if(status == UMFPACK_ERROR_invalid_system)
    cell_error("UMFPACK reports an invalid system. Is the matrix square?.");
  else if(status == UMFPACK_ERROR_invalid_Numeric_object)
    cell_error("UMFPACK reports an invalid Numeric object.");
  else if(status == UMFPACK_ERROR_invalid_Symbolic_object)
    cell_error("UMFPACK reports an invalid Symbolic object.");
  else if(status != UMFPACK_OK)
    cell_warning("UMFPACK is reporting an unknown error.");
}
//-----------------------------------------------------------------------------
#else
//-----------------------------------------------------------------------------
boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_symbolic(size_t M, size_t N,
                                                  const std::size_t* Ap,
                                                  const std::size_t* Ai,
                                                  const double* Ax)
{
  cell_error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
  return boost::shared_ptr<void>();
}
//-----------------------------------------------------------------------------
boost::shared_ptr<void> UmfpackLUSolver::umfpack_factorize_numeric(const std::size_t* Ap,
                                                 const std::size_t* Ai,
                                                 const double* Ax,
                                                 void* symbolic)
{
  cell_error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
  return boost::shared_ptr<void>();
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::umfpack_solve(const std::size_t* Ap,
                                    const std::size_t* Ai,
                                    const double* Ax, double* x,
                                    const double* b, void* numeric)
{
  cell_error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
}
//-----------------------------------------------------------------------------
void UmfpackLUSolver::umfpack_check_status(long int status,
                                            std::string function)
{
  cell_error("Umfpack not installed. Cannot perform LU solver using Umfpack.");
}
//-----------------------------------------------------------------------------
#endif
