// Copyright (C) 2006-2010 Garth N. Wells
//
// Modified by Anders Logg 2006.
// Modified by Dag Lindbo 2008.

#ifndef _UMFPACK_LU_SOLVER_H_
#define _UMFPACK_LU_SOLVER_H_

#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include "ublas.h"
#include "SparseMatrix.h"
#include "DenseVector.h"
#include "parameter/Parametrized.h"

/// This class implements the direct solution (LU factorization) of
/// linear systems of the form Ax = b using UMFPACK
/// (http://www.cise.ufl.edu/research/sparse/umfpack/)

class UmfpackLUSolver : public Parametrized
{

public:

    /// Constructor
    UmfpackLUSolver();

    /// Constructor
    UmfpackLUSolver ( boost::shared_ptr<const SparseMatrix> A );

    /// Destructor
    ~UmfpackLUSolver();

    /// Set operator (matrix)
    void set_operator ( const boost::shared_ptr<const SparseMatrix> A );

    /// Return the operator (matrix)
    const SparseMatrix& get_operator() const;

    /// Solve linear system Ax = b for a sparse matrix using UMFPACK if installed
    size_t solve ( DenseVector& x, const DenseVector& b, bool erase = true );

    /// Solve linear system
    size_t solve ( const SparseMatrix& A, DenseVector& x, const DenseVector& b, bool erase = true );

    /// Default parameter values
    void readParameters();
//     static Parameters default_parameters();

private:

    // Perform symbolic factorisation
    void symbolic_factorize();

    /// LU factorisation
    void numeric_factorize();

    /// Solve factorized system (UMFPACK).
    size_t solve_factorized ( DenseVector& x, const DenseVector& b ) const;

    // Return pointer to symbolic factorisation
    static boost::shared_ptr<void> umfpack_factorize_symbolic ( size_t M, size_t N, const std::size_t* Ap, const std::size_t* Ai, const double* Ax );

    // Return pointer to the numerical factorisation
    static boost::shared_ptr<void> umfpack_factorize_numeric ( const std::size_t* Ap, const std::size_t* Ai, const double* Ax, void* symbolic );

    static void umfpack_solve ( const std::size_t* Ap, const std::size_t* Ai, const double* Ax, double* x, const double* b, void* numeric, boost::shared_ptr<const SparseMatrix> a );

    /// Check status flag returned by an UMFPACK function
    static void umfpack_check_status ( long int status, std::string function );

    // UMFPACK data (note that boost::scoped_ptr cannot hold a void pointer)
    boost::shared_ptr<void> symbolic;
    boost::shared_ptr<void> numeric;

    // Operator (the matrix)
    boost::shared_ptr<const SparseMatrix> A;

    /// True if we have read parameters
    bool parameters_read;

};

#endif
