// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "matrix/rot_mat.h"
//-----------------------------------------------------------------------------
rot_mat::rot_mat(Vector3 const& e1, Vector3 const& e2, Vector3 const& e3, Vector3 const& e1g, Vector3 const& e2g, Vector3 const& e3g)
{
    d[0][0] = e1*e1g;
    d[0][1] = e2*e1g;
    d[0][2] = e3*e1g;
    d[1][0] = e1*e2g;
    d[1][1] = e2*e2g;
    d[1][2] = e3*e2g;
    d[2][0] = e1*e3g;
    d[2][1] = e2*e3g;
    d[2][2] = e3*e3g;
}
//-----------------------------------------------------------------------------
void rot_mat::assign(Vector3 const& e1, Vector3 const& e2, Vector3 const& e3)
{
    d[0][0] = e1.x;
    d[0][1] = e2.x;
    d[0][2] = e3.x;
    d[1][0] = e1.y;
    d[1][1] = e2.y;
    d[1][2] = e3.y;
    d[2][0] = e1.z;
    d[2][1] = e2.z;
    d[2][2] = e3.z;
}
//-----------------------------------------------------------------------------
Vector3 rot_mat::base_vec(int i) const
{
    return Vector3(d[0][i], d[1][i], d[2][i]);
}
//-----------------------------------------------------------------------------
Vector3 rot_mat::rotate(Vector3 const& v) const
{
    //calculate d*v
    return operator*(v);
}
//-----------------------------------------------------------------------------
Vector3 rot_mat::inv_rotate(Vector3 const& v) const
{
    //transpose(d)*v
    return Vector3(d[0][0]*v.x + d[1][0]*v.y + d[2][0]*v.z,
                 d[0][1]*v.x + d[1][1]*v.y + d[2][1]*v.z,
                 d[0][2]*v.x + d[1][2]*v.y + d[2][2]*v.z);
}
//-----------------------------------------------------------------------------
Matrix3 rot_mat::rotate(Matrix3 const& m) const
{
    //calculate m*transpose(d)
    double mdT00 = m(0,0)*d[0][0]+m(0,1)*d[0][1]+m(0,2)*d[0][2];
    double mdT01 = m(0,0)*d[1][0]+m(0,1)*d[1][1]+m(0,2)*d[1][2];
    double mdT02 = m(0,0)*d[2][0]+m(0,1)*d[2][1]+m(0,2)*d[2][2];

    double mdT10 = m(1,0)*d[0][0]+m(1,1)*d[0][1]+m(1,2)*d[0][2];
    double mdT11 = m(1,0)*d[1][0]+m(1,1)*d[1][1]+m(1,2)*d[1][2];
    double mdT12 = m(1,0)*d[2][0]+m(1,1)*d[2][1]+m(1,2)*d[2][2];

    double mdT20 = m(2,0)*d[0][0]+m(2,1)*d[0][1]+m(2,2)*d[0][2];
    double mdT21 = m(2,0)*d[1][0]+m(2,1)*d[1][1]+m(2,2)*d[1][2];
    double mdT22 = m(2,0)*d[2][0]+m(2,1)*d[2][1]+m(2,2)*d[2][2];

    //calculate d*m*transpose(d)
    return Matrix3(d[0][0]*mdT00+d[0][1]*mdT10+d[0][2]*mdT20,
                 d[0][0]*mdT01+d[0][1]*mdT11+d[0][2]*mdT21,
                 d[0][0]*mdT02+d[0][1]*mdT12+d[0][2]*mdT22,

                 d[1][0]*mdT00+d[1][1]*mdT10+d[1][2]*mdT20,
                 d[1][0]*mdT01+d[1][1]*mdT11+d[1][2]*mdT21,
                 d[1][0]*mdT02+d[1][1]*mdT12+d[1][2]*mdT22,

                 d[2][0]*mdT00+d[2][1]*mdT10+d[2][2]*mdT20,
                 d[2][0]*mdT01+d[2][1]*mdT11+d[2][2]*mdT21,
                 d[2][0]*mdT02+d[2][1]*mdT12+d[2][2]*mdT22);
}
//-----------------------------------------------------------------------------
Matrix3 rot_mat::inv_rotate(Matrix3 const& m) const
{
    //calculate m*d
    double md00 = m(0,0)*d[0][0]+m(0,1)*d[1][0]+m(0,2)*d[2][0];
    double md01 = m(0,0)*d[0][1]+m(0,1)*d[1][1]+m(0,2)*d[2][1];
    double md02 = m(0,0)*d[0][2]+m(0,1)*d[1][2]+m(0,2)*d[2][2];

    double md10 = m(1,0)*d[0][0]+m(1,1)*d[1][0]+m(1,2)*d[2][0];
    double md11 = m(1,0)*d[0][1]+m(1,1)*d[1][1]+m(1,2)*d[2][1];
    double md12 = m(1,0)*d[0][2]+m(1,1)*d[1][2]+m(1,2)*d[2][2];

    double md20 = m(2,0)*d[0][0]+m(2,1)*d[1][0]+m(2,2)*d[2][0];
    double md21 = m(2,0)*d[0][1]+m(2,1)*d[1][1]+m(2,2)*d[2][1];
    double md22 = m(2,0)*d[0][2]+m(2,1)*d[1][2]+m(2,2)*d[2][2];

    //calculate transpose(d)*m*d
    return Matrix3(d[0][0]*md00+d[1][0]*md10+d[2][0]*md20,
                 d[0][0]*md01+d[1][0]*md11+d[2][0]*md21,
                 d[0][0]*md02+d[1][0]*md12+d[2][0]*md22,

                 d[0][1]*md00+d[1][1]*md10+d[2][1]*md20,
                 d[0][1]*md01+d[1][1]*md11+d[2][1]*md21,
                 d[0][1]*md02+d[1][1]*md12+d[2][1]*md22,

                 d[0][2]*md00+d[1][2]*md10+d[2][2]*md20,
                 d[0][2]*md01+d[1][2]*md11+d[2][2]*md21,
                 d[0][2]*md02+d[1][2]*md12+d[2][2]*md22);
}
//-----------------------------------------------------------------------------
Matrix3s rot_mat::rotate(Matrix3s const& m) const
{
//calculate m*transpose(d)
    double mdT00 = m.xx()*d[0][0]+m.xy()*d[0][1]+m.xz()*d[0][2];
    double mdT01 = m.xx()*d[1][0]+m.xy()*d[1][1]+m.xz()*d[1][2];
    double mdT02 = m.xx()*d[2][0]+m.xy()*d[2][1]+m.xz()*d[2][2];

    double mdT10 = m.xy()*d[0][0]+m.yy()*d[0][1]+m.yz()*d[0][2];
    double mdT11 = m.xy()*d[1][0]+m.yy()*d[1][1]+m.yz()*d[1][2];
    double mdT12 = m.xy()*d[2][0]+m.yy()*d[2][1]+m.yz()*d[2][2];

    double mdT20 = m.xz()*d[0][0]+m.yz()*d[0][1]+m.zz()*d[0][2];
    double mdT21 = m.xz()*d[1][0]+m.yz()*d[1][1]+m.zz()*d[1][2];
    double mdT22 = m.xz()*d[2][0]+m.yz()*d[2][1]+m.zz()*d[2][2];

    //calculate d*m*transpose(d)
    return Matrix3s(d[0][0]*mdT00+d[0][1]*mdT10+d[0][2]*mdT20, //xx component
                  d[1][0]*mdT01+d[1][1]*mdT11+d[1][2]*mdT21, //yy component
                  d[2][0]*mdT02+d[2][1]*mdT12+d[2][2]*mdT22, //zz component
                  d[0][0]*mdT01+d[0][1]*mdT11+d[0][2]*mdT21, //xy component
                  d[1][0]*mdT02+d[1][1]*mdT12+d[1][2]*mdT22, //yz component
                  d[0][0]*mdT02+d[0][1]*mdT12+d[0][2]*mdT22); //xz component
}
//-----------------------------------------------------------------------------
Matrix3s rot_mat::inv_rotate(Matrix3s const& m) const
{
    //calculate m*d
    double md00 = m.xx()*d[0][0]+m.xy()*d[1][0]+m.xz()*d[2][0];
    double md01 = m.xx()*d[0][1]+m.xy()*d[1][1]+m.xz()*d[2][1];
    double md02 = m.xx()*d[0][2]+m.xy()*d[1][2]+m.xz()*d[2][2];

    double md10 = m.xy()*d[0][0]+m.yy()*d[1][0]+m.yz()*d[2][0];
    double md11 = m.xy()*d[0][1]+m.yy()*d[1][1]+m.yz()*d[2][1];
    double md12 = m.xy()*d[0][2]+m.yy()*d[1][2]+m.yz()*d[2][2];

    double md20 = m.xz()*d[0][0]+m.yz()*d[1][0]+m.zz()*d[2][0];
    double md21 = m.xz()*d[0][1]+m.yz()*d[1][1]+m.zz()*d[2][1];
    double md22 = m.xz()*d[0][2]+m.yz()*d[1][2]+m.zz()*d[2][2];

    //calculate transpose(d)*m*d
    return Matrix3s(d[0][0]*md00+d[1][0]*md10+d[2][0]*md20, //xx component
                  d[0][1]*md01+d[1][1]*md11+d[2][1]*md21, //yy component
                  d[0][2]*md02+d[1][2]*md12+d[2][2]*md22, //zz component
                  d[0][0]*md01+d[1][0]*md11+d[2][0]*md21, //xy component
                  d[0][1]*md02+d[1][1]*md12+d[2][1]*md22, //yz component
                  d[0][0]*md02+d[1][0]*md12+d[2][0]*md22); //xz component
}
//-----------------------------------------------------------------------------
DenseVector rot_mat::rotate(DenseVector const& v) const
{
    const double a = 2.0;
    DenseMatrix A(6,6);
    //first column
    A.set(0, 0, d[0][0]*d[0][0]);
    A.set(1, 0, d[1][0]*d[1][0]);
    A.set(2, 0, d[2][0]*d[2][0]);
    A.set(3, 0, d[0][0]*d[1][0]);
    A.set(4, 0, d[1][0]*d[2][0]);
    A.set(5, 0, d[0][0]*d[2][0]);
    //second column
    A.set(0, 1, d[0][1]*d[0][1]);
    A.set(1, 1, d[1][1]*d[1][1]);
    A.set(2, 1, d[2][1]*d[2][1]);
    A.set(3, 1, d[0][1]*d[1][1]);
    A.set(4, 1, d[1][1]*d[2][1]);
    A.set(5, 1, d[0][1]*d[2][1]);
    //third column
    A.set(0, 2, d[0][2]*d[0][2]);
    A.set(1, 2, d[1][2]*d[1][2]);
    A.set(2, 2, d[2][2]*d[2][2]);
    A.set(3, 2, d[0][2]*d[1][2]);
    A.set(4, 2, d[1][2]*d[2][2]);
    A.set(5, 2, d[0][2]*d[2][2]);
    //forth column
    A.set(0, 3, 2.0*d[0][0]*d[0][1]);
    A.set(1, 3, 2.0*d[1][0]*d[1][1]);
    A.set(2, 3, 2.0*d[2][0]*d[2][1]);
    A.set(3, 3, d[0][0]*d[1][1] + d[0][1]*d[1][0]);
    A.set(4, 3, d[1][0]*d[2][1] + d[1][1]*d[2][0]);
    A.set(5, 3, d[0][0]*d[2][1] + d[0][1]*d[2][0]);
    //fifth column
    A.set(0, 4, 2.0*d[0][1]*d[0][2]);
    A.set(1, 4, 2.0*d[1][1]*d[1][2]);
    A.set(2, 4, 2.0*d[2][1]*d[2][2]);
    A.set(3, 4, d[0][1]*d[1][2] + d[0][2]*d[1][1]);
    A.set(4, 4, d[1][1]*d[2][2] + d[1][2]*d[2][1]);
    A.set(5, 4, d[0][1]*d[2][2] + d[0][2]*d[2][1]);
    //sixth column
    A.set(0, 5, 2.0*d[0][0]*d[0][2]);
    A.set(1, 5, 2.0*d[1][0]*d[1][2]);
    A.set(2, 5, 2.0*d[2][0]*d[2][2]);
    A.set(3, 5, d[0][0]*d[1][2] + d[0][2]*d[1][0]);
    A.set(4, 5, d[1][0]*d[2][2] + d[1][2]*d[2][0]);
    A.set(5, 5, d[0][0]*d[2][2] + d[0][2]*d[2][0]);

    DenseVector res(6);
    ublas::axpy_prod(A.mat(), v.vec(), res.vec(), true);//res = A*mv
    return res;
}
//-----------------------------------------------------------------------------
DenseVector rot_mat::inv_rotate(DenseVector const& mv) const
{
    DenseMatrix A(6,6);
    //first column
    A.set(0, 0, d[0][0]*d[0][0]);
    A.set(1, 0, d[0][1]*d[0][1]);
    A.set(2, 0, d[0][2]*d[0][2]);
    A.set(3, 0, d[0][0]*d[0][1]);
    A.set(4, 0, d[0][1]*d[0][2]);
    A.set(5, 0, d[0][0]*d[0][2]);
    //second column
    A.set(0, 1, d[1][0]*d[1][0]);
    A.set(1, 1, d[1][1]*d[1][1]);
    A.set(2, 1, d[1][2]*d[1][2]);
    A.set(3, 1, d[1][0]*d[1][1]);
    A.set(4, 1, d[1][1]*d[1][2]);
    A.set(5, 1, d[1][0]*d[1][2]);
    //third column
    A.set(0, 2, d[2][0]*d[2][0]);
    A.set(1, 2, d[2][1]*d[2][1]);
    A.set(2, 2, d[2][2]*d[2][2]);
    A.set(3, 2, d[2][0]*d[2][1]);
    A.set(4, 2, d[2][1]*d[2][2]);
    A.set(5, 2, d[2][0]*d[2][2]);
    //forth column
    A.set(0, 3, 2.0*d[0][0]*d[1][0]);
    A.set(1, 3, 2.0*d[0][1]*d[1][1]);
    A.set(2, 3, 2.0*d[0][2]*d[1][2]);
    A.set(3, 3, d[0][0]*d[1][1] + d[1][0]*d[0][1]);
    A.set(4, 3, d[0][1]*d[1][2] + d[1][1]*d[0][2]);
    A.set(5, 3, d[0][0]*d[1][2] + d[1][0]*d[0][2]);
    //fifth column
    A.set(0, 4, 2.0*d[1][0]*d[2][0]);
    A.set(1, 4, 2.0*d[1][1]*d[2][1]);
    A.set(2, 4, 2.0*d[1][2]*d[2][2]);
    A.set(3, 4, d[1][0]*d[2][1] + d[2][0]*d[1][1]);
    A.set(4, 4, d[1][1]*d[2][2] + d[2][1]*d[1][2]);
    A.set(5, 4, d[1][0]*d[2][2] + d[2][0]*d[1][2]);
    //sixth column
    A.set(0, 5, 2.0*d[0][0]*d[2][0]);
    A.set(1, 5, 2.0*d[0][1]*d[2][1]);
    A.set(2, 5, 2.0*d[0][2]*d[2][2]);
    A.set(3, 5, d[0][0]*d[2][1] + d[2][0]*d[0][1]);
    A.set(4, 5, d[0][1]*d[2][2] + d[2][1]*d[0][2]);
    A.set(5, 5, d[0][0]*d[2][2] + d[2][0]*d[0][2]);

    DenseVector res(6);
    ublas::axpy_prod(A.mat(), mv.vec(), res.vec(), true);//res = A*mv
    return res;
}
//-----------------------------------------------------------------------------
DenseMatrix rot_mat::rotate(DenseMatrix const& m) const
{
    DenseMatrix A(6,6);
    //first column
    A.set(0, 0, d[0][0]*d[0][0]);
    A.set(1, 0, d[1][0]*d[1][0]);
    A.set(2, 0, d[2][0]*d[2][0]);
    A.set(3, 0, d[0][0]*d[1][0]);
    A.set(4, 0, d[1][0]*d[2][0]);
    A.set(5, 0, d[0][0]*d[2][0]);
    //second column
    A.set(0, 1, d[0][1]*d[0][1]);
    A.set(1, 1, d[1][1]*d[1][1]);
    A.set(2, 1, d[2][1]*d[2][1]);
    A.set(3, 1, d[0][1]*d[1][1]);
    A.set(4, 1, d[1][1]*d[2][1]);
    A.set(5, 1, d[0][1]*d[2][1]);
    //third column
    A.set(0, 2, d[0][2]*d[0][2]);
    A.set(1, 2, d[1][2]*d[1][2]);
    A.set(2, 2, d[2][2]*d[2][2]);
    A.set(3, 2, d[0][2]*d[1][2]);
    A.set(4, 2, d[1][2]*d[2][2]);
    A.set(5, 2, d[0][2]*d[2][2]);
    //forth column
    A.set(0, 3, d[0][0]*d[0][1]);
    A.set(1, 3, d[1][0]*d[1][1]);
    A.set(2, 3, d[2][0]*d[2][1]);
    A.set(3, 3, d[0][0]*d[1][1] + d[0][1]*d[1][0]);
    A.set(4, 3, d[1][0]*d[2][1] + d[1][1]*d[2][0]);
    A.set(5, 3, d[0][0]*d[2][1] + d[0][1]*d[2][0]);
    //fifth column
    A.set(0, 4, d[0][1]*d[0][2]);
    A.set(1, 4, d[1][1]*d[1][2]);
    A.set(2, 4, d[2][1]*d[2][2]);
    A.set(3, 4, d[0][1]*d[1][2] + d[0][2]*d[1][1]);
    A.set(4, 4, d[1][1]*d[2][2] + d[1][2]*d[2][1]);
    A.set(5, 4, d[0][1]*d[2][2] + d[0][2]*d[2][1]);
    //sixth column
    A.set(0, 5, d[0][0]*d[0][2]);
    A.set(1, 5, d[1][0]*d[1][2]);
    A.set(2, 5, d[2][0]*d[2][2]);
    A.set(3, 5, d[0][0]*d[1][2] + d[0][2]*d[1][0]);
    A.set(4, 5, d[1][0]*d[2][2] + d[1][2]*d[2][0]);
    A.set(5, 5, d[0][0]*d[2][2] + d[0][2]*d[2][0]);

    //coonstruct inverse tensor rotation matrix
    DenseMatrix temp( trans(A.mat()) );
    quadrant_multiply(temp, 2.0, 0, 3);
    //coonstruct forward tensor rotation matrix
    quadrant_multiply(A, 2.0, 0, 3);

    DenseMatrix temp2(6,6);
    ublas::axpy_prod(m.mat(), temp.mat(), temp2.mat(), true);
    DenseMatrix res(6,6);

    ublas::axpy_prod(A.mat(), temp2.mat(), res.mat(), true);//res = R*m*R^-1
    return res;
}
//-----------------------------------------------------------------------------
DenseMatrix rot_mat::inv_rotate(DenseMatrix const& m) const
{
    DenseMatrix A(6,6);
    //first column
    A.set(0, 0, d[0][0]*d[0][0]);
    A.set(1, 0, d[1][0]*d[1][0]);
    A.set(2, 0, d[2][0]*d[2][0]);
    A.set(3, 0, d[0][0]*d[1][0]);
    A.set(4, 0, d[1][0]*d[2][0]);
    A.set(5, 0, d[0][0]*d[2][0]);
    //second column
    A.set(0, 1, d[0][1]*d[0][1]);
    A.set(1, 1, d[1][1]*d[1][1]);
    A.set(2, 1, d[2][1]*d[2][1]);
    A.set(3, 1, d[0][1]*d[1][1]);
    A.set(4, 1, d[1][1]*d[2][1]);
    A.set(5, 1, d[0][1]*d[2][1]);
    //third column
    A.set(0, 2, d[0][2]*d[0][2]);
    A.set(1, 2, d[1][2]*d[1][2]);
    A.set(2, 2, d[2][2]*d[2][2]);
    A.set(3, 2, d[0][2]*d[1][2]);
    A.set(4, 2, d[1][2]*d[2][2]);
    A.set(5, 2, d[0][2]*d[2][2]);
    //forth column
    A.set(0, 3, d[0][0]*d[0][1]);
    A.set(1, 3, d[1][0]*d[1][1]);
    A.set(2, 3, d[2][0]*d[2][1]);
    A.set(3, 3, d[0][0]*d[1][1] + d[0][1]*d[1][0]);
    A.set(4, 3, d[1][0]*d[2][1] + d[1][1]*d[2][0]);
    A.set(5, 3, d[0][0]*d[2][1] + d[0][1]*d[2][0]);
    //fifth column
    A.set(0, 4, d[0][1]*d[0][2]);
    A.set(1, 4, d[1][1]*d[1][2]);
    A.set(2, 4, d[2][1]*d[2][2]);
    A.set(3, 4, d[0][1]*d[1][2] + d[0][2]*d[1][1]);
    A.set(4, 4, d[1][1]*d[2][2] + d[1][2]*d[2][1]);
    A.set(5, 4, d[0][1]*d[2][2] + d[0][2]*d[2][1]);
    //sixth column
    A.set(0, 5, d[0][0]*d[0][2]);
    A.set(1, 5, d[1][0]*d[1][2]);
    A.set(2, 5, d[2][0]*d[2][2]);
    A.set(3, 5, d[0][0]*d[1][2] + d[0][2]*d[1][0]);
    A.set(4, 5, d[1][0]*d[2][2] + d[1][2]*d[2][0]);
    A.set(5, 5, d[0][0]*d[2][2] + d[0][2]*d[2][0]);

    //coonstruct inverse tensor rotation matrix
    DenseMatrix temp( trans(A.mat()) );
    quadrant_multiply(temp, 2.0, 0, 3);
    //coonstruct forward tensor rotation matrix
    quadrant_multiply(A, 2.0, 0, 3);

    DenseMatrix temp2(6,6);
    ublas::axpy_prod(m.mat(), A.mat(), temp2.mat(), true);
    DenseMatrix res(6,6);

    ublas::axpy_prod(temp.mat(), temp2.mat(), res.mat(), true);//res = R^-1*m*R
    return res;
}
//-----------------------------------------------------------------------------
void rot_mat::quadrant_multiply(DenseMatrix &m, double d, int i, int j) const
{
    for (int a = i; a < i+3; ++a)
    {
        for (int b = j; b < j+3; ++b)
            m(a,b) *= d;
    }
}
//-----------------------------------------------------------------------------
