// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _ROTATION_MATRIX_H_
#define _ROTATION_MATRIX_H_

#include "matrix/Matrix3.h"
#include "utils/types.h"
#include "matrix/DenseMatrix.h"
#include "matrix/DenseVector.h"
//-----------------------------------------------------------------------------
class rot_mat : public Matrix3
{
public:
    rot_mat() {}
    rot_mat(Matrix3 const& m) : Matrix3(m) {}
    ///rotation matrix from the coordinate frame defined by the vectors  e1, e2, e3 to the global coordinate system
    rot_mat(Vector3 const& e1, Vector3 const& e2, Vector3 const& e3) : Matrix3(e1.x, e2.x,e3.x,e1.y,e2.y,e3.y,e1.z,e2.z,e3.z) {}
    ///rotation from coordinate system defined by first three base vectors to the coordinate system defined by next three base vectors
    rot_mat(Vector3 const& e1, Vector3 const& e2, Vector3 const& e3, Vector3 const& e1g, Vector3 const& e2g, Vector3 const& e3g);

    void assign(Vector3 const& e1, Vector3 const& e2, Vector3 const& e3);
    ///give i-th base vector of the original frame
    Vector3 base_vec(int i) const;
    ///get element of transposed/inverse rotation
    double transposed(int i, int j) const {
        return operator ()(j,i);
    }
    ///rotate the vector from local to global coordinates
    ///v'(m) = R(mi)v(i)
    Vector3 rotate(Vector3 const& v) const;
    ///rotate the vector from global to local coordinates
    ///v'(m) = R^T(mi) v(i)
    Vector3 inv_rotate(Vector3 const& v) const;
    ///rotate the second order tensor from local to global coordinates
    ///A'(mn) = R(mi) A(ij) R^T(jn)
    Matrix3 rotate(Matrix3 const& m) const;
    ///rotate the second order tensor from global to local coordinates
    ///A'(mn) = R^T(mi) A(ij) R(jn)
    Matrix3 inv_rotate(Matrix3 const& m) const;
    ///rotate the symmetric second order tensor from local to global coordinates
    Matrix3s rotate(Matrix3s const& m) const;
    ///rotate the symmetric second order tensor from global to local coordinates
    Matrix3s inv_rotate(Matrix3s const& m) const;
    ///rotate the symmetric second order tensor in Voigt notation from local to global coordinates
    DenseVector rotate(DenseVector const& mv) const;
    ///rotate the symmetric second order tensor in Voigt notation from global to local coordinates
    DenseVector inv_rotate(DenseVector const& mv) const;
    ///rotate the fourth order tensor in Voigt notation from local to global coordinates
    ///A'(mnop) = R^T(mi)R^T(nj) A(ijkl) R(ko)R(lp)
    DenseMatrix rotate(DenseMatrix const& m) const;
    ///rotate the fourth order tensor in Voigt notation from global to local coordinates
    ///A'(mnop) = R(mi)R(nj) A(ijkl) R^T(ko)R^T(lp)
    DenseMatrix inv_rotate(DenseMatrix const& m) const;
private:
    /// multiplies 3x3 part of 6x6 matrix by a number, i,j upper left index to start with
    void quadrant_multiply(DenseMatrix &m, double d, int i, int j) const;
};
//-----------------------------------------------------------------------------
#endif
