// Copyright (C) 2006-2009 Garth N. Wells
// Modified by Anders Logg, 2006-2009.
// Modified by Ola Skavhaug, 2007-2008.
// Modified by Kent-Andre Mardal, 2008.
// Modified by Martin Sandve Alnes, 2008.
// Modified by Dag Lindbo, 2008
// Modified by Pawel Krupinski
#ifndef _UBLAS_MATRIX_H_
#define _UBLAS_MATRIX_H_

#include <sstream>
#include <iomanip>
#include <vector>
#include "matrix/ublas.h"
#include "matrix/uBlasVector.h"
#include "utils/error_macros.h"
#include "boost/tuple/tuple.hpp"
#include "utils/utility.h"
/// This class provides a simple matrix class based on uBLAS.
/// It is a simple wrapper for a uBLAS matrix implementing the
/// uBlasMatrix interface.
///
/// The interface is intentionally simple. For advanced usage,
/// access the underlying uBLAS matrix and use the standard
/// uBLAS interface which is documented at
/// http://www.boost.org/libs/numeric/ublas/doc/index.htm.
///
/// Developer note: specialised member functions must be
/// inlined to avoid link errors.

template<typename Mat>
class uBlasMatrix
{
public:

    typedef Mat internal_type;
    /// Create empty matrix
    uBlasMatrix();

    /// Create M x N matrix
    uBlasMatrix ( size_t M, size_t N );

    /// Copy constructor
    uBlasMatrix ( const uBlasMatrix& A );

    /// Create matrix from given uBLAS matrix expression
    template <typename E>
    explicit uBlasMatrix ( const ublas::matrix_expression<E>& A ) : A ( A ) {}

    /// Destructor
    ~uBlasMatrix();

    void init ( size_t M, size_t N, size_t nnz );

    /// Return copy of tensor
    uBlasMatrix<Mat>* copy() const;

    /// Return size of given dimension
    size_t size ( size_t dim ) const;

    /// Return local ownership range
    std::pair<size_t, size_t> local_range ( size_t dim ) const
    {
        return std::make_pair ( 0, size ( dim ) );
    }

    /// Set all entries to zero and keep any sparse structure
    void zero();

    /// Finalize assembly of tensor
    void apply();

    /// Finalize assembly of tensor
    void unapply();

    /// Return informal string representation (pretty-print)
    std::string str ( bool verbose ) const;

    //--- Implementation of the uBlasMatrix interface ---
    /// Resize vector y such that is it compatible with matrix for
    /// multuplication Ax = b (dim = 0 -> b, dim = 1 -> x) In parallel
    /// case, size and layout are important.
    void resize ( uBlasVector& y, size_t dim ) const;

    /// Get block of values
    void get ( double* block, size_t m, const size_t* rows, size_t n, const size_t* cols ) const;

    /// Set block of values
    void set ( const double* block, size_t m, const size_t* rows, size_t n, const size_t* cols );

    /// Add block of values
    void add ( const double* block, size_t m, const size_t* rows, size_t n, const size_t* cols );

    /// Get a value (const)
    double get ( size_t rows, size_t cols ) const;

    /// Set a value
    void set ( size_t rows, size_t cols, double val );

    /// Add multiple of given matrix (AXPY operation)
    void axpy ( double a, const uBlasMatrix& A, bool same_nonzero_pattern );

    /// Return norm of matrix
    double norm ( std::string norm_type ) const;

    /// Get non-zero values of given row
    void getrow ( size_t row, std::vector<size_t>& columns, std::vector<double>& values ) const;

    /// Set values for given row
    void setrow ( size_t row_idx, const std::vector<size_t>& columns, const std::vector<double>& values );

    /// Set given rows to zero
    void zero ( size_t m, const size_t* rows );

    /// Set all entries less than eps to zero
    void eliminate ( double eps );

    /// Set given rows to identity matrix
    void ident ( size_t m, const size_t* rows );

    /// Set given rows to identity matrix. Use assembly matrix to make sure that zeros on the diagonal (not present in sparse representation are set to one as well)
    void ident_insert ( size_t m, const size_t* rows );

    /// Matrix-vector product, y = Ax
    void mult ( const uBlasVector& x, uBlasVector& y, bool init = true ) const;

    /// Matrix-vector product, y = A^T x
    void transpmult ( const uBlasVector& x, uBlasVector& y ) const;

    /// Multiply matrix by given number
    const uBlasMatrix<Mat>& operator*= ( double a );

    /// Return multiplication of matrix by given number
    const uBlasMatrix<Mat> operator* ( double a );

    /// Divide matrix by given number
    const uBlasMatrix<Mat>& operator/= ( double a );

    /// Return division of matrix by given number
    const uBlasMatrix<Mat> operator/ ( double a );
    
    /// Plus assign operator
    const uBlasMatrix<Mat>& operator+= ( const uBlasMatrix<Mat>& A );

    /// Return pointers to underlying compresssed storage data
    /// See uBlasMatrix for documentation.
    boost::tuple<const std::size_t*, const std::size_t*, const double*, int> data() const;

    //--- Special uBLAS functions ---

    /// Return reference to uBLAS matrix (const version)
    const Mat& mat() const
    {
        return A;
    }

    /// Return reference to uBLAS matrix (non-const version)
    Mat& mat()
    {
        return A;
    }

    /// Solve Ax = b out-of-place using uBLAS (A is not destroyed)
    void solve ( uBlasVector& x, const uBlasVector& b ) const;

    /// Solve Ax = b in-place using uBLAS(A is destroyed)
    void solveInPlace ( uBlasVector& x, const uBlasVector& b );

    /// Compute inverse of matrix
    void invert();

    /// Lump matrix into vector m
    void lump ( uBlasVector& m ) const;

    /// Compress matrix (eliminate all non-zeros from a sparse matrix)
    void compress();

    /// Access value of given entry
    double operator() ( size_t i, size_t j ) const
    {
        return A ( i, j );
    }

    /// Access value of given entry
    double& operator() ( size_t i, size_t j )
    {
        return A ( i, j );
    }

    /// Assignment operator
    const uBlasMatrix<Mat>& operator= ( const uBlasMatrix<Mat>& A );
    
private:

    /// General uBLAS LU solver which accepts both vector and matrix right-hand sides
    template<typename B>
    void solveInPlace ( B& X );

    /// Resize matrix to M x N
    void resize ( size_t M, size_t N );

    /// uBLAS matrix object
    Mat A;

    /// Matrix used internally for assembly of sparse matrices
    ublas_assembly_matrix Assembly_matrix;

    /// Matrix state
    bool assembled;

};

//---------------------------------------------------------------------------
// Implementation of uBlasMatrix
//---------------------------------------------------------------------------
template <typename Mat>
uBlasMatrix<Mat>::uBlasMatrix() : A ( 0, 0 ), assembled ( true )
{
}
//---------------------------------------------------------------------------
template <typename Mat>
uBlasMatrix<Mat>::uBlasMatrix ( size_t M, size_t N ) : A ( M, N ), assembled ( true )
{
    init ( M,N,0 );
}
//---------------------------------------------------------------------------
template <typename Mat>
uBlasMatrix<Mat>::uBlasMatrix ( const uBlasMatrix& A ) : A ( A.A ), assembled ( A.assembled )
{
}
//---------------------------------------------------------------------------
template <typename Mat>
uBlasMatrix<Mat>::~uBlasMatrix()
{
    // Do nothing
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix< Mat >::resize ( size_t M, size_t N )
{
    if ( size ( 0 ) != M || size ( 1 ) != N )
        A.Mat::resize ( M, N, false );
}
//---------------------------------------------------------------------------
template <typename Mat>
uBlasMatrix<Mat>* uBlasMatrix<Mat>::copy() const
{
    return new uBlasMatrix<Mat> ( *this );
}
//---------------------------------------------------------------------------
template <typename Mat>
size_t uBlasMatrix<Mat>::size ( size_t dim ) const
{
    assert ( dim < 2 );
    return ( dim == 0 ? A.Mat::size1() : A.Mat::size2() );
}
//---------------------------------------------------------------------------
template <typename Mat>
double uBlasMatrix<Mat>::norm ( std::string norm_type ) const
{
    if ( norm_type == "l1" )
        return norm_1 ( A );
    else if ( norm_type == "linf" )
        return norm_inf ( A );
    else if ( norm_type == "frobenius" )
        return norm_frobenius ( A );
    else
    {
        cell_error ( "Unknown norm type in uBlasMatrix." );
        return 0.0;
    }
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::getrow ( size_t row_idx, std::vector<size_t>& columns,
                                std::vector<double>& values ) const
{
    assert ( row_idx < this->size ( 0 ) );

    // Reference to matrix row
    const ublas::matrix_row<const Mat> row ( A, row_idx );

    // Insert values into std::vectors
    columns.clear();
    values.clear();
    typename ublas::matrix_row<const Mat>::const_iterator component;
    for ( component = row.begin(); component != row.end(); ++component )
    {
        columns.push_back ( component.index() );
        values.push_back ( *component );
    }
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::setrow ( size_t row_idx, const std::vector<size_t>& columns,
                                const std::vector<double>& values )
{
    assert ( columns.size() == values.size() );
    assert ( row_idx < this->size ( 0 ) );

    ublas::matrix_row<Mat> row ( A, row_idx );
    assert ( columns.size() <= row.size() );

    row *= 0;
    for ( size_t i = 0; i < columns.size(); i++ )
        row ( columns[i] ) =values[i];
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::resize ( uBlasVector& y, size_t dim ) const
{
    y.resize ( size ( dim ) );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::set ( const double* block, size_t m, const size_t* rows,
                             size_t n, const size_t* cols )
{
    for ( size_t i = 0; i < m; i++ )
        for ( size_t j = 0; j < n; j++ )
            A ( rows[i] , cols[j] ) = block[i*n + j];
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::add ( const double* block, size_t m, const size_t* rows,
                             size_t n, const size_t* cols )
{
    for ( size_t i = 0; i < m; i++ )
        for ( size_t j = 0; j < n; j++ )
            A ( rows[i] , cols[j] ) += block[i*n + j];
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::get ( double* block, size_t m, const size_t* rows,
                             size_t n, const size_t* cols ) const
{
    for ( size_t i = 0; i < m; ++i )
        for ( size_t j = 0; j < n; ++j )
            block[i*n + j] = A ( rows[i], cols[j] );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::set ( size_t rows, size_t cols, double val )
{
    A ( rows , cols ) = val;
}

//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::lump ( uBlasVector& m ) const
{
    const size_t n = size ( 1 );
    m.resize ( n );
    m.zero();
    ublas::scalar_vector<double> one ( n, 1.0 );
    ublas::axpy_prod ( A, one, m.vec(), true );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::solve ( uBlasVector& x, const uBlasVector& b ) const
{
    // Make copy of matrix and vector
    uBlasMatrix<Mat> Atemp;
    Atemp.mat().resize ( size ( 0 ), size ( 1 ) );
    Atemp.mat().assign ( A );
    x.vec().resize ( b.vec().size() );
    x.vec().assign ( b.vec() );

    // Solve
    Atemp.solveInPlace ( x.vec() );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::solveInPlace ( uBlasVector& x, const uBlasVector& b )
{
    const size_t M = A.size1();
    assert ( M == b.size() );

    // Initialise solution vector
    if ( x.vec().size() != M )
        x.vec().resize ( M );
    x.vec().assign ( b.vec() );

    // Solve
    solveInPlace ( x.vec() );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::invert()
{
    const size_t M = A.size1();
    assert ( M == A.size2() );

    // Create indentity matrix
    Mat X ( M, M );
    X.assign ( ublas::identity_matrix<double> ( M ) );

    // Solve
    solveInPlace ( X );
    A.assign_temporary ( X );
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::zero()
{
    // Iterate through no-zero pattern and zero entries
    typename Mat::iterator1 row;    // Iterator over rows
    typename Mat::iterator2 entry;  // Iterator over entries
    for ( row = A.begin1(); row != A.end1(); ++row )
        for ( entry = row.begin(); entry != row.end(); ++entry )
            *entry = 0;
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::zero ( size_t m, const size_t* rows )
{
    for ( size_t i = 0; i < m; ++i )
        ublas::row ( A, rows[i] ) *= 0.0;
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::eliminate ( double eps )
{
    // Iterate through no-zero pattern and zero entries
    typename Mat::iterator1 row;    // Iterator over rows
    typename Mat::iterator2 entry;  // Iterator over entries
    for ( row = A.begin1(); row != A.end1(); ++row )
        for ( entry = row.begin(); entry != row.end(); ++entry )
            if ( *entry < eps )
                *entry = 0;
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::ident ( size_t m, const size_t* rows )
{
    // Copy row indices to a vector
    std::vector<size_t> _rows ( rows, rows+m );

    size_t counter = 0;
    typename Mat::iterator1 row;    // Iterator over rows
    typename Mat::iterator2 entry;  // Iterator over entries
    for ( row = A.begin1(); row != A.end1(); ++row )
    {
        entry = row.begin();
        if ( std::find ( _rows.begin(), _rows.end(), entry.index1() ) != _rows.end() )
        {
            // Iterate over entries to zero and place one on the diagonal
            for ( entry = row.begin(); entry != row.end(); ++entry )
            {
                    *entry = 0.0;
            }
            ++ counter;
        }
        if ( counter == _rows.size() )
            continue;
    }
    for (std::vector<size_t>::iterator it = _rows.begin(); it != _rows.end(); ++it) 
       A( *it , *it ) = 1.0;
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::ident_insert ( size_t m, const size_t* rows )
{
    assert ( !assembled );
    for ( size_t i = 0; i < m; i++ )
        Assembly_matrix ( rows[i] , rows[i] ) = 1.0;
}
//---------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::mult ( const uBlasVector& x, uBlasVector& y, bool init ) const
{
  ublas::axpy_prod ( A, x.vec(), y.vec(), init );
}
//-----------------------------------------------------------------------------
template <typename Mat>
void uBlasMatrix<Mat>::transpmult ( const uBlasVector& x, uBlasVector& y ) const
{
    cell_error ( "The transposed version of the uBLAS matrix-vector product is not yet implemented" );
}
//-----------------------------------------------------------------------------
template <typename Mat>
const uBlasMatrix<Mat>& uBlasMatrix<Mat>::operator*= ( double a )
{
    A *= a;
    return *this;
}
//-----------------------------------------------------------------------------
template <typename Mat>
const uBlasMatrix<Mat> uBlasMatrix<Mat>::operator* ( double a )
{
  uBlasMatrix M(*this);
  M *= a;
  return M;
}
//-----------------------------------------------------------------------------
template <typename Mat>
const uBlasMatrix<Mat>& uBlasMatrix<Mat>::operator/= ( double a )
{
    A /= a;
    return *this;
}
//-----------------------------------------------------------------------------
template <typename Mat>
const uBlasMatrix<Mat> uBlasMatrix<Mat>::operator/ ( double a )
{
  uBlasMatrix M(*this);
  M /= a;
  return M;
}
//-----------------------------------------------------------------------------
template <typename Mat>
inline const uBlasMatrix<Mat>& uBlasMatrix<Mat>::operator+= ( const uBlasMatrix<Mat>& A )
{
    this->A += A.mat();
    return *this;
}
//-----------------------------------------------------------------------------
template <typename Mat>
inline const uBlasMatrix<Mat>& uBlasMatrix<Mat>::operator= ( const uBlasMatrix<Mat>& A )
{
    // Check for self-assignment
    if ( this != &A )
    {
        // Assume uBLAS take care of deleting an existing Matrix
        // using its assignment operator
        this->A = A.mat();
    }
    return *this;
}
//-----------------------------------------------------------------------------
template <typename Mat>
inline void uBlasMatrix<Mat>::compress()
{
    Mat A_temp ( this->size ( 0 ), this->size ( 1 ) );
    A_temp.assign ( A );
    A.swap ( A_temp );
}
//-----------------------------------------------------------------------------
template <typename Mat>
std::string uBlasMatrix<Mat>::str ( bool verbose ) const
{
    typename Mat::const_iterator1 it1;  // Iterator over rows
    typename Mat::const_iterator2 it2;  // Iterator over entries

    std::stringstream s;

    if ( verbose )
    {
        s << str ( false ) << std::endl;
        for ( it1 = A.begin1(); it1 != A.end1(); ++it1 )
        {
            s << "|";
            for ( it2 = it1.begin(); it2 != it1.end(); ++it2 )
            {
                std::stringstream entry;
                entry << std::setiosflags ( std::ios::scientific );
                entry << std::setprecision ( 16 );
                entry << " (" << it2.index1() << ", " << it2.index2() << ", " << *it2 << ")";
                s << entry.str();
            }
            s << " |" << std::endl;
        }
    }
    else
    {
        s << "<uBlasMatrix of size " << size ( 0 ) << " x " << size ( 1 ) << ">";
    }

    return s.str();
}
//-----------------------------------------------------------------------------
// Specialised member functions (must be inlined to avoid link errors)
//-----------------------------------------------------------------------------
template <>
inline double uBlasMatrix<ublas_dense_matrix>::get ( size_t rows, size_t cols ) const
{
    return A.operator()( rows , cols );
}
//-----------------------------------------------------------------------------
template <>
inline void uBlasMatrix<ublas_sparse_matrix>::init ( size_t M, size_t N, size_t nnz )
{
    resize ( M, N );
    A.clear();
    // Reserve space for non-zeroes
    A.reserve ( nnz );
    // Resize assembly matrix
    if ( Assembly_matrix.size1() != M && Assembly_matrix.size2() != N )
        Assembly_matrix.resize ( M, N, false );
}

//   //---------------------------------------------------------------------------
template <typename Mat>
inline void uBlasMatrix<Mat>::init ( size_t M, size_t N, size_t nnz )
{
    resize ( M, N );
    A.clear();
}
//---------------------------------------------------------------------------
template <>
inline void uBlasMatrix<ublas_sparse_matrix>::unapply()
{
    // Assign sparse matrix to the temporary assembly matrix
    if ( assembled )
    {
        // Assign temporary assembly matrix to the matrix
        Assembly_matrix.resize ( size ( 0 ), size ( 1 ), A.nnz() );
        Assembly_matrix.assign ( A );
        assembled = false;
    }
}
//---------------------------------------------------------------------------
template <typename Mat>
inline void uBlasMatrix<Mat>::unapply()
{
}
//---------------------------------------------------------------------------
template <>
inline void uBlasMatrix<ublas_sparse_matrix>::apply()
{
    // Assign temporary assembly matrix to the sparse matrix
    if ( !assembled )
    {
        // Assign temporary assembly matrix to the matrix
        A.assign ( Assembly_matrix );
        assembled = true;

        // Free memory
        Assembly_matrix.resize ( 0,0, false );
    }
    // Make sure matrix assembly is complete
    A.complete_index1_data();

}
//---------------------------------------------------------------------------
template <typename Mat>
inline void uBlasMatrix<Mat>::apply()
{
    // Do nothing
}
//---------------------------------------------------------------------------
template <typename Mat>
inline void uBlasMatrix<Mat>::axpy ( double a, const uBlasMatrix& A, bool same_nonzero_pattern )
{
    assert ( assembled );
    // Check for same size
    assert ( size ( 0 ) == A.size ( 0 ) );
    assert ( size ( 1 ) == A.size ( 1 ) );
    this->A += ( a ) * ( A.mat() );
}
//---------------------------------------------------------------------------
template <>
inline boost::tuple<const std::size_t*, const std::size_t*,
const double*, int> uBlasMatrix<ublas_sparse_matrix>::data() const
{
    assert ( assembled );
    typedef boost::tuple<const std::size_t*, const std::size_t*, const double*, int> tuple;
    return tuple ( &A.index1_data() [0], &A.index2_data() [0], &A.value_data() [0], A.nnz() );
}
//---------------------------------------------------------------------------
template <>
inline void uBlasMatrix<ublas_sparse_matrix>::set ( const double* block, size_t m, const size_t* rows, size_t n, const size_t* cols )
{
    assert ( !assembled );
    for ( size_t i = 0; i < m; i++ )
        for ( size_t j = 0; j < n; j++ )
            Assembly_matrix ( rows[i] , cols[j] ) = block[i*n + j];
}
//---------------------------------------------------------------------------
template <>
inline void uBlasMatrix<ublas_sparse_matrix>::add ( const double* block, size_t m, const size_t* rows, size_t n, const size_t* cols )
{
    assert ( !assembled );
    for ( size_t i = 0; i < m; i++ )
        for ( size_t j = 0; j < n; j++ )
        {
            Assembly_matrix ( rows[i] , cols[j] ) += block[i*n + j];
        }
}
//---------------------------------------------------------------------------
template <typename Mat>
inline boost::tuple<const std::size_t*, const std::size_t*, const double*, int>
uBlasMatrix<Mat>::data() const
{
    cell_error ( "Unable to return pointers to underlying data for this uBlasMatrix type." );
    return boost::tuple<const std::size_t*, const std::size_t*, const double*, int> ( 0, 0, 0, 0 );
}
//---------------------------------------------------------------------------
template<typename Mat> template<typename B>
void uBlasMatrix<Mat>::solveInPlace ( B& X )
{
    const size_t M = A.size1();
    assert ( M == A.size2() );

    // Create permutation matrix
    ublas::permutation_matrix<std::size_t> pmatrix ( M );

    // Factorise (with pivoting)
    size_t singular = ublas::lu_factorize ( A, pmatrix );
    if ( singular > 0 )
        cell_error1 ( "Singularity detected in uBLAS matrix factorization on line %u.", singular-1 );

    // Back substitute
    ublas::lu_substitute ( A, pmatrix, X );
}
//-----------------------------------------------------------------------------
#endif
