// Copyright (C) 2006 Garth N. Wells.
// Licensed under the GNU GPL Version 2.
//
// Modified by Anders Logg 2006.
// Modified by Pawel Krupinski 2007
#include <algorithm>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/vector_expression.hpp>
#include <boost/unordered_set.hpp>

#include "matrix/uBlasVector.h"
#include "utils/error_macros.h"
#include "utils/utility.h"
//-----------------------------------------------------------------------------
uBlasVector::uBlasVector(): x(new ublas_vector(0))
{
  // Do nothing
}
//-----------------------------------------------------------------------------
uBlasVector::uBlasVector(size_t N): x(new ublas_vector(N))
{
  // Set all entries to zero
  x->clear();
}
//-----------------------------------------------------------------------------
uBlasVector::uBlasVector(const uBlasVector& x): x(new ublas_vector(*(x.x)))
{
  // Do nothing
}
//-----------------------------------------------------------------------------
uBlasVector::uBlasVector(boost::shared_ptr<ublas_vector> x) : x(x)
{
  // Do nothing
}
//-----------------------------------------------------------------------------
uBlasVector::~uBlasVector()
{
  // Do nothing
}
//-----------------------------------------------------------------------------
uBlasVector* uBlasVector::copy() const
{
  return new uBlasVector(*this);
}
//-----------------------------------------------------------------------------
void uBlasVector::resize(size_t N)
{
  if (x->size() == N)
    return;
  x->resize(N, false);
  x->clear();
}
//-----------------------------------------------------------------------------
size_t uBlasVector::size() const
{
  return x->size();
}
//-----------------------------------------------------------------------------
void uBlasVector::zero()
{
  x->clear();
}
//-----------------------------------------------------------------------------
double uBlasVector::norm(Norm_type norm_type) const
{
  if (norm_type == l1)
    return ublas::norm_1(*x);
  else if (norm_type == l2)
    return ublas::norm_2(*x);
  else if (norm_type == linf)
    return ublas::norm_inf(*x);
  else
    cell_error("Requested vector norm type for uBlasVector unknown");
  
  return 0.0;
}
//-----------------------------------------------------------------------------
double uBlasVector::min() const
{
  double value = *std::min_element(x->begin(), x->end());
  return value;
}
//-----------------------------------------------------------------------------
double uBlasVector::max() const
{
  double value = *std::max_element(x->begin(), x->end());
  return value;
}
//-----------------------------------------------------------------------------
double uBlasVector::sum() const
{
  return ublas::sum(*x);
}
//-----------------------------------------------------------------------------
void uBlasVector::axpy(double a, const uBlasVector& y)
{
  if (size() != y.size())
    cell_error("Vectors must be of same size.");
  (*x) += a * y.vec();
}
//-----------------------------------------------------------------------------
void uBlasVector::toggle_sign()
{
  assert(x);
  const size_t size = x->size();
  for (size_t i = 0; i < size; i++)
    (*x)[i] = -((*x)[i]);
}
//-----------------------------------------------------------------------------
void uBlasVector::abs()
{
  assert(x);
  const size_t size = x->size();
  for (size_t i = 0; i < size; i++)
    (*x)[i] = std::abs((*x)[i]);
}
//-----------------------------------------------------------------------------
void uBlasVector::div(const uBlasVector& y)
{
  assert(x);
  const size_t size = x->size();
  for (size_t i = 0; i < size; i++)
    (*x)[i] /= (y.vec())[i];
}
//-----------------------------------------------------------------------------
double uBlasVector::inner(const uBlasVector& y) const
{
  return ublas::inner_prod(*x, y.vec());
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator= (const uBlasVector& y)
{
  *x = y.vec();
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator= (double a)
{
  x->ublas_vector::assign(ublas::scalar_vector<double> (x->size(), a));
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator*= (const double a)
{
  (*x) *= a;
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator*= (const uBlasVector& y)
{
  uBlasVector const& yy = dynamic_cast<uBlasVector const& >(y);

  *x = ublas::element_prod(*x,yy.vec());
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator/= (const double a)
{
  (*x) /= a;
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator+= (const uBlasVector& y)
{
  uBlasVector const& yy = dynamic_cast<uBlasVector const& >(y);
  *x += yy.vec();
  return *this;
}
//-----------------------------------------------------------------------------
const uBlasVector& uBlasVector::operator-= (const uBlasVector& y)
{
  uBlasVector const& yy = dynamic_cast<uBlasVector const& >(y);
  *x -= yy.vec();
  return *this;
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const uBlasVector& v)
{
  size_t size = v.size();
  os << "[" << size << "](";
  if (size > 0)
    os << v(0);
  for (size_t i = 1; i < size; i++)
    os << "," << v(i);
  os << ")";
  return os;
}
//-----------------------------------------------------------------------------