// Copyright (C) 2006 Garth N. Wells
// Licensed under the GNU GPL Version 2.
//
// Modified by Anders Logg 2006.
// Modified by Pawel Krupinski 2007

#ifndef __UBLAS_VECTOR_H
#define __UBLAS_VECTOR_H

#include <boost/shared_ptr.hpp>
#include "ublas.h"
// #include "GenericVector.h"
// #include <fstream>

// namespace ublas = boost::numeric::ublas;
//   typedef ublas::vector<double> ublas_vector;

/// This class represents a dense vector of dimension N.
/// It is a simple wrapper for a Boost ublas vector.
///
/// The interface is intentionally simple. For advanced usage,
/// refer to the documentation for ublas which can be found at
/// http://www.boost.org/libs/numeric/ublas/doc/index.htm.

class uBlasVector 
{
public:

  typedef ublas_vector Data;
  enum Norm_type { l1, l2, linf };
  
/// Create empty vector
    uBlasVector();
/// Create vector of size N
    explicit uBlasVector(size_t N);
/// Copy constructor
    uBlasVector(const uBlasVector& x);
/// Construct vector from a ublas_vector
    explicit uBlasVector(boost::shared_ptr<ublas_vector> x);
/// Destructor
    ~uBlasVector();
//--- Implementation of the GenericTensor interface ---
/// Create copy of tensor
    uBlasVector* copy() const;
/// Set all entries to zero and keep any sparse structure
    void zero();
/// Resize vector to size N
    void resize(size_t N);
/// Init vector to size N
    void init(size_t N){ resize(N); }
/// Return size of vector
    size_t size() const;
/// Access element value
    inline double get(size_t i) const {
        return (*x)(i);
    }
/// Set element value
    inline void set(size_t i, double value) {
        (*x)(i) = value;
    }
/// Add element value
    inline void add(size_t i, double value) {
        (*x)(i) += value;
    }
/// Set block of values
    template<class T>
    void set(const T block, const size_t* pos, int n);
/// Add block of values
    template<class T>
    void add(const T block, const size_t* pos, int n);
/// Get block of values
    template<class T>
    void get(T block, const size_t* pos, int n) const;
/// Set block of values
    void set(const double* block, const size_t* pos, int n){ set<const double*>(block, pos, n); }
    /// Add block of values    
    void add(const double* block, const size_t* pos, int n){ add<const double*>(block, pos, n); }
/// Get block of values
    void get(double* block, const size_t* pos, int n) const { get<double*>(block, pos, n); }
/// Add multiple of given vector (AXPY operation)
void axpy(double a, const uBlasVector& x);
/// Change the sign of all entries in the vector to the opposite
    void toggle_sign();
/// Replace all entries in the vector by their absolute values
    void abs();
/// Divide two vectors element by element    
    void div(const uBlasVector& y);
/// Return inner product with given vector
    double inner(const uBlasVector& x) const;
/// Compute norm of vector
    double norm(Norm_type norm_type) const;
/// Return minimum value of vector
    double min() const;
/// Return maximum value of vector
    double max() const;
/// Return sum of values of vector
    double sum() const;
/// Multiply vector by given number
    const uBlasVector& operator*= (double a);
/// Multiply vector by another vector pointwise
    const uBlasVector& operator*= (const uBlasVector& x);
/// Divide vector by given number
    const uBlasVector& operator/= (double a);
/// Add given vector
    const uBlasVector& operator+= (const uBlasVector& x);
/// Subtract given vector
    const uBlasVector& operator-= (const uBlasVector& x);
/// Assignment operator
    const uBlasVector& operator= (const uBlasVector& x);
/// Assignment operator
    const uBlasVector& operator= (double a);
/// Return pointer to underlying data (const version)
    const double* data() const
    {
        return &x->data()[0];
    }
/// Return pointer to underlying data
    double* data()
    {
        return &x->data()[0];
    }
    //--- Special uBLAS functions ---
/// Return reference to uBLAS vector (const version)
    const ublas_vector& vec() const
    {
        return *x;
    }
/// Return reference to uBLAS vector (non-const version)
    ublas_vector& vec()
    {
        return *x;
    }
/// Access value of given entry (const version)
    double operator() (size_t i) const
    {
        return (*x)(i);
    };
/// Access value of given entry (non-const version)
    double& operator() (size_t i)
    {
        return (*x)(i);
    };
/// Assignment operator
//     const uBlasVector& operator= (const uBlasVector& x);

private:
    // Smart pointer to uBLAS vector object
    boost::shared_ptr<ublas_vector> x;
};
//-----------------------------------------------------------------------------
template<class T>
void uBlasVector::set(const T block, const size_t pos[], int n)
{
    for (size_t i = 0; i < n; ++i)
        (*x)(pos[i]) = block[i];
}
//-----------------------------------------------------------------------------
template<class T>
void uBlasVector::add(const T block, const size_t pos[], int n)
{
    for (size_t i = 0; i < n; ++i)
        (*x)(pos[i]) += block[i];
}
//-----------------------------------------------------------------------------
template<class T>
void uBlasVector::get(T block, const size_t pos[], int n) const
{
    for (size_t i = 0; i < n; ++i)
        block[i] = (*x)(pos[i]);
}
//-----------------------------------------------------------------------------
std::ostream& operator<<(std::ostream& os, const uBlasVector& v);
//-----------------------------------------------------------------------------
#endif
