// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "mesh/mesh.h"
#include "utils/utility.h"

//-----------------------------------------------------------------------------

Mesh::Mesh ( Mesh const& m )
{
// 	std::cout << "Mesh copy constructor... \n";
    reserve_nodes ( m.size_of_nodes() );
    reserve_edges ( m.size_of_edges() );
    reserve_edge_uses ( m.size_of_edge_uses() );
    reserve_faces ( m.size_of_faces() );
// 	reserve_face_uses( m.size_of_face_uses() );
    m_type = m.type();

    Node_const_iterator n_iter, n_end;
    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        add_node ( new Mesh_node ( n_iter ) );
    }

    Edge_const_iterator e_iter, e_end;
    for ( e_iter = m.edges_begin(), e_end = m.edges_end(); e_iter != e_end; ++e_iter )
    {
        int id0 = e_iter->node0()->id();
        int id1 = e_iter->node1()->id();
        add_edge ( new Mesh_edge ( node ( id0 ), node ( id1 ) ) );
    }

    Edge_use_const_iterator eu_iter, eu_end;
    for ( eu_iter = m.edge_uses_begin(), eu_end = m.edge_uses_end(); eu_iter != eu_end; ++eu_iter )
    {
        int id = eu_iter->edge()->id();
        add_edge_use ( new Mesh_edge_use ( edge ( id ), eu_iter->direction() ) );
    }

    for ( eu_iter = m.edge_uses_begin(), eu_end = m.edge_uses_end(); eu_iter != eu_end; ++eu_iter )
    {
        Edge_use_handle eu = edge_use ( eu_iter->id() );
        int loop_id = eu_iter->loop_next()->id();
        int radial_id = eu_iter->radial_next()->id();
        eu->set_loop_next ( edge_use ( loop_id ) );
        eu->set_radial_next ( edge_use ( radial_id ) );
    }

    for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
    {
        Node_handle n = node ( n_iter->id() );
        Edge_use_const_iterator eu_i, eu_e;
        for ( eu_i = n_iter->edge_uses_begin(), eu_e = n_iter->edge_uses_end(); eu_i != eu_e; ++eu_i )
        {
            Edge_use_handle eu = edge_use ( eu_i->id() );
            n->add_edge_use ( eu );
        }
    }

    Face_const_iterator f_iter, f_end;
    for ( f_iter = m.faces_begin(), f_end = m.faces_end(); f_iter != f_end; ++f_iter )
    {
        int id = f_iter->edge_use()->id();
        add_face ( new Mesh_face ( edge_use ( id ) ) );
    }
}
//-----------------------------------------------------------------------------
Mesh::~Mesh()
{
    //delete faces
    Face_iterator f_iter = m_faces.begin();

    while ( f_iter!= m_faces.end() )
    {
        delete f_iter.ptr();
        f_iter = m_faces.erase ( f_iter.current_iterator() );
    }

    //delete edges
    Edge_iterator e_iter = m_edges.begin();

    while ( e_iter != m_edges.end() )
    {
        delete e_iter.ptr();
        e_iter = m_edges.erase ( e_iter.current_iterator() );
    }

    //delete nodes
    Node_iterator n_iter = m_nodes.begin();

    while ( n_iter != m_nodes.end() )
    {
        delete n_iter.ptr();
        n_iter = m_nodes.erase ( n_iter.current_iterator() );
    }
}
//-----------------------------------------------------------------------------
void Mesh::statistics ( bool detailed, std::ostream &out )
{
    out << "Mesh statistics:" << std::endl;

    out << "Number of nodes:" << size_of_nodes() << std::endl;
    if ( detailed )
    {
        int i;
        Node_iterator it;
        for ( ( i = 0, it = nodes_begin() ); it != nodes_end(); ( ++i, ++it ) )
        {
            out << "node[" << it->id() << "] at point " << it->point() <<"; degree : " << it->size_of_edge_uses() << std::endl;
        }
    }
    out << "Number of edges:" << size_of_edges() << std::endl;
    out << "Number of edge_uses:" << size_of_edge_uses() << std::endl;
    if ( detailed )
    {
        Edge_use_iterator it;
        for ( it = edge_uses_begin(); it != edge_uses_end(); ++it )
            it->statistics ( detailed, out );
    }
    out << "Number of faces:" << size_of_faces() << std::endl;
    if ( detailed )
    {
        int i;
        Face_iterator it;
        for ( ( i = 0, it = faces_begin() ); it != faces_end(); ( ++i, ++it ) )
        {
            out << "face[" << it->id() << "] ; degree : " << it->edge_use()->size_of_loop() << std::endl;
        }
    }
}
//-----------------------------------------------------------------------------
