// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_MESH_H_
#define _CELL_MESH_H_

#include <vector>
#include <iostream>
#include "mesh/mesh_node.h"
#include "mesh/mesh_edge.h"
#include "mesh/mesh_edge_use.h"
#include "mesh/mesh_face.h"
#include "mesh/mesh_handle.h"
#include "utils/handle_iterator.h"
#include "utils/utility.h"
#include "mesh/iterator_projector_functors.h"

/*!Mesh class handles memory allocation for all mesh elements, faces, edges and nodes. All other classes operate on pointers*/

class Mesh
{
public:
//VOLUME mesh -3D cells, SURFACE mesh -2D cells
    enum Type { VOLUME, SURFACE, VOLUME_OUTLINE, SURFACE_OUTLINE, NONE};

    typedef std::vector< Face_handle >		Face_container;
    typedef Handle_iterator<Face_container::iterator>			Face_iterator;
    typedef Handle_iterator<Face_container::const_iterator>	Face_const_iterator;

    typedef std::vector< Edge_handle >		Edge_container;
    typedef Handle_iterator<Edge_container::iterator>			Edge_iterator;
    typedef Handle_iterator<Edge_container::const_iterator>	Edge_const_iterator;

    typedef std::vector< Edge_use_handle >		Edge_use_container;
    typedef Handle_iterator<Edge_use_container::iterator>			Edge_use_iterator;
    typedef Handle_iterator<Edge_use_container::const_iterator>	Edge_use_const_iterator;

    typedef std::vector< Node_handle >		Node_container;
    typedef Handle_iterator<Node_container::iterator>			Node_iterator;
    typedef Handle_iterator<Node_container::const_iterator>	Node_const_iterator;

    typedef Project_point<Node_handle>	Point_projector;
    typedef Iterator_projector<Node_container::iterator, Point_projector>	Point_iterator;

    Mesh ( Type t = Mesh::NONE );
    Mesh ( Mesh const& m );
    ~Mesh();

    Type type() const
    {
        return m_type;
    }
    void set_type ( Type t )
    {
        m_type = t;
    }

    void reserve_nodes ( size_t n )
    {
        m_nodes.reserve ( n );
    }
    void reserve_edges ( size_t n )
    {
        m_edges.reserve ( n );
    }
    void reserve_edge_uses ( size_t n )
    {
        m_edge_uses.reserve ( n );
    }
    void reserve_faces ( size_t n )
    {
        m_faces.reserve ( n );
    }

    Edge_iterator edges_begin()
    {
        return m_edges.begin();
    }
    Edge_iterator edges_end()
    {
        return m_edges.end();
    }

    Edge_use_iterator edge_uses_begin()
    {
        return m_edge_uses.begin();
    }
    Edge_use_iterator edge_uses_end()
    {
        return m_edge_uses.end();
    }

    Node_iterator nodes_begin()
    {
        return m_nodes.begin();
    }
    Node_iterator nodes_end()
    {
        return m_nodes.end();
    }

    Face_iterator faces_begin()
    {
        return m_faces.begin();
    }
    Face_iterator faces_end()
    {
        return m_faces.end();
    }

    Point_iterator points_begin()
    {
        return m_nodes.begin();
    }
    Point_iterator points_end()
    {
        return m_nodes.end();
    }

    Edge_use_const_iterator edge_uses_begin() const
    {
        return m_edge_uses.begin();
    }
    Edge_use_const_iterator edge_uses_end() const
    {
        return m_edge_uses.end();
    }

    Edge_const_iterator edges_begin() const
    {
        return m_edges.begin();
    }
    Edge_const_iterator edges_end() const
    {
        return m_edges.end();
    }

    Node_const_iterator nodes_begin() const
    {
        return m_nodes.begin();
    }
    Node_const_iterator nodes_end() const
    {
        return m_nodes.end();
    }

    Face_const_iterator faces_begin() const
    {
        return m_faces.begin();
    }
    Face_const_iterator faces_end() const
    {
        return m_faces.end();
    }

    int node_index ( Mesh_node const *n ) const
    {
        return n->id();
    }
    int edge_index ( Edge_handle e ) const
    {
        return e->id();
    }
    int face_index ( Face_handle f ) const
    {
        return f->id();
    }

    size_t size_of_edges() const
    {
        return m_edges.size();
    }
    size_t size_of_nodes() const
    {
        return m_nodes.size();
    }
    size_t size_of_faces() const
    {
        return m_faces.size();
    }
    size_t size_of_edge_uses() const
    {
        return m_edge_uses.size();
    }

    Node_handle const& node ( size_t i ) const
    {
        return m_nodes[i];
    }
    Edge_handle const& edge ( size_t i ) const
    {
        return m_edges[i];
    }
    Edge_use_handle const& edge_use ( size_t i ) const
    {
        return m_edge_uses[i];
    }
    Face_handle const& face ( size_t i ) const
    {
        return m_faces[i];
    }

    Node_handle& node ( size_t i )
    {
        return m_nodes[i];
    }
    Edge_handle& edge ( size_t i )
    {
        return m_edges[i];
    }
    Edge_use_handle& edge_use ( size_t i )
    {
        return m_edge_uses[i];
    }
    Face_handle& face ( size_t i )
    {
        return m_faces[i];
    }

    void statistics ( bool detailed = false, std::ostream &out = std::cout );
    void check() const;

private:
    Type m_type;
    Face_container m_faces;
    Edge_container m_edges;
    Edge_use_container m_edge_uses;
    Node_container m_nodes;

    void add_node ( Node_handle n )
    {
        add_entity ( n, m_nodes );
    }
    void add_face ( Face_handle f )
    {
        add_entity ( f, m_faces );
    }
    void add_edge ( Edge_handle e )
    {
        add_entity ( e, m_edges );
    }
    void add_edge_use ( Edge_use_handle eu )
    {
        add_entity ( eu, m_edge_uses );
    }

    Node_iterator erase_node ( Node_iterator ni )
    {
        return erase_entity ( ni, m_nodes );
    }
    Edge_iterator erase_edge ( Edge_iterator ei )
    {
        return erase_entity ( ei, m_edges );
    }
    Edge_use_iterator erase_edge_use ( Edge_use_iterator eui )
    {
        return erase_entity ( eui, m_edge_uses );
    }
    Face_iterator erase_face ( Face_iterator fi )
    {
        return erase_entity ( fi, m_faces );
    }

    Node_handle delete_node ( Node_handle n )
    {
        return delete_entity ( n, m_nodes );
    }
    Edge_handle delete_edge ( Edge_handle e )
    {
        return delete_entity ( e, m_edges );
    }
    Edge_use_handle delete_edge_use ( Edge_use_handle eu )
    {
        return delete_entity ( eu, m_edge_uses );
    }
    Face_handle delete_face ( Face_handle f )
    {
        return delete_entity ( f, m_faces );
    }

    friend class Mesh_manipulator;
    friend class Mesh_builder;
    friend class Boundary_builder;
    friend class cell::Cell_manipulator;
};
//-----------------------------------------------------------------------------
inline Mesh::Mesh ( Type t ) :m_type ( t )
{

}
//-----------------------------------------------------------------------------
void check_doubled_nodes ( Mesh* m );
void check_doubled_edges ( Mesh* m );
void check_doubled_faces ( Mesh* m );
//-----------------------------------------------------------------------------
#endif
