// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "mesh/mesh_builder.h"
#include "mesh/mesh_utils.h"
#include "utils/determinant.h"
#include "utils/utility.h"
#include "utils/error_macros.h"
#include "utils/string_utils.h"
#include <CGAL/Timer.h>
using namespace cell;
// extern CGAL::Timer tmr;
//-----------------------------------------------------------------------------
Mesh_builder* Mesh_builder::m_instance = 0;// initialize pointer
//-----------------------------------------------------------------------------
Mesh_builder* Mesh_builder::get_instance()
{
    if (m_instance == 0)
    {
        m_instance = new Mesh_builder();
    }
    return m_instance;
}
//-----------------------------------------------------------------------------
//subdivision for meshes without tissue topology
Mesh_handle Mesh_builder::subdivide_mesh( Mesh_handle m_org )
{
    //clear own maps
    emn_map.clear(NULL);
    fmn_map.clear(NULL);
    nn_map.clear(NULL);

    Mesh::Type t = m_org->type();
    Mesh_handle m(new Mesh(t));

    {
        Mesh_manipulator mm(m);

        Face_const_iterator f_iter, f_end;
        for ( f_iter = m_org->faces_begin(), f_end = m_org->faces_end(); f_iter != f_end; ++f_iter )
        {
            divide_std_face( f_iter.handle(), mm );
        }
        //set edge flags in the new mesh from the edge flags of the old mesh using the edge_use_map
        mark_edge_flags();
    }

    return m;
}
//-----------------------------------------------------------------------------
void Mesh_builder::divide_std_face( Face_handle f_p, Mesh_manipulator &mm )
{
    divide_std_face_base( f_p, mm );
}
//-----------------------------------------------------------------------------
void Mesh_builder::divide_std_face_base(Face_handle f_p, Mesh_manipulator &mm )
{
    //get first and next edge_use of the face
    Mesh_face::Edge_use_circulator eu_circ = f_p->edge_uses_begin(), eu_start = eu_circ, eu_next = eu_circ;
    ++eu_next;
    Mesh_handle m = mm.mesh();

    Node_handle center_node;
    if ( (center_node = fmn_map[f_p]) == fmn_map.default_value() )
    {
        //add a face center node first
        center_node = mm.add_node( centroid( f_p ) );
        fmn_map.insert( f_p, center_node );
        Node_handle new_node, mid_node, next_mid_node;
        //add first middle edge node
        if ( (mid_node = emn_map[eu_circ->edge()]) == emn_map.default_value() )
        {
            mid_node = new Mesh_node( midpoint( eu_circ.handle() ) );
            emn_map.insert( eu_circ->edge(), mid_node );
            m->add_node( mid_node );
        }
        int count = 0;
        //next for all edges add middle nodes and connect with old vertex nodes and center node to build quads
        do {
            Node_handle old_node = eu_circ->other_node();
            if ( (new_node = nn_map[old_node]) == nn_map.default_value())
            {
                new_node = new Mesh_node( old_node );
                nn_map.insert( old_node, new_node );
                m->add_node( new_node );
            }
            if ( (next_mid_node = emn_map[eu_next->edge()]) == emn_map.default_value() )
            {
                next_mid_node = new Mesh_node( midpoint( eu_next.handle() ) );
                emn_map.insert( eu_next->edge(), next_mid_node );
                m->add_node( next_mid_node );
            }
            Face_handle quad_face = mm.add_quad( mid_node, new_node, next_mid_node, center_node );
            //store map of old to new edge uses only not default flags
            Mesh_edge::Flag_type def_flag = eu_circ->edge()->default_flag();
            if (eu_circ->edge()->flag() != def_flag)
            {
                eu_map[eu_circ.handle()].second = quad_face->edge_use();
            }
            if (eu_next->edge()->flag() != def_flag)
            {
                eu_map[eu_next.handle()].first = quad_face->edge_use()->loop_next();
            }
            //next step
            mid_node = next_mid_node;
            ++eu_next;
        } while (++eu_circ != eu_start);
    }
}
//-----------------------------------------------------------------------------
void Mesh_builder::mark_edge_flags()
{
    Edge_uses_map::iterator it, it_end;
    for (it = eu_map.begin(), it_end = eu_map.end(); it != it_end; ++it)
    {
        Edge_use_handle old_eu = it->first;
        Mesh_edge::Flag_type flag = old_eu->edge()->flag();
        Edge_use_handle new_eu = it->second.first;
        new_eu->edge()->set_flag(flag);
        new_eu = it->second.second;
        new_eu->edge()->set_flag(flag);
    }
}
//-----------------------------------------------------------------------------
