// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_COARSE_MESH_BUILDER_H_
#define _CELL_COARSE_MESH_BUILDER_H_

#include "mesh/mesh.h"
#include "mesh/mesh_manipulator.h"
#include "cell/cell_handle.h"
#include "utils/hash_map.h"
#include <list>
#include <CGAL/iterator.h>
class Mesh_builder
{
public:

    typedef Mesh::Face_container       Face_container;
    typedef Mesh::Edge_container       Edge_container;
    typedef Mesh::Node_container       Node_container;
    typedef Mesh::Face_iterator        Face_iterator;
    typedef Mesh::Face_const_iterator  Face_const_iterator;
    typedef Mesh::Edge_iterator        Edge_iterator;
    typedef Mesh::Edge_const_iterator  Edge_const_iterator;
    typedef Mesh::Node_iterator        Node_iterator;
    typedef Mesh::Node_const_iterator  Node_const_iterator;

    Mesh_handle subdivide_mesh( Mesh_handle m_org );
    Mesh_handle subdivide_mesh( Mesh_handle m_org, cell::Tissue_handle t );

    static Mesh_builder* get_instance();

protected:

    Mesh_builder() {}

private:

    typedef Unique_hash_map< Edge_handle, Node_handle > Edge_mid_node_map;
    typedef Unique_hash_map< Face_handle, Node_handle > Face_mid_node_map;
    typedef Unique_hash_map< Node_handle, Node_handle > Node_node_map;
    typedef std::list< std::pair< size_t, size_t > >    Indices_list;
    // MAp of old edges to pairs of new edges resulting from division
    typedef std::map< Edge_use_handle, std::pair< Edge_use_handle, Edge_use_handle > > Edge_uses_map;
    //set of subdivision methods for surface mesh
    template<class OI>
    void divide_std_face_base( Face_handle f_p, Mesh_manipulator &mm, OI out_iter );
    void divide_std_face_base( Face_handle f_p, Mesh_manipulator &mm );
    void divide_std_face( Face_handle f_p, Mesh_manipulator &mm, cell::Tissue_handle t );
    void divide_std_face( Face_handle f_p, Mesh_manipulator &mm );
    //mark new edges flags based on old edges flags and Edge_uses_map
    void mark_edge_flags();
   //singleton
    static Mesh_builder* m_instance;
    Edge_mid_node_map emn_map;
    Face_mid_node_map fmn_map;
    Node_node_map nn_map;
    Edge_uses_map eu_map;
    //local data
    static const int EDGE_FLAG = 1;
};
//-----------------------------------------------------------------------------
template<class OI>
void Mesh_builder::divide_std_face_base(Face_handle f_p, Mesh_manipulator &mm, OI out_iter )
{
    Mesh_face::Edge_use_circulator eu_circ = f_p->edge_uses_begin(), eu_start = eu_circ, eu_next = eu_circ;
    ++eu_next;
    Mesh_handle m = mm.mesh();

    Node_handle center_node;
    //get the face centroid node from the map or create new one if necessary
    if ( (center_node = fmn_map[f_p]) == fmn_map.default_value() )
    {
        center_node = mm.add_node( centroid( f_p ) );
        fmn_map.insert( f_p, center_node );
        Node_handle new_node, mid_node, next_mid_node;
        //get existing or create new edge mid nodes
        if ( (mid_node = emn_map[eu_circ->edge()]) == emn_map.default_value() )
        {
            mid_node = new Mesh_node( midpoint( eu_circ ) );
            emn_map.insert( eu_circ->edge(), mid_node );
            m->add_node( mid_node );
        }
        do {
            Node_handle old_node = eu_circ->other_node();
            if ( (new_node = nn_map[old_node]) == nn_map.default_value())
            {
                new_node = new Mesh_node( old_node );
                nn_map.insert( old_node, new_node );
                m->add_node( new_node );
            }
            if ( (next_mid_node = emn_map[eu_next->edge()]) == emn_map.default_value() )
            {
                next_mid_node = new Mesh_node( midpoint( eu_next ) );
                emn_map.insert( eu_next->edge(), next_mid_node );
                m->add_node( next_mid_node );
            }
            Face_handle quad_face = mm.add_quad( mid_node, new_node, next_mid_node, center_node );
            *out_iter++ = quad_face;
            //store map of old to new edge uses only not default flags
            Mesh_edge::Flag_type def_flag = eu_circ->edge()->default_flag();
            if(eu_circ->edge()->flag() != def_flag)
            {
              eu_map[eu_circ.handle()].second = quad_face->edge_use();
            }
            if(eu_next->edge()->flag() != def_flag)
            {
              eu_map[eu_next.handle()].first = quad_face->edge_use()->loop_next();
            }
            //next step
            mid_node = next_mid_node;
            ++eu_next;
        } while (++eu_circ != eu_start);
    }
}
//-----------------------------------------------------------------------------
#endif
