// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "mesh/mesh_manipulator.h"
#include "mesh/mesh_edge_use.h"
#include "fem/Function.h"
#include "utils/direction.h"
#include "utils/error_macros.h"
#include "utils/intersection_point.h"
#include "utils/globals.h"
#undef DEBUG_PRINT

//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::add_triangle( Node_handle const& n0, Node_handle const& n1, Node_handle const& n2 )
{
    cell_assert(m_mesh != INVALID_MESH && n0 != INVALID_NODE && n1 != INVALID_NODE && n2 != INVALID_NODE );

    Edge_use_handle eu0 = insert_radial(n0, n1);
    Edge_use_handle eu1 = insert_radial(n1, n2);
    Edge_use_handle eu2 = insert_radial(n2, n0);

    eu0->insert_loop(eu1);
    eu1->insert_loop(eu2);


    return add_face(eu0);
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::add_quad( Node_handle const& n0, Node_handle const& n1, Node_handle const& n2, Node_handle const& n3 )
{
    cell_assert(m_mesh != INVALID_MESH && n0 != INVALID_NODE && n1 != INVALID_NODE && n2 != INVALID_NODE && n3 != INVALID_NODE );

    Edge_use_handle eu0 = insert_radial(n0, n1);
    Edge_use_handle eu1 = insert_radial(n1, n2);
    Edge_use_handle eu2 = insert_radial(n2, n3);
    Edge_use_handle eu3 = insert_radial(n3, n0);

    eu0->insert_loop(eu1);
    eu1->insert_loop(eu2);
    eu2->insert_loop(eu3);

    return add_face(eu0);
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::find_edge_use( Node_handle const& n0, Node_handle const& n1 )
{
    cell_assert( n0 != INVALID_NODE && n1 != INVALID_NODE );
    Mesh_node::Edge_use_iterator e_it, e_end;
    for ( e_it = n0->edge_uses_begin(), e_end = n0->edge_uses_end(); e_it != e_end; ++e_it )
    {
        if ( e_it->other_node() == n1 )
            return e_it.handle();
    }
    return INVALID_EDGE_USE;
}
//-----------------------------------------------------------------------------
Edge_handle Mesh_manipulator::get_edge( Node_handle const& n0, Node_handle const& n1 )
{
    cell_assert( n0 != INVALID_NODE && n1 != INVALID_NODE );
    Edge_use_handle eu = find_any_edge_use(n0, n1);
    if ( eu == INVALID_EDGE_USE )
        return add_edge(n0, n1);
    return eu->edge();
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::find_any_edge_use( Node_handle const& n0, Node_handle const& n1 )
{
    cell_assert( n0 != INVALID_NODE && n1 != INVALID_NODE );
    Edge_use_handle eu = find_edge_use(n0, n1);
    if (eu == INVALID_EDGE_USE)
    {
        eu = find_edge_use(n1, n0);
    }
    return eu;
}
//-----------------------------------------------------------------------------
Direction Mesh_manipulator::get_direction( Edge_handle const& e, Node_handle const& n0, Node_handle const& n1 )
{
    cell_assert( e != INVALID_EDGE && n0 != INVALID_NODE && n1 != INVALID_NODE );
    if ( e->node0() == n0 )
    {
        if ( e->node1() == n1 )
            return DIR_ORIGINAL;
        else
            return DIR_INVALID;
    }
    else if ( e->node0() == n1 )
    {
        if ( e->node1() == n0 )
            return DIR_REVERSED;
        else
            return DIR_INVALID;
    }
    return DIR_INVALID;
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::reverse_loop( Edge_use_handle &head )
{
    Edge_use_handle eu = head;
    Edge_use_handle n_eu = eu->loop_next();
    do
    {
        eu->reverse();
        Edge_use_handle nn_eu;
        if (n_eu != INVALID_EDGE_USE)
        {
            nn_eu = n_eu->loop_next();
            n_eu->m_loop_next = eu;
        }
        else
        {
            nn_eu = INVALID_EDGE_USE;
        }
        eu = n_eu;
        n_eu = nn_eu;
    } while (eu != head && eu != INVALID_EDGE_USE);
    if (eu == INVALID_EDGE_USE)
        head->m_loop_next = INVALID_EDGE_USE;
}
//-----------------------------------------------------------------------------
Node_handle Mesh_manipulator::add_node( Point const &p )
{
    cell_assert(m_mesh != INVALID_MESH);
    Node_handle n = new Mesh_node(p);
    m_mesh->add_node(n);
    return n;
}
//-----------------------------------------------------------------------------
Edge_handle Mesh_manipulator::add_edge( Node_handle const& n0, Node_handle const& n1 )
{
    cell_assert(m_mesh != INVALID_MESH);
    Edge_handle e = new Mesh_edge( n0, n1 );
    m_mesh->add_edge( e );
    return e;
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::add_edge_use( Edge_handle const& e, Direction d )
{
    cell_assert(m_mesh != INVALID_MESH && e != INVALID_EDGE);
    Edge_use_handle eu = new Mesh_edge_use( e, d );
    m_mesh->add_edge_use( eu );
    return eu;
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::add_face( Edge_use_handle const& eu )
{
    cell_assert(m_mesh != INVALID_MESH && eu != INVALID_EDGE_USE);
    Face_handle f = new Mesh_face(eu);
    m_mesh->add_face( f );
    return f;
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::reverse( Face_handle &f1 )
{
    reverse_loop( f1->edge_use() );
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::remove_edge( Edge_use_handle &eu )
{
    cell_assert(eu->size_of_radial() == 2);
    Edge_use_handle rad = eu->radial_next();

    if (eu->direction() == rad->direction())
    {
        reverse_loop(rad);
    }

    Edge_use_handle eu_prev = eu->previous_loop().handle();
    Edge_use_handle eu_next = eu->loop_next();
    Edge_use_handle rad_prev = rad->previous_loop().handle();
    Edge_use_handle rad_next = rad->loop_next();

    eu_prev->set_loop_next(rad_next);
    rad_prev->set_loop_next(eu_next);

    eu->node()->remove_edge_use( eu );
    rad->node()->remove_edge_use( rad );

    //  delete edge and both edge uses
    m_mesh->delete_edge_use(rad);
    m_mesh->delete_edge_use(eu);
    m_mesh->delete_edge(eu->edge());

    return eu_prev;
}
#undef DEBUG_PRINT
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::remove_edge2( Edge_use_handle &eu )
{
    //store all edge_uses to remove in separate list (not internal linked list)
    std::list<Edge_use_handle> rad_eus;
    Mesh_edge_use::circulator radial_circ=eu->radial_begin(), radial_start=radial_circ;
    do
    {
      rad_eus.push_back(radial_circ.handle());
    }while(++radial_circ != radial_start);
    
    std::list<Edge_use_handle>::iterator it, it_end;
    for(it = rad_eus.begin(), it_end = rad_eus.end(); it != it_end; ++it)
    {
      remove_edge_use(*it);
    }

}
#undef DEBUG_PRINT
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::remove_edge_use( Edge_use_handle &eu, bool radial)
{
    Edge_use_handle eu_prev = eu->previous_loop().handle();
    Edge_use_handle eu_next = eu->loop_next();
    eu_prev->set_loop_next(eu_next);
    
    if(radial)
    {
      Edge_use_handle rad_prev = eu->previous_radial().handle();
      Edge_use_handle rad_next = eu->radial_next();
      rad_prev ->set_radial_next(rad_next);
    }
    m_mesh->delete_edge_use(eu);
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::insert_radial( Node_handle const& n0, Node_handle const&  n1 )
{
    cell_assert(n0 != INVALID_NODE && n1 != INVALID_NODE);
    Edge_use_handle exist_eu = find_any_edge_use( n0, n1 );
    if ( exist_eu  == INVALID_EDGE_USE )
    {
        Edge_handle e = add_edge( n0, n1 );
        return add_edge_use( e, DIR_ORIGINAL );
    }
    Edge_handle e = exist_eu->edge();
    Direction d = get_direction( e, n0, n1 );
    Edge_use_handle eu = add_edge_use( e, d );
    exist_eu->insert_radial(eu);
    return eu;
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::insert_radial( Node_handle const& n0, Node_handle const&  n1, Edge_handle const& e)
{
    cell_assert(n0 != INVALID_NODE && n1 != INVALID_NODE);
    Edge_use_handle exist_eu = find_any_edge_use( n0, n1 );
    Direction d = get_direction( e, n0, n1 );
    Edge_use_handle eu = add_edge_use( e, d );
    if ( exist_eu  != INVALID_EDGE_USE )
    {
        exist_eu->insert_radial(eu);
    }
    return eu;
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::split_edge( Edge_use_handle &eu, Node_handle const& n )
{
    cell_assert(eu != INVALID_EDGE_USE && n != INVALID_NODE);

    Edge_handle e = eu->edge();
    Node_handle n0 = e->node0();
    Node_handle n1 = e->node1();
    //move node0 of edge
    e->set_node0(n);
    //remove affected edge uses from node0 and put them at node n
    Mesh_node::Edge_use_iterator eu_it, eu_end;
    for ( eu_it = n0->edge_uses_begin(), eu_end = n0->edge_uses_end(); eu_it != eu_end; ++eu_it )
    {
        if ( !eu_it->edge()->has_node(n0) )
        {
            n->add_edge_use(eu_it);
            eu_it = n0->erase_edge_use(eu_it);
        }
    }

    Edge_handle new_e = add_edge(n, n0);
    Edge_use_handle prev_eu = INVALID_EDGE_USE;

    Mesh_edge_use::circulator rad_cir = eu->radial_begin(), rad_end = rad_cir;
    do {
        Edge_use_handle new_eu;
        if ( n == rad_cir->other_node() )
        {
            new_eu = add_edge_use(new_e, DIR_ORIGINAL);
            rad_cir->insert_loop(new_eu);
        }
        else
        {
            new_eu = add_edge_use(new_e, DIR_REVERSED );
            rad_cir->previous_loop()->insert_loop(new_eu);
        }
        if ( prev_eu != INVALID_EDGE_USE )
            prev_eu->insert_radial(new_eu);
        prev_eu = new_eu;
    } while (++rad_cir != rad_end);

    return prev_eu->radial_next();
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::split_edge( Edge_use_handle &eu, int n )
{
    cell_assert(eu != INVALID_EDGE_USE && n > 0);
    if (n ==1)
        return eu;
    Edge_handle e = eu->edge();
    Node_handle n0 = e->node0();
    Node_handle n1 = e->node1();

    std::vector<Node_handle> in_nodes(n-1);

    Vector v(n0->point(), n1->point());
    double l = sqrt(v.squared_length());
    double dl = l/n;
    double d = dl;
    v = v/l;

    for (int i = 0; i < n-1; ++i)
    {
        in_nodes[i] = add_node(n0->point() + v*d);
        d += dl;
    }
    //move node0 of edge
    e->set_node0(in_nodes.back());
    //remove affected edge uses from node0 and put them at node n
    Mesh_node::Edge_use_iterator eu_it, eu_end;
    for ( eu_it = n0->edge_uses_begin(), eu_end = n0->edge_uses_end(); eu_it != eu_end; ++eu_it )
    {
        if ( !eu_it->edge()->has_node(n0) )
        {
            eu_it = n0->erase_edge_use(eu_it);
            in_nodes.back()->add_edge_use(eu_it);
        }
    }
    std::vector<Edge_handle> new_e(n-1);
    for (int i = n-2; i > 0; --i)
    {
        new_e[i] = add_edge(in_nodes[i], in_nodes[i-1]);
    }
    new_e[0] = add_edge(in_nodes[0], n0);
    std::vector<Edge_use_handle> prev_eu;

    Mesh_edge_use::circulator rad_cir = eu->radial_begin(), rad_end = rad_cir;
    do {
        std::vector<Edge_use_handle> new_eu(n-1);
        if ( in_nodes.back() == rad_cir->other_node() )
        {
            for (int i = 0; i < n-1; ++i)
            {
                new_eu[i] = add_edge_use(new_e[i], DIR_ORIGINAL);
                rad_cir->insert_loop(new_eu[i]);
            }
        }
        else
        {
            Edge_use_handle base_eu = rad_cir->previous_loop().handle();
            for (int i = n-2; i >= 0; --i)
            {
                new_eu[n-2-i] = add_edge_use(new_e[i], DIR_REVERSED );
                base_eu->insert_loop(new_eu[n-2-i]);
            }
        }
        if ( !prev_eu.empty() )
        {
            for (int i = 0; i < n-1; ++i)
            {
                prev_eu[i]->insert_radial(new_eu[i]);
            }
        }
        prev_eu = new_eu;
    } while (++rad_cir != rad_end);
    return prev_eu[0]->radial_next();
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::split_face_edges( Face_handle &f, double d)
{
    Mesh_face::Edge_use_circulator eu_circ = f->edge_uses_begin(), eu_start = eu_circ;
    do
    {

        Point &p0 = eu_circ->node()->point();
        Point &p1 = eu_circ->other_node()->point();
        Vector v(p0,p1);
        double l = sqrt(v.squared_length());
        int n = l/d;
        split_edge( eu_circ.handle(), n );
    } while (++eu_circ != eu_start);
}
//-----------------------------------------------------------------------------
Edge_use_handle Mesh_manipulator::split_face( Edge_use_handle eu1, Node_handle const& n1, Edge_use_handle eu2, Node_handle const& n2, int c1, int c2 )
{
    //take care of edge inclusion cases
    cell_assert((eu1 != INVALID_EDGE_USE) && (eu2 != INVALID_EDGE_USE)  && (n1 != INVALID_NODE) && (n2 != INVALID_NODE) );
    if (c1 == -1)
        return eu1;
    if (c2 == -1)
        return eu2;
    //Edge_use_handles for second edge uses at division nodes
    Edge_use_handle neu1, neu2;
    //division point in the middle of the adge
    if (c1 == 1)
    {
        neu1 = split_edge( eu1, n1 );
    }
    //division point at the beginig of the edge use. traverse the loop to get other eu
    else if (c1 == 2)
    {
        Mesh_edge_use::circulator cir = eu1->loop_begin();
        while (cir->loop_next() != eu1)
            ++cir;
        neu1 = cir.handle();
    }
    //division point at the end of the edge. next in the loop is the other eu
    else if (c1 == 3)
        neu1 = eu1->loop_next();

    if (c2 == 1)
    {
        neu2 = split_edge( eu2, n2 );
    }
    else if (c2 == 2)
    {
        Mesh_edge_use::circulator cir = eu2->loop_begin();
        while (cir->loop_next() != eu2)
            ++cir;
        neu2 = cir.handle();
    }
    else if (c2 == 3)
        neu2 = eu2->loop_next();

    cell_assert(neu1 != INVALID_EDGE_USE && neu2 != INVALID_EDGE_USE);
    //create new edge and two edge uses for division edge
    Edge_handle new_e = add_edge(n1, n2);
    Edge_use_handle new_eu_org = add_edge_use(new_e, DIR_ORIGINAL);
    Edge_use_handle new_eu_rev = add_edge_use(new_e, DIR_REVERSED );
    new_eu_org->insert_radial(new_eu_rev);

    if (eu1->other_node() == n1 && neu1->node() == n1)
    {
        eu1->m_loop_next = new_eu_org;
        new_eu_rev->m_loop_next = neu1;
    }
    else if (eu1->node() == n1 && neu1->other_node() == n1)
    {
        neu1->m_loop_next = new_eu_org;
        new_eu_rev->m_loop_next = eu1;
    }
    else
        cell_error("not consistent orientation of edge uses at division point n1");

    if (eu2->other_node() == n2 && neu2->node() == n2)
    {
        eu2->m_loop_next = new_eu_rev;
        new_eu_org->m_loop_next = neu2;
    }
    else if (eu2->node() == n2 && neu2->other_node() == n2)
    {
        neu2->m_loop_next = new_eu_rev;
        new_eu_org->m_loop_next = eu2;
    }

    else
        cell_error("not consistent orientation of edge uses at division point n2");
    return new_eu_org;
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::split_face( Face_handle &f, Point const& p, Vector const& n )
{
    cell_assert(f != INVALID_FACE);
    Intersection_point inter;
    Point inter_p;
    Node_handle div_n[2];
    Edge_use_handle div_eu[2];
    Mesh_edge_use::circulator eu_circ = f->edge_use()->loop_begin(), eu_end = eu_circ;
    Node_handle n0 = eu_circ->node();
    int e_count = 0;
    int codes[2] = {-3,-3};
    bool no_edge_mode = true;
    bool edge_ended = false;
    do
    {
        inter_p = p;
        Node_handle n1 = eu_circ->other_node();
        int code = inter(inter_p, n, n0->point(), n1->point());
//code==-1 - edge in plane
//code==0 - no division
//code==1 - division in the middle of edge
//code==2 - division at n0 of current edge
//code==3 - division at n1 of current edge
#ifdef DEBUG_PRINT
        std::cout << "#######dividing edge[" << eu_circ->edge()->id() << "](" << eu_circ->edge()->node0()->id() << "," << eu_circ->edge()->node1()->id() << ")";
#endif
        //if edge lies in division plane
        if (code == -1)
        {
            //check the edge should be added to the beginig or the end of chain
            if (codes[0] != -1 || edge_ended)
            {
                e_count = 0;
                edge_ended = false;
            }
            else
            {
                e_count = 1;
            }
            div_n[e_count] = n0;
            div_eu[e_count] = eu_circ.handle();
            codes[e_count] = code;
            no_edge_mode = false;
        }
        //if edge in division plane wasn't detected so far
        else if (no_edge_mode)
        {
            if (code == 1)
            {
                div_n[e_count] = add_node(inter_p);
                div_eu[e_count] = eu_circ.handle();
                codes[e_count] = code;
                ++e_count;
            }
            else if (code == 2)
            {
                div_n[e_count] = n0;
                div_eu[e_count] = eu_circ.handle();
                codes[e_count] = code;
                ++e_count;

            }
            else if (code == 3)
            {
                div_n[e_count] = n1;
                div_eu[e_count] = eu_circ.handle();
                codes[e_count] = code;
                ++e_count;
            }
        }
        else
        {
            edge_ended = true;
        }
        //check if the same node was added from two different edges or need to search for edge division further
        if (e_count == 2 && ( div_n[0] == div_n[1] || ( codes[0] == 2 && codes[1] == 3 )))
            --e_count;
        n0 = n1;
    } while (++eu_circ != eu_end && e_count < 2);
    if (codes[0] == -1)
    {
        f->set_edge_use(div_eu[0]);
        return f;
    }
    else if (e_count == 2)
    {
        Edge_use_handle new_eu = split_face( div_eu[0], div_n[0], div_eu[1], div_n[1], codes[0], codes[1] );

        f->set_edge_use(new_eu);
        return add_face( new_eu->radial_next() );
    }
    return INVALID_FACE;
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::join_faces( Face_handle &f1, Face_handle &f2, Edge_use_handle & eu )
{
    Edge_use_handle n_eu = remove_edge(eu);
    f1->set_edge_use( n_eu );
    return f1;
}
//-----------------------------------------------------------------------------
Face_handle Mesh_manipulator::quad_to_triangles(Face_handle &f)
{
    Mesh_face::Edge_use_circulator h_start = f->edge_uses_begin();
    Edge_use_handle e0 = h_start;
    ++h_start;
    ++h_start;
    Edge_use_handle e1 = h_start;
    Edge_use_handle e = split_face( e0, e0->other_node(), e1, e1->other_node(), 3, 3 );
    return add_face(e);
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::init_node_directors()
{
    Mesh::Node_iterator n_iter, n_end;

    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        Vector dir(0,0,0);
        Mesh_node::Edge_use_iterator eu_iter, eu_end;
        for (eu_iter = n_iter->edge_uses_begin(), eu_end = n_iter->edge_uses_end(); eu_iter != eu_end; ++eu_iter)
        {
            dir = dir + normal(Edge_use_handle(eu_iter));
        }
        normalize(dir);
        n_iter->set_director( dir );
        n_iter->set_director0( dir );
    }
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::normalize_node_directors()
{
    Mesh::Node_iterator n_iter, n_end;

    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        Vector dir = n_iter->director();
        normalize(dir);
        n_iter->set_director(dir);
        dir = n_iter->director0();
        normalize(dir);
        n_iter->set_director0(dir);
    }
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::init_nodes( double h )
{
    Mesh::Node_iterator n_iter, n_end;
    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        n_iter->set_point0( n_iter->point() );
        n_iter->set_h0(h);
    }
    init_node_directors();
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::displace_mesh(Function const& f)
{
    Mesh::Node_iterator n_iter, n_end;
    Function & u = const_cast<Function &>(f);
    for ( n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter )
    {
        Point const& p = n_iter->point();
        Point np(p.x() + u(n_iter.ptr(), 0), p.y() + u(n_iter.ptr(), 1), p.z() + u(n_iter.ptr(), 2));
        n_iter->set_point(np);
        Vector const& v = n_iter->director();
        Vector nv(v.x() + u(n_iter.ptr(), 3), v.y() + u(n_iter.ptr(), 4), v.z() + u(n_iter.ptr(), 5));
        n_iter->set_director(nv);
    }
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::translate_mesh(Vector const v)
{
    Mesh::Node_iterator n_iter, n_end;
    for ( n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter )
    {
      Point const& p = n_iter->point();
      n_iter->set_point(Point(p.x()+v.x(), p.y()+v.y(), p.z()+v.z()));
    }
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::reset_to_current_mesh_state(Mesh_handle &m)
{
    Mesh::Node_iterator n_iter, n_end;
    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        n_iter->set_point0( n_iter->point() );
        n_iter->set_director0(n_iter->director());
    }
}
//-----------------------------------------------------------------------------
void Mesh_manipulator::reset_to_init_mesh_state(Mesh_handle &m)
{
    Mesh::Node_iterator n_iter, n_end;
    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        n_iter->set_point( n_iter->point0() );
        n_iter->set_director(n_iter->director0());
    }
}
//-----------------------------------------------------------------------------
// void Mesh_manipulator::merge_short_edges(double treshold)
// {
//   //create map of edge uses to faces
//   //loop over mesh edge uses and check the length of their edges
//   //if lenght below treshold make sure that no radial eus are defineing the faces 
//   //remove edge uses and edge merging the rest of things in the loop
// }
//-----------------------------------------------------------------------------
void Mesh_manipulator::delete_edge( Edge_use_handle &eu, bool flaged)
{
  //gather objects to remove
  Edge_handle e = eu->edge();
  Direction dir0 = eu->direction(); 
  std::list<Edge_use_handle> radial_eus;
  //get radial eus and if flag is set so eu is used in a face specification set face eu to the next in loop
  Mesh_edge_use::circulator cir = eu->radial_begin();
  do{
    radial_eus.push_back( cir.handle() );
    if(flaged && cir->is_set())
    {
      m_mesh->face(cir->flag())->set_edge_use(cir->loop_next());
    }
  }while(++cir != eu);
  Node_handle n = eu->node();
  Node_handle on = eu->other_node();
  
  //remove edge from the loop in each of radial eus
  std::list<Edge_use_handle>::iterator it, beyond;
  for(it = radial_eus.begin(), beyond = radial_eus.end(); it != beyond; ++it)
  {
    Edge_use_handle prev = (*it)->previous_loop().handle();
    Edge_use_handle next = (*it)->loop_next();
    prev->set_loop_next(next);
    if((*it)->direction() == dir0)
    {
      n->remove_edge_use((*it));
      prev->other_node() = on;
    }
    else
    {
      on->remove_edge_use((*it));
      on->add_edge_use(next);
      next->node() = on;
    }
    m_mesh->delete_edge_use((*it));
  }
  //replace all the handles to the node to be deleted in the edge uses
  Mesh_node::Edge_use_iterator eu_it, eu_end;
  for(eu_it = n->edge_uses_begin(), eu_end = n->edge_uses_end(); eu_it != eu_end; ++eu_it)
  {
    Edge_use_handle eu_prev = eu_it->previous_loop().handle();
    eu_it->node() = on;
    eu_prev->other_node() = on;
  }
  m_mesh->delete_node(n);
  m_mesh->delete_edge(e);
}
//-----------------------------------------------------------------------------
