// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _MESH_MANIPULATOR_H_
#define _MESH_MANIPULATOR_H_

#include "mesh/mesh_handle.h"
#include "mesh/mesh.h"
#include "mesh/mesh_utils.h"
#include "utils/error_macros.h"
#include "material/generic_material.h"
#include "matrix/Vector3.h"
#include <set>
#undef DEBUG_PRINT
// #define DEBUG_PRINT
namespace cell
{
class Cell_manipulator;
}
class Function;
//-----------------------------------------------------------------------------
class Mesh_manipulator
{
public:
    Mesh_manipulator( Mesh_handle &m ) : m_mesh(m) {}

    Face_handle add_triangle( Node_handle const& n0, Node_handle const& n1, Node_handle const& n2 );
    //adds quad spaned by four nodes in cw direction. Edges and edge uses are created, nodes are not
    Face_handle add_quad( Node_handle const& n0, Node_handle const& n1, Node_handle const& n2, Node_handle const& n3 );
    Face_handle quad_to_triangles(Face_handle &f);
    //adds polygonal face to the mesh. The input is the sequence of nodes. No checks of planarity, selfcrosing, circularity are made whatsoever.
    template<class I>
    Face_handle add_polygon( I first, I last );

    //creates new node at the point and adds it to the mesh. No checking for existing entities.
    Node_handle add_node( Point const &p );
    //creates new edge spaned by the nodes and adds it to the mesh. No checking for existing entities.
    Edge_handle add_edge( Node_handle const& n0, Node_handle const& n1 );
    //creates new edge_use with edge and direction. No checking for existing entities.
    Edge_use_handle add_edge_use( Edge_handle const& e, Direction d );
    //creates new face with edge_use. No checking for existing entities.
    Face_handle add_face( Edge_use_handle const& eu );
    //splits edge by adding a node (node doesn't have to lie on the edge)
    Edge_use_handle split_edge( Edge_use_handle & eu, Node_handle const& n );
    //splits edge to n equal segments.
    Edge_use_handle split_edge( Edge_use_handle & eu, int n );
    //splits face by adding an edge spaned by two new nodes
    Edge_use_handle split_face( Edge_use_handle eu1, Node_handle const& n1, Edge_use_handle eu2, Node_handle const& n2, int c1 = 1, int c2 = 1 );
    //splits face with the plane given by the point and normal vector
    Face_handle split_face( Face_handle &f, Point const& p, Vector const& n);
    //splits face's edges so the edges are approximately of length d
    Face_handle split_face_edges( Face_handle &f, double d);
    //joins two faces along the edge. Second face gets deleted
    Face_handle join_faces( Face_handle &f1, Face_handle &f2, Edge_use_handle & eu );
    //reverse direction of the face circulation
    void reverse( Face_handle &f1 );
    //reverse direction of the faces circulation
    template< class I >
    void reverse_faces(I from, I beyond);
    //reverses direction of the loop of edge uses
    void reverse_loop( Edge_use_handle &head );
    //returns new edge_use created on two nodes. Creates edge if needed. Attaches new edge_use to radial list of edge_uses on the edge in NONSORTED manner.
    Edge_use_handle insert_radial( Node_handle const& n0, Node_handle const& n1 );
    //returns new edge_use created on two nodes. Uses supplied edge if a first edge_use. Attaches new edge_use to radial list of edge_uses on the edge in NONSORTED manner.
    Edge_use_handle insert_radial( Node_handle const& n0, Node_handle const&  n1, Edge_handle const& e );
    //insert next_eu in the loop after eu. R
    void insert_loop(Edge_use_handle &eu, Edge_use_handle &next_eu)
    {
        eu->insert_loop(next_eu);
    }
    //merge edges which lenght is below treshold
    void merge_short_edges(double treshold);
    
    //finds edge_use spaned by two nodes in given direction(n0->n1).Returns INVALID_EDGE_USE if no edge_use found
    Edge_use_handle find_edge_use( Node_handle const& n0, Node_handle const& n1 );
    //finds edge_use spaned by two nodes in any direction(n0->n1 or n1->n0).Returns INVALID_EDGE_USE if no edge_use found
    Edge_use_handle find_any_edge_use( Node_handle const& n0, Node_handle const& n1 );
    //returns edge use spaned by two nodes.If no approporiate edge use found creates edge and edge use and places them in the mesh
    Edge_handle get_edge( Node_handle const& n0, Node_handle const& n1 );
    //returns direction that edge use has to have to point from n0  to n1
    Direction get_direction( Edge_handle const& e, Node_handle const& n0, Node_handle const& n1 );

    //resets initial state of the mesh (0-th node positions and directors) to the current state of given mesh
    void reset_to_current_mesh_state() {
        reset_to_current_mesh_state(m_mesh);
    }
    void reset_to_current_mesh_state(Mesh_handle &m);
    //resets current state of the mesh (0-th node positions and directors) to the initial state of given mesh
    void reset_to_init_mesh_state() {
        reset_to_init_mesh_state(m_mesh);
    }
    //reset deformed configuration to the initial one
    void reset_to_init_mesh_state(Mesh_handle &m);
    // normalize directors of all nodes
    void normalize_node_directors();
    // sets initial directors for all nodes
    void init_node_directors();
    // sets initial directors for nodes
    template< class I >
    void init_node_directors(I from, I beyond) {}
    // set initial directors for nodes belonging to faces
    template< class I >
    void init_face_directors(I from, I beyond);
    // reverses directors for nodes
    template< class I >
    void reverse_node_directors(I from, I beyond);
    // sets initial quanities and thickness for all nodes
    void init_nodes( double h );
    void set_face_edge_use(Face_handle &f, Edge_use_handle const& e) {
        f->set_edge_use(e);
    }
    template< class I, class T >
    void triangulate_face( Face_handle &f, I it, T tr );

    void displace_mesh(Function const& f);
    //translates mesh by the vector
    void translate_mesh(Vector const v);
    // sets material for the faces in the range of iterators. Iterators are handle iterators
    template< class I >
    void set_material(Generic_material*m, I from, I beyond);
    // sets material for the faces in the range of iterators
    template< class I >
    void set_material2(Generic_material*m, I from, I beyond);
    // sets fiber direction for the faces in the range of iterators. Iterator are assumed to be pointing to faces
    template< class I >
    void set_fiber_direction(Vector3 f, I from, I beyond);
    // sets fiber direction for the faces in the range of iterators. Iterator are assumed to be pointing to 'pointers' to faces
    template< class I >
    void set_fiber_direction2(Vector3 f, I from, I beyond);
    // sets thickness for the nodes in the range of iterators. Iterator are assumed to be pointing to nodes
    template< class I >
    void set_shell_thickness(double h, I from, I beyond);
    // sets thickness for the nodes in the range of iterators. Iterator are assumed to be pointing to pointers to nodes
    template< class I >
    void set_shell_thickness2(double h, I from, I beyond);
    Mesh_handle const& mesh() const {
        return m_mesh;
    }
    void set_mesh( Mesh_handle &m ) {
        m_mesh = m;
    }
    void delete_edge( Edge_use_handle &eu, bool flaged);
private:

    //removes edge from mesh joining approporiately remaining edge_uses. This is only possible for mid-wall edges since wall boundary edges can have more than two edge uses atached to them and joining edge uses becomes ambigous
    Edge_use_handle remove_edge( Edge_use_handle &eu );
    //removes edge taking care of all its edge_uses. Should be done only for simple meshes, without cell logic since it removes entities only in mesh
    Edge_use_handle remove_edge2( Edge_use_handle &eu );
    //removes single edge_use from mesh joining pervious and next loop edge_uses. If 'radial' is set it does the same for radial loop
    Edge_use_handle remove_edge_use( Edge_use_handle &eu, bool radial = false);
    //removes a face from the mesh. It deals with mesh entities only. Compartments need to be adjsted separately
//     Edge_use_handle remove_face( Face_handle &f );
    //delete edge and all the objects associated with it from the mesh. Flagged set means that eus used in a face definition are expected to be marked with id of the face (other eus are not set (set with default value)) and then a care is taken of eus used in aface definition 
    
    //private members
    Mesh_handle m_mesh;
    friend class cell::Cell_manipulator;
    friend class Mesh_builder;
};
//-----------------------------------------------------------------------------
//I is output iterator for Face_handle container. Newly created faces will be outputed there
template< class I, class T >
void Mesh_manipulator::triangulate_face( Face_handle &f, I it, T tr )
{
    tr.triangulate(f,it, *this);
}

//-----------------------------------------------------------------------------
//I is not Handle_iterator. ie. *i == Handle<T> not *i == T.
template<class I>
Face_handle Mesh_manipulator::add_polygon( I first, I last )
{
    cell_assert(m_mesh != INVALID_MESH);
    if (first == last)
        return INVALID_FACE;
    std::vector<Edge_use_handle> eus;
    I start = first;
    I next = first;
    ++next;
    while (next != last)
    {
        eus.push_back( insert_radial( *first, *next ) );
        ++first;
        ++next;
    }
    eus.push_back( insert_radial( *first, *start ) );

    for (int i = 0; i < eus.size()-1; ++i )
    {
        eus[i]->insert_loop(eus[i+1]);
    }
    return add_face(eus[0]);
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::reverse_node_directors(I from, I beyond)
{
    do
    {
        from->set_director( -from->director() );
    } while (++from != beyond);
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::reverse_faces(I from, I beyond)
 {
    do
    {
      reverse_loop( from->edge_use() );
    } while (++from != beyond);
}   
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::set_material(Generic_material *m, I from, I beyond)
{
    do
    {
        from->set_material(m);
    } while (++from != beyond);
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::set_material2(Generic_material *m, I from, I beyond)
{
    do
    {
        (*from)->set_material(m);
    } while (++from != beyond);
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::init_face_directors(I from, I beyond)
{
    std::set<Node_handle> nodes;
    while (from != beyond)
    {
        Mesh_face::Edge_use_circulator eu_circ = from->edge_uses_begin(), eu_start = eu_circ;
        do
        {
            Node_handle nod = eu_circ->other_node();
            if (!nodes.count(nod))
            {
                nodes.insert(nod);
                Point const& p0 = eu_circ->node()->point();
                Point const& p1 = nod->point();
                Point const& p2 = eu_circ->loop_next()->other_node()->point();
                Vector v1(p0,p1);
                Vector v2(p1,p2);
                Vector dir = normalized(CGAL::cross_product(v1, v2)) + nod->director();
                nod->set_director(dir);
            }
        } while (++eu_circ != eu_start);
        ++from;
    }
    std::set<Node_handle>::iterator n_iter, n_end;
    for (n_iter = nodes.begin(), n_end = nodes.end(); n_iter != n_end; ++n_iter)
    {
        Node_handle n = *n_iter;
        Vector dir = n->director();
        normalize(dir);
        n->set_director(dir);
        n->set_director0(dir);
    }
}
//-----------------------------------------------------------------------------
template< class I>
void Mesh_manipulator::set_fiber_direction(Vector3 f, I from, I beyond)
{
    while (from != beyond)
    {
        from->fem_data().fiber = f;
        ++from;
    }
}
//-----------------------------------------------------------------------------
template< class I>
void Mesh_manipulator::set_fiber_direction2(Vector3 f, I from, I beyond)
{
    while (from != beyond)
    {
        (*from)->fem_data().fiber = f;
        ++from;
    }
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::set_shell_thickness(double h, I from, I beyond)
{
    while (from != beyond)
    {
        from->set_h0(h);
        ++from;
    }
}
//-----------------------------------------------------------------------------
template< class I >
void Mesh_manipulator::set_shell_thickness2(double h, I from, I beyond)
{
    while (from != beyond)
    {
        (*from)->set_h0(h);
        ++from;
    }
}
//-----------------------------------------------------------------------------
#endif
