// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "mesh/mesh_utils.h"
#include <fstream>
#include "mesh_manipulator.h"
#include "matrix/principal_directions.h"
#include "postprocessing/stress_interpolator.h"
#include "material/isotropic_material.h"

//-----------------------------------------------------------------------------
void convert_to_spring_format( std::ofstream &out, Mesh_handle const& m, int n_dim, bool ext )
{
    if ( !out.good() )
        cell_error( "Unusable output stream->" );
    if (m->type() != Mesh::SURFACE)
    {
        cell_warning("Not suported mesh type for format conversion. No output file written.\n");
        return;
    }
    if ( n_dim != 3 && n_dim != 2 )
    {
        cell_warning("Number of dimensions has to be 2 or 3. No output file written.\n");
        return;
    }

    size_t n_faces = m->size_of_faces(), n_edges = m->size_of_edges(), n_nodes = m->size_of_nodes();
    out << n_faces << " " << n_edges << " " << n_nodes << "\n";

    //construct the map from edges to faces
    std::vector< std::pair<int, int> > edge_face_map(n_edges, std::make_pair<int, int>(-1,-1));
    Mesh::Face_const_iterator f_it, f_end;
    for ( f_it = m->faces_begin(), f_end = m->faces_end(); f_it != f_end; ++f_it )
    {
        size_t f_id = f_it->id();
        Mesh_face::Edge_use_const_circulator e_circ = f_it->edge_uses_begin(), e_start = e_circ;
        do {
            size_t e_id = e_circ->edge()->id();
            std::pair<int, int> &faces = edge_face_map[e_id];
            if (faces.first < 0)
                faces.first = f_id;
            else if (faces.second < 0)
                faces.second = f_id;
            else
            {
                std::cout << "faces: " << faces.first << ", " << faces.second << ", " << f_id << "\n";
                cell_error("Edge connected to more than two faces in a surface mesh\n");
            }
        } while (++e_circ != e_start);
    }

    Mesh::Edge_const_iterator e_iter, e_end;
    for ( e_iter = m->edges_begin(), e_end = m->edges_end(); e_iter != e_end; ++e_iter )
    {
        size_t e_id = e_iter->id();
        out << e_id << " ";
        std::pair<int, int> &faces = edge_face_map[e_id];
        out << faces.first << " " << faces.second << " ";
        out << e_iter->node0()->id() << " " <<  e_iter->node1()->id() << "\n";
    }
    out << "\n";

    out << n_nodes << " " << n_dim << "\n";
    Mesh::Node_const_iterator n_iter, n_end;
    if (n_dim == 3)
    {
        for ( n_iter = m->nodes_begin(), n_end = m->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() << " " << p.y() << " " << p.z() << "\n";
        }
    }
    else if (n_dim == 2)
    {
        for ( n_iter = m->nodes_begin(), n_end = m->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Point const& p = n_iter->point();
            out << p.x() << " " << p.y() << "\n";
        }
    }
    else
        cell_error("Number of dimensions for mesh output should be 2 or 3.\n");
    out << "\n";

    out << n_edges << " 1 ";
    if (ext)
        out << "2\n";
    else
        out << "0\n";

    for ( e_iter = m->edges_begin(), e_end = m->edges_end(); e_iter != e_end; ++e_iter )
    {
        Point const& p1 = e_iter->node0()->point();
        Point const& p2 = e_iter->node1()->point();
        out << sqrt( CGAL::squared_distance(p1, p2) );
        if (ext)
            out << " 0 0";
        out << "\n";
    }
    out << "\n";

    out << n_faces << " ";
    if (ext)
        out << (n_dim+1)*2+5;
    else
        out << n_dim+1;
    out<< "\n";

//     Mesh::Face_const_iterator f_iter, f_end;
    if (n_dim == 3)
    {
//         for ( f_iter = m->faces_begin(), f_end = m->faces_end(); f_iter != f_end; ++f_iter )
        for (int i = 0; i < n_faces; ++i)
        {
            out <<"1 0 0 1";
            if (ext)
            {
                out <<" 1 0 0 1";
                out <<" 1 1 0 1 1";
            }
            out <<"\n";
        }
    }
    else if (n_dim == 2)
    {
//         for ( f_iter = m->faces_begin(), f_end = m->faces_end(); f_iter != f_end; ++f_iter )
        for (int i = 0; i < n_faces; ++i)
        {
            out <<"1 0 1";
            if (ext)
            {
                out <<" 1 0 1";
                out <<" 1 1 0 1 1";
            }
            out <<"\n";
        }
    }
    else
        cell_warning("Number of dimensions should be 2 or 3.\n");

    return;
}
//-----------------------------------------------------------------------------
void write_mesh(Mesh_handle m, std::ofstream &out)
{
    if (!out.is_open() || !out.good())
        cell_error("unusable output stream.");
    size_t n_points = m->size_of_nodes(), n_faces = m->size_of_faces();
    out << n_points << " nodes\n";
    Mesh::Node_iterator n_it, n_end;
    for (n_it = m->nodes_begin(), n_end = m->nodes_end(); n_it != n_end; ++ n_it)
    {
        Point &p = n_it->point();
        out << p.x() << " " << p.y() << " " << p.z();
        if (n_it->flag() != n_it->default_flag())
            out << " : " << n_it->flag();
        out << "\n";
    }
    
    out << n_faces << " faces\n";
    Mesh::Face_iterator f_it, f_end;
    std::vector<size_t> indices;
    indices.reserve(10);
    for(f_it = m->faces_begin(), f_end = m->faces_end(); f_it != f_end; ++f_it)
    {
      indices.clear();
      Mesh_face::Edge_use_circulator eu_circ = f_it->edge_uses_begin(), eu_start = eu_circ;
      do{
        indices.push_back(eu_circ->node()->id());
      }while(++eu_circ != eu_start);
      out << indices.size() << " :";
      std::vector<size_t>::iterator it, it_end;
      for(it = indices.begin(), it_end = indices.end(); it != it_end; ++it)
        out << " " << *it;
      if (f_it->flag() != f_it->default_flag())
            out << " : " << f_it->flag();
        out << "\n";
    }
    
}
//-----------------------------------------------------------------------------
void write_spreadheet_timestep(std::ofstream &out, Mesh_handle m, State_interpolator const& si, double t)
{
    Mesh::Node_const_iterator n_it, n_end;
    principal_directions pd;
    n_it = m->nodes_begin();
        Point const&p = n_it->point();
        out << p.x() << " " << p.y() << " " << p.z() << " ";
        pd.calculate_principal_directions( si.stress(n_it), true );
        for ( int i = 0 ; i < 3; ++i )
        {
            out << pd.val(i) << " ";
        }
    ++n_it;++n_it;
    Point const&p1 = n_it->point();
    out << 0.5*(p.x()-p1.x()) << " " << 0.5*(p.x()-p1.x())*(p.y()-p1.y());
    out << "\n";
}
//-----------------------------------------------------------------------------
void write_3d_plot(std::ofstream &out, Node_handle n, State_interpolator const& si, double t, double nu, double Y)
{
    principal_directions pd;
    pd.calculate_principal_directions( si.stress(n), true );
    out << Y << " " << nu << " ";
    for ( int i = 0 ; i < 3; ++i )
    {
        out << pd.val(i) << " ";
    }
    out << "\n";
}
//-----------------------------------------------------------------------------
void write_3d_plot2(std::ofstream &out, int point, Face_handle fh, Affine_map_shell<Mesh_face> &aff_map)
{
    aff_map.set_flag(Affine_map_shell< Mesh_face >::UPDATE_ALL);
    aff_map.update(*fh);
    aff_map.update_stress();
    principal_directions pd;
    pd.calculate_principal_directions( aff_map.stress(point), true );

    double nu;
    double Y;
    Isotropic_material const* mater = dynamic_cast<Isotropic_material const*>(aff_map.material());
    if (mater)
    {
        nu = mater->nu();
        Y = mater->E();
    }
    else
    {
     cell_error("Material not supported by the \"write_3d_plot2\" routine\n.");
    }
    out << Y << " " << nu << " ";
    for ( int i = 0 ; i < 3; ++i )
    {
        out << pd.val(i) << " ";
    }
    out << "\n";
}
//-----------------------------------------------------------------------------
void check_mesh_faces_circularity(Mesh_handle m)
{
    //check if every edge_use radial counterpart is in opposite direction (that ensures that circularity of the faces is consistent across surface mesh)
    Mesh::Edge_use_const_iterator eu_it, eu_end;
    for (eu_it = m->edge_uses_begin(), eu_end = m->edge_uses_end(); eu_it != eu_end; ++eu_it)
    {
        Edge_use_handle const& other_eu = eu_it->radial_next();
        if (other_eu != INVALID_EDGE_USE && other_eu != eu_it.handle())
        {
            if (other_eu->node()->id() == eu_it->node()->id() && other_eu->other_node()->id() == eu_it->other_node()->id())
            {
                std::cout << "Edge uses: [" << eu_it->id() << "](" <<  eu_it->node()->id() << ", " <<  eu_it->other_node()->id() << ") and [" << other_eu->id() << "](" <<  other_eu->node()->id() << ", " <<  other_eu->other_node()->id() << ") are alligned in the same direction\n";
            }
            else if (other_eu->node()->id() != eu_it->other_node()->id() || other_eu->other_node()->id() != eu_it->node()->id())
            {
                std::cout << "Edge uses: [" << eu_it->id() << "](" <<  eu_it->node()->id() << ", " <<  eu_it->other_node()->id() << ") and [" << other_eu->id() << "](" <<  other_eu->node()->id() << ", " <<  other_eu->other_node()->id() << ") are misaligned\n";
            }
        }
    }
}
//-----------------------------------------------------------------------------
void fix_mesh_faces_circularity(Mesh_handle m)
{
    //allign faces circularity to be consistent with the one of the first face
    std::list<int> process_faces;
    std::list<int> next_faces;

    Face_handle f = m->faces_begin().handle();
    //set face flags (0-unprocessed, 1-processed) and edge_use flags (= face id)
    Mesh::Face_iterator f_it, f_end;
    for (f_it = m->faces_begin(), f_end = m->faces_end(); f_it != f_end; ++f_it)
    {
        f_it->set_flag(0);
        Mesh_face::Edge_use_circulator eu_circ = f_it->edge_uses_begin() , eu_start = eu_circ;
        do {
            eu_circ->set_flag(f_it->id());
        } while (++eu_circ != eu_start);
    }
    next_faces.push_back(0);
    while (!next_faces.empty())
    {
        process_faces.assign(next_faces.begin(), next_faces.end());
        next_faces.clear();
        while (!process_faces.empty())
        {
            process_faces.pop_back();
        }
    }
    Mesh::Edge_use_const_iterator eu_it, eu_end;
    for (eu_it = m->edge_uses_begin(), eu_end = m->edge_uses_end(); eu_it != eu_end; ++eu_it)
    {
        Edge_use_handle const& other_eu = eu_it->radial_next();
        if (other_eu != INVALID_EDGE_USE)
        {
            if (other_eu->node()->id() == eu_it->node()->id() && other_eu->other_node()->id() == eu_it->other_node()->id())
            {
                std::cout << "Edge uses: [" << eu_it->id() << "](" <<  eu_it->node()->id() << ", " <<  eu_it->other_node()->id() << ") and [" << other_eu->id() << "](" <<  other_eu->node()->id() << ", " <<  other_eu->other_node()->id() << ") are alligned in the same direction\n";
            }
            else if (other_eu->node()->id() != eu_it->other_node()->id() || other_eu->other_node()->id() != eu_it->node()->id())
            {
                std::cout << "Edge uses: [" << eu_it->id() << "](" <<  eu_it->node()->id() << ", " <<  eu_it->other_node()->id() << ") and [" << other_eu->id() << "](" <<  other_eu->node()->id() << ", " <<  other_eu->other_node()->id() << ") are misaligned\n";
            }
        }
    }
}
//-----------------------------------------------------------------------------
Mesh_handle 
import_from_spring_format( std::ifstream &in )
{
    if ( !in.good() )
        cell_error( "Unusable intput stream->" );

    int n_faces, n_edges, n_nodes;
    in >> n_faces >> n_edges >> n_nodes;

    //edge nodes map
    std::vector< std::pair<int, int> > edges_nodes(n_edges, std::make_pair<int, int>(-1,-1));
    //edge faces map
    std::vector< std::pair<int, int> > edges_faces(n_edges, std::make_pair<int, int>(-1,-1));
    //face edge map
    std::vector< std::list<int> > face_edges(n_faces, std::list<int>());
    //read in edges
    int id, f1, f2, n1, n2;
//     std::cout << "edges maps\n";
    for (int i = 0; i < n_edges; ++i)
    {
        in >> id >> f1 >> f2 >> n1 >> n2;
        if (id > n_edges || f1 > n_faces || f2 > n_faces || n1 > n_nodes || n2 > n_nodes)
        {
            cell_error( "indices excede declared sizes.\n" );
        }

        edges_nodes[id].first = n1;
        edges_nodes[id].second = n2;
        edges_faces[id].first = f1;
        edges_faces[id].second = f2;
        if( f1 >=0 )
          face_edges[f1].push_back(id);
        if( f2 >=0 )
          face_edges[f2].push_back(id);
    }

    //prepare mesh
    Mesh_handle m = new Mesh(Mesh::SURFACE);
    m->reserve_nodes(n_nodes);
    m->reserve_edges(n_edges);
    m->reserve_faces(n_faces);
    Mesh_manipulator manip(m);
    //read in and construct nodes
    int temp, dim;
    double x, y, z;
    in >> temp >> dim;
    if (temp != n_nodes)
        cell_error( "inconsistent declaration of number of nodes.\n" );
    if ( dim != 3 && dim != 2 )
    {
        cell_warning("Number of dimensions has to be 2 or 3. No data read.\n");
        return m;
    }
    if (dim == 2)
        for (int i = 0; i < n_nodes; ++i)
        {
            in >> x >> y;
            manip.add_node(Point(x,y,0));
        }
    else if (dim == 3)
        for (int i = 0; i < n_nodes; ++i)
        {
            in >> x >> y >> z;
            manip.add_node(Point(x,y,z));
        }
//     construct edges
    for (int i = 0; i < n_edges; ++i)
    {
      manip.add_edge(m->node(edges_nodes[i].first), m->node(edges_nodes[i].second));
    }
    //construct faces
    for(int i = 0; i < n_faces; ++i)
    {
      std::list<int> &f = face_edges[i];
      std::list<int>::iterator it = f.begin(), f_end;
      
      Edge_handle e = m->edge(*it);     
      Node_handle n1 = e->node0();
      Node_handle n2 = e->node1();
      int n1_id  = n1->id(), n2_id = n2->id();
      Edge_use_handle eu = manip.insert_radial(n1, n2, e);
      f.erase(it);
      while(!f.empty())
      {
            Direction d = DIR_INVALID;
            it = f.begin();
            do
            {
              e = m->edge(*it);
              n1 = e->node0();
              n2 = e->node1();
              if(n1->id() == n2_id)
              {
                d = DIR_ORIGINAL;
              }
              else if(n2->id() == n2_id)
              {
                d = DIR_REVERSED;
                Node_handle temp = n1;
                n1 = n2;
                n2 = temp;
              }
              else 
              {
                ++it;
              }
            }while ((it != f.end()) && (d == DIR_INVALID));
            if(d == DIR_INVALID)
              cell_error("face edges can not form closed loop.\n");
            else
              f.erase(it);
            Edge_use_handle new_eu = manip.insert_radial(n1, n2, e);
            manip.insert_loop(eu, new_eu);
            eu = new_eu;
            n1_id = n1->id();
            n2_id = n2->id();
        }
        Face_handle fc = manip.add_face(eu);
        fc->check_circularity();
    }
    //orient faces in consistent direction on the surface
    //marker for already processed edge uses indices
    std::vector<char> processed(m->size_of_edge_uses(), 0);
    //collection of edge uses indices to process
    std::list<int> heap;
    heap.push_back(m->edge_uses_begin()->id());

    Edge_use_handle eu, loop, radial;
    eu = m->edge_uses_begin().handle();

    while (!heap.empty())
    {
        Edge_use_handle eu, loop, radial, first;
        eu = m->edge_use(heap.front());
        first = eu;
        heap.pop_front();

        do {
            loop = eu->loop_next();
            if ( processed[eu->id()] == 0 )
            {
                processed[eu->id()] = 1;
                radial = eu->radial_next();
                if (processed[radial->id()] == 0)
                {
                    if (reverse(radial->direction()) != eu->direction())
                        manip.reverse_loop(radial);
                    heap.push_back(radial->id());
                }
            }
            eu = loop;
        } while (loop != first);
    }
    return m;
}
//-----------------------------------------------------------------------------
void project_on_triangular_mesh( Mesh_handle &src_m, Mesh_handle const& trg_m )
{
    //list of nodes that couldnt be projected on supplied mesh
    std::list<Node_handle> up_nodes;
    //iterate mesh points and project them on the triangular mesh
    Mesh::Node_iterator n_it, n_end;
    for (n_it = src_m->nodes_begin(), n_end = src_m->nodes_end(); n_it != n_end; ++n_it)
    {
        //list of triangles that contain the point when projected on xy plane
        std::list<Face_handle> trs;
        //list of projection points on those triangles
        std::list<Point> points;
        Point p = n_it->point();
        //Check all faces of target mesh
        Mesh::Face_iterator f_it, f_end;
        for(f_it = trg_m->faces_begin(), f_end = trg_m->faces_end(); f_it != f_end; ++f_it)
        {
          if( point_in_triangle( p, f_it.handle() ) )
          {
            trs.push_back(f_it.handle());
          }
        }
        
        if(!trs.empty())
        { 
          //take projection points for the faces
          std::list<Face_handle>::iterator t_it, t_end;
          for(t_it = trs.begin(), t_end = trs.end(); t_it != t_end; ++t_it)
          {
            points.push_back(project_point(p, *t_it));
          }
          //choose the top one in z direction
          while(points.size() > 1)
          {
              std::list<Point>::iterator i1 = points.begin(), i2 = i1;
              ++i2;
              if(i1->z() > i2->z())
                points.erase(i2);
              else
                points.erase(i1);
          }
          n_it->set_point( points.front() );
        }
        else
        {
            std::cout << "no projection possible\n";
            up_nodes.push_back(n_it.handle());
        }
    }
    //move unprojected nodes z coordinate to the average of its neighbors
    std::list<Node_handle>::iterator un_it, un_end;
    //TODO check for unprojectable nodes displacement and continue until it is larger than treshold. Now done 5 times
    for (int i = 0; i < 5; ++i)
    {
        for (un_it = up_nodes.begin(), un_end = up_nodes.end(); un_it != un_end; ++un_it)
        {
            double z = 0.0;
            int count = 0;
            Mesh_node::Edge_use_iterator eu_it, eu_end;
            for (eu_it = (*un_it)->edge_uses_begin(), eu_end = (*un_it)->edge_uses_end(); eu_it != eu_end; ++eu_it, ++count)
            {
                z += eu_it->other_node()->point().z();
            }
            z /= count;
            Point &p = (*un_it)->point();
            (*un_it)->set_point(Point(p.x(), p.y(),z));
        }
    }
}
//-----------------------------------------------------------------------------
void smooth_mesh_Z( Mesh_handle &m, int r, double d )
{
    if (d < 0.0 ) d = 0.0;
    if (d > 1.0 ) d = 1.0;
    //iterate mesh points and put them at z position which is average of all neighbors (including source point)
    Mesh::Node_iterator n_it, n_end;
    for (int i = 0; i < r; i++)
    {
        for (n_it = m->nodes_begin(), n_end = m->nodes_end(); n_it != n_end; ++n_it)
        {
            Point &p = n_it->point();
            double z = 0.0;
            int count = 0;
            Mesh_node::Edge_use_iterator eu_it, eu_end;
            for (eu_it = n_it->edge_uses_begin(), eu_end = n_it->edge_uses_end(); eu_it != eu_end; ++eu_it, ++count)
            {
                z += eu_it->other_node()->point().z();
            }
            z /= count;
            z -= p.z();
            z *= d;
            n_it->set_point(Point(p.x(), p.y(),p.z()+z));
        }
    }
}
//-----------------------------------------------------------------------------
void scale_mesh( Mesh_handle &m, double sx, double sy, double sz)
{
    Mesh::Node_iterator n_it, n_end;
    for (n_it = m->nodes_begin(), n_end = m->nodes_end(); n_it != n_end; ++n_it)
    {
        Point p = n_it->point();
        n_it->set_point(Point(sx*p.x(), sy*p.y(), sz*p.z()));
    }
}
//-----------------------------------------------------------------------------
void flag_faces_edge_uses(Mesh_handle &m, bool reset)
{
    if (reset)
    {
        Mesh::Edge_use_iterator e_it, e_end;
        for (e_it = m->edge_uses_begin(), e_end = m->edge_uses_end(); e_it != e_end; ++e_it)
            e_it->reset_flag();
    }
    Mesh::Face_iterator it, f_end;
    for (it = m->faces_begin(), f_end = m->faces_end(); it != f_end; ++it)
    {
        it->edge_use()->flag() = it->id();
    }
}
//-----------------------------------------------------------------------------
int remove_short_edges (Mesh_handle &m, double tresh)
{
    //maps edge to a single edge use of it
    std::vector<Edge_use_handle> edge_map(m->size_of_edges(), INVALID_EDGE_USE);
    Mesh::Edge_use_iterator eu_it, eu_end;
    for (eu_it = m->edge_uses_begin(), eu_end = m->edge_uses_end(); eu_it != eu_end; ++eu_it)
    {
        int e_id = eu_it->edge()->id();
        edge_map[e_id] = eu_it.handle();
    }
    
    std::list<Edge_use_handle> remove_list;
    tresh *= tresh;
    Mesh::Edge_iterator e_it, e_end;
    for (e_it = m->edges_begin(), e_end = m->edges_end(); e_it != e_end; ++e_it)
    {
      if(edge_squared_length(e_it.handle()) < tresh)
       remove_list.push_back(edge_map[e_it->id()]); 
    }
    
    flag_faces_edge_uses(m);
    Mesh_manipulator manip(m);
    
    int count = 0;
    std::list<Edge_use_handle>::iterator it, it_end;
    for(it = remove_list.begin(), it_end = remove_list.end(); it != it_end; ++it)
    {
      manip.delete_edge(*it, true);
      ++count;
    }
    return count;
}
//-----------------------------------------------------------------------------
