// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _MESH_UTILS_H_
#define _MESH_UTILS_H_
#include "mesh/mesh.h"
#include "mesh/mesh_handle.h"
#include "utils/types.h"
#include "utils/utility.h"
#include "utils/geometry_utils.h"
#include "utils/la_utils.h"
#include <set>
#undef DEBUG_PRINT
// #define DEBUG_PRINT
const double EPS = 1.0e-24;
class State_interpolator;
template<class ME> class Affine_map_shell;
//-----------------------------------------------------------------------------
void check_mesh_faces_circularity(Mesh_handle m);
//-----------------------------------------------------------------------------
void fix_mesh_faces_circularity(Mesh_handle m);
//-----------------------------------------------------------------------------
void convert_to_spring_format( std::ofstream &out, Mesh_handle const& m, int n_dim, bool ext );
//-----------------------------------------------------------------------------
Mesh_handle import_from_spring_format( std::ifstream &in );
//-----------------------------------------------------------------------------
void write_spreadheet_timestep(std::ofstream &out, Mesh_handle m, State_interpolator const& si, double t);
//-----------------------------------------------------------------------------
//writes to a file stress at given node togetheter with poisson ration and Young modudlus of the material
void write_3d_plot(std::ofstream &out, Node_handle n, State_interpolator const& si, double t, double mu, double Y);
//-----------------------------------------------------------------------------
//writes to a file stress at given integration point of element togetheter with poisson ratio and Young modudlus of the material
void write_3d_plot2(std::ofstream &out, int int_point, Face_handle fh, Affine_map_shell<Mesh_face> &aff_map);
//-----------------------------------------------------------------------------
void write_mesh(Mesh_handle m, std::ofstream &out);
//-----------------------------------------------------------------------------
Point inline midpoint( Edge_use_handle eu )
{
    return eu->node()->point() + Vector( eu->node()->point(), eu->other_node()->point() )*0.5;
}
//-----------------------------------------------------------------------------
Point inline centroid( Edge_use_handle eu )
{
    Mesh_edge_use::circulator eu_cir = eu->loop_begin(), eu_end = eu_cir;
    Vector v = CGAL::NULL_VECTOR;
    size_t nb_points = 0;
    do
    {
        v = v + ( eu_cir->node()->point() - CGAL::ORIGIN );
        ++nb_points;
    } while (++eu_cir != eu_end);
    return CGAL::ORIGIN + v / nb_points;
}
//-----------------------------------------------------------------------------
Point inline centroid( Face_handle f )
{
    return centroid( f->edge_use() );
}

//-----------------------------------------------------------------------------
//check on which side of the plane face is
//-1 negative side
//0 in plane
//1 positive side
//2 crossing(both sides)
int inline side_of_plane( Face_handle f, Point const &p, Vector const& n )
{
    bool pos = false;
    bool neg = false;
    double d = n*( CGAL::ORIGIN - p );
#ifdef DEBUG_PRINT
    std::cout << "p(" << p << "); n(" << n << "); d = " << d << "; n_edge = " << f->size_of_edges() << "\n";
#endif
    Mesh_face::Edge_use_circulator eu_cir = f->edge_uses_begin(), eu_end = eu_cir;
    int counter = 0;
    do
    {
//      Vector v(p, eu_cir->node()->point() );
        double res = n*( eu_cir->node()->point() - CGAL::ORIGIN ) + d;
#ifdef DEBUG_PRINT
        std::cout  << "vec = " << ( eu_cir->node()->point() - CGAL::ORIGIN ) << " ;point(" << eu_cir->node()->point() << ") ;res = " << res << "\n";
#endif
//      if(res > CELL_EPS)
        if (res > PROXIMITY_THRESHOLD)
            pos = true;
//      if(res < -CELL_EPS)
        if (res < -PROXIMITY_THRESHOLD)
            neg = true;
        ++counter;
    } while (++eu_cir != eu_end && eu_cir != INVALID_EDGE_USE && counter < 10);
    if (counter == 10 )
        wait_kb_hit();
    if (pos)
    {
        if (neg)
            return 2;
        else
            return 1;
    }
    else if (neg)
        return -1;
    else
        return 0;
}
//-----------------------------------------------------------------------------
//returns vector normal to the plane given by two first edges of the face. It is true normal for triangular faces but not for nonplanar polygonal faces
inline Vector normal(Face_handle f)
{
    Mesh_face::Edge_use_circulator circ = f->edge_uses_begin();
    Vector vec1 = Vector(circ->node()->point(), circ->other_node()->point());
    ++circ;
    Vector vec2 = Vector(circ->node()->point(), circ->other_node()->point());
    Vector normal = cross_product(vec1, vec2);
    normalize(normal);
    return normal;
}
//-----------------------------------------------------------------------------
//returns vector normal at the mesh node which is average of normals of all the faces connected to this node. It is intendendent to use for surface triangular meshes. Oherwise it may not give accurate result
inline Vector normal(Node_handle n)
{
    Vector normal(0,0,0);
    Mesh_node::Edge_use_iterator eu_it = n->edge_uses_begin(), eu_end = n->edge_uses_end();
    for (;eu_it != eu_end; ++eu_it)
    {
        Vector vec1 = Vector(eu_it->node()->point(), eu_it->other_node()->point());
        Edge_use_handle next_eu = eu_it->loop_next();
        Vector vec2 = Vector(next_eu->node()->point(), next_eu->other_node()->point());
        Vector temp = cross_product(vec1, vec2);
        if ((temp*normal) < 0.0)
            normal = normal - temp;
        else
            normal = normal + temp;
    }
    return normal;
}
//-----------------------------------------------------------------------------
//returns vector normal at the mesh edge use. It is average of the normals for the faces joined at this edge
inline Vector normal(Edge_use_handle eu)
{
    Vector normal(0,0,0);

    Mesh_edge_use::circulator circ = eu->radial_begin(), start  = circ;
    do {
        Mesh_edge_use::circulator loop_c = circ->loop_begin(), loop_start = loop_c;
        Vector temp(0,0,0);
        do {
            Vector vec1 = Vector(loop_c->node()->point(), loop_c->other_node()->point());
            Edge_use_handle next_eu = loop_c->loop_next();
            Vector vec2 = Vector(next_eu->node()->point(), next_eu->other_node()->point());
            temp = cross_product(vec1, vec2);
            if (++loop_c == loop_start)
            {
//                 cell_error("Unable to determine normal of the edge_use.\n");
              std::cout << "Unable to determine normal of the edge_use.\n";
              return Vector(0,0,0);
            }
        } while (temp.squared_length() < EPS );
        normalize(temp);
        normal = normal + temp;
    } while (++circ != start);
    normalize(normal);
    return normal;
}
//-----------------------------------------------------------------------------
void inline check_nodes(Mesh_handle m)
{
    Mesh::Node_iterator n_it, n_end;
    for (n_it = m->nodes_begin(), n_end = m->nodes_end(); n_it != n_end; ++n_it)
    {
        bool ch = n_it->check_node_edges();
        if (!ch)
        {
            wait_kb_hit();
        }
    }
}
//-----------------------------------------------------------------------------
void inline faces_statistics(Mesh_handle m, bool det = true)
{
    Mesh::Face_iterator f_it, f_end;
    for (f_it = m->faces_begin(), f_end = m->faces_end(); f_it != f_end; ++f_it)
    {
        f_it->statistics(det);
    }
}
//-----------------------------------------------------------------------------
void inline nodes_statistics(Mesh_handle m, bool det = true)
{
    Mesh::Node_iterator n_it, n_end;
    for (n_it = m->nodes_begin(), n_end = m->nodes_end(); n_it != n_end; ++n_it)
    {
        n_it->statistics(det);
    }
}
//-----------------------------------------------------------------------------
double inline edge_squared_length(Edge_handle e)
{
  Vector v(e->node0()->point(), e->node1()->point());
  return v.squared_length();
}
//-----------------------------------------------------------------------------
//version for handle iterator
template<class II, class OI, class PRED>
void select(II from, II beyond, OI o_it, PRED const& pred)
{
    while ( from != beyond )
    {
//         std::cout << "node: " << from->point() << "\n";
        if (pred(from))
            o_it = from;
        ++from;
    }
}
//-----------------------------------------------------------------------------
//version for normal iterator
template<class II, class OI, class PRED>
void select2(II from, II beyond, OI o_it, PRED const& pred)
{
    while ( from != beyond )
    {
//         std::cout << "node: " << from->point() << "\n";
        if (pred(*from))
            o_it = *from;
        ++from;
    }
}
//-----------------------------------------------------------------------------
struct Less_id
{
  template<class H>
    bool operator()(H h1, H h2)
    {
        return h1->id() < h2->id();
    }
};
//-----------------------------------------------------------------------------
template<class II, class OI>
void select_edge_use_nodes(II from, II beyond, OI o_it)
{
  std::set<Node_handle,Less_id> nodes;
    while ( from != beyond )
    {
      nodes.insert((*from)->node());
      nodes.insert((*from)->other_node());
        ++from;
    }
    std::set<Node_handle,Less_id>::iterator n_it,n_end;
    for(n_it = nodes.begin(), n_end = nodes.end(); n_it != n_end; ++n_it)
    {
      *o_it = *n_it;
    }
}
//-----------------------------------------------------------------------------
///finds an edge use which starts at the ending node of given edge_use chosen from the range between iterators. Iterators point to Edge_use_handles
template<class I>
I find_connected_edge_use(Edge_use_handle const& eu, I from, I beyond)
{
  do{
    if(eu->other_node() == (*from)->node())
      return from;
  }while(++from != beyond);
  return from;
}
//-----------------------------------------------------------------------------
///Project a point onto surface mesh (parallel projection in z direction)
inline Point parallel_project( Point const& p, Mesh_handle surf_m)
{
  
}
//-----------------------------------------------------------------------------
///checks if given 2D point is within triangle (only x, y coordinated used)
inline bool point_in_triangle(Point const & p, Face_handle f)
{
  Mesh_face::Edge_use_const_circulator ec = f->edge_uses_begin();
  
  Point const & a = ec->node()->point();
  Point const & b = (++ec)->node()->point();
  Point const & c = (++ec)->node()->point();
  
  assert(++ec == f->edge_uses_begin());
  double ux = b.x()-a.x(), uy = b.y()-a.y();
  double vx = c.x()-a.x(), vy = c.y()-a.y();
  double wx = p.x()-a.x(), wy = p.y()-a.y();
  
  //inverse of v x u
  double inv_den = 1.0/(vx*uy-vy*ux);
  double r = (vx*wy-vy*wx) * inv_den;
  double t = (wx*uy-wy*ux) * inv_den;
  
  return (r >= 0) && (t >= 0) && (r + t < 1);
}
//-----------------------------------------------------------------------------
///Projects 2D point on 3D triangle along z axis (no checks of inclusion)
inline Point project_point( Point const & p, Face_handle f )
{
  Mesh_face::Edge_use_const_circulator ec = f->edge_uses_begin();
  
  Point const & a = ec->node()->point();
  Point const & b = (++ec)->node()->point();
  Point const & c = (++ec)->node()->point();
  
  assert(++ec == f->edge_uses_begin());
  
  double vx   = p.x()-a.x(), vy   = p.y()-a.y();
  double v21x = b.x()-a.x(), v21y = b.y()-a.y(), v21z = b.z()-a.z();
  double v31x = c.x()-a.x(), v31y = c.y()-a.y(), v31z = c.z()-a.z();
  
  double z = (vx*(v21z*v31y-v21y*v31z) + vy*(v21x*v31z-v21z*v31x))/(v21x*v31y-v21y*v31x) + a.z();
  
  return Point(p.x(), p.y(), z);
}
//-----------------------------------------------------------------------------
//projects source mesh on the target triangular mesh along z axis
void project_on_triangular_mesh( Mesh_handle &src_m, Mesh_handle const& trg_m );
//-----------------------------------------------------------------------------
//smooth mesh in z direction. r is repetitions, d is a scale factor in range (0,1)
void smooth_mesh_Z( Mesh_handle &m, int r = 1, double d = 1.0);
//-----------------------------------------------------------------------------
//scales the mesh in x, y, z directions
void scale_mesh( Mesh_handle &m, double sx, double sy, double sz);
//-----------------------------------------------------------------------------
//returns a vector which is an average of all node directors (interpolated director to the face center?)
inline Vector3 average_director(Face_handle f)
{
    Vector3 d(0,0,0);

    Mesh_face::Edge_use_const_circulator circ = f->edge_uses_begin(), start  = circ;
    do {    
      
        d += normalized(circ->node()->director());
    } while (++circ != start);
    d.unit();
    return d;
}
//flags edge uses definig the faces with the face id
//-----------------------------------------------------------------------------
void flag_faces_edge_uses(Mesh_handle &m, bool reset = true);
//-----------------------------------------------------------------------------
//remove edges shorter than a given treshold. returns number of removed edges
int remove_short_edges (Mesh_handle &m, double tresh);
//-----------------------------------------------------------------------------
#endif
