// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "mesh_generator/mesh_generator.h"
// #include "mesh/boundary_builder.h"
#include "utils/string_utils.h"
#include "utils/utility.h"
// #include "cell/tissue.h"
#include "cell/cell_handle.h"
#include <map>

using namespace cell;
//-----------------------------------------------------------------------------
Mesh_generator* Mesh_generator::m_instance = 0;// initialize pointer

Mesh_generator* Mesh_generator::get_instance(/* Surface_transform * t */)
{
    if (m_instance == 0)
    {
        m_instance = new Mesh_generator( );
    }
    return m_instance;
}
//-----------------------------------------------------------------------------
Mesh_handle  Mesh_generator::project( Voronoi2D const& c_v, Reference_points &points, Surface_transform const*tr )
{
    Voronoi2D &v = const_cast< Voronoi2D& >(c_v);
    points.resize( v.number_of_vertices() );
    Mesh_handle  m = new Mesh();
    Mesh_manipulator manip(m);
    Point_node_map pn_map;

    pn_map.clear();
    Voronoi2D::Inner_faces_iterator f_iter, f_end;
    for ( f_iter = v.inner_faces_begin(), f_end = v.inner_faces_end(); f_iter != f_end; ++f_iter )
    {
        if ((*f_iter)->is_unbounded())
            continue;

        Voronoi2D::Face::Ccb_halfedge_circulator h_start  = (*f_iter)->outer_ccb();
        Voronoi2D::Face::Ccb_halfedge_circulator h_circ = h_start;
        std::list<Node_handle> node_list;
        do
        {
            //add mesh node
            Point pt =  tr->transform( h_circ->target()->point() );
            Node_handle n = insert_point(  pt , manip, pn_map );

            cell_assert( n->id()<points.size() );
            points[n->id()] = h_circ->target()->point();

            node_list.push_back(n);

        } while ( ++h_circ != h_start );
        Face_handle f = manip.add_polygon( node_list.begin(), node_list.end() );
    }
    points.resize( m->size_of_nodes() );
    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle  Mesh_generator::project( Voronoi2D const& c_v, Reference_points &points, Surface_transform const*tr, double z_tresh )
{
    Voronoi2D &v = const_cast< Voronoi2D& >(c_v);
    points.resize( v.number_of_vertices() );
    Mesh_handle  m = new Mesh();
    Mesh_manipulator manip(m);
    Point_node_map pn_map;

    pn_map.clear();
    Voronoi2D::Inner_faces_iterator f_iter, f_end;
    for ( f_iter = v.inner_faces_begin(), f_end = v.inner_faces_end(); f_iter != f_end; ++f_iter )
    {
        if ((*f_iter)->is_unbounded())
            continue;
        Voronoi2D::Face::Ccb_halfedge_circulator h_start  = (*f_iter)->outer_ccb();
        Voronoi2D::Face::Ccb_halfedge_circulator h_circ = h_start;
        bool skip = true;
        //check if face poits are above z treshold
        do
        {
            Point pt =  tr->transform( h_circ->target()->point() );
            if (pt.z() > z_tresh)
            {
                skip = false;
                break;
            }
        } while ( ++h_circ != h_start );
        //skip the face if below z treshold
        if ( skip )
            continue;
        //start over halfedge circulators
        h_circ = h_start;
        std::list<Node_handle> node_list;
        do
        {
            Point pt =  tr->transform( h_circ->target()->point() );
            Node_handle n = insert_point(  pt , manip, pn_map );
            cell_assert( n->id()<points.size() );
            points[n->id()] = h_circ->target()->point();
            node_list.push_back(n);
        } while ( ++h_circ != h_start );
        Face_handle f = manip.add_polygon( node_list.begin(), node_list.end() );
    }
    points.resize( m->size_of_nodes() );
    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle  Mesh_generator::project( Mesh_handle in_m, Surface_transform const*tr)
{
    Mesh_handle  m = new Mesh( *in_m );

    Mesh::Node_iterator n_iter, n_end;
    for ( n_iter = m->nodes_begin(), n_end = m->nodes_end(); n_iter != n_end; ++n_iter )
    {
        Point p = tr->transform( n_iter->point());
        n_iter->set_point(p);
    }

    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::project_on_surface_mesh( Mesh_handle in_m, Mesh_handle surf_m )
{
    Mesh_handle  m = new Mesh( *in_m );

    Mesh::Node_iterator n_iter, n_end;
    for ( n_iter = m->nodes_begin(), n_end = m->nodes_end(); n_iter != n_end; ++n_iter )
    {
        Point p = parallel_project( n_iter->point(), surf_m);
        n_iter->set_point(p);
    }

    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_cylindrical_boundary( Voronoi2D const& v, double r)
{
    //  points.resize( v.number_of_vertices() );
    Mesh_handle  m = new Mesh();
    Mesh_manipulator manip(m);
    Point_node_map pn_map;
    pn_map.clear();

    Voronoi2D::Inner_faces_iterator f_iter, f_end;
    for ( f_iter = v.inner_faces_begin(), f_end = v.inner_faces_end(); f_iter != f_end; ++f_iter )
    {
        if ((*f_iter)->is_unbounded())
            continue;

        Voronoi2D::Face::Ccb_halfedge_circulator h_start  = (*f_iter)->outer_ccb();
        Voronoi2D::Face::Ccb_halfedge_circulator h_circ = h_start;
        std::list<Node_handle> node_list;
        do
        {
            //add mesh node
            Point_2 const& org_pt = h_circ->target()->point();
            double phi = org_pt.y()/r;
            Point pt(org_pt.x(), r*sin(phi), r*cos(phi));
            Node_handle n = insert_point(  pt , manip, pn_map );

            node_list.push_back(n);

        } while ( ++h_circ != h_start );

        Face_handle f = manip.add_polygon( node_list.begin(), node_list.end() );

    }
    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_cylindrical_boundary( Mesh_handle m, double r )
{
    Mesh_handle mesh = new Mesh();
    Mesh_manipulator manip(mesh);
    Point_node_map pn_map;
    pn_map.clear();

    Mesh::Face_iterator f_iter, f_end;
    for ( f_iter = m->faces_begin(), f_end = m->faces_end(); f_iter != f_end; ++f_iter )
    {
        Mesh_face::Edge_use_circulator h_start = f_iter->edge_uses_begin();
        Mesh_face::Edge_use_circulator h_circ = h_start;
        std::list<Node_handle> node_list;
        do
        {
            //add mesh node
            Point const& org_pt = h_circ->node()->point();
            double phi = org_pt.y()/r;
            Point pt(org_pt.x(), r*sin(phi), r*cos(phi));
            Node_handle n = insert_point(  pt , manip, pn_map );
            node_list.push_back(n);

        } while ( ++h_circ != h_start );

        Face_handle f = manip.add_polygon( node_list.begin(), node_list.end() );

    }
    return mesh;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_quad_cylinder(double r, double h, int n_r, int n_h)
{
    Mesh_handle mesh = new Mesh();
    Mesh_manipulator manip(mesh);
    double d_theta = 2*PI/n_r;
    double theta = 0;
    std::vector<Node_handle> *top_nodes, *bottom_nodes;
    std::vector<Node_handle> nodes1(n_r); //top circle nodes
    std::vector<Node_handle> nodes2(n_r); //bottom circle nodes
    top_nodes = &nodes1;
    bottom_nodes = &nodes2;
    for (int i = 0; i < n_r; ++i)
    {
        double x = r*sin(theta);
        double y = r*cos(theta);
//       circle[i] = Point(x,y,0);
        Node_handle n_h = manip.add_node(Point(x,y,0));
        nodes1[i] = n_h;
        theta += d_theta;
    }
    //add quads layer by layer in z direction
    double dz = h/n_h;
    for (int j = 0; j < n_h; ++j)
    {
        for (int i = 0; i < n_r; ++i)
        {
            Node_handle n1_1 = top_nodes->operator[](i), n1_2;
            Node_handle n2_1, n2_2;
            if (i < n_r-1)
                n1_2 = top_nodes->operator[](i+1);
            else
                n1_2 = top_nodes->operator[](0);

            Point const& p1 = n1_1->point();
            Point const& p2 = n1_2->point();
            if (i == 0)
            {
                n2_1 = manip.add_node(Point(p1.x(), p1.y(), p1.z()+dz));
                bottom_nodes->operator[](i) = n2_1;
            }
            else
                n2_1 = bottom_nodes->operator[](i);

            if (i < n_r-1)
            {
                n2_2 = manip.add_node(Point(p2.x(), p2.y(), p2.z()+dz));
                bottom_nodes->operator[](i+1) = n2_2;
            }
            else
                n2_2 = bottom_nodes->operator[](0);
            manip.add_quad(n1_1, n1_2, n2_2, n2_1);
        }
        //swap bottom and top nodes space;
        std::vector<Node_handle> *temp;
        temp = top_nodes;
        top_nodes = bottom_nodes;
        bottom_nodes = temp;
    }
    return mesh;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_staggered_rectangle_cylinder(double r, double h, int n_r, int n_h)
{
    Mesh_handle mesh = new Mesh();
    Mesh_manipulator manip(mesh);
    double d_theta = 2*PI/n_r;
    double theta = 0;
    std::vector<Node_handle> *top_nodes, *mid_nodes,*bot_nodes;
    std::vector<Node_handle> nodes1(n_r); //top circle nodes
    std::vector<Node_handle> nodes2(n_r); //middle circle nodes
    std::vector<Node_handle> nodes3(n_r); //bottom circle nodes
    top_nodes = &nodes1;
    mid_nodes = &nodes2;
    bot_nodes = &nodes3;

    double dz = h/n_h;
    for (int i = 0; i < n_r; ++i)
    {
        double x = r*sin(theta);
        double y = r*cos(theta);

        top_nodes->operator[](i) = manip.add_node(Point(x,y,0));
        theta += d_theta;
    }
    //creating bottom cap
    for (int i = 0; i != top_nodes->size(); ++i)
        mid_nodes->operator[](i) = top_nodes->operator[](i);

    const double rad_div = 2*n_r;
    double d_phi = 2*PI/(rad_div+4);
    double phi = d_phi;
    for (int j = 0; j < rad_div/4; ++j)
    {
        theta = 0;
        double rr = r*cos(phi);
        double z = r*sin(phi);
        for (int i = 0; i < n_r; ++i)
        {
            double x = rr*sin(theta);
            double y = rr*cos(theta);
            bot_nodes->operator[](i) = manip.add_node(Point(x, y, -z));
            theta += d_theta;
        }
        for (int i = 0; i < n_r; ++i)
        {
            int ii = i+1;
            if (i == n_r-1)
                ii = 0;
            manip.add_quad(mid_nodes->operator[](i), mid_nodes->operator[](ii), bot_nodes->operator[](ii), bot_nodes->operator[](i));
        }
        phi += d_phi;
        std::vector<Node_handle> *temp;
        temp = bot_nodes;
        bot_nodes = mid_nodes;
        mid_nodes = temp;
    }
    //add triangles at the bottom
    Node_handle nod = manip.add_node(Point(0, 0, -r));
    for (int i = 0; i < n_r; ++i)
    {
        int ii = i+1;
        if (i == n_r-1)
            ii = 0;
        manip.add_triangle(mid_nodes->operator[](i), mid_nodes->operator[](ii), nod);
    }
    //reassign the pointers
    mid_nodes = &nodes2;
    bot_nodes = &nodes3;
    for (int i = 0; i < n_r; ++i)
    {
        double x = top_nodes->operator[](i)->point().x();
        double y = top_nodes->operator[](i)->point().y();
        mid_nodes->operator[](i) = manip.add_node(Point(x,y,dz));
    }
    //add half rectangles in a first layer
    for (int i = 1; i < n_r; i+=2)
    {
        int ii = i+1;
        if (i == n_r-1)
            ii = 0;
        manip.add_quad(top_nodes->operator[](i), top_nodes->operator[](ii), mid_nodes->operator[](ii), mid_nodes->operator[](i));
    }
    //add rectangles layer by layer in z direction
    for (int j = 0; j < n_h-1; ++j) //half layer steps
    {
        for (int i = 0; i < n_r; ++i)
        {
            Point const& p = mid_nodes->operator[](i)->point();
            bot_nodes->operator[](i) = manip.add_node(Point(p.x(), p.y(), p.z()+dz));
        }
        for (int i = 0; i < n_r; ++i) //go around circle
        {
            if ( ((i+j)&1) == 0)
            {
                int ii = i+1;
                if (i == n_r-1)
                    ii = 0;
                std::list<Node_handle> polygon_list;
                polygon_list.push_back(top_nodes->operator[](i));
                polygon_list.push_back(top_nodes->operator[](ii));
                polygon_list.push_back(mid_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](i));
                polygon_list.push_back(mid_nodes->operator[](i));
                manip.add_polygon(polygon_list.begin(), polygon_list.end());
            }
        }
        //swap bottom, middle and top nodes space;
        std::vector<Node_handle> *temp;
        temp = top_nodes;
        top_nodes = mid_nodes;
        mid_nodes = bot_nodes;
        bot_nodes = temp;
    }
    //add half rectangles in a last layer
    for (int i = 1; i < n_r; i+=2)
    {
        int ii = i+1;
        if (i == n_r-1)
            ii = 0;
        manip.add_quad(top_nodes->operator[](i), top_nodes->operator[](ii), mid_nodes->operator[](ii), mid_nodes->operator[](i));
    }

    //creating top cap
    phi = d_phi;
    for (int j = 0; j < rad_div/4; ++j)
    {
        theta = 0;
        double rr = r*cos(phi);
        double z = r*sin(phi);
        for (int i = 0; i < n_r; ++i)
        {
            double x = rr*sin(theta);
            double y = rr*cos(theta);
            bot_nodes->operator[](i) = manip.add_node(Point(x, y, h+z));
            theta += d_theta;
        }
        for (int i = 0; i < n_r; ++i)
        {
            int ii = i+1;
            if (i == n_r-1)
                ii = 0;
            manip.add_quad(mid_nodes->operator[](i), mid_nodes->operator[](ii), bot_nodes->operator[](ii), bot_nodes->operator[](i));
        }
        phi += d_phi;
        std::vector<Node_handle> *temp;
        temp = bot_nodes;
        bot_nodes = mid_nodes;
        mid_nodes = temp;
    }
    //add triangles at the bottom
    nod = manip.add_node(Point(0, 0, h+r));
    for (int i = 0; i < n_r; ++i)
    {
        int ii = i+1;
        if (i == n_r-1)
            ii = 0;
        manip.add_triangle(mid_nodes->operator[](i), mid_nodes->operator[](ii), nod);
    }
    return mesh;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_staggered_rectangle_cylinder2(double r, double h, int n_r, int n_h)
{
    Mesh_handle mesh = new Mesh();
    Mesh_manipulator manip(mesh);
    double d_theta = 2*PI/n_r;
    double theta = 0;
//     std::vector<Point> circle(n_r); //points on the circle
    std::vector<Node_handle> *top_nodes, *mid_nodes, *bot_nodes, *m_top_nodes, *m_mid_nodes, *m_bot_nodes;
    std::vector<Node_handle> init_nodes(n_r); //initial circle nodes
    std::vector<Node_handle> nodes1(n_r); //top circle nodes
    std::vector<Node_handle> nodes2(n_r); //middle circle nodes
    std::vector<Node_handle> nodes3(n_r); //bottom circle nodes
    std::vector<Node_handle> m_nodes1(n_r); //minus top circle nodes
    std::vector<Node_handle> m_nodes2(n_r); //minus middle circle nodes
    std::vector<Node_handle> m_nodes3(n_r); //minus bottom circle nodes
    top_nodes = &nodes1;
    mid_nodes = &nodes2;
    bot_nodes = &nodes3;
    m_top_nodes = &m_nodes1;
    m_mid_nodes = &m_nodes2;
    m_bot_nodes = &m_nodes3;

    double dz = 0.5*h/n_h;
    for (int i = 0; i < n_r; ++i)
    {
        double x = r*sin(theta);
        double y = r*cos(theta);

        init_nodes[i] = manip.add_node(Point(x,y,0));
        top_nodes->operator[](i) = init_nodes[i];
        m_top_nodes->operator[](i) = init_nodes[i];
        theta += d_theta;
    }
//reassign the pointers
    for (int i = 0; i < n_r; ++i)
    {
        double x = top_nodes->operator[](i)->point().x();
        double y = top_nodes->operator[](i)->point().y();
        mid_nodes->operator[](i) = manip.add_node(Point(x,y,dz));
        m_mid_nodes->operator[](i) = manip.add_node(Point(x,y,-dz));
    }

    //add mid cells
    for (int i = 0; i < n_r; ++i) //go around circle
    {
        if ( ((i)&1) != 0)
        {
            int ii = i+1;
            if (i == n_r-1)
                ii = 0;
            std::list<Node_handle> polygon_list;
            polygon_list.push_back(top_nodes->operator[](i));
            polygon_list.push_back(mid_nodes->operator[](i));
            polygon_list.push_back(mid_nodes->operator[](ii));
            polygon_list.push_back(top_nodes->operator[](ii));
            polygon_list.push_back(m_mid_nodes->operator[](ii));
            polygon_list.push_back(m_mid_nodes->operator[](i));

            manip.add_polygon(polygon_list.begin(), polygon_list.end());
        }
    }
    //add rectangles layer by layer starting from 0 line symetrically in z direction
    int j = 0;
    for (j = 0; j < n_h-1; ++j) //half layer steps
    {
        for (int i = 0; i < n_r; ++i)
        {
            Point const& p = mid_nodes->operator[](i)->point();
            bot_nodes->operator[](i) = manip.add_node(Point(p.x(), p.y(), p.z()+dz));
            Point const& m_p = m_mid_nodes->operator[](i)->point();
            m_bot_nodes->operator[](i) = manip.add_node(Point(m_p.x(), m_p.y(), m_p.z()-dz));
        }
        for (int i = 0; i < n_r; ++i) //go around circle
        {
            if ( ((i+j)&1) == 0)
            {
                int ii = i+1;
                if (i == n_r-1)
                    ii = 0;
                std::list<Node_handle> polygon_list;
                polygon_list.push_back(top_nodes->operator[](i));
                polygon_list.push_back(top_nodes->operator[](ii));
                polygon_list.push_back(mid_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](i));
                polygon_list.push_back(mid_nodes->operator[](i));
                manip.add_polygon(polygon_list.begin(), polygon_list.end());

                polygon_list.clear();
                polygon_list.push_back(m_top_nodes->operator[](i));
                polygon_list.push_back(m_top_nodes->operator[](ii));
                polygon_list.push_back(m_mid_nodes->operator[](ii));
                polygon_list.push_back(m_bot_nodes->operator[](ii));
                polygon_list.push_back(m_bot_nodes->operator[](i));
                polygon_list.push_back(m_mid_nodes->operator[](i));
                manip.add_polygon(polygon_list.begin(), polygon_list.end());
            }
        }
        //swap bottom, middle and top nodes space;
        std::vector<Node_handle> *temp;
        temp = top_nodes;
        top_nodes = mid_nodes;
        mid_nodes = bot_nodes;
        bot_nodes = temp;
        temp = m_top_nodes;
        m_top_nodes = m_mid_nodes;
        m_mid_nodes = m_bot_nodes;
        m_bot_nodes = temp;
    }

    int pivot = (j+1)&1;
    //add caps
    double d_phi = dz/r;
    double dd_phi = 0;
    double phi_tresh = 0.5*PI-PI/180.0*5;
    double d_phi_tresh = PI/180.0*5; 

    double phi = d_phi;
    for (j = 0; phi < phi_tresh; ++j, phi += d_phi, d_phi -= dd_phi)
    {
        theta = 0;
        double rr = r*cos(phi);
        double z = r*sin(phi);
        for (int i = 0; i < n_r; ++i)
        {
            double x = rr*sin(theta);
            double y = rr*cos(theta);
            Point const& p = mid_nodes->operator[](i)->point();
            bot_nodes->operator[](i) = manip.add_node(Point(x, y, 0.5*h+z));
            Point const& m_p = m_mid_nodes->operator[](i)->point();
            m_bot_nodes->operator[](i) = manip.add_node(Point(x, y, -0.5*h-z));
            theta += d_theta;
        }
        for (int i = 0; i < n_r; ++i) //go around circle
        {
            if ( ((i+j)&1) != pivot)
            {
                int ii = i+1;
                if (i == n_r-1)
                    ii = 0;
                std::list<Node_handle> polygon_list;
                polygon_list.push_back(top_nodes->operator[](i));
                polygon_list.push_back(top_nodes->operator[](ii));
                polygon_list.push_back(mid_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](ii));
                polygon_list.push_back(bot_nodes->operator[](i));
                polygon_list.push_back(mid_nodes->operator[](i));
                manip.add_polygon(polygon_list.begin(), polygon_list.end());

                polygon_list.clear();
                polygon_list.push_back(m_top_nodes->operator[](i));
                polygon_list.push_back(m_top_nodes->operator[](ii));
                polygon_list.push_back(m_mid_nodes->operator[](ii));
                polygon_list.push_back(m_bot_nodes->operator[](ii));
                polygon_list.push_back(m_bot_nodes->operator[](i));
                polygon_list.push_back(m_mid_nodes->operator[](i));
                manip.add_polygon(polygon_list.begin(), polygon_list.end());
            }
        }
        //swap bottom, middle and top nodes space;
        if(d_phi > d_phi_tresh)
          d_phi = 0.5*d_theta*rr/r;
        std::vector<Node_handle> *temp;
        temp = top_nodes;
        top_nodes = mid_nodes;
        mid_nodes = bot_nodes;
        bot_nodes = temp;
        temp = m_top_nodes;
        m_top_nodes = m_mid_nodes;
        m_mid_nodes = m_bot_nodes;
        m_bot_nodes = temp;
    }
    //add half rectangles in a last layer
    for (int i = (j+1)&1; i < n_r; i+=2)
    {
        int ii = i+1;
        if (i == n_r-1)
            ii = 0;
        manip.add_quad(top_nodes->operator[](i), top_nodes->operator[](ii), mid_nodes->operator[](ii), mid_nodes->operator[](i));
        manip.add_quad(m_top_nodes->operator[](i), m_top_nodes->operator[](ii), m_mid_nodes->operator[](ii), m_mid_nodes->operator[](i));
    }

    Node_handle top_n = manip.add_node(Point(0, 0, 0.5*h+r));
    Node_handle m_top_n = manip.add_node(Point(0, 0, -0.5*h-r));
    int const N = 8;
    int start_i[N+1];
    int end_i[N];
    for (int i = 0 ;i < N; ++i)
    {
        start_i[i] = ceil(i*n_r/double(N));
        end_i[i] = ceil((i+1)*n_r/double(N));
    }
    start_i[N] = n_r;
    end_i[N-1] = 0;

    for (int n = 0; n < N; ++n)
    {
        std::list<Node_handle> polygon_list;
        std::list<Node_handle> m_polygon_list;
        polygon_list.push_back(top_n);
        m_polygon_list.push_back(m_top_n);
        for (int i = start_i[n]; i < start_i[n+1]; ++i)
        {
            polygon_list.push_back(mid_nodes->operator[](i));
            m_polygon_list.push_back(m_mid_nodes->operator[](i));
        }
        polygon_list.push_back(mid_nodes->operator[](end_i[n]));
        m_polygon_list.push_back(m_mid_nodes->operator[](end_i[n]));
        manip.add_polygon(polygon_list.begin(), polygon_list.end());
        manip.add_polygon(m_polygon_list.begin(), m_polygon_list.end());
    }
    //add a top division
    return mesh;
}
//-----------------------------------------------------------------------------
Mesh_handle  Mesh_generator::create_quad_mesh(double a, double r)
{
    Mesh_handle m = new Mesh();
    //  Quad_builder *qb = Quad_builder::get_instance(m);
    Mesh_manipulator manip(m);

    std::vector<Node_handle> nodes(4);

    std::map<std::pair<int,int>, Node_handle> node_map;
    double left = -a/2.0;
    double right = a/2.0;
    double up = a/2.0;
    double down = -a/2.0;

    bool go = true;
    int i = 0;
    bool go_left = true;
    bool go_right = true;
    while (go)
    {
        for (int alt = -1; alt <= 1; alt+=2)
        {
            int pos = 2*alt*i;
            double d_left = 0.5*a*(pos+1);
            double d_right = 0.5*a*(pos-1);

            if (alt == -1)
            {
                if ( sqrt(d_left*d_left + down*down) > r || sqrt(d_left*d_left + up*up) > r )
                {
                    go_left = false;
                    continue;
                }
            }
            else
            {
                if ( sqrt(d_right*d_right + down*down) > r || sqrt(d_right*d_right + up*up) > r )
                {
                    go_right = false;
                    continue;
                }
            }

            add_stripe( m, manip, pos, a, r, nodes, node_map );
            if (i ==0)
                break;
        }
        go = go_right || go_left;
        ++i;
    }
    return m;
}
//-----------------------------------------------------------------------------
//quad of size 'a' mesh of 'r' rows and 'c' columns
Mesh_handle Mesh_generator::create_quad_mesh(double a, int r, int c)
{
    Mesh_handle m = new Mesh();
    Mesh_manipulator manip(m);
    std::vector<Node_handle> nodes(4);

    std::map<std::pair<int,int>, Node_handle> node_map;

    int alt = -1;
    int alt2 = -1;
    for (int rows = 0; rows < r; ++rows)
    {
        int i = (rows+1)/2;
        int pos = 2*alt*i;
        alt *= -1;
        for (int cols = 0; cols < c; ++cols)
        {
            int j = (cols+1)/2;
            int pos2 = 2*alt2*j;
            alt2 *= -1;
            int i_up = pos2+1;
            int i_down = pos2-1;
            add_quad(m, manip, pos, pos2, a, nodes, node_map);
        }
    }
    return m;
}
//-----------------------------------------------------------------------------
//create circular stripe of n quads of width a with inner radius r
Mesh_handle Mesh_generator::create_quad_circle(double a, double r, int n)
{
    double R = r+a;
    double d_phi = 2.0*PI/n;
    Mesh_handle m = new Mesh();
    Mesh_manipulator manip(m);
    std::vector<Node_handle> nodes;
    nodes.reserve(2*n);
    Node_handle node;

    double phi = 0;
    for (int i = 0; i < n; ++i)
    {
        double s = sin(phi);
        double c = cos(phi);
        node = manip.add_node(Point(r*c, r*s, 0.0));
        nodes.push_back(node);
        node = manip.add_node(Point(R*c, R*s, 0.0));
        nodes.push_back(node);

        phi += d_phi;
    }

    for (int i = 0; i < n-1; ++i)
    {
        //circular order of nodes
        manip.add_quad(nodes[2*i], nodes[2*i+1], nodes[2*(i+1)+1], nodes[2*(i+1)]);
    }
    manip.add_quad(nodes[2*(n-1)], nodes[2*(n-1)+1], nodes[1], nodes[0]);
    return m;
}
//-----------------------------------------------------------------------------
void Mesh_generator::add_quad(Mesh_handle m, Mesh_manipulator &manip, int pos, int pos2, double a, std::vector<Node_handle> &n, std::map<std::pair<int,int>, Node_handle> &node_map)
{
    int i_left = pos+1;
    int i_right = pos-1;

    int i_up = pos2+1;
    int i_down = pos2-1;

    n[0] = find_node(i_right, i_down, a, node_map, m, manip);
    n[1] = find_node(i_left, i_down, a, node_map, m, manip);
    n[2] = find_node(i_left, i_up, a, node_map, m, manip);
    n[3] = find_node(i_right, i_up, a, node_map, m, manip);

    manip.add_polygon(n.begin(), n.end());
}
//-----------------------------------------------------------------------------
void Mesh_generator::add_stripe(Mesh_handle m, Mesh_manipulator &qb, int pos, double aa, double r, std::vector<Node_handle> &nodes, std::map<std::pair<int,int>, Node_handle> &node_map)
{
    double a = aa * 0.5;
    int i_left = pos+1;
    int i_right = pos-1;
    double d_left = a*i_left;
    double d_right = a*i_right;

    bool go = true;
    int i = 0;
    bool go_up = true;
    bool go_down = true;
    while (go)
    {
        for (int alt = -1; alt <= 1; alt+=2)
        {
            int pos2 = 2*alt*i;
            int i_up = pos2+1;
            int i_down = pos2-1;
            double d_up = a * i_up;
            double d_down = a * i_down;
            if (alt == -1)
            {
                if ( sqrt(d_left*d_left + d_down*d_down) > r || sqrt(d_right*d_right + d_down*d_down) > r )
                {
                    go_down = false;
                    continue;
                }
            }
            else
            {
                if ( sqrt(d_right*d_right + d_up*d_up) > r || sqrt(d_left*d_left + d_up*d_up) > r )
                {
                    go_up = false;
                    continue;
                }
            }

            add_quad(m, qb, pos, pos2, aa, nodes, node_map);
            if (i ==0)
                break;
        }

        go = go_up || go_down;

        ++i;
    }
}
//-----------------------------------------------------------------------------
Node_handle Mesh_generator::find_node(int x, int y, double a, std::map<std::pair<int,int>, Node_handle> &node_map, Mesh_handle m, Mesh_manipulator &manip)
{
    Node_handle node;
    a *= 0.5;
    std::map<std::pair<int,int>, Node_handle>::iterator iter;
    iter = node_map.find(std::make_pair(x, y));
    if (iter == node_map.end())
    {
        node = manip.add_node(Point(a*x, a*y, 0.0));
        node_map.insert(std::make_pair(std::make_pair(x, y), node));
    }
    else
        node = iter->second;
    return node;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::quads_to_triangles(Mesh_handle m)
{
    Mesh_manipulator manip(m);
    int n_faces = m->size_of_faces();
    m->reserve_faces(2*n_faces);
    for (int i = 0; i < n_faces; ++i)
    {
        manip.quad_to_triangles(m->face(i));
    }
    return m;
}
//-----------------------------------------------------------------------------
Mesh_handle Mesh_generator::create_regular_polygon_mesh(double r, int n, double theta, double ran)
{
    double const T_EPS = 1.0e-15;
    //sanity checks
    if (r <= 0.0 || n < 3)
        return INVALID_MESH;
    //scale randomization factor by the circumcircle radius and divide by max random fro further optimization
    ran = r *ran/RAND_MAX;

    Mesh_handle m = new Mesh();
    Mesh_manipulator manip ( m );
    std::vector<Node_handle> nodes;
    nodes.reserve ( n );

    double d_ang = 2.0*PI/n;
    Node_handle node;

    for ( int i = 0; i < n; ++i )
    {
        double c = cos(theta);
        double s = sin(theta);
        if (fabs(s) < T_EPS)
            s = 0.0;
        if (fabs(c) < T_EPS)
            c = 0.0;
        double x = r*c;
        double y = r*s;
        if (ran != 0.0)
        {
            x += (rand() - RAND_MAX/2) * ran;
            y += (rand() - RAND_MAX/2) * ran;
        }
        node = manip.add_node ( Point ( x, y, 0.0 ) );
        nodes.push_back ( node );
        theta += d_ang;
    }
    manip.add_polygon( nodes.begin(), nodes.end() );
    m->set_type(Mesh::SURFACE);
    return m;
}
//-----------------------------------------------------------------------------
