// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_MESH_GENERATOR_H_
#define _CELL_MESH_GENERATOR_H_

#include "mesh/mesh.h"
#include "mesh_generator/voronoi2D.h"
#include "mesh_generator/surface_transform.h"
#include "mesh_generator/transformations.h"
#include "mesh/mesh_manipulator.h"
#include "cell/cell_handle.h"

//-----------------------------------------------------------------------------
class Mesh_generator
{
public:
    static Mesh_generator* get_instance();
    ///create 3D boundary mesh directly from Voronoi diagram by repeatedly applying to it the transformation with shift
    Mesh_handle create_boundary ( Voronoi2D const& const_v, cell::Tissue_handle t, Surface_transform *tr, double shift, int rep = 1, double ran = 0.0 );
    ///create 3D boundary mesh from surface mesh by repeatedly applying to it the transformation with shift
    Mesh_handle create_boundary ( Mesh_handle  v, cell::Tissue_handle t, Surface_transform *tr, double shift, int rep = 1, double ran = 0.0 );
    ///Attach aproporiate tissue information to the surface mesh assuming each face is a single cell. Mesh is not affected in this operation
    Mesh_handle attach_tissue ( Mesh_handle  v, cell::Tissue_handle t );

    Mesh_handle create_cylindrical_boundary ( Voronoi2D const& v, double r );
    Mesh_handle create_cylindrical_boundary ( Mesh_handle m, double r );
    Mesh_handle project ( Voronoi2D const& cv )
    {
        Surface_transform *tr = new Identity();
        project ( cv, tr );
        delete tr;
    }
    Mesh_handle project ( Voronoi2D const& c_v, Surface_transform const*tr )
    {
        Reference_points p;
        return project ( c_v, p, tr );
    }
    Mesh_handle project ( Voronoi2D const& c_v, Surface_transform const*tr, double z_tresh )
    {
        Reference_points p;
        return project ( c_v, p, tr, z_tresh );
    }
    ///project a 2D mesh to 3D given surface transform
    Mesh_handle project ( Mesh_handle in_m, Surface_transform const*tr );
    ///make a parallel projection of 2D mesh onto 3D given surface transform
    Mesh_handle project_on_surface_mesh ( Mesh_handle in_m, Mesh_handle surf_m );

    //quad of size 'a', radius 'r'
    Mesh_handle create_quad_mesh ( double a, double r );
    //quad of size 'a' mesh of 'r' rows and 'c' columns
    Mesh_handle create_quad_mesh ( double a, int r, int c );
    //create circular stripe of 'n' quads of width 'a' with inner radius 'r'
    Mesh_handle create_quad_circle ( double a, double r, int n );
    //create single regular polygon face
    Mesh_handle create_regular_polygon_mesh ( double r, int n, double theta = 0.0, double ran = 0.0 );
    //create quad meshed cylinder
    Mesh_handle create_quad_cylinder ( double r, double h, int n_r, int n_h );
    //create overlayed rectangle meshed cylinder
    Mesh_handle create_staggered_rectangle_cylinder ( double r, double h, int n_r, int n_h );
    //create overlayed rectangle meshed cylinder
    Mesh_handle create_staggered_rectangle_cylinder2 ( double r, double h, int n_r, int n_h );
    //divide quads diagonaly
    Mesh_handle quads_to_triangles ( Mesh_handle m );
    //void bind_transform( Surface_transform * t ) { m_transform = t; }
private:
    typedef Voronoi2D::Point_2                          Point_2;
    typedef std::vector< Face_handle >  Reference_faces;
    typedef std::vector<Point_2>                        Reference_points;
    typedef std::map< Point, Node_handle >      Point_node_map;

    typedef Point_node_map::iterator                    Pn_map_iterator;

    Mesh_generator() {}
    Mesh_handle project ( Voronoi2D const& c_v, Reference_points &points, Surface_transform const*tr );
    Mesh_handle project ( Voronoi2D const& c_v, Reference_points &points, Surface_transform const*tr, double z_tresh );
    void add_layer ( size_t i, Mesh_handle m, cell::Tissue_handle t, Reference_points &points, Reference_faces& faces, Surface_transform const*tr, double ran = 0.0 );
    inline Node_handle insert_point ( Point_2 const &p, Mesh_manipulator &manip, Point_node_map& map );
    inline Node_handle insert_point ( Point const &p, Mesh_manipulator &manip, Point_node_map& map );
    void add_quad ( Mesh_handle m, Mesh_manipulator &manip, int pos, int pos2, double a, std::vector<Node_handle > &n, std::map<std::pair<int,int>, Node_handle > &node_map );
    void add_stripe ( Mesh_handle m, Mesh_manipulator &manip, int pos, double a, double r, std::vector<Node_handle > &nodes, std::map<std::pair<int,int>, Node_handle > &node_map );
    Node_handle  find_node ( int x, int y, double a, std::map<std::pair<int,int>, Node_handle > &node_map, Mesh_handle m, Mesh_manipulator &manip );

    static Mesh_generator *m_instance;
};
//-----------------------------------------------------------------------------
Node_handle Mesh_generator::insert_point ( Point_2 const &p_2, Mesh_manipulator &manip, Point_node_map& map )
{
    Point p ( p_2.x(), p_2.y(), 0 );
    insert_point ( p, manip, map );

}
//-----------------------------------------------------------------------------
Node_handle Mesh_generator::insert_point ( Point const &p, Mesh_manipulator &manip, Point_node_map& map )
{
    Pn_map_iterator map_iter = map.find ( p );
    if ( map_iter == map.end() )
    {
        Node_handle n = manip.add_node ( p );
        map.insert ( std::make_pair ( p, n ) );
        return n;
    }

    return map_iter->second;
}
//-----------------------------------------------------------------------------
#endif
