#include "postprocessing/stress_interpolator.h"
#include "utils/la_utils.h"
#include "boost/numeric/ublas/operation.hpp"
//-----------------------------------------------------------------------------
State_interpolator::State_interpolator ( Mesh_handle const& mesh, Affine_map_shell<Mesh_face> &map ) : m_mesh ( mesh ), m_map ( map ), m_I ( Matrix3d ( 1,1,1 ) )
{
    m_weights = new double[map.q_rule().spacedim()];
    init_storage();
    reset_local_node_data();
    reset_local_element_data();
    construct_extrapolation_matrix();
}
//-----------------------------------------------------------------------------
State_interpolator::~State_interpolator()
{
    delete[] m_weights;

}
//-----------------------------------------------------------------------------
Matrix3s const& State_interpolator::stress ( Node_handle const& n ) const
{
    return m_stress[n->id()];
}
//-----------------------------------------------------------------------------
Matrix3s const& State_interpolator::element_stress ( Face_handle const& f ) const
{
    return m_elem_stress[f->id()];
}
//-----------------------------------------------------------------------------
Matrix3s const& State_interpolator::strain ( Node_handle const& n ) const
{
    return m_strain[n->id()];
}
//-----------------------------------------------------------------------------
Matrix3s const& State_interpolator::element_strain ( Face_handle const& f ) const
{
    return m_elem_strain[f->id()];
}
//-----------------------------------------------------------------------------
void State_interpolator::interpolate_stress()
{
    int son = m_mesh->size_of_nodes();
    m_stress.resize ( son );
    m_conectivity_count.resize ( son );
    Mesh::Face_const_iterator f_it, f_end;
    //find out number of nodes of the element
    const size_t n = m_map.element().spacedim() /m_map.element().tensordim();
    m_temp.resize ( n );
    //prepare space for stress representation at all nodes of the element
    std::vector<DenseVector> loc_element_stress ( n,DenseVector ( 6 ) );
    //map local nodes of the element_stress to global nodes of the m_stress
    size_t* nodes = new size_t[n];
    reset_local_stress_data();
    typedef std::vector<DenseVector>::iterator IT;
    for ( f_it = m_mesh->faces_begin(), f_end = m_mesh->faces_end(); f_it != f_end; ++f_it )
    {
        ;
        //zero the storage for element stress
        for ( IT it = loc_element_stress.begin(); it != loc_element_stress.end(); ++it )
            it->zero();

        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( *f_it );
        m_map.update_stress();
        //extrapolate the stress from integration points to the element nodes
        extrapolate_gauss_element_stress ( loc_element_stress, m_map.q_rule() );
        //add each face extrapolated stress to the node stress and count faces contributing to node stress
        Mesh_face::Edge_use_const_circulator eu_cir = f_it->edge_uses_begin(), eu_start = eu_cir;
        size_t count = 0;
        do
        {
            Node_handle const& n = eu_cir->node();
            size_t id = n->id();
            m_stress[id] += loc_element_stress[count];
            ++m_conectivity_count[id];
            ++count;
        }
        while ( ++eu_cir != eu_start );
    }
    //average the stress by dividing by the number of element connected at the node
    Mesh::Node_const_iterator n_it, n_end;
    for ( n_it = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_it != n_end; ++n_it )
    {
        size_t id = n_it->id();
        m_stress[id] /= m_conectivity_count[id];
    }
    delete[] nodes;
}
//-----------------------------------------------------------------------------
void State_interpolator::interpolate_strain()
{
    int son = m_mesh->size_of_nodes();
    m_strain.resize ( son );
    m_conectivity_count.resize ( son );
    Mesh::Face_const_iterator f_it, f_end;
    //find out number of nodes of the element
    const size_t n = m_map.element().spacedim() /m_map.element().tensordim();
    m_temp.resize ( n );
    //prepare space for strain representation at all nodes of the element
    std::vector<DenseVector> loc_element_strain ( n,DenseVector ( 6 ) );
    //map local nodes of the element_stress to global nodes of the m_stress
    size_t* nodes = new size_t[n];
    reset_local_strain_data();
    typedef std::vector<DenseVector>::iterator IT;
    for ( f_it = m_mesh->faces_begin(), f_end = m_mesh->faces_end(); f_it != f_end; ++f_it )
    {
        //zero the storage for element stress
        for ( IT it = loc_element_strain.begin(); it != loc_element_strain.end(); ++it )
            it->zero();

        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( *f_it );

        //extrapolate the stress from integration points to the element nodes
        extrapolate_gauss_element_strain ( loc_element_strain, m_map.q_rule() );

        //add each face extrapolated stress to the node stress and count faces contributing to node stress
        Mesh_face::Edge_use_const_circulator eu_cir = f_it->edge_uses_begin(), eu_start = eu_cir;
        size_t count = 0;
        do
        {
            Node_handle const& n = eu_cir->node();
            size_t id = n->id();
            m_strain[id] += loc_element_strain[count];
            ++m_conectivity_count[id];
            ++count;
        }
        while ( ++eu_cir != eu_start );
    }

    //average the stress by dividing by the number of element connected at the node
    Mesh::Node_const_iterator n_it, n_end;
    for ( n_it = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_it != n_end; ++n_it )
    {
        size_t id = n_it->id();
        m_strain[id] /= m_conectivity_count[id];
    }
    delete[] nodes;

}
//-----------------------------------------------------------------------------
void State_interpolator::interpolate_tensors()
{
    int son = m_mesh->size_of_nodes();
    m_stress.resize ( son );
    m_strain.resize ( son );
    m_conectivity_count.resize ( son );

    Mesh::Face_const_iterator f_it, f_end;
    //find out number of nodes of the element
    const size_t n = m_map.element().spacedim() /m_map.element().tensordim();
    m_temp.resize ( n );
    m_temp2.resize ( n );
    //prepare space for stress representation at all nodes of the element
    std::vector<DenseVector> loc_element_stress ( n,DenseVector ( 6 ) );
    std::vector<DenseVector> loc_element_strain ( n,DenseVector ( 6 ) );
    //map local nodes of the element_stress to global nodes of the m_stress
    size_t* nodes = new size_t[n];
    reset_local_stress_data();
    reset_local_strain_data();
    typedef std::vector<DenseVector>::iterator IT;
    for ( f_it = m_mesh->faces_begin(), f_end = m_mesh->faces_end(); f_it != f_end; ++f_it )
    {
        //zero the storage for element stress and strain
        for ( IT it = loc_element_stress.begin(); it != loc_element_stress.end(); ++it )
            it->zero();
        for ( IT it = loc_element_strain.begin(); it != loc_element_strain.end(); ++it )
            it->zero();
        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( *f_it );
        m_map.update_stress();
        //extrapolate the stress from integration points to the element nodes
        extrapolate_gauss_element_tensors ( loc_element_stress, loc_element_strain, m_map.q_rule() );
        //add each face extrapolated stress to the node stress and count faces contributing to node stress
        Mesh_face::Edge_use_const_circulator eu_cir = f_it->edge_uses_begin(), eu_start = eu_cir;
        size_t count = 0;
        do
        {
            Node_handle const& n = eu_cir->node();
            size_t id = n->id();
            m_stress[id] += loc_element_stress[count];
            m_strain[id] += loc_element_strain[count];
            ++m_conectivity_count[id];
            ++count;
        }
        while ( ++eu_cir != eu_start );
    }
    //average the stress by dividing by the number of element connected at the node
    Mesh::Node_const_iterator n_it, n_end;
    for ( n_it = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_it != n_end; ++n_it )
    {
        size_t id = n_it->id();
        m_stress[id] /= m_conectivity_count[id];
        m_strain[id] /= m_conectivity_count[id];
    }
    delete[] nodes;
}
//-----------------------------------------------------------------------------
void State_interpolator::average_stress()
{
    size_t n_faces = m_mesh->size_of_faces();
    m_elem_stress.resize ( n_faces );
    typedef std::vector<Matrix3s>::iterator IT;
    for ( IT it = m_elem_stress.begin(); it != m_elem_stress.end(); ++it )
        it->zero();
    for ( int i = 0; i < n_faces; ++i )
    {
        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( * ( m_mesh->face ( i ) ) );
        m_map.update_stress();
        //extrapolate the stress
        average_element_stress ( m_elem_stress[i], m_map.q_rule() );
    }
}
//-----------------------------------------------------------------------------
void State_interpolator::average_strain()
{
    size_t n_faces = m_mesh->size_of_faces();
    m_elem_strain.resize ( n_faces );
    typedef std::vector<Matrix3s>::iterator IT;
    for ( IT it = m_elem_strain.begin(); it != m_elem_strain.end(); ++it )
        it->zero();
    for ( int i = 0; i < n_faces; ++i )
    {
        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( * ( m_mesh->face ( i ) ) );
        //extrapolate the stress
        average_element_strain ( m_elem_strain[i], m_map.q_rule() );
    }
}
//-----------------------------------------------------------------------------
void State_interpolator::average_tensors()
{
    size_t n_faces = m_mesh->size_of_faces();
    m_elem_stress.resize ( n_faces );
    m_elem_strain.resize ( n_faces );
    typedef std::vector<Matrix3s>::iterator IT;
    for ( IT it = m_elem_stress.begin(); it != m_elem_stress.end(); ++it )
        it->zero();
    for ( IT it = m_elem_strain.begin(); it != m_elem_strain.end(); ++it )
        it->zero();
    for ( int i = 0; i < n_faces; ++i )
    {
        //update the affine map for element
        m_map.set_flag ( Affine_map_shell< Mesh_face >::UPDATE_ALL );
        m_map.update ( * ( m_mesh->face ( i ) ) );
        m_map.update_stress();

        average_element_strain ( m_elem_strain[i], m_map.q_rule() );
        average_element_stress ( m_elem_stress[i], m_map.q_rule() );
    }
}
//-----------------------------------------------------------------------------
void State_interpolator::extrapolate_gauss_element_stress ( std::vector<DenseVector> &loc_element_stress, Quadrature_rule const& qrule )
{
    int const EL_NODES = 4;
    int const INT_POINTS_2D = 4;
    //this procedure works only for 12 point quad integration rule

    //WARNINIG get the thickness at each xy integration point (use first 4)
    double lambda[EL_NODES];
    double lambda_int[INT_POINTS_2D];
    //thickness at nodes
    for ( int i = 0; i < EL_NODES; ++i )
    {
        lambda[i] = sqrt ( ( m_map.D ( i ) ).squared_length() ) *m_map.h0 ( i );
    }
    //interpolate thickness from nodes to integration points (use first 4 as xy cooradinates repeat)
    for ( int j = 0; j < INT_POINTS_2D; ++j )
    {
        lambda_int[j] = 0.0;
        for ( int i = 0; i < EL_NODES; ++i )
        {
            lambda_int[j] += lambda[i]*m_map.shape_fun ( i,j );
        }
    }
    //get the weights for thickness integration (the weights are for 3D rule, so assumption is that in plane weights are 1 (true for 12 point quad rule))
    m_map.q_rule().weights ( m_weights );
    //integrate stress over the thickness
    //loop over z layers of gauss noodes (el = offset)
    for ( int i = 0; i < INT_POINTS_2D; ++i )
    {
        m_temp[i].zero();
        for ( int el = 0; el <= 2*INT_POINTS_2D; el += INT_POINTS_2D )
        {
            m_temp[i] += m_map.stress ( i+el ) *m_weights[i+el]*0.5*lambda_int[i];
        }
    }
    //loop over element nodes
    for ( int l = 0; l < EL_NODES; ++l )
    {
        //loop over gauss nodes in a layer
        for ( int i = 0; i < INT_POINTS_2D; ++i )
        {
            add_prod ( m_extr_mat ( l,i ), m_temp[i], loc_element_stress[l] );
        }
    }
}
//-----------------------------------------------------------------------------
void State_interpolator::extrapolate_gauss_element_strain ( std::vector<DenseVector> &loc_element_strain, Quadrature_rule const& qrule )
{
    int const EL_NODES = 4;
    int const INT_POINTS_2D = 4;
    //WARNINIG this procedure works only for 12 point quad integration rule
    //get the thickness at each xy integration point (use first 4)

    double lambda[EL_NODES];
    double lambda_int[INT_POINTS_2D];
    //thickness at nodes
    //do not multiply by the thickness as the strain is not area dependennt for 2D case
    for ( int i = 0; i < EL_NODES; ++i )
    {
        lambda[i] = sqrt ( ( m_map.D ( i ) ).squared_length() ) *m_map.h0 ( i );
    }
    //interpolate thickness from nodes to integration points (use first 4 as xy cooradinates repeat)
    for ( int j = 0; j < INT_POINTS_2D; ++j )
    {
        lambda_int[j] = 0.0;
        for ( int i = 0; i < EL_NODES; ++i )
        {
            lambda_int[j] += lambda[i]*m_map.shape_fun ( i,j );
        }
    }
    //get the weights for thickness integration (the weights are for 3D rule, so assumption is that in plane weights are 1 (true for 12 point quad rule))
    m_map.q_rule().weights ( m_weights );
    //integrate strain over the thickness
    //loop over z layers of gauss noodes (el = offset)
    Matrix3s b;//left Cauchy-Green
    for ( int i = 0; i < INT_POINTS_2D; ++i )
    {
        m_temp[i].zero();
        for ( int el = 0; el <= 2*INT_POINTS_2D; el += INT_POINTS_2D )
        {
            m_map.left_cauchy_green ( b,i+el );
            m_temp[i] += ( m_I - b.inverse() ) *m_weights[i+el]*0.25*lambda_int[i]; //0.5(from jackobian)*0.5(from Almansi starain)
        }
    }
    //divide the strain by thickness since we want average over thickness
    for ( int i = 0; i < INT_POINTS_2D; ++i )
    {
        m_temp[i] = m_temp[i]/lambda_int[i];
    }
    //loop over element nodes
    for ( int l = 0; l < EL_NODES; ++l )
    {
        //loop over gauss nodes in a layer
        for ( int i = 0; i < INT_POINTS_2D; ++i )
        {
            add_prod ( m_extr_mat ( l,i ), m_temp[i], loc_element_strain[l] );
        }
    }
}
//-----------------------------------------------------------------------------
void State_interpolator::extrapolate_gauss_element_tensors ( std::vector<DenseVector> &loc_element_stress, std::vector<DenseVector> &loc_element_strain, Quadrature_rule const& qrule )
{
    int const EL_NODES = 4;
    int const INT_POINTS_2D = 4;
    //this procedure works only for 12 point quad integration rule

    //get the thickness at each xy integration point (use first 4)
    double lambda[EL_NODES];
    double lambda_int[INT_POINTS_2D];
    //thickness at nodes
    for ( int i = 0; i < EL_NODES; ++i )
    {
        lambda[i] = sqrt ( ( m_map.D ( i ) ).squared_length() ) *m_map.h0 ( i );
    }
    //interpolate thickness from nodes to integration points (use first 4 as xy cooradinates repeat)
    for ( int j = 0; j < INT_POINTS_2D; ++j )
    {
        lambda_int[j] = 0.0;
        for ( int i = 0; i < EL_NODES; ++i )
        {
            lambda_int[j] += lambda[i]*m_map.shape_fun ( i,j );
        }
    }
    //get the weights for thickness integration (the weights are for 3D rule, so assumption is that in plane weights are 1 (true for 12 point quad rule))
    m_map.q_rule().weights ( m_weights );
    //integrate stress over the thickness
    //loop over z layers of gauss noodes (el = offset)
    Matrix3s b;//left Cauchy-Green
    for ( int i = 0; i < INT_POINTS_2D; ++i )
    {
        m_temp[i].zero();
        m_temp2[i].zero();
        for ( int el = 0; el <= 2*INT_POINTS_2D; el += INT_POINTS_2D )
        {
            double tmp = m_weights[i+el]*0.5*lambda_int[i];
            m_temp[i] += m_map.stress ( i+el ) *tmp;
            m_map.left_cauchy_green ( b,i+el );
            m_temp2[i] += ( m_I - b.inverse() ) *0.5*tmp; //0.5(from jackobian)*0.5(from Almansi starain)
        }
    }
    //divide the strain by thickness since we want average over thickness
    for ( int i = 0; i < INT_POINTS_2D; ++i )
    {
        m_temp2[i] = m_temp2[i]/lambda_int[i];
    }
    //loop over element nodes
    for ( int l = 0; l < EL_NODES; ++l )
    {
        //loop over gauss nodes in a layer
        for ( int i = 0; i < INT_POINTS_2D; ++i )
        {
            add_prod ( m_extr_mat ( l,i ), m_temp[i], loc_element_stress[l] );
            add_prod ( m_extr_mat ( l,i ), m_temp2[i], loc_element_strain[l] );
        }
    }
}
//integrate stress over element and divide by volume. Assumes affine map is updated for the element
//-----------------------------------------------------------------------------
void State_interpolator::average_element_stress ( Matrix3s& element_stress, Quadrature_rule const& qrule )
{
    int nint = qrule.spacedim();
    m_map.q_rule().weights ( m_weights );
    Matrix3s av_stress;
    av_stress.zero();
    double vol = 0;
    //loop over integration points
    for ( int i = 0; i < nint; ++i )
    {
        double temp = m_weights[i]*m_map.det ( i );
        av_stress += m_map.stress ( i ) *temp;
        vol += m_weights[i]*m_map.det ( i );
    }
    vol = 1.0/vol;
    element_stress = av_stress * vol;
}
//integrate strain over element and divide by volume. Assumes affine map is updated for the element
//-----------------------------------------------------------------------------
void State_interpolator::average_element_strain ( Matrix3s& element_strain, Quadrature_rule const& qrule )
{
    int nint = qrule.spacedim();
    m_map.q_rule().weights ( m_weights );
    Matrix3s av_strain;
    Matrix3s b;
    av_strain.zero();
    double vol = 0;
    //loop over integration points
    for ( int i = 0; i < nint; ++i )
    {
        double temp = m_weights[i]*m_map.det ( i );
        m_map.left_cauchy_green ( b,i );
        av_strain += ( m_I - b.inverse() ) *temp*0.5;
        vol += temp;
    }
    vol = 1.0/vol;
    element_strain = av_strain * vol;
}
//-----------------------------------------------------------------------------
void State_interpolator::construct_extrapolation_matrix()
{
    //extrapolation matrix has in its rows shape functions calculated in the basis of gauss element
    std::string qr_type = m_map.q_rule().type();
    std::string el_shape = m_map.element().spec().shape();
    if ( el_shape == "quadrilateral" )
    {
        if ( qr_type == "[ 12 point rule on a quad shell ]" )
        {
            m_extr_mat.init ( 4,4,0 );
            const double sqrt3 = sqrt ( 3. );
            const double A = ( 1.+0.5*sqrt3 );
            const double B = ( 1.-0.5*sqrt3 );

            m_extr_mat.set ( 0,0, A );
            m_extr_mat.set ( 0,1, -0.5 );
            m_extr_mat.set ( 0,2, B );
            m_extr_mat.set ( 0,3, -0.5 );
            m_extr_mat.set ( 1,0, -0.5 );
            m_extr_mat.set ( 1,1, A );
            m_extr_mat.set ( 1,2, -0.5 );
            m_extr_mat.set ( 1,3, B );
            m_extr_mat.set ( 2,0, B );
            m_extr_mat.set ( 2,1, -0.5 );
            m_extr_mat.set ( 2,2, A );
            m_extr_mat.set ( 2,3, -0.5 );
            m_extr_mat.set ( 3,0, -0.5 );
            m_extr_mat.set ( 3,1, B );
            m_extr_mat.set ( 3,2, -0.5 );
            m_extr_mat.set ( 3,3, A );
        }
        else
            cell_error ( "stress extrapolation for supplied quadrature rule not implemented yet." );
    }
    else
        cell_error ( "stress extrapolation for supplied element shape not implemented yet." );
}
//-----------------------------------------------------------------------------
void State_interpolator::reset_local_stress_data()
{
    for ( int i = 0; i < m_stress.size(); ++i )
        m_stress[i].clean();
    for ( int i = 0; i < m_conectivity_count.size(); ++i )
        m_conectivity_count[i] = 0;
}
//-----------------------------------------------------------------------------
void State_interpolator::reset_local_strain_data()
{
    for ( int i = 0; i < m_strain.size(); ++i )
        m_strain[i].clean();
    for ( int i = 0; i < m_conectivity_count.size(); ++i )
        m_conectivity_count[i] = 0;
}
//-----------------------------------------------------------------------------
void State_interpolator::init_storage()
{
    m_stress.resize ( m_mesh->size_of_nodes() );
    m_elem_stress.resize ( m_mesh->size_of_faces() );
    m_strain.resize ( m_mesh->size_of_nodes() );
    m_elem_strain.resize ( m_mesh->size_of_faces() );
    m_conectivity_count.resize ( m_mesh->size_of_nodes() );
    m_temp.resize ( m_map.element().spacedim() /m_map.element().tensordim() );
    m_temp2.resize ( m_map.element().spacedim() /m_map.element().tensordim() );
}
//-----------------------------------------------------------------------------
void State_interpolator::reset_local_node_data()
{
    for ( int i = 0; i < m_stress.size(); ++i )
        m_stress[i].clean();
    for ( int i = 0; i < m_strain.size(); ++i )
        m_strain[i].clean();
}
//-----------------------------------------------------------------------------
void State_interpolator::reset_local_element_data()
{
    for ( int i = 0; i < m_elem_stress.size(); ++i )
        m_elem_stress[i].clean();
    for ( int i = 0; i < m_elem_strain.size(); ++i )
        m_elem_strain[i].clean();
}
//-----------------------------------------------------------------------------
