// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _STRESS_INTERPOLATOR_H_
#define _STRESS_INTERPOLATOR_H_

#include "mesh/mesh.h"
#include "fem/Affine_map_shell.h"
#include "cell/cell_handle.h"
//-----------------------------------------------------------------------------
class State_interpolator
{
public:
    State_interpolator(Mesh_handle const& mesh, Affine_map_shell<Mesh_face> &map);
    ~State_interpolator();
    ///get stress tensor at the node
    Matrix3s const& stress(Node_handle const& n) const;
    ///get stress tensor averaged for the elelmnt
    Matrix3s const& element_stress(Face_handle const& f) const;
    ///get strain tensor at the node
    Matrix3s const& strain(Node_handle const& n) const;
    ///get strain tensor averaged for the elelmnt
    Matrix3s const& element_strain(Face_handle const& f) const;
    
    ///interpolate stress for the mesh
    void interpolate_stress();
    ///interpolate strain for the mesh
    void interpolate_strain();
    ///interpolate strain and stress for the mesh
    void interpolate_tensors();
    
    ///average stress for the mesh (each element)
    void average_stress();
    ///average strain for the mesh (each element)
    void average_strain();
    ///average strain and stress for the mesh (each element)
    void average_tensors();
    
    ///average stress over each compartment (by averaging element averaged tensors)
    template<class CIterator, class OutIterator>
    void average_stress_over_compartments(CIterator from, CIterator beyond, OutIterator oit);
    ///average stress over each compartment (by averaging element averaged tensors)
    template<class CIterator, class OutIterator>
    void average_strain_over_compartments(CIterator from, CIterator beyond, OutIterator oit);
    
    void construct_extrapolation_matrix();

    Mesh_handle const& mesh() const {
        return m_mesh;
    }
    Affine_map_shell<Mesh_face>& map() const {
        return m_map;
    }
private:

    /// add extrapolation from an element to the global nodal_stress
    void extrapolate_gauss_element_stress(std::vector<DenseVector> &loc_element_stress, Quadrature_rule const& qrule);
    /// add extrapolation from an element to the global nodal_strain
    void extrapolate_gauss_element_strain(std::vector<DenseVector> &loc_element_strain, Quadrature_rule const& qrule);
    /// add extrapolation from an element to the global nodal_strain
    void extrapolate_gauss_element_tensors(std::vector<DenseVector> &loc_element_stress, std::vector<DenseVector> &loc_element_strain, Quadrature_rule const& qrule);
    /// average stress for an element from the integration points (integrate stress and divide by element volume)
    void average_element_stress(Matrix3s& element_stress, Quadrature_rule const& qrule);
    /// average strain for an element from the integration points (integrate strain and divide by element volume)
    void average_element_strain(Matrix3s& element_strain, Quadrature_rule const& qrule);
    /// resets local stress and connectivity matrices
    void reset_local_stress_data();
    /// resets local strain and connectivity matrices
    void reset_local_strain_data();
    ///initialize storage to the current size of the mesh
    void init_storage();
    ///zero the strain and stress tensors at the nodes
    void reset_local_node_data();
    ///zero the strain and stress tensors for the elements
    void reset_local_element_data();
    
    /// associated mesh
    Mesh_handle m_mesh;
    /// associated affine map
    Affine_map_shell<Mesh_face> & m_map;
    /// interpolated stresses at each mesh node
    std::vector<Matrix3s> m_stress;
    /// interpolated strains at each mesh node
    std::vector<Matrix3s> m_strain;
    /// averaged stresses for each mesh element
    std::vector<Matrix3s> m_elem_stress;
    /// averaged strains for each mesh element
    std::vector<Matrix3s> m_elem_strain;
    /// tensor at each of elements nodes used as temporary in computations
    std::vector<Matrix3s> m_temp, m_temp2;
    /// counts how many faces are connected to each node
    std::vector<size_t> m_conectivity_count;
    ///extrapolation matrix from gauss element to finite element
    DenseMatrix m_extr_mat;
    /// storage place for gauss weights
    double *m_weights;
    ///Identity 3x3 matrix
    const Matrix3d m_I;

};
//-----------------------------------------------------------------------------
#endif