// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _EDGE_USE_PREDICATE_H_
#define _EDGE_USE_PREDICATE_H_

#include "predicates/predicate.h"
#include "mesh/mesh_edge_use.h"
#include <mesh/mesh_face.h>
typedef Predicate<Mesh_edge_use> Edge_use_predicate;
namespace edge_use_predicates
{
//-----------------------------------------------------------------------------
class Predicate_list: public Edge_use_predicate
{
public:
    typedef std::list<Edge_use_predicate*> collection;
    typedef collection::iterator iterator;
    typedef collection::const_iterator const_iterator;

    Predicate_list( ) {}

    void add ( Edge_use_predicate *np )
    {
        m_predicates.push_back ( np );
    }

    void clear() 
    {
      m_predicates.clear();
    }
    
    bool operator() ( Edge_use_handle const& f ) const
    {
        const_iterator it, it_end;
        bool result = true;
        for ( it = m_predicates.begin(), it_end = m_predicates.end(); it != it_end; ++it )
        {
            result = result && ( *it )->operator() ( f );
        }
        return result;
    }
private:
    collection m_predicates;
};
//-----------------------------------------------------------------------------
//'inclusive' template parameter specifies if all the edge nodes have to satisfy the constraint (if true) or just one (if false)
template<class BP, bool incusive = true>
class X_position: public Edge_use_predicate
{
public:
    X_position ( double x ) : m_x ( x ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        bool cond = true;
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            cond = cond && BP() ( p.x(), m_x );
        }
        return cond;
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP>
class X_position<BP, false>: public Edge_use_predicate
{
public:
    X_position ( double x ) : m_x ( x ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            if( BP() ( p.x(), m_x ) )
              return true;
        }
        return false;
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP, bool incusive = true>
class Y_position: public Edge_use_predicate
{
public:
    Y_position ( double y ) : m_y ( y ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        bool cond = true;
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            cond = cond && BP() ( p.y(), m_y );
        }
        return cond;
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP>
class Y_position<BP, false>: public Edge_use_predicate
{
public:
    Y_position ( double y ) : m_y ( y ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            if( BP() ( p.y(), m_y ) )
              return true;
        }
        return false;
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP, bool incusive = true>
class Z_position: public Edge_use_predicate
{
public:
    Z_position ( double z ) : m_z ( z ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        bool cond = true;
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            cond = cond && BP() ( p.z(), m_z );
        }
        return cond;
    }
private:
    double m_z;
};
//-----------------------------------------------------------------------------
template<class BP>
class Z_position<BP, false>: public Edge_use_predicate
{
public:
    Z_position ( double z ) : m_z ( z ) {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            if( BP() ( p.z(), m_z ) )
              return true;
        }
        return false;
    }
private:
    double m_z;
};
//-----------------------------------------------------------------------------
//tests distance of booth nodes of the edge against predicate BP
template<class BP>
class radial_position: public Edge_use_predicate
{
public:
    radial_position( double r, Point p = Point(0,0,0)) : m_r2(r*r), m_p(p) {}

    bool operator() ( Edge_use_handle const& eu ) const
    {
        bool cond = true;
        Edge_handle const& e = eu->edge();
        Mesh_edge::Node_const_iterator n_it, n_end;
        for ( n_it = e->nodes_begin(), n_end = e->nodes_end(); n_it != n_end; ++n_it )
        {
            Point const& p = n_it->point();
            double x = m_p.x() - p.x();
            double y = m_p.y() - p.y();
            double z = m_p.z() - p.z();
            double r02 = x*x+y*y+z*z;
            cond = cond && BP()( r02, m_r2);
        }
        return cond;
    }
private:
    double m_r2;
    Point m_p;
};
///Returns true if and edge is on the boundary of 2D mesh
//-----------------------------------------------------------------------------
class Boundary: public Edge_use_predicate
{
public:
    Boundary () {}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        return ( eu->size_of_radial() == 1 );
    }
};
//-----------------------------------------------------------------------------
class Edge_flag: public Edge_use_predicate
{
public:
    Edge_flag (int fl) { m_flag = fl;}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        return ( eu->edge()->flag() == m_flag );
    }
private:
  int m_flag;
};
//-----------------------------------------------------------------------------
class Edge_id: public Edge_use_predicate
{
public:
    Edge_id (int id) { m_id = id;}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        return ( eu->edge()->id() == m_id );
    }
private:
  int m_id;
};
//-----------------------------------------------------------------------------
//check if edge use belongs to faces in the range of iterators (iterators to Face_handle)
template<class I>
class Belongs_to_faces: public Edge_use_predicate
{
public:
    Belongs_to_faces (I from, I beyond) : m_start(from), m_beyond(beyond){}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        I from = m_start;
        while (from != m_beyond)
        {
          Mesh_face::Edge_use_const_circulator e_circ = (*from)->edge_uses_begin(), e_start = e_circ;
          do{
            if(eu == e_circ.handle())
              return true;
          }while(++e_circ != e_start);
          ++from;
        }
        return false;
    }
private:
  I m_start, m_beyond;
};
//-----------------------------------------------------------------------------
//check if edge use doesn't belong to faces in the range of iterators (iterators to Face_handle)
template<class I>
class Excluded_from_faces: public Edge_use_predicate
{
public:
    Excluded_from_faces (I from, I beyond) : m_start(from), m_beyond(beyond){}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        I from = m_start;
        while (from != m_beyond)
        {
          Mesh_face::Edge_use_const_circulator e_circ = (*from)->edge_uses_begin(), e_start = e_circ;
          do{
            if(eu == e_circ.handle())
              return false;
          }while(++e_circ != e_start);
          ++from;
        }
        return true;
    }
private:
  I m_start, m_beyond;
};
//-----------------------------------------------------------------------------
//check if edge use doesn't belong to edges belonging to edge_uses in the range of iterators (iterators to Edge_use_handle)
template<class I>
class Excluded_from_edges: public Edge_use_predicate
{
public:
    Excluded_from_edges (I from, I beyond) : m_start(from), m_beyond(beyond){}
    bool operator() ( Edge_use_handle const& eu ) const
    {
        I from = m_start;
        while (from != m_beyond)
        {
            if(eu->edge()->id() == (*from)->edge()->id())
              return false;
          ++from;
        }
        return true;
    }
private:
  I m_start, m_beyond;
};
}//end namespace
//-----------------------------------------------------------------------------
#endif
