//Elasticity bilinear and linear forms

#ifndef _ELASTICITY_SHELL_H
#define _ELASTICITY_SHELL_H

#include "fem/Affine_map_shell.h"
#include "fem/Linear_form.h"
#include "fem/fem_utils.h"
#include "fem/Bilinear_form.h"
#include "matrix/Vector3.h"
#include "utils/constants.h"
#include "io/pvd_file.h"

// #define LAMINA_COORDINATES
bool const GLOBAL_BENDING = true;
bool const GLOBAL_NLGEOM = true;
namespace Elasticity_shell {

/// This class contains the form to be evaluated, including
/// contributions from the interior and boundary of the domain.
//-----------------------------------------------------------------------------
class Bilinear_form : public ::Bilinear_form
{
public:

    Bilinear_form() : ::Bilinear_form(0), temp_mat(6,6), res_mat(6,6), c_mat(6,6)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);
        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        // Create finite element for trial space
        _trial = Finite_element::makeElement(specs);
        // Nonlinear geometry initial stress stiffening switch
        NLGEOM = GLOBAL_NLGEOM;
        BENDING = GLOBAL_BENDING;
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //local stiffness matrix for quad shell. Variables structure:(x*4,y*4,z*4,Dx*4,Dy*4,Dz*4)
        for (int i = 0; i < 576; ++i)
        {
            block[i] = 0.0;
        }

        //number of integration points
        size_t nint = map.q_rule().spacedim();
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;
        //ofset for the next row (nect dof) in the block
        int ind_offset = ntotd * nen;

        int ind, transp_ind;

        double con, temp_x, temp_y, temp_z;
        double *wgts = new double[nint];

        map.q_rule().weights(wgts);
        //storage for b matrix rotated to lamina coordinates
#ifdef LAMINA_COORDINATES
        DenseMatrix bq_mat(6,6);
#endif
        //loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            //elasticity tensor in global coordinates
            DenseMatrix elastic_mat(6,6);
            map.material()->elasticity( elastic_mat, map, l );
#ifdef LAMINA_COORDINATES
            //rotate elasticity tensor to lamina coordinates
            reduce_tangent_matrix(elastic_mat);
#endif
            con = wgts[l]*map.det(l);
            c_mat.mat() = con * elastic_mat.mat();
            //constitutive component
            //loop over test element nodes
            for ( int b = 0; b < nen; ++b )
            {
                DenseMatrix const& b_mat = map.sfd_matrix(l,b);
#ifdef LAMINA_COORDINATES
                inv_rotate_B_matrix(bq_mat, b_mat, map.coord_rot(l));
                ublas::axpy_prod(c_mat, bq_mat, temp_mat, true);
#else
                ublas::axpy_prod(c_mat.mat(), b_mat.mat(), temp_mat.mat(), true);
#endif
                for ( int a = 0; a < nen; ++a )
                {
                    DenseMatrix const& bt_mat  = map.sfd_matrix(l,a);
#ifdef LAMINA_COORDINATES
                    inv_rotate_B_matrix(bq_mat, bt_mat, map.coord_rot(l));
                    ublas::axpy_prod(trans(bq_mat), temp_mat, res_mat, true);
#else
                    ublas::axpy_prod(trans(bt_mat.mat()), temp_mat.mat(), res_mat.mat(), true);
#endif
                    //index of the xx elements(b,a) block in the matrix
                    ind = a*ntotd + b;

                    block[ind] += res_mat(0,0);         //xx
                    block[ind + nen] += res_mat(0,1);   //xy
                    block[ind + 2*nen] += res_mat(0,2); //xz
                    if (BENDING)
                    {
                        block[ind + 3*nen] += res_mat(0,3); //xdx
                        block[ind + 4*nen] += res_mat(0,4); //xdy
                        block[ind + 5*nen] += res_mat(0,5); //xdz
                    }
                    ind  += ind_offset;

                    block[ind] += res_mat(1,0);         //yx
                    block[ind + nen] += res_mat(1,1);   //yy
                    block[ind + 2*nen] += res_mat(1,2); //yz
                    if (BENDING)
                    {
                        block[ind + 3*nen] += res_mat(1,3); //ydx
                        block[ind + 4*nen] += res_mat(1,4); //ydy
                        block[ind + 5*nen] += res_mat(1,5); //ydz
                    }
                    ind += ind_offset;

                    block[ind] += res_mat(2,0);         //zx
                    block[ind + nen] += res_mat(2,1);   //zy
                    block[ind + 2*nen] += res_mat(2,2); //zz
                    if (BENDING)
                    {
                        block[ind + 3*nen] += res_mat(2,3); //zdx
                        block[ind + 4*nen] += res_mat(2,4); //zdy
                        block[ind + 5*nen] += res_mat(2,5); //zdz
                        ind += ind_offset;

                        block[ind] += res_mat(3,0);         //dxx
                        block[ind + nen] += res_mat(3,1);   //dxy
                        block[ind + 2*nen] += res_mat(3,2); //dxz
                        block[ind + 3*nen] += res_mat(3,3); //dxdx
                        block[ind + 4*nen] += res_mat(3,4); //dxdy
                        block[ind + 5*nen] += res_mat(3,5); //dxdz
                        ind += ind_offset;

                        block[ind] += res_mat(4,0);         //dyx
                        block[ind + nen] += res_mat(4,1);   //dyy
                        block[ind + 2*nen] += res_mat(4,2); //dyz
                        block[ind + 3*nen] += res_mat(4,3); //dydx
                        block[ind + 4*nen] += res_mat(4,4); //dydy
                        block[ind + 5*nen] += res_mat(4,5); //dydz
                        ind += ind_offset;

                        block[ind] += res_mat(5,0);         //dzx
                        block[ind + nen] += res_mat(5,1);   //dzy
                        block[ind + 2*nen] += res_mat(5,2); //dzz
                        block[ind + 3*nen] += res_mat(5,3); //dzdx
                        block[ind + 4*nen] += res_mat(5,4); //dzdy
                        block[ind + 5*nen] += res_mat(5,5); //dzdz
                    }
                }
            }
            if (NLGEOM)
            {
                // initial stress component
                //get Cauchy stress
                Matrix3s const&s = map.stress(l);
                //loop over trial element nodes
                for ( int a = 0; a < nen; ++a )
                {
                    DenseMatrix const& sfd_a = map.sfd_matrix(l,a);
                    double NXa = sfd_a(0,0);
                    double NYa = sfd_a(1,1);
                    double NZa = sfd_a(2,2);

                    double MXa = sfd_a(0,3);
                    double MYa = sfd_a(1,4);
                    double MZa = sfd_a(2,5);

                    double SNax = s.xx()*NXa+s.xy()*NYa+s.xz()*NZa;
                    double SNay = s.xy()*NXa+s.yy()*NYa+s.yz()*NZa;
                    double SNaz = s.xz()*NXa+s.yz()*NYa+s.zz()*NZa;
                    double SMax = s.xx()*MXa+s.xy()*MYa+s.xz()*MZa;
                    double SMay = s.xy()*MXa+s.yy()*MYa+s.yz()*MZa;
                    double SMaz = s.xz()*MXa+s.yz()*MYa+s.zz()*MZa;

                    for ( int b = 0; b < nen; ++b )
                    {
                        DenseMatrix const& sfd_b = map.sfd_matrix(l,b);
                        double NXb = sfd_b(0,0);
                        double NYb = sfd_b(1,1);
                        double NZb = sfd_b(2,2);

                        double MXb = sfd_b(0,3);
                        double MYb = sfd_b(1,4);
                        double MZb = sfd_b(2,5);

                        //index of the xx elements(b,a) block in the matrix
                        ind = a*ntotd + b;

                        // the v-v component
                        double kba = (NXb*SNax + NYb*SNay + NZb*SNaz)*con;
                        block[ind] += kba;                          //xx
                        block[ind + ind_offset + nen] += kba;       //yy
                        block[ind + 2*ind_offset + 2*nen] += kba;   //zz
                        if (BENDING)
                        {
                            // the v-t component
                            kba = (NXb*SMax + NYb*SMay + NZb*SMaz)*con;
                            block[ind + 3*nen] += kba;                  //xdx
                            block[ind + ind_offset + 4*nen] += kba;     //ydy
                            block[ind + 2*ind_offset + 5*nen] += kba;   //zdz

                            // the t-v component
                            kba = (MXb*SNax + MYb*SNay + MZb*SNaz)*con;

                            block[ind + 3*ind_offset] += kba;           //dxx
                            block[ind + 4*ind_offset + nen] += kba;     //dyy
                            block[ind + 5*ind_offset + 2*nen] += kba;   //dzz

                            // the t-t component
                            kba = (MXb*SMax + MYb*SMay + MZb*SMaz)*con;

                            block[ind + 3*ind_offset + 3*nen] += kba;   //dxdx
                            block[ind + 4*ind_offset + 4*nen] += kba;   //dydy
                            block[ind + 5*ind_offset + 5*nen] += kba;   //dzdz
                        }
                    }
                }
            }
        }

        delete [] wgts;
    }

    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const {}

private:
    mutable DenseMatrix temp_mat;
    mutable DenseMatrix res_mat;
    mutable DenseMatrix c_mat;
    bool NLGEOM;
    bool BENDING;
};
//-----------------------------------------------------------------------------
class Pressure_stiffness : public ::Bilinear_form
{
public:
    //pressure 'p' acting on the surface located at t in the director direction reference space (t should be in range -1:1)
    Pressure_stiffness() : ::Bilinear_form(0), m_p(0), m_t(0)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);
        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        // Create finite element for trial space
        _trial = Finite_element::makeElement(specs);
        BENDING = GLOBAL_BENDING;
//              BENDING = false;
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        for (int i = 0; i < 576; ++i)
        {
            block[i] = 0.0;
        }
        //number of integration points
        size_t nint = map.surf_q_rule().spacedim();
        //number of element's total degrees of freedom
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;
        //ofset for the next row (nect dof) in the block
        int ind_offset = ntotd * nen;
        int ind;
        double *wgts = new double[nint];

        map.surf_q_rule().weights(wgts);

        //loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            //calculate pressure force at a
            double tr = 0;
            Vector3 dxr = Vector3(0,0,0);
            Vector3 dxs = Vector3(0,0,0);
            //calculate traction force and moments at the integration point
            for ( int b = 0; b < nen; ++b )
            {
                Point const& P = map.P(b);
                Vector3 rb = Vector3( P.x(), P.y(), P.z() );
                tr += map.shape_fun(l,b)*m_p;
                dxr += rb*map.shape_fun_der_x(l,b);
                dxs += rb*map.shape_fun_der_y(l,b);
            }

            tr *= 0.5 * wgts[l];
            //loop over test element nodes
            for ( int b = 0; b < nen; ++b )
            {
                double Nb = map.shape_fun(l,b);
                double Nrb = map.shape_fun_der_x(l,b);
                double Nsb = map.shape_fun_der_y(l,b);
                double zb = 0.5*map.h0(b)*m_t;
                //loop over trial element nodes
                for ( int a = 0; a < nen; ++a )
                {
                    double Na = map.shape_fun(l,a);
                    double Nra = map.shape_fun_der_x(l,a);
                    double Nsa = map.shape_fun_der_y(l,a);

                    Vector3 kba = (dxr*(Na*Nsb - Nb*Nsa) - dxs*(Na*Nrb - Nb*Nra))*tr;
                    //index of the xx elements(b,a) block in the matrix
                    ind = a*ntotd + b;

                    block[ind + nen] -= kba.z;      //xy
                    block[ind + 2*nen] += kba.y;    //xz
                    ind  += ind_offset;

                    block[ind] += kba.z;            //yx

                    block[ind + 2*nen] -= kba.x;    //yz
                    ind += ind_offset;

                    block[ind] -= kba.y;            //zx
                    block[ind + nen] += kba.x;      //zy

                    if (BENDING)
                    {
                        double za = 0.5*map.h0(a)*m_t;
                        ind = a*ntotd + b;

                        block[ind + 4*nen] -= kba.z*za; //xdy
                        block[ind + 5*nen] += kba.y*za; //xdz
                        ind  += ind_offset;

                        block[ind + 3*nen] += kba.z*za; //ydx
                        block[ind + 5*nen] -= kba.x*za; //ydz
                        ind += ind_offset;

                        block[ind + 3*nen] -= kba.y*za; //zdx
                        block[ind + 4*nen] += kba.x*za; //zdy
                        ind += ind_offset;

                        block[ind + nen] -= kba.z*zb;       //dxy
                        block[ind + 2*nen] = kba.y*zb;      //dxz
                        block[ind + 4*nen] -= kba.z*za*zb;  //dxdy
                        block[ind + 5*nen] += kba.y*za*zb;  //dxdz
                        ind += ind_offset;

                        block[ind] += kba.z*zb;             //dyx
                        block[ind + 2*nen] -= kba.x*zb;     //dyz
                        block[ind + 3*nen] += kba.z*za*zb;  //dydx
                        block[ind + 5*nen] -= kba.x*za*zb;  //dydz
                        ind += ind_offset;

                        block[ind] -= kba.y*zb;             //dzx
                        block[ind + nen] += kba.x*zb;       //dzy
                        block[ind + 3*nen] -= kba.y*za*zb;  //dzdx
                        block[ind + 4*nen] += kba.x*za*zb;  //dzdy
                    }
                }
            }
        }
        delete [] wgts;
    }

    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const {}
private:
    double m_p;
    double m_t;
    bool BENDING;
};
//-----------------------------------------------------------------------------
/// This class contains the form to be evaluated, including
/// contributions from the interior and boundary of the domain.

class Linear_form : public ::Linear_form
{
public:

    Linear_form(Function& w0) : ::Linear_form(1)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);
        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        // Add functions
        add(w0, Finite_element::makeElement(specs));
        BENDING = GLOBAL_BENDING;
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
        //number of integration points
        size_t nint = map.q_rule().spacedim();
        //number of total degrees of freedom
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;

        int ind_offset = ntotd * nen;
        int ind, offset;
        double con, temp;
        double *wgts = new double[nint];

        map.q_rule().weights(wgts);

        // loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            con = wgts[l]*map.det(l);
            double gt = 0.5*map.int_point_Z(l);
            //loop over test shape functions
            for ( int b = 0; b < nen; ++b )
            {
                //vector c[0] contains nodal values of the forces in displacement dofs and 0s in rotation dofs
                double zb = gt*map.h0(b)*con;
                //displacement degrees of freedom
                block[b        ] += con * map.shape_fun(l,b) * c[0][b];
                block[b + nen  ] += con * map.shape_fun(l,b) * c[0][b + nen];
                block[b + 2*nen] += con * map.shape_fun(l,b) * c[0][b + 2*nen];
                //rotation degrees of freedom
                if (BENDING)
                {
                    block[b + 3*nen] += zb * map.shape_fun(l,b) * c[0][b];
                    block[b + 4*nen] += zb * map.shape_fun(l,b) * c[0][b + nen];
                    block[b + 5*nen] += zb * map.shape_fun(l,b) * c[0][b + 2*nen];
                }
            }
        }
        delete [] wgts;
    }
    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
    }
private:
    bool BENDING;
};
//-----------------------------------------------------------------------------
/// This class contains the form to be evaluated, including
/// contributions from the interior and boundary of the domain.

class Concentrated_load : public ::Linear_form
{
public:

    Concentrated_load(Function& w0) : ::Linear_form(1)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);
        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        // Add functions
        add(w0, Finite_element::makeElement(specs));
        BENDING = GLOBAL_BENDING;
//              BENDING = false;
    }

    //adds force f on the nodes in iterator range
    template<class I>
    void add_load(I from, I beyond, Vector const& f)
    {
        while (from != beyond)
        {

            ++from;
        }
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
        //number of integration points
        size_t nint = map.q_rule().spacedim();
        //number of total degrees of freedom
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;

        int ind_offset = ntotd * nen;
        int ind, offset;
        double con, temp;
        double *wgts = new double[nint];

        map.q_rule().weights(wgts);

        // loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            con = wgts[l]*map.det(l);
            double gt = 0.5*map.int_point_Z(l);
            //loop over test shape functions
            for ( int b = 0; b < nen; ++b )
            {
                //vector c[0] contains nodal values of the forces in displacement dofs and 0s in rotation dofs
                double temp = map.shape_fun(l,b)*con;
                double zb = gt*map.h0(b)*temp;
                //displacement degrees of freedom
                block[b        ] += temp * c[0][b];
                block[b + nen  ] += temp * c[0][b + nen];
                block[b + 2*nen] += temp * c[0][b + 2*nen];
                //rotation degrees of freedom
                if (BENDING)
                {
                    block[b + 3*nen] += zb * c[0][b];
                    block[b + 4*nen] += zb * c[0][b + nen];
                    block[b + 5*nen] += zb * c[0][b + 2*nen];
                }
            }
        }
        delete [] wgts;
    }
    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
    }
private:
    bool BENDING;
};
//-----------------------------------------------------------------------------
class Pressure_load : public ::Linear_form
{
public:
    //pressure 'p' acting on the surface located at t in the director direction reference space (t should be in range -1:1)
//          Pressure_load(double p, double t) : m_p(p), m_t(t)
    Pressure_load(): m_p(0), m_t(0)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);

        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        BENDING = GLOBAL_BENDING;
//              BENDING = false;
    }

    void update(double p, double *t)
    {
        m_p = p;
        m_t = *t;
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }

        //number of surface integration points
        size_t nint = map.surf_q_rule().spacedim();
        //number of total degrees of freedom
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;

        int ind_offset = ntotd * nen;
        int ind, offset;
        double con, temp;
        double *wgts = new double[nint];

        map.surf_q_rule().weights(wgts);

        // loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            con = wgts[l];
            //position of the integration point with respect to the reference surface at 0
            double gt = 0.5*m_t;
            //calculate pressure force at a
            double tr = 0;
            Vector3 dxr = Vector3(0,0,0);
            Vector3 dxs = Vector3(0,0,0);
//                  for (i=0; i< nen; ++i)
            //calculate traction force and moments at the integration point
            for ( int b = 0; b < nen; ++b )
            {
                Point const& P = map.P(b);
                Vector3 rb = Vector3( P.x(), P.y(), P.z() );
                ///FIXME shape functions and their local derivatives should be of the surface element. Luckily they are the same for the shell quad and a plate quad in x, y so far
                tr += map.shape_fun(l,b)*m_p;
                dxr += rb*map.shape_fun_der_x(l,b);
                dxs += rb*map.shape_fun_der_y(l,b);;
            }
            //dxr^dxs includes area of the face, so we don't need jacobian
            Vector3 f = (dxr ^ dxs)*tr*con;
            //loop over test shape functions
            for ( int b = 0; b < nen; ++b )
            {
                double zb = gt*map.h0(b);
                double sf = map.shape_fun(l,b);
                //displacement degrees of freedom
                block[b        ] += sf * f.x;
                block[b + nen  ] += sf * f.y;
                block[b + 2*nen] += sf * f.z;
                if (BENDING)
                {
                    //rotation degrees of freedom
                    block[b + 3*nen] += zb * sf * f.x;
                    block[b + 4*nen] += zb * sf * f.y;
                    block[b + 5*nen] += zb * sf * f.z;
                }
            }
        }

        delete [] wgts;
    }
    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
    }
private:
    bool BENDING;
    double m_p; //uniform pressure value
    double m_t; //load surface position
};
//-----------------------------------------------------------------------------
class Internal_forces : public ::Linear_form
{
public:

  Internal_forces() : m_pvd(NULL)
    {
        Finite_element_spec specs("Shell", "quadrilateral", 1);

        // Create finite element for test space
        _test = Finite_element::makeElement(specs);
        BENDING = GLOBAL_BENDING;
//              BENDING = false;
    }
    
    Internal_forces(PVD_file *pvd_file) : m_pvd(pvd_file)
    {
      Finite_element_spec specs("Shell", "quadrilateral", 1);
      
      // Create finite element for test space
      _test = Finite_element::makeElement(specs);
      BENDING = GLOBAL_BENDING;
      //              BENDING = false;
    }

    void eval(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
        //number of integration points
        size_t nint = map.q_rule().spacedim();
        //number of total degrees of freedom
        size_t ntotd = map.element().spacedim();
        //number of degrees of freedom per node
        size_t ned = map.element().tensordim();
        //number of element nodes
        size_t nen = ntotd/ned;

        int ind_offset = ntotd * nen;

        int ind, offset;

        double con, temp;
        double *wgts = new double[nint];

        map.q_rule().weights(wgts);

        // loop over integration points
        for ( int j = 0; j < nint; ++j )
        {
          std::vector<double> const& Nr = map.shape_fun_der_x(j);
          std::vector<double> const& Ns = map.shape_fun_der_y(j);
          std::vector<double> const& N = map.shape_fun(j);
          DenseMatrix const& J = map.invJ0(j);
          double gt = map.int_point_Z(j);
          
            double Nx, Ny, Nz, Mx, My, Mz;
            Matrix3s const&s = map.stress(j);
            con = wgts[j]*map.det(j);

            for ( int i = 0; i < nen; ++i )
            {
                //shape function derivative matrix (B-matrix) for integration point j and node i
                DenseMatrix const& sfdji = map.sfd_matrix(j, i);

                // calculate global gradient of shape functions
                // note that we need the transposed of J0i, not J0i itself !
                Nx = sfdji(0,0);
                Ny = sfdji(1,1);
                Nz = sfdji(2,2);
                // the '-' sign is so that the internal forces get subtracted from the global residual vector
                //the indices are gruped by (4 nodes) for each of 6 dofs
                block[i      ] -= ( Nx*s.xx() + Ny*s.xy() + Nz*s.xz() )*con;
                block[i+nen  ] -= ( Ny*s.yy() + Nx*s.xy() + Nz*s.yz() )*con;
                block[i+2*nen] -= ( Nz*s.zz() + Ny*s.yz() + Nx*s.xz() )*con;

                if (BENDING)
                {
                    Mx = sfdji(0,3);
                    My = sfdji(1,4);
                    Mz = sfdji(2,5);

                    block[i+3*nen] -= ( Mx*s.xx() + My*s.xy() + Mz*s.xz() )*con;
                    block[i+4*nen] -= ( My*s.yy() + Mx*s.xy() + Mz*s.yz() )*con;
                    block[i+5*nen] -= ( Mz*s.zz() + My*s.yz() + Mx*s.xz() )*con;
                }
            }
        }
        delete [] wgts;
    }

    // Contribution from the boundary
    void eval_boundary(double block[], const Affine_map_shell<Mesh_face>& map) const
    {
        //zero the block 1x24
        for (int i = 0; i < 24; ++i)
        {
            block[i] = 0.0;
        }
    }
private:
    bool BENDING;
    PVD_file *m_pvd;
};
//-----------------------------------------------------------------------------
}

#endif
