// Licensed under the GNU GPL Version 2.

#ifndef _MASS_MATRIX_SHELL_H_
#define _MASS_MATRIX_SHELL_H_

#include "fem/Affine_map_shell.h"
#include "fem/Linear_form.h"
#include "fem/Bilinear_form.h"

namespace Mass_matrix_shell
{

/// This class contains the form to be evaluated, including
/// contributions from the interior and boundary of the domain.

class Bilinear_form : public ::Bilinear_form
{
public:

    Bilinear_form ( Function& rho ) : ::Bilinear_form ( 1 )
    {
        Finite_element_spec specs ( "Shell", "quadrilateral", 1 );
        // Create finite element for test space
        _test = Finite_element::makeElement ( specs );
        // Create finite element for trial space
        _trial = Finite_element::makeElement ( specs );
        // Create finite element for function
        add ( rho, Finite_element::makeElement ( specs ) );
    }

    void eval ( double block[], const Affine_map_shell<Mesh_face>& map ) const
    {
        //zero the block 24x24
        for ( int i = 0; i < 576; ++i )
        {
            block[i] = 0.0;
        }

        //number of integration points
        size_t nint = map.q_rule().spacedim();
        //number of element nodes
        size_t nen = map.cell().size_of_nodes();
        //number of element's total degrees of freedom
        size_t ntotd = test().spacedim();
        //number of degrees of freedom per node
        size_t ned = ntotd/nen;
        //number of function element degrees of freedom
        size_t nfuncd = elements[0]->spacedim();

        int ind_offset = ntotd * nen;
        double con;

        //update form function coeficients (c[n_funct][n_deg_of_freedom]) for current map element
        update ( map );

        double *wgts = new double[nint];
        map.q_rule().weights ( wgts );

        //loop over integration points
        for ( int l = 0; l < nint; ++l )
        {
            //calculate desnity at integration point
            double rho_l = 0;
            for ( int b = 0; b < nen; ++b )
            {
                rho_l += map.shape_fun ( l,b ) *c[0][b];
            }

            //gauss weight, density and jacobian for given point
            con = wgts[l]*map.det ( l ) *rho_l;

            //construct matrix elements
            //loop over test element nodes
            for ( int b = 0; b < nen; ++b )
            {
                //loop over trial element nodes
                for ( int a = 0; a < nen; ++a )
                {
                    double res = map.shape_fun ( l,a ) * con * map.shape_fun ( l,b );
                    //index of the xx elements(b,a) block in the matrix
                    int ind = b*ntotd + a;
                    block[ind] += res;          //xx
                    ind  += ind_offset;
                    block[ind + nen] += res;    //yy
                    ind += ind_offset;
                    block[ind + 2*nen] += res;  //zz
                    ind += ind_offset;
                    block[ind + 3*nen] += res;  //dxdx
                    ind += ind_offset;
                    block[ind + 4*nen] += res;  //dydy
                    ind += ind_offset;
                    block[ind + 5*nen] += res;  //dzdz
                }
            }
        }

        delete [] wgts;

    }

    // No contribution from the boundary
    void eval_boundary ( double block[], const Affine_map_shell<Mesh_face>& map ) const {}

};
}
#endif
