// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _NEWTON_SOLVER_H_
#define _NEWTON_SOLVER_H_

#include "fem/Affine_map_shell.h"
#include "fem/BC_evaluator_list.h"
#include "fem/FEM.h"
#include "fem/fem_utils.h"

#include "solver/Elasticity_shell.h"
#include "solver/pressure_load.h"
#include "solver/boundary_conditions.h"

#include "mesh/mesh_manipulator.h"
#include "mesh/mesh_utils.h"
#include "mesh/mesh.h"

#include "matrix/SparseMatrix.h"
#include "matrix/DenseVector.h"
#include "matrix/UmfpackLUSolver.h"

#include "io/VTK_ostream.h"
#include "io/pvd_file.h"

#include "utils/string_utils.h"
#include "utils/globals.h"

#include "parameter/Parametrized.h"
#include "predicates/node_predicates.h"
#include <CGAL/Timer.h>

#include "postprocessing/stress_interpolator.h"
#include "postprocessing/postproc_utils.h"

#include "material/isotropic_material.h"
#include "material/fiber_manager.h"

//-----------------------------------------------------------------------------
class Newton_solver : public Parametrized
{
    double  m_Rtol;     // residual tolerance
    int m_niter;        // nr newton iterations

    PVD_file *m_result;
    PVD_file *m_internal;
    PVD_file *m_R_file;
    PVD_file *m_load;
    PVD_file *m_load_increment;
    PVD_file *m_marked_edges;
    PVD_file *m_cell_outline;

    std::ofstream *m_spreadsheet;
    std::ofstream *m_3dplot;
    bool m_detailed_info;

public:
    enum Output_type
    {
        result,
        internal,
        residual,
        load,
        load_increment,
        marked_edges,
        cell_outline,
        spreadsheet,
        threedplot
    };

    Newton_solver ( Affine_map_shell<Mesh_face> &a, Mesh_handle m );
    //reset state of the solver
    void init_step();
    void reset();
    void set_output(PVD_file &file, Output_type ot);
    void set_output(std::ofstream& ofs, Output_type ot);
    void set_print_details(bool b);
    void set_R_tolerance(double t);

    //return reference to the internal state interpolator
    State_interpolator & state_interpolator() {
        return stress_int;
    }
    //take n steps out of m total increments in load_case
    template<class BC, class EL, class LC>
    void solve ( int n, BC_evaluator_list<BC, EL> const& bcl, LC const& lc, int m);
    //solver which takes boundary condition evaluator list and load case
    template<class BC, class EL, class LC>
    void solve( int n, BC_evaluator_list<BC, EL> const& bcl, LC const& lc );

    SparseMatrix m_K;    // stiffness matrix
    DenseVector m_Fn;    // concentrated nodal force vector
    DenseVector m_Fr;    // nodal reaction forces
    DenseVector m_ui;    // displacement increment vector
    DenseVector m_Ui;    // Total displacement vector for iteration
    DenseVector m_Ut;    // Total dispalcement vector at time t (incl all previous timesteps)
    DenseVector m_R1;    // residual at iteration i


    Affine_map_shell<Mesh_face> &m_aff_map;
    Mesh_handle m_mesh;
    State_interpolator stress_int;
};
//-----------------------------------------------------------------------------
template<class BC, class EL, class LC>
void Newton_solver::solve( int n, BC_evaluator_list<BC, EL> const& bcl, LC const& lc)
{
    solve(n, bcl, lc, n);
}
//-----------------------------------------------------------------------------
template<class BC, class EL, class LC>
void Newton_solver::solve( int n, BC_evaluator_list<BC, EL> const& bcl, LC const& lc, int m )
{
    if (m_detailed_info)
        std::cout << "Newton solver\n";

    Mesh_manipulator manip ( m_mesh );
    VTK_ofstream vtk_out;
    CGAL::Timer timer;

    //incremental step for load factor
    double dlf = 1.0/m;
    //tolerance
    double tol = 1.0e-5;
    //init internal variables
    init_step();
    Elasticity_shell::Bilinear_form a;
    Elasticity_shell::Internal_forces IF;

    int fem_size = FEM::size ( *m_mesh, a.test() );
    int n_nodes = m_mesh->size_of_nodes();
    m_R1.init ( fem_size );//residual force
    m_R1 = 0.0;

    m_Ut.init ( fem_size );//total displacement
    m_Fn.init ( fem_size );//Load force
    m_Fn = 0.0;
    m_Fr.init ( fem_size );//internal reaction force
    m_Fr = 0.0;
    DenseVector Ftemp;//temporary storage for forces calculation
    Ftemp.init ( fem_size );
    Ftemp = 0.0;

    Function out_residual ( m_R1, *m_mesh, a.test() );
    Function out_load ( m_Fn, *m_mesh, a.test() );
    Function out_internal ( m_Fr, *m_mesh, a.test() );

    bool align_fibers = false;
    Fiber_manager *fiber_man = NULL;
    if (get("align_material_fibers_with_stress"))
    {
        fiber_man = new Align_with_stress(stress_int);
        align_fibers = true;
    }
    else if (get("align_material_fibers_with_strain"))
    {
        fiber_man = new Align_with_strain(stress_int);
        align_fibers = true;
    }
    double time = 0.0;
    double dt = dlf;
    for ( int i = 0; i < n; ++i )
    {
        time = lc.time();
        //pressure load forms
        Elasticity_shell::Pressure_load PL;
        Elasticity_shell::Pressure_stiffness PS;;
        stress_int.interpolate_tensors();
        if (m_result)
            m_result->write_tensors(stress_int, time);
        if (m_load)
            m_load->write_loads ( out_load, time );
        if (m_R_file)
            m_R_file->write_loads ( out_residual, time );
        if (m_internal)
            m_internal->write_loads ( out_internal, time );
        if (m_marked_edges)
            m_marked_edges->write_edges(m_mesh, time, true, 1);
        if (m_spreadsheet)
            write_spreadheet_timestep(*m_spreadsheet, m_mesh, stress_int, time);
        lc.update_time(time + dlf);
        if (m_detailed_info)
            std::cout << "incremented load factor time  = " << lc.time() << "\n";
        FEM::assemble ( a, m_K, *m_mesh, m_aff_map );
        //m_Fn is load force for current iteration, Ftemp from previous iteration
        Ftemp = m_Fn;
        FEM::assemble_surface_loads ( PL, m_Fn, lc, *m_mesh, m_aff_map );
        FEM::assemble_line_loads ( m_Fn, lc, *m_mesh, m_aff_map, false );
        FEM::assemble_point_loads ( m_Fn, lc, *m_mesh, m_aff_map, false );

        //iterator to the pointer to BC_evaluator
        typename BC_evaluator_list<BC, EL>::const_iterator bcl_iter, bcl_end;
        for ( bcl_iter = bcl.begin(), bcl_end = bcl.end(); bcl_iter != bcl_end; ++bcl_iter )
        {
            FEM::applyBC ( m_K, m_Fn, *m_mesh, m_aff_map, **bcl_iter );
        }
        //residual force increased by a step increment
        m_R1.vec() += ( m_Fn.vec()-Ftemp.vec() );
        double normF = norm ( m_Fn, n_nodes );
        double R_norm = norm ( m_R1, n_nodes );
        double prev_R_norm = R_norm;
        double U_norm = 0.0, u_norm = 0.0;
        m_Ui.init ( fem_size );//displacement total increment
        m_Ui.zero();
        int count = 0;
        //solve to the tolerance
        do
        {
            dt = 1.0e-7;
            UmfpackLUSolver umf_lu;
            umf_lu.set("LU report", m_detailed_info);

            m_ui.init ( fem_size );//displacement convergence step increment
            m_ui.zero();

            //set to identity bending degrees of freedonm
            if (!GLOBAL_BENDING)
            {
                m_K.unapply();
                FEM::Indent_bending_DOF ( m_K );
                m_K.apply();
            }
            
            m_K.compress();
            umf_lu.solve ( m_K, m_ui, m_R1 );
            //update the geomenty
            m_Ui.vec() += m_ui.vec();
            Function u ( m_ui, *m_mesh, a.test() );
            manip.displace_mesh ( u );

            //write the loads in updated geometry
            if (m_R_file)
                m_R_file->write_loads ( out_residual, time+dt );
            if (m_load)
                m_load->write_loads ( out_load, time+dt );
            //update the stiffness

            FEM::assemble ( a, m_K, *m_mesh, m_aff_map );
            //update the load
            FEM::assemble ( IF, m_Fr, *m_mesh, m_aff_map );
            FEM::assemble_surface_loads ( PL, m_Fn, lc, *m_mesh, m_aff_map );
            FEM::assemble_line_loads ( m_Fn, lc, *m_mesh, m_aff_map, false );
            FEM::assemble_point_loads ( m_Fn, lc, *m_mesh, m_aff_map, false );

            m_R1.vec().assign ( m_Fr.vec() +m_Fn.vec() );

            //iterator to the pointer to BC_evaluator
            for ( bcl_iter = bcl.begin(), bcl_end = bcl.end(); bcl_iter != bcl_end; ++bcl_iter )
            {
                FEM::applyBC ( m_K, m_Fn, *m_mesh, m_aff_map, **bcl_iter );
                FEM::applyBC_residual ( m_R1, *m_mesh, m_aff_map, **bcl_iter );
            }
            //find norm of F
            normF = norm ( m_Fn, n_nodes );
            U_norm = norm ( m_Ui, n_nodes );
            u_norm = norm ( m_ui, n_nodes );
            R_norm = norm ( m_R1, n_nodes );
            if (m_detailed_info)
            {
                std::cout <<"time = " << time << "\n";
                std::cout << "norm u= " << u_norm << "; norm U = " << U_norm << "; U tol = " << u_norm/U_norm << "\n";
                std::cout << "norm F = " << normF << ", norm R = " << R_norm << ", prev norm R = " << prev_R_norm << ", tol = " << R_norm/normF << "\n";
            }
            time +=dt;
            if (m_internal)
                m_internal->write_loads ( out_internal, time );
            if (count != 0 && R_norm > prev_R_norm)
            {
                //undo diplace mesh
                m_ui.toggle_sign();
                manip.displace_mesh ( u );
                cell_warning("Residual force norm tolerance couldnt be reached.");
                break;
            }
            prev_R_norm = R_norm;
            ++count;
        }
        while ( R_norm > m_Rtol);
        m_Ut += m_Ui;
        if (align_fibers)
        {
            stress_int.average_stress();
            fiber_man->align_material_fibers(m_mesh);
            align_material_fibers(m_mesh, stress_int);
        }
        ++m_niter;
    }
    if (m_R_file)
        m_R_file->write_loads ( out_residual, lc.time());
    if (m_internal)
        m_internal->write_loads ( out_internal, lc.time());

    stress_int.interpolate_tensors();
    if (m_result)
        m_result->write_tensors(stress_int, lc.time());
    if (m_marked_edges)
        m_marked_edges->write_edges(m_mesh, time, true, 1);
    if (m_spreadsheet)
        write_spreadheet_timestep(*m_spreadsheet, m_mesh, stress_int, lc.time());
    if (m_3dplot)
    {
        Isotropic_material const* mater = dynamic_cast<Isotropic_material const*>(m_aff_map.material());
        if (mater)
            write_3d_plot(*m_3dplot, m_mesh->node(0), stress_int, lc.time(), mater->nu(), mater->E());
        else
        {
            cell_error("Material not supported by the \"write_3d_plot\" routine\n.");
        }
    }
    delete fiber_man;
}
//-----------------------------------------------------------------------------
#endif
