// Copyright (C) 2007 Pawel Krupinski
// Licensed under the GNU GPL Version 2
#ifndef _BOUNDARY_CONDITIONS_H_
#define _BOUNDARY_CONDITIONS_H_
#include "fem/Boundary_condition.h"
#include "fem/Boundary_value.h"
#include "predicates/point_predicates.h"

//-----------------------------------------------------------------------------
class Marcus_setup : public Boundary_condition
{
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        double sy = 4.0;
        double sx = 4.0;
        double y_lim = 5.5;
        double x_lim = 8.5;
        if ( i==1 )
        {
            double y = p.y();
            if (y > y_lim)
                value = sy;
            else if (y < -y_lim)
                value = -sy;
        }
        if ( i==0 )
        {
            double x = p.x();
            if (x > x_lim)
                value = sx;
            else if (x < -x_lim)
                value = -sx;
        }
    }
};
//-----------------------------------------------------------------------------
class BarBC : public Boundary_condition
{
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        // Width of clamp
        const double w = 0.1;

        //height of the bar
        const double h = 1.0;

        // Center of rotation
        double x0 = 0.5;
        double y0 = 0.5;

        // Angle of rotation
        double theta = 0.5236; //(30 degrees)
        //double theta = 0.174532; //(10 degrees)

        // New coordinates
        double x = x0 + (p.x() - x0)*cos(theta) - (p.y() - y0)*sin(theta);
        double y = y0 + (p.x() - x0)*sin(theta) + (p.y() - y0)*cos(theta);

        // Clamp at the bottom
        if ( p.z() < w )
            value = 0.0;

        // Clamp at the top
        if ( p.z() > (h - w) )
        {
            if ( i == 0 )
                value = x - p.x();
            else if ( i == 1 )
                value = y - p.y();
            else if ( i == 2 )
                value = 0.0;
        }
    }
};
//-----------------------------------------------------------------------------
class DodecBC : public Boundary_condition
{
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        // Width of clamp
        const double w = 0.06;

        // Height of the bar
        const double h = 1.0;

        // Center of rotation
        double x0 = 0.0;
        double y0 = 0.0;

        // Angle of rotation
        double theta = 0.5236; //(30 degrees)
        //double theta = 0.174532; //(10 degrees)

        // New coordinates
        double x = x0 + (p.x() - x0)*cos(theta) - (p.y() - y0)*sin(theta);
        double y = y0 + (p.x() - x0)*sin(theta) + (p.y() - y0)*cos(theta);

        // Clamp at the bottom
        if ( p.z() < -(h - w) )
            value = 0.0;

        // Clamp at the top
        if ( p.z() > (h - w) )
        {
            if ( i == 0 )
                value = x - p.x();
            else if ( i == 1 )
                value = y - p.y();
        }
    }
};
//-----------------------------------------------------------------------------
class Clamp_cellBC : public Boundary_condition
{
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        // Width of clamp
        const double w = 0.1;

        // bottom z coordinate
        const double z_bott = -0.109469;
        // Height of the cell in z dir
        const double h = 0.24767;


        // Center of rotation
        double x0 = 0.0;
        double y0 = 0.0;

        // Angle of rotation
        double theta = 0.5236; //(30 degrees)
        //double theta = 0.174532; //(10 degrees)

        // New coordinates
        double x = x0 + (p.x() - x0)*cos(theta) - (p.y() - y0)*sin(theta);
        double y = y0 + (p.x() - x0)*sin(theta) + (p.y() - y0)*cos(theta);

        // Clamp at the bottom
        if ( p.z() < z_bott+w )
            value = 0.0;

        // Clamp at the top
        if ( p.z() > (z_bott + h - w) )
        {
            if ( i == 0 )
                value = x - p.x();
            //value = 0;
            else if ( i == 1 )
                value = y - p.y();
            //value = 0;
            else if ( i == 2 )
                value = 0.08;
        }
    }
};
//-----------------------------------------------------------------------------
class StrechBC : public Boundary_condition
{
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        // Width of clamp
        const double w = 0.1;

        // +-Height of the dodec
        const double h = 1.58;

        // Amount of streach
        const double s = 0.3;

        // New coordinates
        //double x = x0 + (p.x() - x0)*cos(theta) - (p.y() - y0)*sin(theta);
        //double y = y0 + (p.x() - x0)*sin(theta) + (p.y() - y0)*cos(theta);

        // Clamp at the bottom
        if ( p.z() < w )
            value = 0.0;

        // Clamp at the top
        if ( p.z() > (h - w) )
        {
            if ( i == 2 )
                value = s;
        }
    }
};
//-----------------------------------------------------------------------------
class Embryo_stem_bc : public Boundary_condition
{
public:
    Embryo_stem_bc( double z ): m_z_lim(z) {}
    void eval( Boundary_value& value, const Point& p, size_t i ) const
    {
        //no z displacement at the botom
        if ( p.z() < m_z_lim )
            if ( i == 2 )
                value = 0.0;
    }
private:
    double m_z_lim;
};
//-----------------------------------------------------------------------------
class PinPointBC : public Boundary_condition
{
public:
    PinPointBC(double x, double y, double z, double r = 0.1) :x0(x), y0(y), z0(z), r(r) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        double x = p.x();
        double y = p.y();
        double z = p.z();

        if ( (x-x0)*(x-x0)+(y-y0)*(y-y0)+(z-z0)*(z-z0) < r*r )
            value = 0.0;
    }
    //coordinates of point to pin
    double x0, y0, z0, r;

};
//-----------------------------------------------------------------------------
class ProliferationBC : public Boundary_condition
{
public:
    ProliferationBC( double z ) :z0(z) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i < 3)
        {
            double z = p.z();
            if ( z < z0 )
                value = 0.0;
        }
    }
    //coordinates of points to pin
    double z0;

};
//-----------------------------------------------------------------------------
class TestBC : public Boundary_condition
{
public:
    TestBC( double y1, double y2, double dy ) :my1(y1), my2(y2), mdy(dy) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i < 3)
        {
            double y = p.y();
            if ( y < my1 )
                value = 0.0;
            else if ( y > my2 && i == 1)
                value = mdy;
        }
    }
    double my1, my2, mdy;
};
//-----------------------------------------------------------------------------
///Fix coordinates of points beyond given range in x direction
class Fix_edgesX : public Boundary_condition
{
public:
    Fix_edgesX( double x1, double x2) :mx1(x1), mx2(x2) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
// 			if( i <3 )
        {
            double x = p.x();
            if ( x < mx1 || x > mx2)
                value = 0.0;
        }
    }
    double mx1, mx2;
};
//-----------------------------------------------------------------------------
///Fix X coordinates of points beyond given range in x direction
class FixX_edgesX : public Boundary_condition
{
public:
    FixX_edgesX( double x1, double x2, bool r = false) :mx1(x1), mx2(x2), rot(r) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 0 || (rot && i > 2))
        {
            double x = p.x();
            if ( x < mx1 || x > mx2)
                value = 0.0;
        }
    }
    double mx1, mx2;
    bool rot;
};
//-----------------------------------------------------------------------------
///Fix coordinates of points beyond given range in y direction
class Fix_edgesY : public Boundary_condition
{
public:
    Fix_edgesY( double y1, double y2) :my1(y1), my2(y2) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
// 			if( i <3 )
        {
            double y = p.y();
            if ( y < my1 || y > my2)
                value = 0.0;
        }
    }
    double my1, my2;
};
//-----------------------------------------------------------------------------
///Fix Y coordinates of points beyond given range in y direction
class FixY_edgesY : public Boundary_condition
{
public:
    FixY_edgesY( double y1, double y2, bool r = false) :my1(y1), my2(y2), rot(r) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 1 || (rot && i > 2))
        {
            double y = p.y();
            if ( y < my1 || y > my2)
                value = 0.0;
        }
    }
    double my1, my2;
    bool rot;
};
//-----------------------------------------------------------------------------
///Fix coordinates of points beyond given range in z direction
class Fix_edgesZ : public Boundary_condition
{
public:
    Fix_edgesZ( double z1, double z2) :mz1(z1), mz2(z2) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
// 			if( i <3 )
        {
            double z = p.z();
            if ( z < mz1 || z > mz2)
                value = 0.0;
        }
    }
    double mz1, mz2;
};
//-----------------------------------------------------------------------------
///fix x and/or y, z coordinates of nodes fulfiling the predicate
class Fix : public Boundary_condition
{
public:
    Fix(bool X, bool Y, bool Z, bool rot, Point_predicate const& pred): m_x(X), m_y(Y), m_z(Z), m_rot(rot), m_predicate(pred) {};
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if (m_predicate(p))
        {
            if ( (m_x && i == 0) || (m_y && i == 1) || (m_z && i == 2) || (m_rot && i > 2))
                value = 0.0;
        }
    }
    bool m_x, m_y, m_z, m_rot;
    Point_predicate const& m_predicate;
};
//-----------------------------------------------------------------------------
///scale x and/or y, z coordinates of nodes fulfiling the predicate
class Scale : public Boundary_condition
{
public:
    Scale(bool X, bool Y, bool Z, double s, Point_predicate const& pred): m_x(X), m_y(Y), m_z(Z), m_s(s), m_predicate(pred) 
    {
    };
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if (m_predicate(p))
        {
            if(m_x && i == 0)
              value = p.x()*m_s;
            else if(m_y && i == 1)
             value = p.y()*m_s;
            else if(m_z && i == 2)
              value = p.z()*m_s;
        }
    }
    bool m_x, m_y, m_z;
    double m_s;
    Point_predicate const& m_predicate;
};
//-----------------------------------------------------------------------------
///Clamp all coordinates of point
class Clamp_point : public Boundary_condition
{
public:
    Clamp_point() {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        value = 0.0;
    }
};
//-----------------------------------------------------------------------------
///fix z coodrinate displacement to zero
class Fix_Z : public Boundary_condition
{
public:
    Fix_Z() {};
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 2 )
        {
            value = 0.0;
        }
    }
};
//-----------------------------------------------------------------------------
class Radial_stretch_XY : public Boundary_condition
{
public:
    Radial_stretch_XY( double r, double mag ) : r2(r*r), magnitude(mag) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        double x = p.x();
        double y = p.y();
        double r  = x*x + y*y;
        double fact = magnitude/sqrt(r);

        if ( r > r2)
        {
            if ( i == 0 )
            {
                value = x * fact;
            }
            else if ( i == 1 )
            {
                value = y * fact;
            }
        }
    }
    //coordinates of xY center of the radial strech
    double r2;
    double magnitude;
};
//-----------------------------------------------------------------------------
class Radial_clamp_Z : public Boundary_condition
{
public:
    Radial_clamp_Z( double r = 0.0) : r2(r*r) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 2 )
        {
            double x = p.x();
            double y = p.y();
            double r  = x*x + y*y;

            if ( r >= r2)
            {
                value = 0.0;
            }
        }
    }
    //coordinates of xY center of the radial strech
    double r2;
};
//-----------------------------------------------------------------------------
class Clamp_Z : public Boundary_condition
{
public:
    Clamp_Z( double z = 0.0) : z(z) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 2 )
// 			if( i <3 )
        {
            if ( p.z() < z)
            {
                value = 0.0;
            }
        }
    }
    //coordinates of xY center of the radial strech
    double z;
};
//-----------------------------------------------------------------------------
class Radial_clamp_XY : public Boundary_condition
{
public:
    Radial_clamp_XY( double r = 0.0) : r2(r*r) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 0 || i ==1 )
        {
            double x = p.x();
            double y = p.y();
            double r  = x*x + y*y;

            if ( r > r2)
            {
                value = 0.0;
            }
        }
    }
    //coordinates of xY center of the radial strech
    double r2;
};
//-----------------------------------------------------------------------------
class Square_stretch_XY : public Boundary_condition
{
public:
    Square_stretch_XY( double r, double mag ) : r(r), magnitude(mag) {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        double var;
        if ( i == 0 )
            var = p.x();
        else if ( i == 1 )
            var = p.y();
        else
            return;
        if ( fabs(var) > r )
            value = sign(var) * magnitude;
    }
    double r;
    double magnitude;
};

//-----------------------------------------------------------------------------
class Stretch : public Boundary_condition
{
public:
  enum Coordinate {X, Y, Z};
  Stretch( Coordinate coor, double mag ) :  magnitude(mag) 
  {
    if(coor == X) ind = 0;
    else if(coor == Y) ind = 1;
    else if(coor == Z) ind = 2;
  }
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == ind )
            value = magnitude;
    }
    double magnitude;
    int ind;
};
//-----------------------------------------------------------------------------
class Stretch_all : public Boundary_condition
{
public:
  Stretch_all( double x, double y, double z ) :  x(x), y(y), z(z) 
  {}
private:
    void eval(Boundary_value& value, const Point& p, size_t i) const
    {
        if ( i == 0 )
            value = x;
        else if ( i == 1 )
            value = y;
        else if ( i == 2 )
            value = z;
    }
    double x,y,z;
};
//-----------------------------------------------------------------------------
#endif
