// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _DIRECT_BOUNDARY_CONDITIONS_H_
#define _DIRECT_BOUNDARY_CONDITIONS_H_

#include "fem/direct_boundary_condition.h"
#include "fem/Boundary_value.h"
#include "utils/error_macros.h"
#include <fstream>
//-----------------------------------------------------------------------------
class Final_mesh_boundary : public Direct_boundary_condition
{
public:

    Final_mesh_boundary ( std::ifstream &in, double t = 0 ) : m_treshold ( t )
    {
        if ( !in.good() )
            cell_error ( "Unusable input stream." );

        size_t id, n_points;
        double x, y, z;
        std::string text;

        in >> n_points >> text;
        if ( text != "nodes" )
            cell_error ( ( "Unexpected file format. " + text ).c_str() );

        m_final_points.resize ( n_points );

        for ( int i = 0; i < n_points; ++i )
        {
            in >> id >> text >> x >> y >> z;
            if ( text != ":" )
                cell_error ( ( "Unexpected file format. " + text ).c_str() );
            m_final_points[id] = Point ( x, y, z );
        }
    }

    Final_mesh_boundary ( Mesh const& m, double t = 0 ) : m_treshold ( t )
    {
        m_final_points.reserve ( m.size_of_nodes() );
        Mesh::Node_const_iterator n_iter, n_end;
        for ( n_iter = m.nodes_begin(), n_end = m.nodes_end(); n_iter != n_end; ++n_iter )
        {
            m_final_points.push_back ( ( n_iter )->point() );
        }
    }

    void eval ( Boundary_value& value, Mesh_node const& n, size_t i ) const
    {
        size_t ind = n.id();
        Point const& p = n.point();

        switch ( i )
        {
            double val;
        case 0:
            val = m_final_points[ind].x() - p.x();
            if ( val > m_treshold )
                value = val;
            break;
        case 1:
            val = m_final_points[ind].y() - p.y();
            if ( val > m_treshold )
                value = val;
            break;
        case 2:
            val = m_final_points[ind].z() - p.z();
            if ( val > m_treshold )
                value = val;
        }
    }

private:
    std::vector<Point> m_final_points;
    double m_treshold;
};
//-----------------------------------------------------------------------------
///Clamp coordinates of node
class Clamp_node : public Direct_boundary_condition
{
public:
    Clamp_node() {}
private:
    void eval ( Boundary_value& value, Mesh_node const& n, size_t i ) const
    {
        value = 0.0;
    }
};
//-----------------------------------------------------------------------------
///fix x and/or y, z coordinates of nodes fulfiling the predicate
class Fix_nodes : public Direct_boundary_condition
{
public:
    Fix_nodes ( bool X, bool Y, bool Z, bool rot ) : m_x ( X ), m_y ( Y ), m_z ( Z ), m_rot ( rot ) {};
private:
    void eval ( Boundary_value& value, Mesh_node const& n, size_t i ) const
    {
        if ( ( m_x && i == 0 ) || ( m_y && i == 1 ) || ( m_z && i == 2 ) || ( m_rot && i > 2 ) )
            value = 0.0;
    }
    bool m_x, m_y, m_z, m_rot;
};

//-----------------------------------------------------------------------------
class Stretch_nodes : public Direct_boundary_condition
{
public:
    enum Coordinate {X, Y, Z};
    Stretch_nodes ( Coordinate coor, double mag ) : magnitude ( mag )
    {
        if ( coor == X ) ind = 0;
        else if ( coor == Y ) ind = 1;
        else if ( coor == Z ) ind = 2;
    }
private:
    void eval ( Boundary_value& value, Mesh_node const& n, size_t i ) const
    {
        if ( i == ind )
            value = magnitude;
    }
    double magnitude;
    int ind;
};
//-----------------------------------------------------------------------------
#endif
