// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "solver/newmark_solver.h"
#include "io/pvd_file.h"
#include "fem/FEM.h"
#include "matrix/DenseVector.h"
#include "solver/Elasticity_shell.h"
#include "solver/Mass_matrix_shell.h"

//-----------------------------------------------------------------------------
Newmark_solver::Newmark_solver ( Affine_map_shell<Mesh_face> &map, Mesh_handle m, double b, double g )
    : m_map ( map ), m_mesh ( m ), m_forms_local ( false ), beta ( b ), gamma ( g ), m_stress_int ( m, map ), m_detailed ( false ), m_file_output ( false )
{
    init ();
    read_parameters();
    add ( "fix centroid", false );
}
//-----------------------------------------------------------------------------
Newmark_solver::~Newmark_solver()
{

}

//-----------------------------------------------------------------------------
void Newmark_solver:: init ( Function *u0, Function *v0 )
{
    Finite_element const& el = m_map.element();
    size_t s = FEM::size ( *m_mesh, el );
    ui_v.resize ( s );
    uf_v.resize ( s );
    vi_v.resize ( s );
    vf_v.resize ( s );
    ai_v.resize ( s );
    af_v.resize ( s );
    force.resize ( s );
    load.resize ( s );
    internal_force.resize ( s );
    K.init ( s, s, 0 );
    C.init ( s, s, 0 );
    M.init ( s, s, 0 );

    //attach vectors to functions for convenience

    m_u.attach ( uf_v, false );
    m_u.attach ( *m_mesh, false );
    m_u.attach ( el, false );

    m_v.attach ( vf_v, false );
    m_v.attach ( *m_mesh, false );
    m_v.attach ( el, false );

    m_a.attach ( af_v, false );
    m_a.attach ( *m_mesh, false );
    m_a.attach ( el, false );

    //copy info from input functions
    if ( u0 )
        get_function_vector ( *u0, ui_v );
    else
        ui_v.zero();
    if ( v0 )
        get_function_vector ( *v0, vi_v );
    else
        vi_v.zero();
    uf_v = ui_v;
    vf_v = vi_v;

    m_time = 0.0;
}
//-----------------------------------------------------------------------------
void Newmark_solver::read_parameters()
{
    // Set tolerances and other parameters
    rtol    = get ( "Newmark relative tolerance" );
    atol    = get ( "Newmark absolute tolerance" );
    t_step  = get ( "Newmark time step" );
    max_it  = get ( "Newmark maximum iterations" );
    report  = get ( "Newmark report" );

    m_dump_a  = get ( "Rayleigh damping a" );
    m_dump_b  = get ( "Rayleigh damping b" );
    beta = get ( "Newmark beta" );
    gamma = get ( "Newmark gamma" );
    // Remember that we have read parameters
    parameters_read = true;
}
//-----------------------------------------------------------------------------
void Newmark_solver::construct_forms ( Function &rho )
{
    if ( m_forms_local )
    {
        delete m_stiff;
        delete m_mass;;
        delete m_press_stiff;
        delete m_press;
        delete m_internal_force;
    }
    m_stiff = new Elasticity_shell::Bilinear_form();
    m_mass  = new Mass_matrix_shell::Bilinear_form ( rho );
    m_press_stiff = new Elasticity_shell::Pressure_stiffness();
    m_press = new Elasticity_shell::Pressure_load();
    m_internal_force = new Elasticity_shell::Internal_forces();
    m_forms_local = true;
}
//-----------------------------------------------------------------------------
void Newmark_solver::explicit_step_first_order ( double dt )
{
    //predictors
    noalias ( uf_v.vec() ) = ui_v.vec() + ( 1.0-gamma ) *dt*vi_v.vec();

    //noalias(af_v) = force - C*vf_v - A*uf_v;
    //calculate new velocity
    K.mult ( uf_v, vf_v );
    vf_v.vec() = force.vec() - vf_v.vec();
    vf_v.div ( lump_m );
    //calculate final displacement
    uf_v.vec() += gamma * dt * vf_v.vec();
}
//-----------------------------------------------------------------------------
int Newmark_solver::solve_explicit_overdamped ( int n_steps, double dt, PVD_file & file, int n_out )
{
//no acceleration. first order equation solved
    //figure out if to keep mesh centroid at the sane position at each step
    Point centr ( 0,0,0 );
    Vector trans ( 0,0,0 );
    bool fix_centr = get ( "fix centroid" );
    if ( fix_centr )
        centr = centroid ( *m_mesh );
    Function v_out ( vf_v, *m_mesh, m_map.element() );

    file.attach_vector_function ( v_out, "velocity" );
    read_parameters();
    apply_BC ( m_bc_list, K, force );

    double tot, rel;

    M.lump ( lump_m );
//calculate initial v (uf, vf play the role of initial values here)
    K.mult ( uf_v, vf_v );
    vf_v.vec() -= force.vec();
    vf_v.toggle_sign();
    vf_v.div ( lump_m );
    //output initial mesh only for very first step
    if ( m_time == 0.0 )
    {
        if ( m_file_output )
        {
            if ( m_detailed )
                std::cout << "time: " << m_time << "\n";
            file.write ( m_mesh, m_time, &m_stress_int );
        }
    }


    double incr = double ( n_steps ) /n_out;
    int i_out = incr;
    int i = 0, count = 1;
//     std::cout << "rtot = "  << rtol <<"; ";
//     std::cout << "atol = "  << atol <<"\n";
    do
    {
//       std::cout << "step " << i << "\n";
        ui_v = uf_v;
        vi_v = vf_v;

        explicit_step_first_order ( dt );
        if ( fix_centr )
        {
            trans = Vector ( centroid ( *m_mesh, uf_v ), centr );
            int offset = m_mesh->size_of_nodes();
            Mesh::Node_const_iterator it, it_end;
            for ( it = m_mesh->nodes_begin(), it_end = m_mesh->nodes_end(); it != it_end; ++it )
            {
                int id = it->id();
                uf_v ( id ) += trans.x();
                uf_v ( id+offset ) += trans.y();
                uf_v ( id+offset+offset ) += trans.z();
            }
        }
        apply_a_v_BC ( m_bc_list, af_v, vf_v );
        apply_u_BC ( m_bc_list, uf_v );
        m_time += dt;
        ui_v -= uf_v;
        tot = ui_v.norm ( DenseVector::linf );
        rel = tot/uf_v.norm ( DenseVector::linf );
        if ( i == i_out )
        {
//           std::cout << "output...\n";
            //data.add( dt*i, m_u, count);
//             m_stress_int.interpolate_stress();
            m_stress_int.interpolate_tensors();
            if ( m_detailed )
            {
                std::cout << "time: " << m_time << "\n";
                std::cout << "tot = "  << tot <<"; ";
                std::cout << "rel = "  << rel <<"\n";
            }
            if ( m_file_output )
            {
                file.write ( m_mesh, m_u, m_time, &m_stress_int );
            }
            ++count;
            i_out = count*incr;
        }
        ++i;
    }
    while ( i < n_steps && tot > atol && rel > rtol );
    if ( i >= n_steps )
        std::cout << "maximal number of steps (" << n_steps << ") reached.\n";

    m_stress_int.interpolate_tensors();
    if ( m_detailed )
    {
        std::cout << "time: " << m_time << "\n";
        std::cout << "tot = "  << tot <<"; ";
        std::cout << "rel = "  << rel <<"\n";
    }
    if ( m_file_output )
    {
        file.write ( m_mesh, m_u, m_time, &m_stress_int );
    }
    return i;
}
//-----------------------------------------------------------------------------
int Newmark_solver::solve_explicit_overdamped(int n_steps, double dt)
{
  set_file_output(false);
  PVD_file dummy("dummy");
  solve_explicit_overdamped(n_steps, dt, dummy, 1);
}
//-----------------------------------------------------------------------------
void Newmark_solver::assemble_matrices ()
{
    assert ( m_stiff != NULL && m_mesh != NULL );
    FEM::assemble ( *m_stiff, K, *m_mesh, m_map );
    FEM::assemble ( *m_mass, M, *m_mesh, m_map );
}
//-----------------------------------------------------------------------------
void Newmark_solver::assemble_load ( Load_case * lc, double load_factor )
{
    bool init = true;
    lc->update_time ( load_factor );
    if ( m_press != NULL )
    {
        FEM::assemble_surface_loads ( *m_press, load, *lc, *m_mesh, m_map );
        init = false;
    }
    if ( m_press_stiff != NULL )
    {
        FEM::assemble_surface_loads ( *m_press_stiff, K, *lc, *m_mesh, m_map, init );
    }
    FEM::assemble ( *m_internal_force, internal_force, *m_mesh, m_map, true );
    FEM::assemble_line_loads ( load, *lc, *m_mesh, m_map, init );
    FEM::assemble_point_loads ( load, *lc, *m_mesh, m_map, false );
    force.vec() = internal_force.vec() +load.vec();
}
//-----------------------------------------------------------------------------
void Newmark_solver::apply_BC ( BC_list const* bc, SparseMatrix &A, DenseVector &f )
{
    BC_list::const_iterator bcl_iter, bcl_end;
    for ( bcl_iter = m_bc_list->begin(), bcl_end = m_bc_list->end(); bcl_iter != bcl_end; ++bcl_iter )
    {
        FEM::applyBC ( A, f, *m_mesh, m_map, **bcl_iter );
    }
}
//-----------------------------------------------------------------------------
void Newmark_solver::apply_u_BC ( BC_list const* bc, DenseVector &u )
{
    BC_list::const_iterator bcl_iter, bcl_end;
    for ( bcl_iter = m_bc_list->begin(), bcl_end = m_bc_list->end(); bcl_iter != bcl_end; ++bcl_iter )
    {
        FEM::applyBC ( u, *m_mesh, m_map, **bcl_iter );
    }
}
//-----------------------------------------------------------------------------
void Newmark_solver::apply_a_v_BC ( BC_list const* bc, DenseVector &a, DenseVector &v )
{
    BC_list::const_iterator bcl_iter, bcl_end;
    for ( bcl_iter = m_bc_list->begin(), bcl_end = m_bc_list->end(); bcl_iter != bcl_end; ++bcl_iter )
    {
        Boundary_value bv;
        //number of node degrees of freedom
        size_t n = m_map.element().tensordim();
        //indices of node dofs in the matrix/vector
        size_t* nodes = new size_t[n];

        BC_list::Evaluator::Node_iterator n_iter, n_end;

        for ( n_iter = ( *bcl_iter )->nodes_begin(), n_end = ( *bcl_iter )->nodes_end(); n_iter != n_end; ++n_iter )
        {
            Mesh_node *vert = &*n_iter;
            m_map.vertexmap ( nodes, vert, *m_mesh );
            for ( int i = 0; i < n; ++i )
            {
                bv.reset();
                ( *bcl_iter )->eval ( bv, *vert, i );
                //zero acceleration and velocity for Dirichlet boundary conditions
                if ( bv.is_set() )
                {
                    int node = nodes[i];
                    a ( node ) = 0.0;
                    v ( node ) = 0.0;
                }
            }
        }
        // Delete data
        delete [] nodes;
    }
}
//-----------------------------------------------------------------------------
void Newmark_solver::assemble_BC ( BC_list const* bc )
{
    apply_BC ( bc, K, force );
}
//-----------------------------------------------------------------------------
void Newmark_solver::assemble_step_BC ( BC_list const* bc )
{
    apply_a_v_BC ( bc, af_v, vf_v );
}
//-----------------------------------------------------------------------------
void Newmark_solver::get_function_vector ( Function &f, DenseVector &v )
{
    switch ( f.type() )
    {
    case Function::constant:
        v = f ( 0,0 ); //construct a constant vector
        break;
    case Function::discrete:
        v = f.vector(); //get the actual vector associated to function
        break;
    default:
        cell_error ( "Function type not supported in Newmark solver.\n" );
    }
}
//-----------------------------------------------------------------------------
#undef DEBUG_PRINT
