// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _NEWMARK_SOLVER_H_
#define _NEWMARK_SOLVER_H_

#include "matrix/DenseVector.h"
#include "matrix/uBlasVector.h"
#include "matrix/SparseMatrix.h"
#include "fem/Function.h"
#include "fem/Bilinear_form.h"
#include "fem/Linear_form.h"
#include "fem/FEM.h"
#include "parameter/Parametrized.h"
#include "fem/BC_evaluator_list.h"
#include "io/pvd_file.h"
#include "postprocessing/stress_interpolator.h"
#include "mesh/mesh_manipulator.h"

class PVD_file;
class Load_case;
//-----------------------------------------------------------------------------
class Newmark_solver: public Parametrized
{
public:
    typedef BC_evaluator_list<Boundary_condition, Mesh_face> BC_list;
    ///create Newmark solver with default forms
    Newmark_solver ( Affine_map_shell<Mesh_face> &map, Mesh_handle m, double b = 0, double g = 0.5 );
    ~Newmark_solver();
    //single step of explicit solver with just velocity update (no acceleration)
    void explicit_step_first_order ( double dt ); 
    int solve_explicit_overdamped(int n_steps, double dt, PVD_file & file1, int n_out);
int solve_explicit_overdamped(int n_steps, double dt);
    void setup_case(Load_case const* lc, BC_list const* bc)
    {
      m_load_case = lc;
      m_bc_list = bc;
    }
    
    void read_parameters();
    Function & get_u()
    {
        return m_u;
    }
    Function & get_v()
    {
        return m_v;
    }
    Function & get_a()
    {
        return m_a;
    }
    DenseVector& get_force_v()
    {
        return force;
    }
    DenseVector& get_internal_force_v()
    {
        return internal_force;
    }
    DenseVector& get_load_v()
    {
        return load;
    }
    double time() const {return m_time;}
    ///initialize matrices and vectors to correct size
    void init( Function *u0 = NULL, Function *v0 = NULL );
    ///construct the form needed for evaluation of matrices and vectors
    void construct_forms(Function &rho);
    ///assemble K, C and M matrices for curent configuration
    void assemble_matrices ();
    void assemble_load( Load_case * lc, double load_factor );
    void assemble_BC ( BC_list const* bc );
    void assemble_step_BC ( BC_list const* bc );
    State_interpolator& state_interpolator() {return m_stress_int;}
    void set_detailed_info( bool b) {m_detailed = b;}
    void set_file_output( bool b) {m_file_output = b;}
private:
    

    void apply_BC( BC_list const* bc, SparseMatrix &A, DenseVector &f );
    ///apply boundy conditions for displacement
    void apply_u_BC ( BC_list const* bc, DenseVector &u);
    ///apply boundy conditions for acceleration and velocity
    void apply_a_v_BC ( BC_list const* bc, DenseVector &a, DenseVector &v );
    
//     void add_default_parameters();
    bool is_discrete_function ( Function const& f );
    void get_function_vector ( Function &f, DenseVector &v );
    DenseVector ui_v;
    DenseVector uf_v;
    DenseVector vi_v;
    DenseVector vf_v;
    DenseVector ai_v;
    DenseVector af_v;

    DenseVector lump_m;
    DenseVector lump_c;
    DenseVector lump_total;
    DenseVector force;
    DenseVector load;
    DenseVector internal_force;

    Function m_u;
    Function m_v;
    Function m_a;

    double beta;
    double gamma;
    double m_time;
    //
    Bilinear_form *m_stiff, *m_mass, *m_press_stiff;
    Linear_form *m_press, *m_internal_force;
    bool m_forms_local;
    //print detailed info at each step
    bool m_detailed;
 
    bool m_file_output;
    //effective stiffness matrix
    SparseMatrix K;
    //mass matrix updated for each step
    SparseMatrix M;
    //damping matrix updated for each step
    SparseMatrix C;
    //solver parameters
    double rtol, atol ,t_step;
    int max_it;
    bool parameters_read, report;
    Affine_map_shell<Mesh_face> & m_map;
    Mesh_handle m_mesh;
    //load case
    Load_case const* m_load_case;
    //boundary condition evaluator list
    BC_list const* m_bc_list;
    //Rayleigh damping parameters
    double m_dump_a;
    double m_dump_b;
    State_interpolator m_stress_int;
};
//-----------------------------------------------------------------------------
inline bool is_discrete_function ( Function const& f )
{
    Discrete_function const* test_f = dynamic_cast<Discrete_function const*> ( &f );
    return ( test_f != NULL );
}
//-----------------------------------------------------------------------------
#endif
