// Copyright (c) 2003  Utrecht University (The Netherlands),
// ETH Zurich (Switzerland), Freie Universitaet Berlin (Germany),
// INRIA Sophia-Antipolis (France), Martin-Luther-University Halle-Wittenberg
// (Germany), Max-Planck-Institute Saarbruecken (Germany), RISC Linz (Austria),
// and Tel-Aviv University (Israel).  All rights reserved.
//
// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with CGAL.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/STL_Extension/include/CGAL/Iterator_project.h $
// $Id: Iterator_project.h 28567 2006-02-16 14:30:13Z lsaboret $
//
//
// Author(s)     : Michael Hoffmann <hoffmann@inf.ethz.ch>
//                 Lutz Kettner <kettner@mpi-sb.mpg.de>
//                 Sylvain Pion <Sylvain.Pion@sophia.inria.fr>
// Modified by Pawel Krupinski 2008
#ifndef _ITERATOR_PROJECTOR_H_
#define _ITERATOR_PROJECTOR_H_
//-----------------------------------------------------------------------------
// Relies on iterator traits. Quite simplified compared to earlier version.

// The pointer type and the reference type in the Iterator_projector
// are based on the value type from the projector, but the base iterator
// determines whether they are const or mutable. The following template
// class and its partial specialization helps creating the derived types.
//-----------------------------------------------------------------------------
template < class I, class Fct >
class Iterator_projector
{
//-----------------------------------------------------------------------------
    // If T === T1 return R1 else return R2
    template <class T, class T1, class R1, class R2>
    struct I_TYPE_MATCH_IF
    {
        typedef R2 Result;
    };  // else clause
    template <class T, class R1, class R2>
    struct I_TYPE_MATCH_IF<T,T,R1,R2>
    {
        typedef R1 Result;
    }; // then clause
//-----------------------------------------------------------------------------
protected:
    I        m_internal;    // The internal iterator.
public:
    typedef Iterator_projector<I,Fct> 				Self;
    typedef I										Iterator; // base iterator
    typedef std::iterator_traits<I>				traits;
    typedef typename traits::difference_type		difference_type;
    typedef typename traits::iterator_category	iterator_category;
    typedef typename traits::value_type			base_value_type;
    typedef typename traits::pointer				base_pointer;
    typedef typename traits::reference			base_reference;

    typedef typename Fct::argument_type			argument_type;
    typedef typename Fct::result_type			value_type;

    // Use I_TYPE_MATCH_IF to find correct pointer and reference type.
    typedef I_TYPE_MATCH_IF< base_reference, const base_value_type &, const value_type &, value_type &> Match1;
    typedef typename Match1::Result	reference;

    typedef I_TYPE_MATCH_IF< base_pointer, const base_value_type *, const value_type *, value_type *> Match2;
    typedef typename Match2::Result	pointer;

    Iterator_projector() {}
    Iterator_projector ( I j ) : m_internal ( j ) {}

// make two iterators assignable if the underlying iterators are
    template <class I2>
    Iterator_projector ( const Iterator_projector<I2,Fct> &i2 ) : m_internal ( i2.current_iterator() )
    {
        /*std::cout << "iteator projector\n";*/
    }

    template <class I2>
    Self& operator= ( const Iterator_projector<I2,Fct> &i2 )
    {
        m_internal = i2.current_iterator();
        return *this;
    }

// OPERATIONS Forward Category

    Iterator  current_iterator() const
    {
        return m_internal;
    }
    const pointer   ptr() const
    {
        return ptr();
    }
    pointer   ptr()
    {
        Fct fct;
// 		std::cout << "returning pointer\n";
        return & ( fct ( *m_internal ) );
    }
    bool operator== ( const Self& i ) const
    {
        return ( m_internal == i.m_internal );
    }
    bool operator!= ( const Self& i ) const
    {
        return ! ( *this == i );
    }
    reference operator* () const
    {
        Fct fct;
        fct ( *m_internal );
    }
    pointer operator->() const
    {
        return ptr();
    }
    Self& operator++()
    {
        ++m_internal;
        return *this;
    }

    Self operator++ ( int )
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }

    // OPERATIONS Bidirectional Category


    Self& operator--()
    {
        --m_internal;
        return *this;
    }
    Self  operator-- ( int )
    {
        Self tmp = *this;
        --*this;
        return tmp;
    }

    // OPERATIONS Random Access Category

    Self& operator+= ( difference_type n )
    {
        m_internal += n;
        return *this;
    }
    Self  operator+ ( difference_type n ) const
    {
        Self tmp = *this;
        return tmp += n;
    }
    Self& operator-= ( difference_type n )
    {
        return operator+= ( -n );
    }
    Self  operator- ( difference_type n ) const
    {
        Self tmp = *this;
        return tmp += -n;
    }
    difference_type  operator- ( const Self& i ) const
    {
        return m_internal - i.m_internal;
    }
    reference  operator[] ( difference_type n ) const
    {
        Self tmp = *this;
        tmp += n;
        return tmp.operator*();
    }
    bool operator< ( const Self& i ) const
    {
        return ( m_internal < i.m_internal );
    }
    bool operator> ( const Self& i ) const
    {
        return i < *this;
    }
    bool operator<= ( const Self& i ) const
    {
        return ! ( i < *this );
    }
    bool operator>= ( const Self& i ) const
    {
        return ! ( *this < i );
    }
};
//-----------------------------------------------------------------------------
template < class Dist, class Fct, class I >
inline Iterator_projector<I,Fct> operator+ ( Dist n, Iterator_projector<I,Fct> i )
{
    return i += n;
}
//-----------------------------------------------------------------------------
#endif
