#ifndef _CELL_UTILITY_H_
#define _CELL_UTILITY_H_

#include <iostream>

#if defined(WIN32) || defined(_WIN32) || defined(Win32)
# if !defined(WIN32)
#  define WIN32
# endif
#include <conio.h>
#endif

#include "utils/types.h"

#include "utils/direction.h"
#include <CGAL/enum.h>
#include <CGAL/Origin.h>

//emulate getch for non windows
#ifndef WIN32
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include "matrix/DenseVector.h"

class Mesh;

inline int getch( )
{
    struct termios oldt, newt;
    int ch;
    tcgetattr( STDIN_FILENO, &oldt );
    newt = oldt;
    newt.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newt );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
    return ch;
}
#endif

bool fuzzy_equal(const Point &pA, const Point &pB);
//-----------------------------------------------------------------------------
template <class T>
struct Pair
{
    T first;
    T second;
};
//-----------------------------------------------------------------------------
inline void wait_kb_hit()
{
    std::cout << "Press any key to continue..." << std::endl;
    std::cout.flush();
    getch();
}
//-----------------------------------------------------------------------------
inline CGAL::Comparison_result fuzzy_comp(const double dA, const double dB)
{
    double dSum  = fabs(dA) + fabs(dB);
    double dDiff = dA - dB;
    const double dTol = 5.e-9;
    const double dFloor = 4.e-5;

    dSum = (dSum > dFloor) ? dSum : dFloor;
    if (dDiff > dTol * dSum) return (CGAL::LARGER);
    else if (dDiff < - dTol * dSum) return (CGAL::SMALLER);
    else return (CGAL::EQUAL);
}
//-----------------------------------------------------------------------------
inline bool fuzzy_equal(const double dA, const double dB)
{
    return fuzzy_comp( dA, dB ) == CGAL::EQUAL;
}
//-----------------------------------------------------------------------------
template <class T>
inline T sgn(T num)
{
    if ( num >= 0.0 )
        return 1;
    else
        return -1;
}
//-----------------------------------------------------------------------------
//calculates centroid of mesh, mesh edge, face and element. Relays on node_iterator within type T
template< class T >
Point centroid( T const &meshT )
{
    typedef typename T::Node_const_iterator NCI;
    Vector v= CGAL::NULL_VECTOR;
    size_t nb_points = 0;
    for ( NCI iter = meshT.nodes_begin(); iter != meshT.nodes_end(); ++iter )
    {
        v = v + ( iter->point() - CGAL::ORIGIN );
        ++nb_points;
    }
    return CGAL::ORIGIN + v / nb_points;
}
//-----------------------------------------------------------------------------
//calculates centroid of mesh with displacement function vector explicitly defined
Point centroid( Mesh const& mesh, DenseVector const& u );
//-----------------------------------------------------------------------------
template< class Iter >
Point centroid( Iter from, Iter beyond )
{
    Iter i;
    Vector v = CGAL::NULL_VECTOR;
    size_t nb_points = 0;
    for ( i = from; i != beyond; ++i )
    {
        v = v + ( *i - CGAL::ORIGIN );
        ++nb_points;
    }
    return CGAL::ORIGIN + v / nb_points;
}
//-----------------------------------------------------------------------------
//check if point lies on the plane
struct Has_on
{
    Has_on():EPS(1.E-6) {}
    bool operator()( const Plane &plane, const Point& p );
    //allowed discrepancy in satisfing plane equation by the point
    double EPS;
};
//-----------------------------------------------------------------------------
inline int sign( double d )
{
    return d<0.0 ? -1 : 1;
}
//-----------------------------------------------------------------------------
inline Direction reverse( Direction dir )
{
    if ( dir == DIR_REVERSED )
        return DIR_ORIGINAL;
    else if ( dir ==  DIR_ORIGINAL )
        return DIR_REVERSED;
    else
        return DIR_INVALID;
}
//-----------------------------------------------------------------------------
inline std::streambuf* redirect ( std::ostream& in, std::ostream& out )
{
    std::streambuf* in_buffer = in.rdbuf();
    in.rdbuf( out.rdbuf() );
    return in_buffer;
}
//-----------------------------------------------------------------------------
inline int round_pos(double x)
{
    return (int)(x + 0.5);
}
//-----------------------------------------------------------------------------
template<class Continer >
size_t size_of(Continer &c)
{
    return c.size();
}
//-----------------------------------------------------------------------------
template<class Handle, class Continer >
void add_entity( Handle h, Continer &c )
{
    c.push_back(h);
    h->set_id( size_of(c)-1 );
}
//-----------------------------------------------------------------------------
template<class Iterator, class Continer >
Iterator erase_entity( Iterator i, Continer &c )
{
    delete i.ptr();
    return c.erase( i.current_iterator() );
}
//-----------------------------------------------------------------------------
template<class Handle, class Container >
Handle delete_entity( Handle h, Container &c )
{
    int id = h->id();
    Handle n_h = c.back();
    n_h->set_id(id);
    c[id] = n_h;
    c.pop_back();
    delete h.ptr();
    return n_h;
}
//-----------------------------------------------------------------------------
template<class Iter >
void reset_flags( Iter from, Iter beyond )
{
    while (from != beyond)
    {
        from->reset_flag();
        ++from;
    }
}
//-----------------------------------------------------------------------------
template<class Iter, class F >
void set_flags( Iter from, Iter beyond, F flag )
{
    while (from != beyond)
    {
        from->set_flag(flag);
        ++from;
    }
}
//-----------------------------------------------------------------------------
//terribly slow algorithm for gettin an intersection of two unsorted ranges
template <class InputIterator1, class InputIterator2, class OutputIterator>
OutputIterator range_intersection ( InputIterator1 first1, InputIterator1 last1, InputIterator2 first2, InputIterator2 last2, OutputIterator result )
{
    InputIterator2 start2 = first2;
    while (first1!=last1)
    {
        first2 = start2;
        while (first2!=last2)
        {
            if (*first1 != *first2) ++first2;
            else {
                *result++ = *first2++;
            }
        }
        ++first1;
    }
    return result;
}
//-----------------------------------------------------------------------------
#endif

